/*
 * @(#)AboutFrame.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JDialog;

import olivamail.gui.Strings;

import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.lang.Package;

public class AboutFrame extends JDialog {

	static final long serialVersionUID = 92416942075804424L;
	private final String WIN_ID = "about";
	private JPanel jContentPane = null;
	private JButton okButton = null;
	private JEditorPane jEditorPaneAbout = null;
	private JPanel jPanelAbout = null;
	private JPanel jPanelOk = null;
	private final String na = "&lt;N/A&gt;"; //$NON-NLS-1$
	/**
	 * This is the default constructor
	 */
	public AboutFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(336, 342);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("AboutFrame.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	private String getCleanString(String str) {
		if(str == null)
			return na;
		if(str.length() == 0)
			return na;
		if(str.length() > 32)
			str = str.substring(0, 33);
		str = str.replaceAll("\\$", "#");		
		return str;
	}
	
	private String getAboutInfo() {
		String clientAbout = Context.clientAbout;
		
		try {
			String value = null;
			value = Context.storage.getDatabaseProductName();
			value = getCleanString(value);			
			clientAbout = clientAbout.replaceFirst("\\$dbname", value);

			value = null;
			value = Context.storage.getDatabaseProductVersion();
			value = getCleanString(value);
			clientAbout = clientAbout.replaceFirst("\\$dbver", value);

			value = null;
			value = Context.storage.getDriverName();
			value = getCleanString(value);			
			clientAbout = clientAbout.replaceFirst("\\$jdbcname", value);

			value = null;
			value = Context.storage.getDriverVersion();
			value = getCleanString(value);
			clientAbout = clientAbout.replaceFirst("\\$jdbcver", value);
		} catch (Exception e) {
			e.printStackTrace();
		}		
		
		String[][] pkgs = {{"\\$guiver", "\\$corever", "\\$javamailver"}, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		{"olivamail.gui", "olivamail.core.mail", "javax.mail"}}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		Package pkg = null;
		String ver = null;
		for(int i = 0; i < pkgs[1].length; i++) {
			ver = null;
			pkg = null;
			if(pkgs[1][i] != null) {
				pkg = Package.getPackage(pkgs[1][i]);
				if(pkg != null)
					ver = pkg.getSpecificationVersion();				
			}							
			if(ver == null)
				ver = na;
			clientAbout = clientAbout.replaceFirst(pkgs[0][i], ver);			
		}
		String vendor = null;
		if(pkg != null)
			vendor = pkg.getSpecificationVendor();
		if(vendor == null)
			vendor = na;
		clientAbout = clientAbout.replaceFirst("\\$jdbcvendor", vendor); //$NON-NLS-1$
		return clientAbout;
	}
	
	public void center() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        Point center = ge.getCenterPoint();
        Rectangle bounds = ge.getMaximumWindowBounds();
        int w = Math.min(this.getWidth(), bounds.width);
        int h = Math.min(this.getHeight(), bounds.height);              
        int x = center.x - w/2;
        int y = center.y - h/2;
        this.setBounds(x, y, w, h);
        if(this.isVisible())
        	this.validate();
        
    }
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {			
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanelAbout(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanelOk(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton")); //$NON-NLS-1$
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					dispose();
					this_windowClosing(null);
				}
			});
		}
		return okButton;
	}
	
	/**
	 * This method initializes jEditorPaneAbout	
	 * 	
	 * @return javax.swing.JEditorPane	
	 */    
	private JEditorPane getJEditorPaneAbout() {
		if (jEditorPaneAbout == null) {
			jEditorPaneAbout = new JEditorPane();
//			final JDialog parent = this;
			jEditorPaneAbout.addHyperlinkListener(new HyperlinkListener() {				
				public void hyperlinkUpdate(HyperlinkEvent hyperlinkEvent) {
					if(hyperlinkEvent.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
						Ocm ocm = new Ocm();
						try {
							String program = Context.getProgram("htm");
							if(program == null) {
								AlertDialog.showInfoDialog(Strings.getString("MessageFrame.noProgram")); //$NON-NLS-1$					
								return;
							}							
							ocm.run(program.replaceAll("%1", hyperlinkEvent.getURL().toString()), System.out, false); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						} catch (Exception exc) {
							exc.printStackTrace();
							AlertDialog.showErrorDialog(exc);
						}						
					}
				}
			});						
			jEditorPaneAbout.setContentType("text/html");		
			jEditorPaneAbout.setEditable(false);
			jEditorPaneAbout.setText(getAboutInfo());
			TextPopup.registerComponent(jEditorPaneAbout);
		}
		return jEditorPaneAbout;
	}

	/**
	 * This method initializes jPanelAbout	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelAbout() {
		if (jPanelAbout == null) {
			jPanelAbout = new JPanel();
			jPanelAbout.setLayout(new BorderLayout());
			jPanelAbout.add(getJEditorPaneAbout(), java.awt.BorderLayout.CENTER);
		}
		return jPanelAbout;
	}

	/**
	 * This method initializes jPanelOk	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelOk() {
		if (jPanelOk == null) {
			jPanelOk = new JPanel();
			jPanelOk.add(getOkButton(), null);
		}
		return jPanelOk;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
