/*
 * @(#)AccountCredentialsDialog.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import java.awt.FlowLayout;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.JButton;

public class AccountCredentialsDialog extends JDialog {

	static final long serialVersionUID = 2744072763524837289L;
	private final String WIN_ID = "account_credentials";
	private JPanel jContentPane = null;
	private JLabel jLabelCaption = null;
	private JLabel jLabelAccount = null;
	private JLabel jLabelUser = null;
	private JLabel jLabelPass = null;
	private JTextField jTextFieldUser = null;
	private JPasswordField jPasswordFieldPass = null;
	private JButton jButtonOk = null;
	private JButton jButtonCancel = null;
	private Account acnt = null;
	private int type = 0; 
	private int response = 3;
	public static final int TYPE_IN = 0;
	public static final int TYPE_OUT = 1;
	public static final int RESPONSE_OK = 2;
	public static final int RESPONSE_CANCEL = 3;

	/**
	 * This is the default constructor
	 */
	private AccountCredentialsDialog() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	public static int showCredentialsDialog(Account acnt, int type) {
		AccountCredentialsDialog acd = new AccountCredentialsDialog();
		acd.setAccount(acnt, type);
		acd.setModal(true);
		acd.setVisible(true);
		int response = acd.response;
		acd.dispose();
		return response;
	}
	
	public void setAccount(Account acnt, int type) {
		this.acnt = acnt;
		this.type = type;
		if(acnt == null)
			return;
		jLabelAccount.setText(acnt.name);
		if(type == TYPE_IN) {
			jTextFieldUser.setText(acnt.inUser);
			jPasswordFieldPass.setText(acnt.inPass);
		}
		if(type == TYPE_OUT) {
			jTextFieldUser.setText(acnt.outUser);
			jPasswordFieldPass.setText(acnt.outPass);
		}		
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(301, 172);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("AccountCredentialsDialog.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}	
	
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabelPass = new JLabel();
			jLabelPass.setText(Strings.getString("AccountCredentialsDialog.password")); //$NON-NLS-1$
			jLabelPass.setPreferredSize(new java.awt.Dimension(100,16));
			jLabelPass.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelUser = new JLabel();
			jLabelUser.setText(Strings.getString("AccountCredentialsDialog.username")); //$NON-NLS-1$
			jLabelUser.setPreferredSize(new java.awt.Dimension(100,16));
			jLabelUser.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelAccount = new JLabel();
			jLabelAccount.setText(Strings.getString("AccountCredentialsDialog.acntName")); //$NON-NLS-1$
			jLabelAccount.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
			jLabelAccount.setPreferredSize(new java.awt.Dimension(250,16));
			jLabelCaption = new JLabel();
			jLabelCaption.setText(Strings.getString("AccountCredentialsDialog.promptToEnter")); //$NON-NLS-1$
			jContentPane = new JPanel();
			jContentPane.setLayout(new FlowLayout());
			jContentPane.add(jLabelCaption, null);
			jContentPane.add(jLabelAccount, null);
			jContentPane.add(jLabelUser, null);
			jContentPane.add(getJTextFieldUser(), null);
			jContentPane.add(jLabelPass, null);
			jContentPane.add(getJPasswordFieldPass(), null);
			jContentPane.add(getJButtonOk(), null);
			jContentPane.add(getJButtonCancel(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTextFieldUser	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJTextFieldUser() {
		if (jTextFieldUser == null) {
			jTextFieldUser = new JTextField();
			jTextFieldUser.setPreferredSize(new java.awt.Dimension(170,20));
			jTextFieldUser.addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyPressed(java.awt.event.KeyEvent e) {    
					if(e.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER)
						okDialog();
				}
			});
			TextPopup.registerComponent(jTextFieldUser);
		}
		return jTextFieldUser;
	}

	/**
	 * This method initializes jPasswordFieldPass	
	 * 	
	 * @return javax.swing.JPasswordField	
	 */    
	private JPasswordField getJPasswordFieldPass() {
		if (jPasswordFieldPass == null) {
			jPasswordFieldPass = new JPasswordField();
			jPasswordFieldPass.setPreferredSize(new java.awt.Dimension(170,20));
			jPasswordFieldPass.addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyPressed(java.awt.event.KeyEvent e) {    
					if(e.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER)
						okDialog();
				}
			});
		}
		return jPasswordFieldPass;
	}

	/**
	 * This method initializes jButtonOk	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonOk() {
		if (jButtonOk == null) {
			jButtonOk = new JButton();
			jButtonOk.setText(Strings.getString("okButton")); //$NON-NLS-1$
			jButtonOk.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					okDialog();
				}
			});
		}
		return jButtonOk;
	}

	void okDialog() {
		if(acnt != null) {
			if(type == TYPE_IN) {
				acnt.inUser = jTextFieldUser.getText();
				acnt.inPass = String.valueOf(jPasswordFieldPass.getPassword());
			}
			if(type == TYPE_OUT) {
				acnt.outUser = jTextFieldUser.getText();
				acnt.outPass = String.valueOf(jPasswordFieldPass.getPassword());
			}						
		}
		response = RESPONSE_OK;
		this_windowClosing(null);
		setVisible(false);		
	}
	
	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					response = RESPONSE_CANCEL;
					this_windowClosing(null);
					setVisible(false);
				}
			});
		}
		return jButtonCancel;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
