/*
 * @(#)AcntSelDialog.java
 *
 * Copyright (C) 2007 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Hashtable;
import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JCheckBox;

public class AcntSelDialog extends JDialog {

	static final long serialVersionUID = 6767896100355641640L;	
	private JPanel jContentPane = null;
	private JPanel buttonPanel = null;
	private JButton okButton = null;
	private JButton cancelButton = null;
	private JPanel centerPanel = null;
	private JComboBox jcbAccount = null;
	private JLabel jlAccount = null;
	private static Account result = null;
	private String email = null;
	private JTextArea jtaMatchAddr = null;
	private JCheckBox jcbMatchAddr = null;
	private JLabel jlAddrFrom = null;
	private JLabel jlAddrFromVal = null;
	private JCheckBox jcbForSameSender = null;
	private JCheckBox jcbForAll = null;
	private static Hashtable htForSameSender = null;
	private static Account acntForAll = null;
	public final static Account DUMMY_ACCOUNT = new Account();
	
	public static void resetAcntPrefs() {
		htForSameSender = new Hashtable();
		acntForAll = null;
	}
			
	/**
	 * This is the default constructor
	 */
	public AcntSelDialog(String email) {
		super();
		this.email = email;		
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(324, 255);
		this.setTitle(Strings.getString("AcntSelDialog.title"));
		this.setContentPane(getJContentPane());
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getButtonPanel(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getCenterPanel(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes buttonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getButtonPanel() {
		if (buttonPanel == null) {
			buttonPanel = new JPanel();
			buttonPanel.setPreferredSize(new java.awt.Dimension(10,35));
			buttonPanel.add(getOkButton(), null);
			buttonPanel.add(getCancelButton(), null);
		}
		return buttonPanel;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton"));
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					result = (Account) jcbAccount.getSelectedItem();
					result = (result == null) ? DUMMY_ACCOUNT : result;
					
					if(jcbForAll.isSelected())
						acntForAll = result;
					else if(jcbForSameSender.isSelected())
						htForSameSender.put(jlAddrFromVal.getText(), result);
					
					dispose();
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					result = null;
					result = (result == null) ? DUMMY_ACCOUNT : result;
					
					if(jcbForAll.isSelected())
						acntForAll = result;
					else if(jcbForSameSender.isSelected())
						htForSameSender.put(jlAddrFromVal.getText(), result);
					
					dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes centerPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getCenterPanel() {
		if (centerPanel == null) {
			jlAddrFromVal = new JLabel();
			jlAddrFromVal.setText((email == null)?Strings.getString("n_a"):email);
			jlAddrFromVal.setFont(jlAddrFromVal.getFont().deriveFont(Font.BOLD));
			jlAddrFromVal.setPreferredSize(new java.awt.Dimension(300,16));
			jlAddrFrom = new JLabel();
			jlAddrFrom.setText(Strings.getString("ComposeFrame.fromFiled"));
			jlAddrFrom.setPreferredSize(new java.awt.Dimension(300,16));
			jlAccount = new JLabel();
			jlAccount.setText(Strings.getString("accountComboBox"));
			jlAccount.setPreferredSize(new java.awt.Dimension(300,16));
			centerPanel = new JPanel();
			centerPanel.add(jlAddrFrom, null);
			centerPanel.add(jlAddrFromVal, null);
			centerPanel.add(jlAccount, null);
			centerPanel.add(getJcbAccount(), null);
			centerPanel.add(getJcbMatchAddr(), null);
			centerPanel.add(getJtaMatchAddr(), null);
			centerPanel.add(getJcbForSameSender(), null);
			centerPanel.add(getJcbApplyForAll(), null);
		}
		return centerPanel;
	}

	/**
	 * This method initializes jcbAccount	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbAccount() {
		if (jcbAccount == null) {
			Object senderAddresses[] = null;
			try {
				senderAddresses = Context.storage.getAccounts(email).toArray();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			Account defAcnt = null;
			if (email == null) {
				try {
					defAcnt = Context.storage.getDefaultAccount();
				} catch (Exception exc) {
					AlertDialog.showErrorDialog(exc);
				}
			}
			if (senderAddresses == null)
				jcbAccount = new JComboBox();
			else
				jcbAccount = new JComboBox(senderAddresses);
			if (defAcnt != null)
				jcbAccount.setSelectedItem(defAcnt);
			jcbAccount.setPreferredSize(new java.awt.Dimension(300,20));
		}
		return jcbAccount;
	}

	private void center() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        Point center = ge.getCenterPoint();
        Rectangle bounds = ge.getMaximumWindowBounds();
        int w = Math.min(this.getWidth(), bounds.width);
        int h = Math.min(this.getHeight(), bounds.height);              
        int x = center.x - w/2;
        int y = center.y - h/2;
        this.setBounds(x, y, w, h);
        if(this.isVisible())
        	this.validate();        
    }
	
	public static Account getPrefAcnt(String email) {
		Account result = acntForAll;
		if (result == null)
			result = (Account) htForSameSender.get(email);
		return result;
		//return (result == DUMMY_ACCOUNT) ? null : result;
	}
	
	public static Account showAccountDialog(String email) {
		AcntSelDialog asd = new AcntSelDialog(email);
		asd.center();		
/*		
		asd.setVisible(true);
		asd.okButton.requestFocus();
		asd.setVisible(false);
*/		
		asd.setModal(true);
		asd.setVisible(true);
		
		return result;
	}

	/**
	 * This method initializes jtaMatchAddr	
	 * 	
	 * @return javax.swing.JTextArea	
	 */    
	private JTextArea getJtaMatchAddr() {
		if (jtaMatchAddr == null) {
			jtaMatchAddr = new JTextArea();
			jtaMatchAddr.setPreferredSize(new java.awt.Dimension(270,38));
			jtaMatchAddr.setEditable(false);
			jtaMatchAddr.setText(Strings.getString("AcntSelDialog.matchAddr"));
			jtaMatchAddr.setLineWrap(true);
			jtaMatchAddr.addMouseListener(new java.awt.event.MouseAdapter() { 
				public void mouseClicked(java.awt.event.MouseEvent e) {    
					jcbMatchAddr.setSelected(!jcbMatchAddr.isSelected());
					jcbMatchAddr_actionPerformed(null);
				}
			});
			jtaMatchAddr.setForeground(java.awt.Color.black);
		}
		return jtaMatchAddr;
	}

	/**
	 * This method initializes jcbMatchAddr	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbMatchAddr() {
		if (jcbMatchAddr == null) {
			jcbMatchAddr = new JCheckBox();
			jcbMatchAddr.setPreferredSize(new java.awt.Dimension(21, 21));
			jcbMatchAddr.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					jcbMatchAddr_actionPerformed(e);
				}
			});
			jcbMatchAddr.setEnabled(email != null);
			jcbMatchAddr.setSelected(email != null);
		}
		return jcbMatchAddr;
	}
		
	void jcbMatchAddr_actionPerformed(java.awt.event.ActionEvent e) {
		jcbAccount.removeAllItems();
		Object senderAddresses[] = null;
		try {
			senderAddresses = Context.storage.getAccounts(
					jcbMatchAddr.isSelected() ? email : null).toArray();
		} catch (Exception exc) {
			exc.printStackTrace();
		}
		Account defAcnt = null;
		if (!jcbMatchAddr.isSelected()) {
			try {
				defAcnt = Context.storage.getDefaultAccount();
			} catch (Exception exc) {
				AlertDialog.showErrorDialog(exc);
			}
		}
		if (senderAddresses != null) {
			for (int i = 0; i < senderAddresses.length; i++)
				jcbAccount.addItem(senderAddresses[i]);
		}
		if (defAcnt != null)
			jcbAccount.setSelectedItem(defAcnt);
	}

	/**
	 * This method initializes jcbForSameSender	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbForSameSender() {
		if (jcbForSameSender == null) {
			jcbForSameSender = new JCheckBox();
			jcbForSameSender.setPreferredSize(new java.awt.Dimension(300,21));
			jcbForSameSender.setText(Strings.getString("AcntSelDialog.forSameSender"));
		}
		return jcbForSameSender;
	}

	/**
	 * This method initializes jcbApplyForAll	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbApplyForAll() {
		if (jcbForAll == null) {
			jcbForAll = new JCheckBox();
			jcbForAll.setPreferredSize(new java.awt.Dimension(300,21));
			jcbForAll.setText(Strings.getString("AcntSelDialog.forAll"));
		}
		return jcbForAll;
	}		
		
}  // @jve:decl-index=0:visual-constraint="10,18"
