/*
 * @(#)AddrFrame.java
 *
 * Copyright (C) 2007 Sergey Bredikhin
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JFrame;
import java.awt.FlowLayout;
import javax.swing.JLabel;
import java.awt.Dimension;
import javax.swing.JTextField;
import javax.swing.JButton;

public class AddrFrame extends JFrame {

	static final long serialVersionUID = 6767896100312300002L;
	private final String WIN_ID = "address_new";
	private JPanel jContentPane = null;
	private JLabel jlFName = null;
	private JTextField jtfFName = null;
	private JLabel jlLName = null;
	private JTextField jtfLName = null;
	private JLabel jlEMail = null;
	private JTextField jtfEMail = null;
	private JButton jbOk = null;
	private JButton jbCancel = null;
	private ParentList parent = null;	

	/**
	 * This is the default constructor
	 */
	public AddrFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(410, 149);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("AddrFormFrame.title"));
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jlEMail = new JLabel();
			jlEMail.setPreferredSize(new java.awt.Dimension(120,16));
			jlEMail.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlEMail.setText(Strings.getString("AddrFormFrame.emailField"));
			jlLName = new JLabel();
			jlLName.setPreferredSize(new java.awt.Dimension(120,16));
			jlLName.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlLName.setText(Strings.getString("AddrFormFrame.lNameField"));
			jlFName = new JLabel();
			jlFName.setPreferredSize(new java.awt.Dimension(120,16));
			jlFName.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlFName.setText(Strings.getString("AddrFormFrame.fNameField"));
			jContentPane = new JPanel();
			jContentPane.setLayout(new FlowLayout());
			jContentPane.add(jlFName, null);
			jContentPane.add(getJtfFName(), null);
			jContentPane.add(jlLName, null);
			jContentPane.add(getJtfLName(), null);
			jContentPane.add(jlEMail, null);
			jContentPane.add(getJtfEMail(), null);
			jContentPane.add(getJbOk(), null);
			jContentPane.add(getJbCancel(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfFName() {
		if (jtfFName == null) {
			jtfFName = new JTextField();
			jtfFName.setPreferredSize(new Dimension(250, 20));
			TextPopup.registerComponent(jtfFName);
		}
		return jtfFName;
	}

	/**
	 * This method initializes jTextField1	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfLName() {
		if (jtfLName == null) {
			jtfLName = new JTextField();
			jtfLName.setPreferredSize(new Dimension(250, 20));
			TextPopup.registerComponent(jtfLName);
		}
		return jtfLName;
	}

	/**
	 * This method initializes jTextField2	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfEMail() {
		if (jtfEMail == null) {
			jtfEMail = new JTextField();
			jtfEMail.setPreferredSize(new Dimension(250, 20));
			TextPopup.registerComponent(jtfEMail);
		}
		return jtfEMail;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbOk() {
		if (jbOk == null) {
			jbOk = new JButton();
			jbOk.setText(Strings.getString("okButton"));
			jbOk.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					olivamail.gui.Address addr = new olivamail.gui.Address(); 
					addr.fname = jtfFName.getText();
					addr.lname = jtfLName.getText();
					addr.email = jtfEMail.getText();
					try {
						addr = Context.storage.saveAddress(addr);
						if (parent != null)
							parent.updateRow(null, -1, addr);
						this_windowClosing(null);
						dispose();
					} catch (Exception exc) {
						exc.printStackTrace();
						AlertDialog.showErrorDialog(exc);
					}
				}
			});
		}
		return jbOk;
	}

	public void setParent(ParentList parent) {
		this.parent = parent;
	}	
	
	/**
	 * This method initializes jButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbCancel() {
		if (jbCancel == null) {
			jbCancel = new JButton();
			jbCancel.setText(Strings.getString("cancelButton"));
			jbCancel.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jbCancel;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
