/*
 * @(#)AlertDialog.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.net.URL;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.JLabel;

import olivamail.core.common.ComUtils;

public class AlertDialog extends JDialog {

	private JPanel jContentPane = null;
	private JScrollPane jScrollPane1 = null;
	private JPanel jPanel1 = null;
	private JButton okButton = null;
	private JButton cancelButton = null;	
	public static final int INFO_ALERT = 0;
	public static final int WARNING_ALERT = 1;
	public static final int ERROR_ALERT = 2;
	public static final int CONFIRMATION_ALERT = 3;
	// Binary bits (3)YES(2)NO(1)OK(0)CANCEL	
	private static final int NONE_OPTION = 0;
	public static final int CANCEL_OPTION = 1;
	public static final int OK_OPTION = 2;
	public static final int NO_OPTION = 4;
	public static final int YES_OPTION = 8;	
	public static final int OK_CANCEL_OPTION = 3;
	public static final int YES_NO_OPTION = 12;
	public static final int YES_NO_CANCEL_OPTION = 13;
	public static final int URL_RESPONSE = 15;	
	private Response response = null;
	static final long serialVersionUID = 4854812333131819902L;
	private JButton moreButton = null;
	private Exception exception = null;
	private String text = null;
	private boolean more = false;	
	private JEditorPane jEditorPane1 = null;
	private final static int widthSmall = 320;
	private final static int widthMedium = 450;
	private final static int widthLarge = 600;
	private int widthLast = 450;
	private final static int heightSmall = 120;//110
	private final static int heightMedium = 150;//130
	private final static int heightLarge = 300;//300
	private int heightLast = 130;
	private JLabel jlIcon = null;
	private JButton yesButton = null;
	private JButton noButton = null;

	/**
	 * This is the default constructor
	 */
	private AlertDialog() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
/*		
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				response.result = NONE_OPTION;
			}
		});
*/		
		this.setResizable(false);
		this.setSize(450, 150);
		this.setContentPane(getJContentPane());
		this.setTitle("");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jlIcon = new JLabel();
			jlIcon.setText("");
			jlIcon.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
			jlIcon.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
			jlIcon.setPreferredSize(new java.awt.Dimension(60,60));
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());		
			jContentPane.add(getJScrollPane1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.SOUTH);			
			jContentPane.add(jlIcon, java.awt.BorderLayout.WEST);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,0,0,0));
			jScrollPane1.setViewportView(getJEditorPane1());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.setPreferredSize(new java.awt.Dimension(10,35));
			jPanel1.add(getYesButton(), null);
			jPanel1.add(getNoButton(), null);
			jPanel1.add(getOkButton(), null);
			jPanel1.add(getCancelButton(), null);
			jPanel1.add(getMoreButton(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton")); //$NON-NLS-1$
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					response.result = OK_OPTION;
					dispose();		
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			cancelButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					response.result = CANCEL_OPTION;
					dispose();
				}
			});
		}
		return cancelButton;
	}
	
	private void setText(String text) {
		text = ComUtils.nvl(text);
		if(text.length() < 35)
			widthLast = widthSmall;
		else
			widthLast = widthMedium;
		if(text.length() < 96) {
			text = (new StringBuffer("<p align=\"center\">")).append(text).append("</p>").toString();
			setSize(widthLast, heightSmall);
		} else if(text.length() < 260) {
			setSize(widthLast, heightMedium);
		} else {
			setSize(widthLast, heightLarge);
		}
		jEditorPane1.setText(text);
		jEditorPane1.setCaretPosition(0);
		this.text = text;
	}

	private void applyTypeView(int type, int option) {
		String iconName = null;
		switch(type) {
		case ERROR_ALERT:
			iconName = "error.gif";			
			break;
		case INFO_ALERT:
			iconName = "info.gif";
			break;
		case CONFIRMATION_ALERT:
			iconName = "confirm.gif";
			break;
		case WARNING_ALERT:
			iconName = "warn.gif";
			break;			
		}
		if (iconName != null) {
			Icon icon = new ImageIcon(getClass().getResource(Context.myPackagePath + iconName));
			if (icon != null)
				jlIcon.setIcon(icon);
		}		
		
		if(!checkOption(option, YES_OPTION))
			yesButton.setVisible(false);
		if(!checkOption(option, NO_OPTION))
			noButton.setVisible(false);
		if(!checkOption(option, OK_OPTION))
			okButton.setVisible(false);
		if(!checkOption(option, CANCEL_OPTION))
			cancelButton.setVisible(false);		
		
		if(exception == null)
			moreButton.setVisible(false);
	}
		
	private boolean checkOption(int options, int checkedOption) {
		return (options & checkedOption) > 0;
	}
	
	private void center() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        Point center = ge.getCenterPoint();
        Rectangle bounds = ge.getMaximumWindowBounds();
        int w = Math.min(this.getWidth(), bounds.width);
        int h = Math.min(this.getHeight(), bounds.height);              
        int x = center.x - w/2;
        int y = center.y - h/2;
        this.setBounds(x, y, w, h);
        if(this.isVisible())
        	this.validate();        
    }
			
	public static void showInfoDialog(String text) {
		showMessageDialog(null, text, null, INFO_ALERT, NONE_OPTION);
	}

	public static void showInfoDialog(String text, RequestHandler rh) {
		showMessageDialog(null, text, null, INFO_ALERT, NONE_OPTION, rh);
	}
		
	public static int showWarningDialog(String text) {
		return showMessageDialog(null, text, null, WARNING_ALERT, NONE_OPTION);
	}	

	public static int showWarningDialog(String text, RequestHandler rh) {
		return showMessageDialog(null, text, null, WARNING_ALERT, NONE_OPTION, rh);
	}	
		
	public static int showWarningDialog(String text, int option) {
		return showMessageDialog(null, text, null, WARNING_ALERT, option);
	}		

	public static int showWarningDialog(String text, int option, RequestHandler rh) {
		return showMessageDialog(null, text, null, WARNING_ALERT, option, rh);
	}		
		
	public static void showErrorDialog(String text) {
		showMessageDialog(null, text, null, ERROR_ALERT, NONE_OPTION);
	}	

	public static void showErrorDialog(String text, RequestHandler rh) {
		showMessageDialog(null, text, null, ERROR_ALERT, NONE_OPTION, rh);
	}	
		
	public static void showErrorDialog(Exception exception) {
		showMessageDialog(null, exception.toString(), exception, ERROR_ALERT, NONE_OPTION);
	}	

	public static void showErrorDialog(Exception exception, RequestHandler rh) {
		showMessageDialog(null, exception.toString(), exception, ERROR_ALERT, NONE_OPTION, rh);
	}	
		
	public static void showErrorDialog(String text, Exception exception) {
		showMessageDialog(null, text, exception, ERROR_ALERT, NONE_OPTION);
	}	

	public static void showErrorDialog(String text, Exception exception, RequestHandler rh) {
		showMessageDialog(null, text, exception, ERROR_ALERT, NONE_OPTION, rh);
	}		
	
	public static int showConfirmDialog(String text, int option) {
		return showMessageDialog(null, text, null, CONFIRMATION_ALERT, option);
	}	

	public static int showConfirmDialog(String text, int option, RequestHandler rh) {
		return showMessageDialog(null, text, null, CONFIRMATION_ALERT, option, rh);
	}	
		
	public static int showMessageDialog(String title, String text, int type) {
		return showMessageDialog(title, text, null, type, NONE_OPTION);
	}

	public static int showMessageDialog(String title, String text, int type, RequestHandler rh) {
		return showMessageDialog(title, text, null, type, NONE_OPTION, rh);
	}
		
	private static int showMessageDialog(String title, String text, Exception exception, int type, int option) {
		return showMessageDialog(title, text, exception, type, option, null);		
	}
	
	private void showDialog(String title, String text, Exception exception, int type, int option, RequestHandler rh) {
		if (title == null) {
			switch (type) {
			case INFO_ALERT:
				title = Strings.getString("informationMsgTitle");
				if(option == NONE_OPTION)
					option = OK_OPTION;
				break;
			case WARNING_ALERT:
				title = Strings.getString("warningMessageTitle");
				if(option == NONE_OPTION)
					option = OK_OPTION;				
				break;
			case ERROR_ALERT:
				title = Strings.getString("errorMessageTitle");
				if(option == NONE_OPTION)
					option = OK_OPTION;				
				break;
			case CONFIRMATION_ALERT:
				title = Strings.getString("confirmationMsgTitle");
				if(option == NONE_OPTION)
					option = OK_CANCEL_OPTION;				
				break;								
			default:
				title = "";
			}
		}
		setTitle(title);
		setText(text);
		this.exception = exception;
		applyTypeView(type, option);
		jEditorPane1.setBackground(jPanel1.getBackground());
		center();		
		setVisible(true);
		okButton.requestFocus();
		setVisible(false);
		setModal(true);
		setVisible(true);
	}
	
	public static int showMessageDialog(String title, String text, Exception exception, int type, int option, RequestHandler rh) {
		AlertDialog af = new AlertDialog();
		Response resp = new Response();
		af.setResponse(resp);
		af.showDialog(title, text, exception, type, option, rh);		
		if(resp.resultURL != null && rh != null) {
			rh.handleRequest(resp.resultURL);
			return URL_RESPONSE;
		}				
		return resp.result;
	}

	/**
	 * This method initializes moreButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getMoreButton() {
		if (moreButton == null) {
			moreButton = new JButton();
			moreButton.setText(Strings.getString("moreButton")); //$NON-NLS-1$
			moreButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					moreButton_actionPerformed(e);
				}
			});
		}
		return moreButton;
	}
	
	private void setResponse(Response response) {
		this.response = response;
	}
	
	void moreButton_actionPerformed(java.awt.event.ActionEvent e) {
		more = !more;
		if(exception == null)
			return;
		if(more) {
			widthLast = getWidth();			
			heightLast = getHeight();
			setSize(widthLarge, heightLarge);
			center();
			moreButton.setText(Strings.getString("lessButton")); //$NON-NLS-1$
			jEditorPane1.setContentType("text/plain");
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			PrintWriter pw = new PrintWriter(baos); 
			exception.printStackTrace(pw);
			pw.flush();
			pw.close();			
			jEditorPane1.setText(new String(baos.toByteArray()));
		} else {
			setSize(widthLast, heightLast);
			center();
			moreButton.setText(Strings.getString("moreButton")); //$NON-NLS-1$
			jEditorPane1.setContentType("text/html");
			jEditorPane1.setText(text);
		}
		jEditorPane1.setCaretPosition(0);		
	}

	/**
	 * This method initializes jEditorPane1	
	 * 	
	 * @return javax.swing.JEditorPane	
	 */    
	private JEditorPane getJEditorPane1() {
		if (jEditorPane1 == null) {
			jEditorPane1 = new JEditorPane();
			jEditorPane1.setEditable(false);
			jEditorPane1.setBackground(new java.awt.Color(204,204,204));
			jEditorPane1.addHyperlinkListener(new HyperlinkListener() {				
				public void hyperlinkUpdate(HyperlinkEvent hyperlinkEvent) {
					if(hyperlinkEvent.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {					
						response.resultURL = hyperlinkEvent.getURL();
						//System.out.println(response.resultURL);						
						dispose();
					}
				}
			});									
			jEditorPane1.setContentType("text/html");
			TextPopup.registerComponent(jEditorPane1);
		}
		return jEditorPane1;
	}

	/**
	 * This method initializes yesButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getYesButton() {
		if (yesButton == null) {
			yesButton = new JButton();
			yesButton.setText(Strings.getString("yesButton")); //$NON-NLS-1$
			yesButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					response.result = YES_OPTION;
					dispose();
				}
			});
		}
		return yesButton;
	}

	/**
	 * This method initializes noButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getNoButton() {
		if (noButton == null) {
			noButton = new JButton();
			noButton.setText(Strings.getString("noButton")); //$NON-NLS-1$
			noButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					response.result = NO_OPTION;
					dispose();
				}
			});
		}
		return noButton;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"

class Response {
	public int result = 0;
	public URL resultURL = null;
}