/*
 * @(#)ComposeFrame.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.mail.internet.InternetAddress;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import olivamail.core.common.IOUtils;
import olivamail.core.mail.MailUtils;
import olivamail.core.mail.MessageHeader;
import olivamail.core.mail.MessagePart;
import olivamail.core.mail.NewMessage;
import olivamail.core.mail.SimpleMessage;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

public class ComposeFrame extends JFrame implements Composer {
	
	static final long serialVersionUID = 1L;
	
	private final String WIN_ID = "compose";	
		
	private JPanel jContentPane = null;

// private JScrollPane jScrollPane1 = null;

	private ComposeHeader headerPanel = null;

	private ComposePane сomposePane1 = null;
	
// private JEditorPane jEditorPane1 = null;

	private javax.mail.Message fwdOrReplyMessage = null;
	
// private String message_id = null;
	
	public static final int ACTION_NEW = 0;
	
	public static final int ACTION_REPLY = 1;

	public static final int ACTION_FORWARD_PARTLY = 2;
	
	public static final int ACTION_FORWARD_AS_ATT = 3;
	
	public static final int ACTION_FORWARD_INPLACE = 4;

	private int action = ACTION_NEW;

// private String refId = null;
	
	private SimpleMessage replyMessage = null;

	private String fwdOrReplyCp = Context.defaultCodepage;
	
	private boolean asUnicode = false; 
	
	private boolean important = false;

	private boolean asHtml = false;	
	
	private boolean requestNotify = false;

	private int sid = -1;

	private int fid = -1;

	private JPanel rightPanel = null;

	private JPanel leftPanel = null;

	private JMenuBar jJMenuBar = null;

	private JMenu jMenuFile = null;
	
	private JMenu jMenuEditor = null;

	private JMenuItem jmiClose = null;

	private JMenu jMenuParams = null;

	private JMenuItem jmiAsUnicode = null;

	private JMenuItem jmiRequestNotify = null;

	private JMenuItem jmiAsHtml = null;

	private JMenuItem jmiSend = null;
	
	private boolean sigHeaderSet = false;

	private JMenuItem jmiImportant = null;	

	int curAttPort = 0;
	
	int curAttXPos = 5;
	
	int curAttYPos = 5;

	private JMenuItem jmiAddAtt = null;

	private JPanel bottomPanel = null;
		
	public void setFwdOrReplyCp(String cp) {
		this.fwdOrReplyCp = cp;
	}

	public void performAction(int action) {			
		try {
			Signature sigFooter = Context.storage.getSignatureToUse(headerPanel.getAcntId(), Signature.SIG_FOOTER);
			if (sigFooter != null && sigFooter.sigLocal != null) {
				StringBuffer sb = new StringBuffer(сomposePane1.getText());
				сomposePane1.setText(sb.append("\n\n").append(sigFooter.sigLocal)
						.toString());
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}					
		
		this.action = action;				
		switch (action) {
		case ACTION_NEW:
			break;
		case ACTION_REPLY:
			reply();
			break;
		case ACTION_FORWARD_PARTLY:
			forward();
			break;
		case ACTION_FORWARD_AS_ATT:
			forward();
			break;
		case ACTION_FORWARD_INPLACE:
			forward();
			break;						
		default:
			break;
		}
	}

	public void setSigHeader() {
		if (sigHeaderSet)
			return;
		try {
			String acntId = headerPanel.getAcntId();
			String sigHeader = getSigHeader(acntId);
			if (sigHeader == null)
				return;
			StringBuffer sb = new StringBuffer(sigHeader);
			sb.append("\r\n\r\n").append(сomposePane1.getText());
			сomposePane1.setText(sb.toString());
			sigHeaderSet = true;
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
	
	private void reply() {
		try {
			replyMessage = Context.mailBean.getReplyMessage(fwdOrReplyMessage, fwdOrReplyCp,
					true);
			if(replyMessage.ReplyTo.equals(""))
				headerPanel.toField.setText(replyMessage.From);
			else
				headerPanel.toField.setText(replyMessage.ReplyTo);			
			headerPanel.subjectField.setText(MailUtils.getPrefixedSubject(replyMessage.Subject, "Re"));
			StringBuffer sb = new StringBuffer(сomposePane1.getText());
			sb.append("\n\n\n----- Original Message -----\nFrom: ");
			sb.append(replyMessage.From).append("\nTo: ").append(replyMessage.To)
			.append("\nSent: ").append(replyMessage.getSentDateToString())
			.append("\nSubject: ").append(replyMessage.Subject).append("\n\n")
			.append(replyMessage.text.trim().replaceAll("\\n", "\n> ").replaceFirst("^", "\n> "));
			
			сomposePane1.setText(sb.toString());
			сomposePane1.setCaretPosition(0);
			
// refId = msg.MessageID;
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
	
	private void addFwdMessageParts() {
		olivamail.core.mail.Message msg = null;
		try {
			msg = Context.mailBean.getMessage(fwdOrReplyMessage, fwdOrReplyCp);
			MessagePart mp = null;
			if (msg.messageParts != null) {
				int attNo = 0;
				for (java.util.Enumeration en = msg.messageParts.elements(); en
						.hasMoreElements();) {
					MessagePart att = (MessagePart) en.nextElement();
					File fout = getOutFile();
					if (fout == null)
						return;
					if (att.type == MessagePart.T_RFC822) {
						mp = new MessagePart(att.partNo, att.alternative,
								att.type, att.contentType, att.fileName);
						mp.fileName = Strings.getString("MessageFrame.rfc822message");
						// If filename was not null; If rfc822 not one then
						// filename should differ by number e.g.
						// CacheFile staff
						OutputStream os = Context.getFileOutputStream(fout);
						msg.getRfc822Part(mp.partNo).apiMessage.writeTo(os);
						Context.closeOutputStream(os);
						headerPanel.addAttachment(new CacheFile(fout,
								mp.fileName, "message/rfc822"));
					} else if (att.fileName != null) {
						// CacheFile staff
						InputStream is = msg.getBinaryPart(att.partNo);
						IOUtils.writeFile(is, fout);
						headerPanel.addAttachment(new CacheFile(fout,
								att.fileName, null));
					} else if (att.cid != null || att.alternative
							|| att.type == MessagePart.T_BINARY) {
						if (att.type == MessagePart.T_TEXT) {
							String attFileName = "ATT"
									+ String.valueOf(attNo++)
									+ "."
									+ olivamail.core.mail.MailUtils
											.getExtension(att.contentType);
							mp = new MessagePart(att.partNo, att.alternative,
									att.type, att.contentType, att.fileName);
							mp.fileName = attFileName;
							// CacheFile staff
							ByteArrayInputStream bais = new ByteArrayInputStream(
									msg.getTextPart(mp.partNo).getBytes());
							IOUtils.writeFile(bais, fout);
							headerPanel.addAttachment(new CacheFile(fout,
									mp.fileName, null));
						} else if (att.type == MessagePart.T_BINARY) {
							String attFileName = "ATT"
									+ String.valueOf(attNo++)
									+ "."
									+ olivamail.core.mail.MailUtils
											.getExtension(att.contentType);
							mp = new MessagePart(att.partNo, att.alternative,
									att.type, att.contentType, att.fileName);
							mp.fileName = attFileName;
							// CacheFile staff
							InputStream is = msg.getBinaryPart(mp.partNo);
							IOUtils.writeFile(is, fout);
							headerPanel.addAttachment(new CacheFile(fout,
									mp.fileName, null));
						}
					} else if (!att.isMimeType("message/partial")) {
						String attFileName = "ATT"
								+ String.valueOf(attNo++)
								+ "."
								+ olivamail.core.mail.MailUtils
										.getExtension(att.contentType);
						mp = new MessagePart(att.partNo, att.alternative,
								att.type, att.contentType, att.fileName);
						mp.fileName = attFileName;
						// CacheFile staff
						ByteArrayInputStream bais = new ByteArrayInputStream(
								msg.getTextPart(mp.partNo).getBytes());
/*
						ByteArrayInputStream bais = new ByteArrayInputStream(
								msg.getTextPart(mp.partNo).getBytes("UTF-8")); 
 */						
						IOUtils.writeFile(bais, fout);
						headerPanel.addAttachment(new CacheFile(fout,
								mp.fileName, mp.contentType));
					}
				}
				// String attList = sb.toString();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
	
	private void forward() {
		try {
			SimpleMessage msg = Context.mailBean.getReplyMessage(fwdOrReplyMessage, fwdOrReplyCp, false);
			
			headerPanel.subjectField.setText(MailUtils.getPrefixedSubject(msg.Subject, "Fwd"));
			
			StringBuffer sb = new StringBuffer(сomposePane1.getText());
			sb.append("\n\n\n----- Original Message -----\nFrom: ");
			sb.append(msg.From).append("\nTo: ").append(msg.To)
			.append("\nSent: ").append(msg.getSentDateToString())
			.append("\nSubject: ").append(msg.Subject).append("\n\n"); 
			
			сomposePane1.setText(sb.toString());
// jEditorPane1.setCaretPosition(jEditorPane1.getText().length());
			if (action == ACTION_FORWARD_PARTLY) {
				addFwdMessageParts();
				action = ACTION_NEW;
				fwdOrReplyMessage = null;				
			} else if (action == ACTION_FORWARD_AS_ATT) {
				File fout = getOutFile();
				if (fout == null)
					return;
				OutputStream os = Context.getFileOutputStream(fout);
				fwdOrReplyMessage.writeTo(os);
				Context.closeOutputStream(os);
				headerPanel.addAttachment(new CacheFile(fout, Strings
						.getString("MessageFrame.rfc822message"),
						"message/rfc822"));
				action = ACTION_NEW;
				fwdOrReplyMessage = null;
			} else if (action == ACTION_FORWARD_INPLACE) {
/*				
				File fout = getOutFile();
				if (fout == null)
					return;
				OutputStream os = Context.getFileOutputStream(fout);
				fwdOrReplyMessage.writeTo(os);
				Context.closeOutputStream(os);
				headerPanel.addAttachment(new CacheFile(fout, Strings
						.getString("MessageFrame.rfc822message"),
						"message/rfc822"));
				//action = ACTION_NEW;
				fwdOrReplyMessage = null;
*/				
			}			
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

/*
 * public void setMessage(javax.mail.Message message, String message_id) {
 * this.message = message; this.message_id = message_id; }
 */	

	public void setFwdOrReplyMessage(javax.mail.Message message) {
		this.fwdOrReplyMessage = message;
	}
		
	/**
	 * This is the default constructor
	 */
	public ComposeFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(535, 373);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);				
			}
		});
		this.setJMenuBar(getJJMenuBar());
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				callExit();
			}
		});
		this.setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("ComposeFrame.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
			
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getСomposePane1().getComponent(), java.awt.BorderLayout.CENTER);			
// jContentPane.add(getJScrollPane1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getHeaderPanel(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getRightPanel(), java.awt.BorderLayout.EAST);
			jContentPane.add(getLeftPanel(), java.awt.BorderLayout.WEST);
			jContentPane.add(getBottomPanel(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes сomposePane1
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private ComposePane getСomposePane1() {
		if (сomposePane1 == null) {
			сomposePane1 = new ComposePane();
		}
		return сomposePane1;
	}
		
	/**
	 * This method initializes jScrollPane1
	 * 
	 * @return javax.swing.JScrollPane
	 */
/*
 * private JScrollPane getJScrollPane1() { if (jScrollPane1 == null) {
 * jScrollPane1 = new JScrollPane();
 * jScrollPane1.setViewportView(getJEditorPane1()); } return jScrollPane1; }
 */	

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private ComposeHeader getHeaderPanel() {
		if (headerPanel == null) {
			headerPanel = new ComposeHeader();
			headerPanel.assignComposer(this);
			//headerPanel.setLayout(new BorderLayout());
			headerPanel.setPreferredSize(new java.awt.Dimension(520,109));
/*			
			headerPanel.setLayout(new BorderLayout());
			headerPanel.setPreferredSize(new java.awt.Dimension(1255,109));
			headerPanel.add(getFromPanel(), java.awt.BorderLayout.NORTH);
			headerPanel.add(getToPanel(), java.awt.BorderLayout.CENTER);
*/			
		}
		return headerPanel;
	}

	/**
	 * This method initializes jEditorPane1
	 * 
	 * @return javax.swing.JEditorPane
	 */
/*
 * private JEditorPane getJEditorPane1() { if (jEditorPane1 == null) {
 * jEditorPane1 = new JEditorPane(); } return jEditorPane1; }
 */

	void callExit() {
		if (AlertDialog
				.showConfirmDialog(
						Strings.getString("confirmClose"), AlertDialog.OK_CANCEL_OPTION) != AlertDialog.OK_OPTION) //$NON-NLS-1$
			return;
		try {
			Context.clearCache("msgs" + sid + "_"); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
		this_windowClosing(null);
		dispose();
	}
	
	void sendMessage() {
		String addrTo =  Context.trimField(headerPanel.toField);
		headerPanel.toField.setText(addrTo);
		String addrCc = Context.trimField(headerPanel.ccField);
		headerPanel.ccField.setText(addrCc);
		String addrBcc = Context.trimField(headerPanel.bccField);
		headerPanel.bccField.setText(addrBcc);
		if (addrTo.equals("") && addrCc.equals("") && addrBcc.equals("")) {
			AlertDialog.showWarningDialog(Strings
					.getString("ComposeFrame.enterAddress")); //$NON-NLS-1$
			headerPanel.toField.requestFocus();
			return;
		}
		String subject = headerPanel.subjectField.getText().trim();
		headerPanel.subjectField.setText(subject);
		if (subject.equals("")) {
			if (AlertDialog
					.showConfirmDialog(
							Strings.getString("ComposeFrame.enterSubject"), AlertDialog.OK_CANCEL_OPTION) != AlertDialog.OK_OPTION) { //$NON-NLS-1$
				headerPanel.subjectField.requestFocus();
				return;
			}
		}
				
		NewMessage newMsg = NewMessage.create(null, headerPanel.toField.getText(),
				headerPanel.ccField.getText(), headerPanel.bccField.getText(),
				headerPanel.subjectField.getText(), null);
		newMsg.asUnicode = asUnicode;
		newMsg.requestNotify = requestNotify;
		if(important)
			newMsg.importance = MessageHeader.IMP_HIGH;
							
		Account acnt = (Account) headerPanel.jcbAddrFrom.getSelectedItem();			
		
		try {
			MailContext.createNewMsgInOutbox(newMsg, acnt, сomposePane1.getText(),
					сomposePane1.getContentType(), headerPanel.cacheFiles,
					action, fwdOrReplyMessage, fwdOrReplyCp, replyMessage);

			Context.clearCache("msgs" + sid + "_"); //$NON-NLS-1$ //$NON-NLS-2$
			MailContext.sendFromOutbox();
			this_windowClosing(null);
			dispose();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
		
	/**
	 * This method initializes jPanel3
	 * 
	 * @return javax.swing.JPanel
	 */    
	private JPanel getRightPanel() {
		if (rightPanel == null) {
			rightPanel = new JPanel();
		}
		return rightPanel;
	}

	/**
	 * This method initializes jPanel4
	 * 
	 * @return javax.swing.JPanel
	 */    
	private JPanel getLeftPanel() {
		if (leftPanel == null) {
			leftPanel = new JPanel();
		}
		return leftPanel;
	}

	/**
	 * This method initializes jJMenuBar
	 * 
	 * @return javax.swing.JMenuBar
	 */    
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getJMenuFile());
			jJMenuBar.add(getJMenuParams());
			jMenuEditor = getСomposePane1().getMenu();
			if(jMenuEditor != null)
				jJMenuBar.add(jMenuEditor);
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes jMenuFile
	 * 
	 * @return javax.swing.JMenu
	 */    
	private JMenu getJMenuFile() {
		if (jMenuFile == null) {
			jMenuFile = new JMenu();
			jMenuFile.setText(Strings.getString("menuFile")); //$NON-NLS-1$
			jMenuFile.add(getJmiSend());
			jMenuFile.add(getJmiAddAtt());
			jMenuFile.add(getJmiClose());
		}
		return jMenuFile;
	}

	/**
	 * This method initializes jmiClose
	 * 
	 * @return javax.swing.JMenuItem
	 */    
	private JMenuItem getJmiClose() {
		if (jmiClose == null) {
			jmiClose = new JMenuItem();
			jmiClose.setText(Strings.getString("menuClose")); //$NON-NLS-1$
			jmiClose.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jmiClose_actionPerformed(e);
				}
			});
		}
		return jmiClose;
	}

	void jmiClose_actionPerformed(java.awt.event.ActionEvent e) {
		callExit();
	}
	
	/**
	 * This method initializes jMenuParams
	 * 
	 * @return javax.swing.JMenu
	 */    
	private JMenu getJMenuParams() {
		if (jMenuParams == null) {
			jMenuParams = new JMenu();
			jMenuParams.setText(Strings.getString("ComposeFrame.menuParams")); //$NON-NLS-1$
			jMenuParams.add(getJmiRequestNotify());
			jMenuParams.add(getJmiAsUnicode());
			jMenuParams.add(getJmiImportant());
			jMenuParams.add(getJmiAsHtml());
		}
		return jMenuParams;
	}

	/**
	 * This method initializes jmiAsUnicode
	 * 
	 * @return javax.swing.JMenuItem
	 */    
	private JMenuItem getJmiAsUnicode() {
		if (jmiAsUnicode == null) {
			jmiAsUnicode = new JMenuItem();
			jmiAsUnicode.setText(Strings.getString("ComposeFrame.miAsUnicode")); //$NON-NLS-1$
			jmiAsUnicode.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jmiAsUnicode_actionPerformed(e);
				}
			});
		}
		return jmiAsUnicode;
	}
	
	void jmiAsUnicode_actionPerformed(java.awt.event.ActionEvent e) {
		asUnicode = !asUnicode;
		if (asUnicode)
			jmiAsUnicode.setIcon(new ImageIcon(getClass().getResource(
					Context.myPackagePath + "check.png")));
		else
			jmiAsUnicode.setIcon(null);
	}
	
/*
	void jButtonAtt_mouseClicked(java.awt.event.MouseEvent e) {
		if (e.getButton() != MouseEvent.BUTTON3 && e.getButton() != MouseEvent.BUTTON1)
			return;
		getJPopupMenu1().show(jepAtt, curAttXPos, curAttYPos);
	}
*/	

	/**
	 * This method initializes jmiRequestNotify
	 * 
	 * @return javax.swing.JMenuItem
	 */    
	private JMenuItem getJmiRequestNotify() {
		if (jmiRequestNotify == null) {
			jmiRequestNotify = new JMenuItem();
			jmiRequestNotify.setText(Strings.getString("ComposeFrame.miRequestNotify")); //$NON-NLS-1$
			jmiRequestNotify.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jmiRequestNotify_actionPerformed(e);
				}
			});
		}
		return jmiRequestNotify;
	}

	void jmiRequestNotify_actionPerformed(java.awt.event.ActionEvent e) {
		requestNotify = !requestNotify;
		if (requestNotify)
			jmiRequestNotify.setIcon(Context.getImageIcon("check.png"));
		else
			jmiRequestNotify.setIcon(null);		
	}

	/**
	 * This method initializes jmiAsHtml
	 * 
	 * @return javax.swing.JMenuItem
	 */    
	private JMenuItem getJmiAsHtml() {
		if (jmiAsHtml == null) {
			jmiAsHtml = new JMenuItem();
			jmiAsHtml.setText(Strings.getString("ComposeFrame.miAsHtml")); //$NON-NLS-1$
			jmiAsHtml.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jmiAsHtml_actionPerformed(e);
				}
			});
		}
		return jmiAsHtml;
	}

	void jmiAsHtml_actionPerformed(java.awt.event.ActionEvent e) {
		if(asHtml)
			return;
		asHtml = !asHtml;
		if (asHtml) {
			String text = сomposePane1.getText().replaceAll("\\n", "\n<br>");			
			jContentPane.remove(сomposePane1.getComponent());
			сomposePane1.setContentType("text/html");
			jContentPane.add(сomposePane1.getComponent(), java.awt.BorderLayout.CENTER);
			сomposePane1.setText(text);
			if(jMenuEditor != null)
				jJMenuBar.remove(jMenuEditor);
			jMenuEditor = сomposePane1.getMenu();
			if(jMenuEditor != null)
				jJMenuBar.add(jMenuEditor);				
			this.setVisible(true);
			
			jmiAsHtml.setIcon(new ImageIcon(getClass().getResource(
					Context.myPackagePath + "check.png")));			
		} else
			jmiAsHtml.setIcon(null);		
	}

	/**
	 * This method initializes jmiSend
	 * 
	 * @return javax.swing.JMenuItem
	 */    
	private JMenuItem getJmiSend() {
		if (jmiSend == null) {
			jmiSend = new JMenuItem();
			jmiSend.setText(Strings.getString("ComposeFrame.sendButton")); //$NON-NLS-1$
			jmiSend.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					sendMessage();
				}
			});
		}
		return jmiSend;
	}
	
	private String parseFirstReceipient(String addr) {
		if(addr == null || addr.equals(""))
			return null;
		InternetAddress[] ia = null;
		try {
			ia = InternetAddress.parse(addr, false);
		} catch (Exception e) {
			return null;
		}
		if(ia.length == 0)
			return null;
		String to = ia[0].getPersonal();
		if (to == null) {
			to = ia[0].getAddress();
			if (to != null) {
				int idx = to.indexOf('@');
				if (idx != -1)
					to = to.substring(0, idx);
			}
		}
		return to;
	}
	
	private String getFirstReceipient() {
		String addr = parseFirstReceipient(headerPanel.toField.getText().trim());
		if(addr == null)
			addr = parseFirstReceipient(headerPanel.ccField.getText().trim());
		return addr;
	}
	
	private String getSigHeader(String acntId) throws Exception {
		String result = null;
		Signature sigHeader = Context.storage.getSignatureToUse(acntId,
				Signature.SIG_HEADER);
		if (sigHeader != null && sigHeader.sigLocal != null) {
			String addr = getFirstReceipient();
			if (addr != null) {
				result = sigHeader.sigLocal.replaceAll("%1", addr);
			}
		}
		return result;
	}
	
	private String getSigFooter(String acntId) throws Exception {
		String result = null;
		Signature sigFooter = Context.storage.getSignatureToUse(acntId,
				Signature.SIG_FOOTER);
		if (sigFooter != null && sigFooter.sigLocal != null) {
			result = sigFooter.sigLocal;
		}
		return result;
	}
	
	public void changeFromAddr() {
		String selText = сomposePane1.getSelectedText();
		if(selText == null) {
			if(AlertDialog.showConfirmDialog(Strings.getString("ComposeFrame.selectText"), 
					AlertDialog.YES_NO_OPTION) == AlertDialog.NO_OPTION) {
				headerPanel.jcbAddrFrom.setSelectedIndex(headerPanel.addrFromIdx);
				return;
			}
		} else {
			if(AlertDialog.showConfirmDialog(Strings.getString("ComposeFrame.selectedText"), 
					AlertDialog.YES_NO_OPTION) == AlertDialog.NO_OPTION) {
				headerPanel.jcbAddrFrom.setSelectedIndex(headerPanel.addrFromIdx);
				return;
			}
		}
		headerPanel.addrFromIdx = headerPanel.jcbAddrFrom.getSelectedIndex();
		
		StringBuffer sb = new StringBuffer();
		try {
			String acntId = headerPanel.getAcntId();
			String sigHeader = getSigHeader(acntId);
			if (sigHeader != null) {
				sb.append(sigHeader).append("\r\n\r\n");
				sigHeaderSet = true;
			}
/*
 * if (сomposePane1.getContentType().equalsIgnoreCase("text/html")) { sb = new
 * StringBuffer(sb.toString().replaceAll("\\n", "\n<br>"));) }
 */			
			int textPos = sb.length();
			if (selText != null) {
				sb.append(selText);
			}
			String sigFooter = getSigFooter(acntId);
			if (sigFooter != null) {
				sb.append("\r\n\r\n").append(sigFooter);
			}
			if (сomposePane1.getContentType().equalsIgnoreCase("text/html"))
				сomposePane1.setText(sb.toString().replaceAll("\\n", "\n<br>"));
			else
				сomposePane1.setText(sb.toString());								
			сomposePane1.setCaretPosition(textPos);
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

	/**
	 * This method initializes jmiImportant
	 * 
	 * @return javax.swing.JMenuItem
	 */    
	private JMenuItem getJmiImportant() {
		if (jmiImportant == null) {
			jmiImportant = new JMenuItem();
			jmiImportant.setText(Strings.getString("ComposeFrame.important"));
			jmiImportant.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					important = !important;
					if (important)
						jmiImportant.setIcon(new ImageIcon(getClass()
								.getResource(Context.myPackagePath + "check.png")));
					else
						jmiImportant.setIcon(null);
				}
			});
		}
		return jmiImportant;
	}

	private synchronized File getOutFile() {
		fid++;
		if (sid == -1)
			sid++;		
		File fout = new File(Context.cache_dir + "msgs" + sid + "_" + fid); //$NON-NLS-1$ //$NON-NLS-2$
		while (fout.exists()) {
			sid++;
			if (sid > 100) {
				AlertDialog
						.showWarningDialog(Strings
								.getString("ComposeFrame.cleanCacheMessage") + Context.cache_dir + ")"); //$NON-NLS-1$ //$NON-NLS-2$						
				return null;
			}
			fout = new File(Context.cache_dir + "msgs" + sid + "_" + fid); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return fout;
	}
	
	void jmiAddAtt_actionPerformed(java.awt.event.ActionEvent e) {
		File[] files = Context.multipleFileOpenDialog(this);
		if (files == null)
			return;
		try {
			for (int i = 0; i < files.length; i++) {
				File fout = getOutFile();
				if (fout == null)
					return;
				IOUtils.copyFile(files[i], fout);
				/*
				 * final JButton jButtonAtt = new JButton();
				 * jButtonAtt.setBorderPainted(false);
				 * jButtonAtt.setText(file.getName());
				 * jButtonAtt.addMouseListener(new java.awt.event.MouseAdapter() {
				 * public void mouseClicked(java.awt.event.MouseEvent e) {
				 * jButtonAtt_mouseClicked(e); } });
				 */
				headerPanel.addAttachment(new CacheFile(fout, files[i]
						.getName(), null));
				if (headerPanel.subjectField.getText().trim().equals("")) {
					headerPanel.subjectField
							.setText(Strings
									.getString("ComposeFrame.fileAttachedSubj").replaceFirst("\\$1", files[i].getName()).toString()); //$NON-NLS-1$ //$NON-NLS-2$
				}

				StringBuffer sb = (new StringBuffer(Strings.getString(
						"ComposeFrame.fileAttachedBody").replaceFirst("\\$1",
						files[i].getName())));
				сomposePane1.setText(sb.append("\r\n").append(
						сomposePane1.getText()).toString());
			}
		} catch (Exception exc) {
			AlertDialog.showErrorDialog(exc);
		}
	}
	
	/**
	 * This method initializes jmiAddAtt
	 * 
	 * @return javax.swing.JMenuItem
	 */    
	private JMenuItem getJmiAddAtt() {
		if (jmiAddAtt == null) {
			jmiAddAtt = new JMenuItem();
			jmiAddAtt.setText(Strings.getString("ComposeFrame.fileButton"));
			jmiAddAtt.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jmiAddAtt_actionPerformed(e);
				}
			});
		}
		return jmiAddAtt;
	}

	/**
	 * This method initializes bottomPanel
	 * 
	 * @return javax.swing.JPanel
	 */    
	private JPanel getBottomPanel() {
		if (bottomPanel == null) {
			bottomPanel = new JPanel();
		}
		return bottomPanel;
	}	
	
} // @jve:decl-index=0:visual-constraint="10,10"

class CacheFile {
	private File file = null;
	private String name = null;
	private String mimeType = null;
	
	public CacheFile(File file, String name, String mimeType) {
		this.file = file;
		this.name = name;
		this.mimeType = mimeType;
	}
	
	public File getFile() {
		return file;
	}
	
	public String getName() {
		return name;
	}
	
	public String getMimeType() {
		return mimeType;
	}
	
}