/*
 * @(#)ComposePane.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.Component;

import javax.swing.JEditorPane;
import javax.swing.JMenu;
import javax.swing.JScrollPane;

import com.hexidec.ekit.EkitCore;

public class ComposePane {

	private JScrollPane jScrollPane1 = null;
	private JEditorPane jEditorPane1 = null;	
	private String contentType = "text/plain";
	private EkitCore ekitCore = null;	
	
	public ComposePane() {
		super();
	}

	public void setContentType(String contentType) {
		this.contentType = contentType.toLowerCase();
	}
	
	public String getContentType() {
		return contentType;
	}
	
	public Component getComponent() {
		if (contentType.equalsIgnoreCase("text/plain")) {
			ekitCore = null;
			return getJScrollPane1();
		} else if (contentType.equalsIgnoreCase("text/html")) {
			jEditorPane1 = null;
			jScrollPane1 = null;
			return getEkitCore();
		}
		return null;
	}

	/**
	 * This method initializes jScrollPane1
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getJEditorPane1());
		}
		return jScrollPane1;
	}
	
	/**
	 * This method initializes jEditorPane1
	 * 
	 * @return javax.swing.JEditorPane
	 */
	private JEditorPane getJEditorPane1() {
		if (jEditorPane1 == null) {
			jEditorPane1 = new JEditorPane();
			TextPopup.registerComponent(jEditorPane1);
		}
		return jEditorPane1;
	}

	private EkitCore getEkitCore() {
		if (ekitCore == null) {
			ekitCore = new EkitCore(null, null, null, null, null, true, false,
					true, true, Context.language, Context.country, false, false, false, false,
					(true ? EkitCore.TOOLBAR_DEFAULT_MULTI
							: EkitCore.TOOLBAR_DEFAULT_SINGLE));
			TextPopup.registerComponent(ekitCore.getTextPane());
		}
		return ekitCore;
	}					

	public void setText(String text) {
		if (contentType.equalsIgnoreCase("text/plain"))
			jEditorPane1.setText(text);
		else if (contentType.equalsIgnoreCase("text/html"))
			ekitCore.setDocumentText(text);				
	}

	public void setCaretPosition(int pos) {
		if (contentType.equalsIgnoreCase("text/plain"))
			jEditorPane1.setCaretPosition(pos);
		else if (contentType.equalsIgnoreCase("text/html"))
			ekitCore.setCaretPosition(pos);						
	}
	
	public String getText() {
		String result = null;
		if (contentType.equalsIgnoreCase("text/plain"))
			result = jEditorPane1.getText();
		else if (contentType.equalsIgnoreCase("text/html"))
			result = ekitCore.getDocumentText();
		return result;
	}

	public String getSelectedText() {
		String result = null;
		if (contentType.equalsIgnoreCase("text/plain"))
			result = jEditorPane1.getSelectedText();
		else if (contentType.equalsIgnoreCase("text/html"))
			result = ekitCore.getTextPane().getSelectedText();
		return result;
	}
		
	public JMenu getMenu() {
		if (contentType.equalsIgnoreCase("text/plain"))
			return null;
		else if (contentType.equalsIgnoreCase("text/html")) {
			JMenu jMenu = new JMenu();
			jMenu.setText(Strings.getString("ComposeFrame.menuHtmlEditor")); //$NON-NLS-1$
			for (int i = 1; i < 6; i++) {
				jMenu.add(ekitCore.getMenuBar().getMenu(i));
			}			
			return jMenu;
		} else
			return null;		
	}
	
}
