/*
 * @(#)CpDialog.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JLabel;

public class CpDialog extends JDialog {
	static final long serialVersionUID = 5641570478330154063L;
	private final String WIN_ID = "cp_select";
	private JPanel jContentPane = null;
	private JPanel jPanel1 = null;
	private JPanel jPanel2 = null;
	private JComboBox jcbCodepage = null;
	private static String result = null;
	private JButton okButton = null;
	private JButton cancelButton = null;
	private JLabel jlCodepage = null;
	private JButton autoButton = null;
	/**
	 * This is the default constructor
	 */
	private CpDialog() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 111);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setTitle(Strings.getString("CpDialog.title")); //$NON-NLS-1$
		this.setContentPane(getJContentPane());
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}	
	
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel2(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jlCodepage = new JLabel();
			jlCodepage.setText(Strings.getString("CpDialog.codepage")); //$NON-NLS-1$
			jPanel1 = new JPanel();
			jPanel1.add(jlCodepage, null);
			jPanel1.add(getJcbCodepage(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();
			jPanel2.setPreferredSize(new java.awt.Dimension(10,40));
			jPanel2.add(getOkButton(), null);
			jPanel2.add(getAutoButton(), null);
			jPanel2.add(getCancelButton(), null);
		}
		return jPanel2;
	}

	/**
	 * This method initializes jcbCodepage	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbCodepage() {
		if (jcbCodepage == null) {
			jcbCodepage = new JComboBox(Charsets.getCpArraySorted());
			jcbCodepage.setPreferredSize(new java.awt.Dimension(150,20));
			jcbCodepage.setEditable(true);
			jcbCodepage.setSelectedItem(Context.defaultCodepage);
		}
		return jcbCodepage;
	}
	
	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton")); //$NON-NLS-1$
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					result = (String) jcbCodepage.getSelectedItem();
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return okButton;
	}

	private void center() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        Point center = ge.getCenterPoint();
        Rectangle bounds = ge.getMaximumWindowBounds();
        int w = Math.min(this.getWidth(), bounds.width);
        int h = Math.min(this.getHeight(), bounds.height);              
        int x = center.x - w/2;
        int y = center.y - h/2;
        this.setBounds(x, y, w, h);
        if(this.isVisible())
        	this.validate();        
    }
		
	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			cancelButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					result = null;
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return cancelButton;
	}
	
	private void setOptions(boolean editable, boolean enableAuto) {
		jcbCodepage.setEditable(editable);
		autoButton.setEnabled(enableAuto);
	}

	public static String selectCodepage() {
		return selectCodepage(true, true);
	}

	public static String selectCodepage(boolean editable, boolean enableAuto) {
		CpDialog cpd = new CpDialog();
		cpd.setOptions(editable, enableAuto);
		cpd.setModal(true);
		cpd.center();
		cpd.setVisible(true);
		return result;
	}

	/**
	 * This method initializes autoButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getAutoButton() {
		if (autoButton == null) {
			autoButton = new JButton();
			autoButton.setText(Strings.getString("cpAuto")); //$NON-NLS-1$
			autoButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					result = "Auto";
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return autoButton;
	}	
		
}  //  @jve:decl-index=0:visual-constraint="10,10"
