/*
 * @(#)Folder.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.JMenuItem;

public class Folder implements PopupFactory {

	protected String id = null;

	protected String name = null;
	
	private char fType = FTYPE_LOCAL;

	protected String fldId = null;

	protected int newMsgCount = 0;

	protected String dispName = null;

	public static final String FOLDER_INBOX = "__INBOX";

	public static final String FOLDER_OUTBOX = "__OUTBOX";

	public static final String FOLDER_SENT = "__SENT";

	public static final String FOLDER_LOCAL = "__LOCAL";

	public static final String FOLDER_TRASH = "__TRASH";

	public static final String FOLDER_DRAFT = "__DRAFT";
	
	public static final char FTYPE_LOCAL = 'L';
	
	public static final char FTYPE_IMAP = 'I';
	
	public static final char FTYPE_FILE = 'F';
	
	public static final char FTYPE_SEARCH = 'S';

	private JPopupMenu folderPopup = null;

	private JMenuItem jmiProperties = null;
	
	protected JTree popupTree = null;
	protected DefaultMutableTreeNode popupDmtn;

	private JMenuItem jmiDelete = null;	
	
	public int getNewMsgCount() {
		return newMsgCount;
	}

	public void setNewMsgCount(int newMsgCount) {
		this.newMsgCount = newMsgCount;
	}

	public Folder() {
	}

	public Folder(String id, String name, String fldId) {
		this.id = id;
		this.name = name;
		this.fldId = fldId;		
	}
	
	public void setId(String id) {
		this.id = id;		
	}
	
	public void setName(String name) {
		this.name = name;		
	}

	public void setFldId(String fldId) {
		this.fldId = fldId;		
	}	
	
	public String getId() {
		return id;
	}
	
	public String getName() {
		return name;
	}

	public String getFldId() {
		return fldId;
	}
	
	public boolean isImap() {
		return fType == FTYPE_IMAP;
	}
		
	public String getDisplayName() {
		String result = name;
		if (result.equals(FOLDER_LOCAL))
			result = Strings.getString("localFolderName");		
		else if (result.equals(FOLDER_INBOX))
			result = Strings.getString("inboxFolderName");
		else if (result.equals(FOLDER_OUTBOX))
			result = Strings.getString("outboxFolderName");
		else if (result.equals(FOLDER_SENT))
			result = Strings.getString("sentFolderName");
		else if (result.equals(FOLDER_DRAFT))
			result = Strings.getString("draftFolderName");
		else if (result.equals(FOLDER_TRASH))
			result = Strings.getString("trashFolderName");
		return result;
	}
		
	public String toString() {
		dispName = getDisplayName();
		newMsgCount = 0;
		try {
			newMsgCount = Context.storage.getNewMsgCount(this.getId());
		} catch (Exception exc) {
			exc.printStackTrace();
		}
		return dispName + ((newMsgCount == 0)?"":" (" + newMsgCount + ")");
	}

	public char getFType() {
		return fType;
	}

	public void setFType(char type) {
		fType = type;
	}

	public JPopupMenu getPopupMenu(JTree tree, DefaultMutableTreeNode dmtn) throws Exception {
		this.popupTree = tree;
		this.popupDmtn = dmtn;
		return getFolderPopup();
	}

	/**
	 * This method initializes folderPopup	
	 * 	
	 * @return javax.swing.JPopupMenu	
	 */    
	private JPopupMenu getFolderPopup() {
		if (folderPopup == null) {
			folderPopup = new JPopupMenu();
			folderPopup.add(getJmiProperties());
			folderPopup.add(getJmiDelete());
		}
		return folderPopup;
	}

	/**
	 * This method initializes jmiProperties	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiProperties() {
		if (jmiProperties == null) {
			jmiProperties = new JMenuItem();
			jmiProperties.setText(Strings.getString("menuProperties"));
			jmiProperties.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					Context.mainFrame.folderProperties(popupDmtn);
				}
			});
		}
		return jmiProperties;
	}

	/**
	 * This method initializes jmiDelete	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiDelete() {
		if (jmiDelete == null) {
			jmiDelete = new JMenuItem();
			jmiDelete.setText(Strings.getString("menuDelete"));
			jmiDelete.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					Context.mainFrame.folderDelete(popupDmtn);
				}
			});
		}
		return jmiDelete;
	}
}
