/*
 * @(#)FolderFormFrame.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JFrame;

import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

public class FolderFormFrame extends JFrame {

	static final long serialVersionUID = 7531720164594326657L;
	private final String WIN_ID = "folder_form";
	private JPanel jContentPane = null;
	private JTextField idField = null;
	private JLabel jLabel1 = null;
	private JLabel jLabel2 = null;
	private JTextField nameField = null;
	private JButton okButton = null;
	private JButton cancelButton = null;
	private JTree tree = null;
	private DefaultMutableTreeNode treeNode = null;
	private static final int OP_CREATE = 0;
	private static final int OP_EDIT = 1;
	private int operation = -1;

	/**
	 * This is the default constructor
	 */
	public FolderFormFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(316, 120);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("FolderFormFrame.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel2 = new JLabel();
			jLabel2.setText(Strings.getString("FolderFormFrame.nameField")); //$NON-NLS-1$
			jLabel2.setPreferredSize(new java.awt.Dimension(70,16));
			jLabel1 = new JLabel();
			jLabel1.setText(Strings.getString("FolderFormFrame.idField")); //$NON-NLS-1$
			jLabel1.setPreferredSize(new java.awt.Dimension(70,16));
			jContentPane = new JPanel();
			jContentPane.setLayout(new FlowLayout());
			jContentPane.add(jLabel1, null);
			jContentPane.add(getIdField(), null);
			jContentPane.add(jLabel2, null);
			jContentPane.add(getNameField(), null);
			jContentPane.add(getOkButton(), null);
			jContentPane.add(getCancelButton(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes idField	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getIdField() {
		if (idField == null) {
			idField = new JTextField();
			idField.setPreferredSize(new java.awt.Dimension(220,20));
			idField.setEnabled(false);
			TextPopup.registerComponent(idField);
		}
		return idField;
	}

	/**
	 * This method initializes nameField	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getNameField() {
		if (nameField == null) {
			nameField = new JTextField();
			nameField.setPreferredSize(new java.awt.Dimension(220,20));
			TextPopup.registerComponent(nameField);
		}
		return nameField;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton")); //$NON-NLS-1$
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					okButton_actionPerformed(e);
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			cancelButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return cancelButton;
	}

/*	
	public void editFolder(String id) {
		try {
			Folder folder = Context.storage.getFolder(id);
			idField.setText(folder.getId());
			nameField.setText(folder.getName());
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
*/	

	void okButton_actionPerformed(ActionEvent e) {
		Folder folder = null;
		IFolder iFolder = null;
		Folder baseFolder = null;
		
		Object folderObject = treeNode.getUserObject();
		baseFolder = (Folder) folderObject;
				
		if (folderObject instanceof IFolder) {
/*			
			if (operation == OP_EDIT) {
				iFolder = (IFolder) folderObject;
			} else if (operation == OP_CREATE) {
				iFolder = new IFolder();
				iFolder.setFldId(baseFolder.getId());
			}
*/			
			iFolder = (IFolder) folderObject;
			iFolder.setName(nameField.getText());
		} else {
			if (operation == OP_EDIT) {
				folder = (Folder) folderObject;
			} else if (operation == OP_CREATE) {
				folder = new Folder();
				folder.setFldId(baseFolder.getId());
			}
			folder.setName(nameField.getText());
		}		
				
		Cursor oldCursor = getCursor();
		setCursor(new Cursor(Cursor.WAIT_CURSOR));
		
		try {
			if (folder != null)
				folder = Context.storage.saveFolder(folder);
			else {
				if (operation == OP_CREATE) {
/*					
					iFolder = IMAPContext.createFolder((IFolder) folderObject,
							iFolder, false);
*/
					iFolder = IMAPContext.operateFolder(iFolder, IMAPContext.FOLDER_CREATE);
				} else if (operation == OP_EDIT) {
					//iFolder = IMAPContext.renameFolder(iFolder);
					iFolder = IMAPContext.operateFolder(iFolder, IMAPContext.FOLDER_RENAME);
				}
								
				iFolder = Context.storage.saveImapFolder(iFolder);
			}
			// parent.listFolders();
			if (operation == OP_CREATE) {
				DefaultMutableTreeNode dmtn = null;
				if (folder != null)
					dmtn = new DefaultMutableTreeNode(folder);
				else
					dmtn = new DefaultMutableTreeNode(iFolder);
				treeNode.add(dmtn);
				tree.updateUI();
			} else if (operation == OP_EDIT) {
				if (folder != null)
					treeNode.setUserObject(folder);
				else
					treeNode.setUserObject(iFolder);
				tree.updateUI();
			}
			try {
				Context.loadFolders();
			} catch (Exception exc) {
				exc.printStackTrace();
			} finally {
				this_windowClosing(null);
				dispose();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		} finally {
			setCursor(oldCursor);
		}
		
	}
		
	public void createChildFolder(JTree tree, DefaultMutableTreeNode parent) {
		operation = OP_CREATE; 
		this.tree = tree;
		this.treeNode = parent;
	}

	public void editFolder(JTree tree, DefaultMutableTreeNode treeNode) {
		operation = OP_EDIT;
		this.tree = tree;
		this.treeNode = treeNode;
		Folder folder = (Folder)treeNode.getUserObject();
		idField.setText(folder.getId());
		nameField.setText(folder.getName());		
	}
		
}  //  @jve:decl-index=0:visual-constraint="10,10"
