/*
 * @(#)FoldersTreeFrame.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JButton;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;

public class FoldersTreeFrame extends JDialog {
	
	static final long serialVersionUID = 7531721917594326657L;	
	
	private final String WIN_ID = "folderstree";	
	
	private JPanel jContentPane = null;

	private JPanel topPanel = null;

	private JPanel leftPanel = null;

	private JPanel rightPanel = null;

	private JPanel bottomPanel = null;

	private JScrollPane jScrollPane1 = null;

	private Object folderObject = null;
	
	private JTree jTree1 = null;

	private JButton selectButton = null;

	private SpecialTreeFolders sf = new SpecialTreeFolders(); 

	private static Object result = null;

	private JButton cancelButton = null;
	
	/**
	 * This is the default constructor
	 */
	public FoldersTreeFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(307, 249);
		this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("FoldersTreeFrame.title"));
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getTopPanel(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getLeftPanel(), java.awt.BorderLayout.WEST);
			jContentPane.add(getRightPanel(), java.awt.BorderLayout.EAST);
			jContentPane.add(getBottomPanel(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getJScrollPane1(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes topPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getTopPanel() {
		if (topPanel == null) {
			topPanel = new JPanel();
		}
		return topPanel;
	}

	/**
	 * This method initializes leftPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getLeftPanel() {
		if (leftPanel == null) {
			leftPanel = new JPanel();
		}
		return leftPanel;
	}

	/**
	 * This method initializes rightPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getRightPanel() {
		if (rightPanel == null) {
			rightPanel = new JPanel();
		}
		return rightPanel;
	}

	/**
	 * This method initializes bottomPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getBottomPanel() {
		if (bottomPanel == null) {
			bottomPanel = new JPanel();
			bottomPanel.setPreferredSize(new java.awt.Dimension(10,30));
			bottomPanel.add(getSelectButton(), null);
			bottomPanel.add(getCancelButton(), null);
		}
		return bottomPanel;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getJTree1());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jTree1	
	 * 	
	 * @return javax.swing.JTree	
	 */    
	private JTree getJTree1() {
		if (jTree1 == null) {
			jTree1 = new JTree(getRootTreeNode(sf));
			jTree1.setRootVisible(false);
			jTree1.setRowHeight(16);			
			jTree1.setCellRenderer(new MyCellRenderer());
			jTree1.addMouseListener(new java.awt.event.MouseAdapter() { 
				public void mouseClicked(java.awt.event.MouseEvent e) {    
					jTree1_valueChanged(null);
				}
			});
			jTree1.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					jTree1_valueChanged(e);
				}
			});
			FoldersTreeFrame.expandTreeNode(jTree1, 0);
			FoldersTreeFrame.expandTreeNode(jTree1, 1);
		}
		return jTree1;
	}

	private Object getCurrentTreeObject() {
		if (jTree1.getSelectionPath() == null)
			return null;
		DefaultMutableTreeNode dutn = (DefaultMutableTreeNode) jTree1
				.getSelectionPath().getLastPathComponent();		
		return dutn.getUserObject();
	}
	
	void jTree1_valueChanged(TreeSelectionEvent e) {
		folderObject = getCurrentTreeObject();
		if (folderObject == null)
			return;
	}
		
	/**
	 * This method initializes selectButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getSelectButton() {
		if (selectButton == null) {
			selectButton = new JButton();
			selectButton.setText(Strings.getString("AddressesFrame.selectButton"));
			selectButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					result = folderObject;					
					this_windowClosing(null);
					dispose();
				}
			});
			selectButton.setPreferredSize(new java.awt.Dimension(85,20));
		}
		return selectButton;
	}

	private static MutableTreeNode getFolderTreeNode(MutableTreeNode parent, SpecialTreeFolders sf) {
		try {
/*			
			olivamail.gui.Folder parentFolder = 
				(olivamail.gui.Folder)((DynamicUtilTreeNode)parent).getUserObject();
*/
			olivamail.gui.Folder parentFolder = 
				(olivamail.gui.Folder)((DefaultMutableTreeNode)parent).getUserObject();
			
			if(parentFolder == null)
				return parent;
			olivamail.gui.Folder[] folders =
				(olivamail.gui.Folder[])Context.storage.getFoldersChildren(parentFolder.getId());
			for(int i = 0; i < folders.length; i++) {
				//folders[i].setName(Context.getFolderName(folders[i].getName()));				
				DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(folders[i]);
				if(folders[i].getId().equals(Context.storage.getInboxFolder()))
					sf.inboxTreeNode = dmtn; 				
				((DefaultMutableTreeNode)parent).add(getFolderTreeNode(dmtn, sf));
			}						
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}		
		return parent;
	}

	private static MutableTreeNode getImapFolderTreeNode(MutableTreeNode parent, boolean showUnsubscr) {
		try {
			IFolder parentFolder = 
				(IFolder)((DefaultMutableTreeNode)parent).getUserObject();			
			if(parentFolder == null)
				return parent;
			IFolder[] folders =
				Context.storage.getImapFoldersChildren(parentFolder.getId());
			for(int i = 0; i < folders.length; i++) {
				DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(folders[i]);
				MutableTreeNode mtn = getImapFolderTreeNode(dmtn, showUnsubscr);
				//IFolder iFolder = (IFolder)(((DefaultMutableTreeNode)mtn).getUserObject());
				if (showUnsubscr) {
					((DefaultMutableTreeNode) parent).add(mtn);
				} else {
					if (folders[i].getSubscrTemp() == IFolder.SUBSCR_ON) {
						((DefaultMutableTreeNode) parent).add(mtn);
						parentFolder.setSubsrTemp(IFolder.SUBSCR_ON);
					}
				}
			}						
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}		
		return parent;
	}	
	
	private static MutableTreeNode getSearchTreeNode(SpecialTreeFolders sf) {
		SearchFolder searchFolder = new SearchFolder(SearchFolder.SEARCH_NONE, Strings.getString("SearchFolder.search"));
		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(searchFolder);
		searchFolder = new SearchFolder(SearchFolder.SEARCH_BASIC, Strings.getString("SearchFolder.searchBasic"));
		sf.searchCommonTreeNode = new DefaultMutableTreeNode(searchFolder); 
		dmtn.add(sf.searchCommonTreeNode);
		searchFolder = new SearchFolder(SearchFolder.SEARCH_BIG_SIZE, Strings.getString("SearchFolder.searchBigSize"));
		dmtn.add(new DefaultMutableTreeNode(searchFolder));
		return dmtn;
	}
		
	public static MutableTreeNode getRootTreeNode(SpecialTreeFolders sf) {
		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode("Root");
		dmtn.add(getLocalTreeNode(sf));
		for(Enumeration en = getImapTreeNodes().elements(); en.hasMoreElements();)
			dmtn.add((MutableTreeNode)en.nextElement());		
		dmtn.add(getSearchTreeNode(sf));
		return dmtn; 
	}
	
	private static MutableTreeNode getLocalTreeNode(SpecialTreeFolders sf) {
		olivamail.gui.Folder rootFolder = null;
		try {
			rootFolder = Context.storage.getFolder(Context.storage.getLocalFolder());
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
/*		
		if(rootFolder != null)
			rootFolder.setName(Context.getFolderName(rootFolder.getName()));
*/			
		DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(
				rootFolder);
		return getFolderTreeNode(dmtn, sf);
	}

	private static Vector getImapTreeNodes() {
		Vector result = new Vector();
		try {
			Vector acnts = Context.storage.getImapAccountList();
			for (Enumeration en = acnts.elements(); en.hasMoreElements();) {
				Account acnt = (Account) en.nextElement();
				IFolder rootFolder = Context.storage.getImapRootFolder(acnt.id);
				if(rootFolder == null)
					continue;
				DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(
						rootFolder);
				result.add(getImapFolderTreeNode(dmtn, acnt.isShowUnsubscr()));
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
		return result;
	}
		
	public static void expandTreeNode(JTree jTree1, int rootChldIdx) {
		DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode) jTree1
				.getModel().getChild(jTree1.getModel().getRoot(), rootChldIdx);
		if (dmtn != null)
			jTree1.expandPath(new TreePath(dmtn.getPath()));
	}

	public synchronized static Object selectFolder() {
		result = null;
		FoldersTreeFrame ftf = new FoldersTreeFrame();
		ftf.setModal(true);
		ftf.setVisible(true);		
		return result;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					result = null;
					this_windowClosing(null);
					dispose();
				}
			});
			cancelButton.setPreferredSize(new java.awt.Dimension(77,20));
		}
		return cancelButton;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
