/*
 * @(#)IFolder.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.JMenuItem;

public class IFolder extends Folder implements PopupFactory {

	private final char fType = FTYPE_IMAP;	
	
	String fullName = null;

	public char subscr = SUBSCR_OFF;	

	public char subscrTemp = SUBSCR_NULL;
	
	public char subscrSrv = SUBSCR_NULL;

	public char syncMode = SYNC_HDR_NEW;

	public int imapType = 0;
	
	private int imapTypeTemp = -1;
	
	public String uidVal = null;
	
	public String acntId = null;
	
	public char dmStat = DM_SYNC;
	
	public static final char SUBSCR_OFF = '0';

	public static final char SUBSCR_ON = '1';	
	
	public static final char SUBSCR_NULL = 'N';
	
	public static final char SYNC_HDR_NEW = '0';
	
	public static final char SYNC_MSG_NEW = '1';
	
	public static final char SYNC_MSG_ALL = '2';
	
	public static final String FOLDER_INBOX = "INBOX";

	public static final String FOLDER_SENT = "INBOX.Sent";

	public static final String FOLDER_TRASH = "INBOX.Trash";

	public static final char DM_NOSYNC = 'N';
	
	public static final char DM_SYNC = 'S';

	public static final String ROOT = "__ROOT";
	
	private JPopupMenu iFolderPopup = null;

	private JMenuItem jmiIFldProps = null;

	private JMenuItem jmiWorkOffline = null;

	private JMenuItem jmiDelete = null;
			
	public char getSubscrTemp() {
		return  (subscrTemp == SUBSCR_NULL)?subscr:subscrTemp;
	}
	
	public void setSubsrTemp(char iSubscr) {
		this.subscrTemp = iSubscr; 
	}
	
	public boolean isSubscrChanged() {
		return subscr != subscrTemp;
	}

	public boolean needUpdateSrvSubscr() {
		return subscrSrv != getSubscrTemp();
	}	
	
	public void setImapTypeTemp(int imapType) {
		imapTypeTemp = imapType;
	}
	
	public boolean isImapTypeChanged() {
		return (imapTypeTemp != -1 && imapTypeTemp != imapType);
	}
	
	public int getImapTypeTemp() {
		return (imapTypeTemp == -1) ? imapType : imapTypeTemp;
	}	
	
	public void saveSubscrTemp() {
		if (subscrTemp != SUBSCR_NULL)
			subscr = subscrTemp;
		subscrTemp = SUBSCR_NULL;
	}

	public void saveImapTypeTemp() {
		if(imapTypeTemp != -1)
			imapType = imapTypeTemp;
		imapTypeTemp = -1;
	}
	
	public boolean isRootFolder() {
		return (fldId == null && name.equals(ROOT));
	}
	
	public String getDisplayName() {
		if(fullName == null)
			return name;
		
		if(isRootFolder()) {
			try {
				Account acnt = Context.storage.getAccount(acntId);
				return acnt.incomingServer;
			} catch (Exception exc) {
				exc.printStackTrace();
				return name;
			}			
		}
		
		String result = fullName;
		if (result.equals(FOLDER_INBOX))
			result = Strings.getString("inboxFolderName");
/*		
		else if (result.equals(FOLDER_SENT))
			result = Strings.getString("sentFolderName");
		else if (result.equals(FOLDER_TRASH))
			result = Strings.getString("trashFolderName");
*/			
		else
			result = name;
		return result;
	}	
		
	public IFolder() {
		super();
		super.setFType(FTYPE_IMAP);
	}
	
	public IFolder(String id, String name, String fldId) {
		super(id, name, fldId);
		super.setFType(FTYPE_IMAP);
	}
		
	public static IFolder create(javax.mail.Folder folder) throws Exception {
		IFolder iFolder = new IFolder();
		iFolder.setName(folder.getName());
		iFolder.fullName = folder.getFullName();
		iFolder.imapType = folder.getType();		
		iFolder.subscrSrv = folder.isSubscribed() ? IFolder.SUBSCR_ON
				: IFolder.SUBSCR_OFF;
		iFolder.subscr = iFolder.subscrSrv;		 
		return iFolder;
	}

	public char getFType() {
		return fType;
	}

	public boolean isOffline() {
		return dmStat == DM_NOSYNC;
	}

	public boolean isOnline() {
		return dmStat == DM_SYNC;
	}

	public String toString() {
		dispName = getDisplayName();
		newMsgCount = 0;
		try {
			newMsgCount = Context.storage.getNewMsgCount(this.getId());
		} catch (Exception exc) {
			exc.printStackTrace();
		}
		return dispName + ((newMsgCount == 0)?"":" (" + newMsgCount + ")") + (isOffline()?" [*]":"");
	}
	
	public boolean holdsMessages() {
		return (getImapTypeTemp() & javax.mail.Folder.HOLDS_MESSAGES) != 0;
	}

	public boolean holdsFolders() {
		return (getImapTypeTemp() & javax.mail.Folder.HOLDS_FOLDERS) != 0;
	}	

	public JPopupMenu getPopupMenu(JTree tree, DefaultMutableTreeNode dmtn) throws Exception {
		this.popupTree = tree;
		this.popupDmtn = dmtn;
		return getIFolderPopup();
	}

	/**
	 * This method initializes iFolderPopup	
	 * 	
	 * @return javax.swing.JPopupMenu	
	 */    
	private JPopupMenu getIFolderPopup() {
		if (iFolderPopup == null) {
			iFolderPopup = new JPopupMenu();
			iFolderPopup.add(getJmiDelete());
			iFolderPopup.addSeparator();
			iFolderPopup.add(getJmiIFldProps());
			if (fullName.equals(ROOT))
				iFolderPopup.add(getJmiWorkOffline());
		}
		if (jmiWorkOffline != null) {
			try {
				Account acnt = Context.storage.getAccount(acntId);
				jmiWorkOffline.setIcon(acnt.isOffline() ? Context
						.getImageIcon("check.png") : null);
			} catch (Exception exc) {
				exc.printStackTrace();
				AlertDialog.showErrorDialog(exc);
			}
		}
		return iFolderPopup;
	}

	/**
	 * This method initializes jmiIFldProps	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiIFldProps() {
		if (jmiIFldProps == null) {
			jmiIFldProps = new JMenuItem();
			jmiIFldProps.setText(Strings.getString("menuProperties"));
			jmiIFldProps.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					Context.mainFrame.folderProperties(popupDmtn);				
				}
			});
		}
		return jmiIFldProps;
	}

	/**
	 * This method initializes jmiWorkOffline	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiWorkOffline() {
		if (jmiWorkOffline == null) {
			jmiWorkOffline = new JMenuItem();
			jmiWorkOffline.setText(Strings.getString("MainFrame.workOffline"));
			jmiWorkOffline
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								Account acnt = Context.storage
										.getAccount(acntId);
								if (acnt.isOffline())
									IMAPContext.applyAcntDMChanges(acntId);
								else {
									acnt.dmStat = Account.DM_NOSYNC;
									Context.storage.saveAccount(acnt, false);
								}
							} catch (Exception exc) {
								exc.printStackTrace();
								AlertDialog.showErrorDialog(exc);
							}
						}
					});
		}
		return jmiWorkOffline;
	}

	/**
	 * This method initializes jmiDelete	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiDelete() {
		if (jmiDelete == null) {
			jmiDelete = new JMenuItem();
			jmiDelete.setText(Strings.getString("menuDelete"));
			jmiDelete.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					Context.mainFrame.folderDelete(popupDmtn);
				}
			});
		}
		return jmiDelete;
	}
		
}
