/*
 * @(#)IMAPContext.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.mail.Flags;

import olivamail.core.mail.MailBean;

public class IMAPContext {

	public static final int FOLDER_CREATE = 0;
	
	public static final int FOLDER_RENAME = 1;

	public static final int FOLDER_CLOSE = 2;	
	
	public static final int FOLDER_EXPUNGE = 3;
	
	public static final int FOLDER_DELETE = 4;
				
	public static Hashtable imapConnections = new Hashtable();

	public static boolean cashImapConnections = true;

	public static Vector loadFolders(Account acnt) throws Exception {
		MailBean mailBean = null;
		Vector result = null;
		try {
			mailBean = IMAPContext.getMailBean(acnt, false);
			if (mailBean == null)
				return result;

			result = mailBean.getAllFolders();

			if (!IMAPContext.cashImapConnections)
				mailBean.closeStore();
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!IMAPContext.cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return result;
	}

	public static boolean saveSubscription(Account acnt, Vector iFolders)
			throws Exception {
		MailBean mailBean = null;
		try {
			mailBean = IMAPContext.getMailBean(acnt, false);
			if (mailBean == null)
				return false;

			for (Enumeration en = iFolders.elements(); en.hasMoreElements();) {
				IFolder iFolder = (IFolder) en.nextElement();
				if (iFolder.needUpdateSrvSubscr()) {
					javax.mail.Folder folder = mailBean.getStore().getFolder(
							iFolder.fullName);
					folder.setSubscribed(iFolder.getSubscrTemp() == IFolder.SUBSCR_ON);
				}
			}

			if (!IMAPContext.cashImapConnections)
				mailBean.closeStore();
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!IMAPContext.cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return true;
	}

/*	
	public static boolean deleteMessage(IFolder iFolder,
			String msgUid) throws Exception {
		MailBean mailBean = null;
		boolean result = false;
		try {
			Account acnt = Context.storage.getAccount(iFolder.acntId);
			mailBean = IMAPContext.getMailBean(acnt);
			if (mailBean == null)
				return result;

			javax.mail.Folder folder = mailBean.getCurFolder();			
			if (folder != null && folder.getFullName().equals(iFolder.fullName)) {
				if (!folder.isOpen())
					folder.open(javax.mail.Folder.READ_WRITE);
			} else
				folder = mailBean.openFolder(iFolder.fullName, false);
*/								
/*			
			javax.mail.Folder folder = mailBean.getStore().getFolder(
					iFolder.fullName);
			folder.open(javax.mail.Folder.READ_WRITE);					
*/		
/*	

			javax.mail.Message message = ((com.sun.mail.imap.IMAPFolder) folder)
					.getMessageByUID(Long.parseLong(msgUid));
			if (message != null)
				message.setFlag(Flags.Flag.DELETED, !message.getFlags()
						.contains(Flags.Flag.DELETED));			

			folder.close(false);
			
			if (!IMAPContext.cashImapConnections) {				
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!IMAPContext.cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return result;
	}
*/	

/*	
	public static void closeFolder(IFolder iFolder,
			boolean expunge) throws Exception {
		MailBean mailBean = null;
		try {
			Account acnt = Context.storage.getAccount(iFolder.acntId);			
			mailBean = IMAPContext.getMailBean(acnt);
			if (mailBean == null)
				return;

			javax.mail.Folder folder = mailBean.getCurFolder();			
			if (folder == null || !folder.isOpen() || !folder.getFullName().equals(iFolder.fullName))
				return;				
			
			folder.close(expunge);

			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
	}	
*/	

	public static IFolder operateFolder(IFolder iFolder, int operation) throws Exception {
		MailBean mailBean = null;
		try {
			Account acnt = Context.storage.getAccount(iFolder.acntId);
			mailBean = IMAPContext.getMailBean(acnt, false);
			if (mailBean == null)
				return iFolder;

			javax.mail.Folder folder = mailBean.getCurFolder();			
			if (folder == null || !folder.getFullName().equals(iFolder.fullName)) {
				//try {} catch (javax.mail.FolderNotFoundException)
				folder = mailBean.openFolder(iFolder.fullName, false);
			}

			switch (operation) {
			case FOLDER_RENAME:
				if (folder.isOpen())
					folder.close(false);
				String newFullName = folder.getParent().getFullName()
						+ folder.getSeparator() + iFolder.getName();
				boolean oldSubscr = folder.isSubscribed();
				javax.mail.Folder nfolder = mailBean.getStore().getFolder(
						newFullName);
				if (folder.renameTo(nfolder)) {
					iFolder.setName(nfolder.getName());
					iFolder.fullName = nfolder.getFullName();
					nfolder.setSubscribed(oldSubscr);
					iFolder.subscr = nfolder.isSubscribed() ? IFolder.SUBSCR_ON
							: IFolder.SUBSCR_OFF;
				}
				break;
			case FOLDER_DELETE:
				if (folder.isOpen())
					folder.close(false);
				folder.delete(true);
				break;
			case FOLDER_CLOSE:
				if (folder.isOpen())
					folder.close(false);
				break;
			case FOLDER_EXPUNGE:
				if (!folder.isOpen())
					folder.open(javax.mail.Folder.READ_WRITE);
				folder.close(true);
				break;
			case FOLDER_CREATE:
				javax.mail.Folder cfolder = mailBean.getStore().getFolder(folder.getFullName() + folder.getSeparator() +  iFolder.getName());
				cfolder.create(javax.mail.Folder.HOLDS_FOLDERS | javax.mail.Folder.HOLDS_MESSAGES);
				cfolder.setSubscribed(true);
				IFolder result = IFolder.create(cfolder);
				result.setFldId(iFolder.getId());
				result.acntId = iFolder.acntId;
				iFolder = result;
				break;				
			}			
			
			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return iFolder;		
	}	
		
/*	
	public static IFolder deleteFolder(IFolder iFolder) throws Exception {
		MailBean mailBean = null;
		try {
			Account acnt = Context.storage.getAccount(iFolder.acntId);
			mailBean = IMAPContext.getMailBean(acnt);
			if (mailBean == null)
				return iFolder;

			javax.mail.Folder folder = mailBean.getCurFolder();			
			if (folder != null && folder.getFullName().equals(iFolder.fullName)) {
				if (folder.isOpen())
					folder.close(false);
			} else {
				//try {} catch (javax.mail.FolderNotFoundException)
				folder = mailBean.openFolder(iFolder.fullName, false);
				folder.close(false);
			}

			folder.delete(true);
			
			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return iFolder;		
	}	
		
	public static IFolder renameFolder(IFolder iFolder) throws Exception {
		MailBean mailBean = null;
		try {
			Account acnt = Context.storage.getAccount(iFolder.acntId);
			mailBean = IMAPContext.getMailBean(acnt);
			if (mailBean == null)
				return iFolder;

			javax.mail.Folder folder = mailBean.getCurFolder();			
			if (folder != null && folder.getFullName().equals(iFolder.fullName)) {
				if (folder.isOpen())
					folder.close(false);
			} else {
				//try {} catch (javax.mail.FolderNotFoundException)
				folder = mailBean.openFolder(iFolder.fullName, false);
				folder.close(false);
			}

			String newFullName = folder.getParent().getFullName() + folder.getSeparator() +  iFolder.getName();
			boolean oldSubscr = folder.isSubscribed();
			javax.mail.Folder nfolder = mailBean.getStore().getFolder(newFullName);			
			if(folder.renameTo(nfolder)) {
				iFolder.setName(nfolder.getName());
				iFolder.fullName = nfolder.getFullName();				
				nfolder.setSubscribed(oldSubscr);
				iFolder.subscr = nfolder.isSubscribed()?IFolder.SUBSCR_ON:IFolder.SUBSCR_OFF;
			}									
			
			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return iFolder;		
	}	
	
	public static IFolder createFolder(IFolder parentFolder, IFolder newFolder, boolean closeFolder) throws Exception {
		MailBean mailBean = null;
		IFolder result = null;
		try {
			Account acnt = Context.storage.getAccount(parentFolder.acntId);
			mailBean = IMAPContext.getMailBean(acnt);
			if (mailBean == null)
				return result;

			javax.mail.Folder folder = mailBean.getCurFolder();			
			if (folder != null && folder.getFullName().equals(parentFolder.fullName)) {
				if (!folder.isOpen())
					folder.open(javax.mail.Folder.READ_WRITE);
			} else
				folder = mailBean.openFolder(parentFolder.fullName, false);				

			javax.mail.Folder nfolder = mailBean.getStore().getFolder(folder.getFullName() + folder.getSeparator() +  newFolder.getName());
			nfolder.create(javax.mail.Folder.HOLDS_FOLDERS | javax.mail.Folder.HOLDS_MESSAGES);
			nfolder.setSubscribed(true);
			result = IFolder.create(nfolder);
			result.setFldId(parentFolder.getId());
			result.acntId = parentFolder.acntId;
			
			if (closeFolder)
				folder.close(false);			
			
			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return result;		
	}	
*/	
	
	public static boolean operateMessages(IFolder iFolder, boolean closeFolder,
			String[] msgUids, int operation, Flags.Flag flag,
			IFolder toIFolder, javax.mail.Message[] appendedMessages)
			throws Exception {
		MailBean mailBean = null;
		boolean result = false;
		try {
			Account acnt = Context.storage.getAccount(iFolder.acntId);
			mailBean = IMAPContext.getMailBean(acnt, false);
			if (mailBean == null)
				return result;

			javax.mail.Folder folder = mailBean.getCurFolder();			
			if (folder != null && folder.getFullName().equals(iFolder.fullName)) {
				if (!folder.isOpen())
					folder.open(javax.mail.Folder.READ_WRITE);
			} else
				folder = mailBean.openFolder(iFolder.fullName, false);										
			
			if (operation == MessageRow.MESSAGES_FLAGS_TOGGLE
					|| operation == MessageRow.MESSAGES_FLAGS_SET
					|| operation == MessageRow.MESSAGES_FLAGS_UNSET) {
				for (int i = 0; i < msgUids.length; i++) {
					javax.mail.Message message = ((com.sun.mail.imap.IMAPFolder) folder)
							.getMessageByUID(Long.parseLong(msgUids[i]));
					if (message != null) {
						if (operation == MessageRow.MESSAGES_FLAGS_TOGGLE)
							message.setFlag(flag, !message.getFlags().contains(
									flag));
						else if (operation == MessageRow.MESSAGES_FLAGS_SET)
							message.setFlag(flag, true);
						else if (operation == MessageRow.MESSAGES_FLAGS_UNSET)
							message.setFlag(flag, false);						
					}
				}
			} else if (operation == MessageRow.MESSAGES_COPY || operation == MessageRow.MESSAGES_MOVE) {
				javax.mail.Message messages[] = new javax.mail.Message[msgUids.length];
				for (int i = 0; i < msgUids.length; i++)
					messages[i] = ((com.sun.mail.imap.IMAPFolder) folder)
							.getMessageByUID(Long.parseLong(msgUids[i]));
				javax.mail.Folder toFolder = mailBean.getStore().getFolder(
						toIFolder.fullName);
				folder.copyMessages(messages, toFolder);
				if (operation == MessageRow.MESSAGES_MOVE) {
					for (int i = 0; i < msgUids.length; i++) {
						if (messages[i] != null) {
							messages[i].setFlag(Flags.Flag.DELETED, true);
						}
					}
				}
			} else if (operation == MessageRow.MESSAGES_APPEND) {
				folder.appendMessages(appendedMessages);
			}
			
			if (closeFolder)
				folder.close(false);

			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return result;
	}

	public static javax.mail.Message getMessage(IFolder iFolder,
			boolean closeFolder, String msgUid) throws Exception {
		javax.mail.Message[] messages = getMessages(iFolder, closeFolder,
				new String[] { msgUid });
		javax.mail.Message message = null;
		if (messages != null)
			message = messages[0];
		return message;
	}
	
	public static javax.mail.Message[] getMessages(IFolder iFolder,
			boolean closeFolder, String[] msgUids) throws Exception {
		MailBean mailBean = null;
		javax.mail.Message messages[] = null;
		try {
			Account acnt = Context.storage.getAccount(iFolder.acntId);
			mailBean = IMAPContext.getMailBean(acnt, false);
			if (mailBean == null)
				return messages;

			javax.mail.Folder folder = mailBean.getCurFolder();
			if (folder != null && folder.getFullName().equals(iFolder.fullName)) {
				if (!folder.isOpen())
					folder.open(javax.mail.Folder.READ_WRITE);
			} else
				folder = mailBean.openFolder(iFolder.fullName, false);

			messages = new javax.mail.Message[msgUids.length];
			for (int i = 0; i < msgUids.length; i++)
				messages[i] = ((com.sun.mail.imap.IMAPFolder) folder)
						.getMessageByUID(Long.parseLong(msgUids[i]));

			if (closeFolder)
				folder.close(false);

			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
		return messages;
	}
		
/*
 * public static boolean moveMessages(IFolder iFolder, IFolder toIFolder,
 * String[] msgUids) throws Exception { MailBean mailBean = null; boolean result =
 * false; try { Account acnt = Context.storage.getAccount(iFolder.acntId);
 * mailBean = IMAPContext.getMailBean(acnt); if (mailBean == null) return
 * result;
 * 
 * javax.mail.Folder folder = mailBean.getCurFolder(); if (folder != null &&
 * folder.getFullName().equals(iFolder.fullName)) { if (!folder.isOpen())
 * folder.open(javax.mail.Folder.READ_WRITE); } else folder =
 * mailBean.openFolder(iFolder.fullName, false);
 * 
 * javax.mail.Message messages[] = new javax.mail.Message[msgUids.length]; for
 * (int i = 0; i < msgUids.length; i++) messages[i] =
 * ((com.sun.mail.imap.IMAPFolder) folder)
 * .getMessageByUID(Long.parseLong(msgUids[i])); javax.mail.Folder toFolder =
 * mailBean.getStore().getFolder(toIFolder.fullName);
 * folder.copyMessages(messages, toFolder); for (int i = 0; i < msgUids.length;
 * i++) { if (messages[i] != null) messages[i].setFlag(Flags.Flag.DELETED,
 * true); }
 * 
 * folder.close(false);
 * 
 * if (!cashImapConnections) { mailBean.closeStore(); } } catch (Exception exc) {
 * exc.printStackTrace(); try { if (!cashImapConnections && mailBean != null)
 * mailBean.closeStore(); } catch (Exception e) { e.printStackTrace(); } throw
 * exc; } return result; }
 */	
		
	protected static MailBean getMailBean(Account acnt, boolean forDMupdate) throws Exception {		
		if(!Context.workOffline && !forDMupdate && acnt.dmStat != Account.DM_SYNC) {
			//applyAcntDMChanges(acnt);
			return null;
		}
				
		MailBean mailBean = null;
		if (cashImapConnections)
			mailBean = (MailBean) imapConnections.get(acnt.id);

		if (mailBean == null) {
			Context.debugStr("CACHED CONNECTION IS NOT FOUND");
			if ((mailBean = MailContext.getMailBean(acnt)) == null)
				return null;
			imapConnections.put(acnt.id, mailBean);
		} else {
			Context.debugStr("CACHED CONNECTION - FOUND");
			if (!mailBean.getStore().isConnected())
				mailBean.getStore().connect();
			else {
				Context.debugStr("THE STORE IS CONNECTED, NO PROBLEM");				
			}
		}
		
		return mailBean;
	}
	
	public static synchronized void applyAllAcntDMChanges() {		
		if (!Context.mainFrame.setReceiving()) {
			AlertDialog.showWarningDialog(Strings
					.getString("MainFrame.simultReceiving"));
			return;
		}			
				
		final SwingWorker worker = new SwingWorker() {

			public Object construct() {
				ProgressFrame pf = null;
				try {
					Vector vacnts = Context.storage.getAccountList(
							"DM_STAT != '" + Account.DM_SYNC + "'", null);
					
					if (vacnts != null && vacnts.size() > 0) {
						pf = new ProgressFrame();
						pf.setVisible(true);
					}
					
					for (Enumeration en = vacnts.elements(); en
							.hasMoreElements();) {												
						String acntId = (String) ((Vector) en.nextElement())
								.get(0);
/*
						if(acntId.equals(String.valueOf(14)))
							continue;
*/							
						Account acnt = Context.storage.getAccount(acntId);
						try {
							applyDMChanges(acnt, pf);
						} catch (Exception exc) {
							if (Context.isConnectionProblem(exc)) {
								AlertDialog.showErrorDialog(Strings
										.getString("srvConnProblem"), exc);
							} else {
								exc.printStackTrace();
								AlertDialog.showErrorDialog(exc);
							}
						} 						
					}
				} catch (Exception exc) {
					exc.printStackTrace();
					AlertDialog.showErrorDialog(exc);
				} finally {
					if(pf != null) {
						pf.dispose();
					}
					Context.mainFrame.unsetReceiving();
				}
				return null;
			}

		};
		worker.start();		
	}

	public static synchronized void applyAcntDMChanges(final String acntId) {
		if (!Context.mainFrame.setReceiving()) {
			AlertDialog.showWarningDialog(Strings
					.getString("MainFrame.simultReceiving"));
			return;
		}

		final SwingWorker worker = new SwingWorker() {

			public Object construct() {
				ProgressFrame pf = null;
				try {
					pf = new ProgressFrame();
					pf.setVisible(true);

					Account acnt = Context.storage.getAccount(acntId);
					try {
						applyDMChanges(acnt, pf);
					} catch (Exception exc) {
						if (Context.isConnectionProblem(exc)) {
							AlertDialog.showErrorDialog(Strings
									.getString("srvConnProblem"), exc);
						} else {
							exc.printStackTrace();
							AlertDialog.showErrorDialog(exc);
						}
					}
				} catch (Exception exc) {
					exc.printStackTrace();
					AlertDialog.showErrorDialog(exc);
				} finally {
					if (pf != null) {
						pf.dispose();
					}
					Context.mainFrame.unsetReceiving();
				}
				return null;
			}

		};
		worker.start();
	}
		
	private static synchronized void applyDMChanges(Account acnt, ProgressFrame pf) throws Exception {
		Context.mainFrame.resetCancelDelivery();
		
		if(acnt.dmStat == Account.DM_SYNC)
			return;
		MailBean mailBean = null;
		boolean warnMissing = true;
		
		try {					
			if (pf != null) {
				pf.setAccount(acnt.name);				
				pf.setOperationDescription(Strings
						.getString("ProgressFrame.descConnecting"));
			}
			
			//2008-12-04 mailBean = IMAPContext.getMailBean(acnt, true);			
/*			
			try {				
			} catch (Exception exc) {
				if (!Context.gonnaWorkOffline(exc))
					throw exc;
			}
*/			
/* 2008-12-04 
			if (mailBean == null)
				return;
*/

			Hashtable htIFolders = Context.storage.getAllImapFolders(acnt.id, false);							
			
			for(Enumeration en = htIFolders.elements(); en.hasMoreElements();) {
				if((pf != null && pf.isCanceled()) || Context.mainFrame.isDeliveryCanceled())
						throw new OperationCancelledException();
				
				IFolder iFolder = (IFolder)en.nextElement();
				if(iFolder.dmStat == IFolder.DM_SYNC)
					continue;
				
				//2008-12-04
				if (mailBean == null) {
					mailBean = IMAPContext.getMailBean(acnt, true);
					if (mailBean == null)
						return;
				}
				
				if(pf != null)
					pf.setOperationDescription("Folder: " + iFolder.fullName);								
				
				javax.mail.Folder folder = null;
//              try {
				folder = mailBean.openFolder(iFolder.fullName, false);
				int imapType = folder.getType();
				if ((imapType & javax.mail.Folder.HOLDS_MESSAGES) == 0) {
					iFolder.imapType = imapType;
					iFolder.dmStat = IFolder.DM_SYNC;
					Context.storage.saveImapFolder(iFolder);
					continue;
				}
/*
 * } catch (Exception exc) { if (exc instanceof javax.mail.MessagingException &&
 * exc.getMessage().equalsIgnoreCase( "folder cannot contain messages")) {
 * exc.printStackTrace(); iFolder.dmStat = IFolder.DM_SYNC;
 * Context.storage.saveImapFolder(iFolder); continue; } else throw exc; }
 */				
				folder.setSubscribed(iFolder.subscr == IFolder.SUBSCR_ON);				
				
				Vector vmsgs = Context.storage.getMessageList(iFolder.id, "DM_STAT != '" + MessageRow.DM_SYNC +
						"'", null, -1);							
				
				if(pf != null)
					pf.setMaximum(htIFolders.size());
				int pos = -1;				
				
				boolean warnSetFlagFailure = true;
				
				for(Enumeration enm = vmsgs.elements(); enm.hasMoreElements();) {
					if((pf != null && pf.isCanceled()) || Context.mainFrame.isDeliveryCanceled())
						throw new OperationCancelledException();
										
					pos++;
					if(pf != null)
						pf.setValue(pos);					
					
					MessageRow mr = (MessageRow)enm.nextElement();					
					
					if(mr.dmStat == MessageRow.DM_FLAG) {
						javax.mail.Message message = ((com.sun.mail.imap.IMAPFolder) folder)
								.getMessageByUID(Long.parseLong(mr.msgUid));
						if (message == null) {
							Context.storage.deleteMessage(mr.id, false);
							Context.mainFrame.removeRowFromTable(mr);
							warnMissing = showDelMissingAlert(mr, warnMissing);
							continue;
						} else {
							try {
								mr.setFlagsToAPIMessage(message);
							} catch (javax.mail.MessagingException me) {
								if (me.getMessage().toLowerCase().indexOf(
										"flag") != -1) {
									if (warnSetFlagFailure) {
										me.printStackTrace();
										int response = AlertDialog
												.showWarningDialog(
														Strings
																.getString("IMAPContext.setFlagFailure"),
														AlertDialog.YES_NO_CANCEL_OPTION);
										if (response == AlertDialog.CANCEL_OPTION)
											throw new OperationCancelledException();
										warnSetFlagFailure = (response == AlertDialog.YES_OPTION);
									}
								} else
									throw me;
							} 
						}
						mr.dmStat = MessageRow.DM_SYNC;
						Context.storage.saveMessageState(mr);
						Context.mainFrame.updateRowInTable(mr);
					} else if (mr.dmStat == MessageRow.DM_NEW) {
						folder
								.appendMessages(new javax.mail.Message[] { Context.storage
										.loadMessage(mr.id) });
						Context.storage.deleteMessage(mr.id, false);
						Context.mainFrame.removeRowFromTable(mr);
					}						
				}
				
				folder.close(false);
				iFolder.dmStat = IFolder.DM_SYNC;
				Context.storage.saveImapFolder(iFolder);
			}
			
			acnt.dmStat = Account.DM_SYNC;
			Context.storage.updateAccountState(acnt);
									
			if (!cashImapConnections) {
				mailBean.closeStore();
			}
		} catch (Exception exc) {
			if (!(exc instanceof OperationCancelledException))
				exc.printStackTrace();
			try {
				if (!cashImapConnections && mailBean != null)
					mailBean.closeStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
			if (!(exc instanceof OperationCancelledException))
				throw exc;
		} finally {
			Context.mainFrame.setCancelDelivery();			
		}
	}

	private static boolean showDelMissingAlert(MessageRow mr, boolean warnMissing) throws Exception {
		if (warnMissing) {
			int response = AlertDialog
					.showWarningDialog(
							Strings
									.getString(
											"MessageRow.skipHeader")
									.replaceAll(
											"\\$from",
											mr.addrFrom)
									.replaceAll(
											"\\$subject",
											mr.subject),
							AlertDialog.YES_NO_CANCEL_OPTION);
			if (response == AlertDialog.CANCEL_OPTION)
				throw new OperationCancelledException();
			warnMissing = (response == AlertDialog.YES_OPTION);
		}
		return warnMissing;
	}
		
}
