/*
 * @(#)IMAPFoldersFrame.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.util.Vector;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.JButton;
import javax.swing.JCheckBox;

public class IMAPFoldersFrame extends JFrame {

	static final long serialVersionUID = 485480432212L;

	private final String WIN_ID = "imapfolders";

	private JPanel jContentPane = null;

	private JScrollPane jScrollPane1 = null;

	private JTable jTable1 = null;

	private JPanel jPanelLeft = null;

	private JPanel jPanelRight = null;

	private JPanel jPanelBottom = null;

	private JPanel jPanelTop = null;

	private JButton loadFoldersButton = null;

	private JButton saveButton = null;

	private Account acnt = null;

	private IMAPFoldersTableModel imapModel = null;

	private JCheckBox jcbShowUnsubscr = null;
	
	/**
	 * This is the default constructor
	 */
	public IMAPFoldersFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(490, 297);
		this.setResizable(false);
		this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("IMAPFoldersFrame.title"));
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJScrollPane1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanelLeft(), java.awt.BorderLayout.WEST);
			jContentPane.add(getJPanelRight(), java.awt.BorderLayout.EAST);
			jContentPane.add(getJPanelBottom(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getJPanelTop(), java.awt.BorderLayout.NORTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getJTable1());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jTable1	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	private JTable getJTable1() {
		if (jTable1 == null) {
			jTable1 = new JTable() {
				static final long serialVersionUID = 3102406552614675292L;

/*
				public boolean isCellEditable(int rowIndex, int vColIndex) {
					return false;
				}
*/								
				
				public Component prepareRenderer(TableCellRenderer renderer,
						int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex,
							vColIndex);
					if (isCellSelected(rowIndex, vColIndex)) {// What else
						c.setBackground(Color.blue);
						c.setForeground(Color.white);
					} else {
						c.setBackground(Color.white);
						c.setForeground(Color.black);
					}
					int style = Font.PLAIN;

/*					
					if (((String)((IMAPFoldersTableModel) this.getModel()).getValueAt(rowIndex, IMAPFoldersTableModel.COL_SUBSCRIBE)).equals("1"))
						style = Font.PLAIN;
*/
					if (((Boolean)((IMAPFoldersTableModel) this.getModel()).getValueAt(rowIndex, IMAPFoldersTableModel.COL_SUBSCRIBE)).booleanValue())
						style = Font.BOLD;					

					c.setFont(c.getFont().deriveFont(style));
					// c.setFont(new java.awt.Font("Dialog", style, 12));
					// //$NON-NLS-1$
					return c;
				}
			};
			// jTable1.setFont(new java.awt.Font("Dialog", 0, 12));
			// //$NON-NLS-1$
//			jTable1.setCellEditor(new MessagesCellEditor());
			jTable1.addMouseListener(new java.awt.event.MouseAdapter() { 
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if(e.getClickCount() == 2)
						;
						//callOpenMessage(null);
				}
			});
/*			
			jTable1.getTableHeader().addMouseListener(
					new ColumnHeaderListener(this));					
			MessageSelectionListener listener = new MessageSelectionListener(jTable1, this);			
			jTable1.getSelectionModel().addListSelectionListener(listener);			
*/			
		}
		return jTable1;
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jPanelLeft	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelLeft() {
		if (jPanelLeft == null) {
			jPanelLeft = new JPanel();
		}
		return jPanelLeft;
	}

	/**
	 * This method initializes jPanelRight	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelRight() {
		if (jPanelRight == null) {
			jPanelRight = new JPanel();
		}
		return jPanelRight;
	}

	public void setAccount(final Account acnt) {
		this.acnt = acnt;
		jcbShowUnsubscr.setSelected(acnt.iShowUnsubscr.equals(Account.SHOW_UNSUBSCR_ON));
	}
	
	public synchronized void listFolders() {
		try {
			imapModel = new IMAPFoldersTableModel(acnt);
			jTable1.setModel(imapModel);
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
	
	private void loadIMAPFolders(final Account acnt) {
		final IMAPFoldersFrame iff = this;
		final SwingWorker worker = new SwingWorker() {
			public Object construct() {
				Cursor oldCursor = iff.getCursor();
				try {
					iff.setCursor(new Cursor(Cursor.WAIT_CURSOR));					
					Vector vFolders = IMAPContext.loadFolders(acnt);
					if (vFolders != null) {
						imapModel = new IMAPFoldersTableModel(acnt, vFolders);
						jTable1.setModel(imapModel);
						IMAPFoldersTableModel.setColumnModel(jTable1);
					}
				} catch (Exception exc) {
					if (Context.isConnectionProblem(exc)) {
						AlertDialog.showErrorDialog(Strings
								.getString("srvConnProblem"), exc);
					} else {
						exc.printStackTrace();
						AlertDialog.showErrorDialog(exc);
					}
				} finally {					
					iff.setCursor(oldCursor);
				}
				return null;
			}
		};
		worker.start();
	}

	/**
	 * This method initializes jPanelBottom	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelBottom() {
		if (jPanelBottom == null) {
			jPanelBottom = new JPanel();
			jPanelBottom.setPreferredSize(new java.awt.Dimension(10,55));
			jPanelBottom.add(getJcbShowUnsubscr(), null);
			jPanelBottom.add(getSaveButton(), null);
		}
		return jPanelBottom;
	}

	/**
	 * This method initializes jPanelTop	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelTop() {
		if (jPanelTop == null) {
			jPanelTop = new JPanel();
			jPanelTop.setPreferredSize(new java.awt.Dimension(10,30));
			jPanelTop.add(getLoadFoldersButton(), null);
		}
		return jPanelTop;
	}

	/**
	 * This method initializes loadFoldersButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getLoadFoldersButton() {
		if (loadFoldersButton == null) {
			loadFoldersButton = new JButton();
			loadFoldersButton.setText(Strings
					.getString("IMAPFoldersFrame.loadButton"));
			loadFoldersButton.setPreferredSize(new java.awt.Dimension(211, 20));
			loadFoldersButton
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								//Refresh offline status
								acnt = Context.storage.getAccount(acnt.id);
							} catch (Exception exc) {
								exc.printStackTrace();
								AlertDialog.showErrorDialog(exc);
							}
							if (acnt.isOffline()) {
								if (AlertDialog
										.showConfirmDialog(
												Strings
														.getString("MainFrame.gonnaWorkOnline"),
												AlertDialog.YES_NO_OPTION) == AlertDialog.YES_OPTION)
									IMAPContext.applyAcntDMChanges(acnt.id);
								return;
							}
							loadIMAPFolders(acnt);
						}

					});
		}
		return loadFoldersButton;
	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setText(Strings.getString("IMAPFoldersFrame.saveButton"));
			saveButton.setPreferredSize(new java.awt.Dimension(214,20));
			saveButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					saveSubscription();
				}
			});
		}
		return saveButton;
	}
	
	private void saveSubscription() {
		Cursor oldCursor = getCursor();
		setCursor(new Cursor(Cursor.WAIT_CURSOR));
		try {
			if (imapModel == null) {
				AlertDialog.showWarningDialog(Strings
						.getString("IMAPFoldersFrame.loadFirst"));
			} else {
				imapModel.saveSubscription(jcbShowUnsubscr.isSelected());
			}
			acnt.iShowUnsubscr = jcbShowUnsubscr.isSelected() ? Account.SHOW_UNSUBSCR_ON
					: Account.SHOW_UNSUBSCR_OFF;
			Context.storage.updateAccountState(acnt);
			//Context.storage.setAcntShowUnsubscr(acnt.id, acnt.iShowUnsubscr);
			// Update MainFrame's JTree
			Context.loadFolders();
			Context.mainFrame.updateTree();
			Context.mainFrame.selectInboxFolder();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		} finally {
			setCursor(oldCursor);
		}
	}

	/**
	 * This method initializes jcbShowUnsubscr	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbShowUnsubscr() {
		if (jcbShowUnsubscr == null) {
			jcbShowUnsubscr = new JCheckBox();
			jcbShowUnsubscr.setText(Strings.getString("IMAPFoldersFrame.showUnsubscr"));
			jcbShowUnsubscr.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
			jcbShowUnsubscr.setPreferredSize(new java.awt.Dimension(350,18));
		}
		return jcbShowUnsubscr;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
