/*
 * @(#)IMAPFoldersTableModel.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.Component;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.mail.Folder;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

public class IMAPFoldersTableModel extends AbstractTableModel {

	static final long serialVersionUID = 2211334L;
	
	public static final int COL_NAME = 0;	
	
	public static final int COL_SUBSCRIBE = 1;		
		
	public static final int shownColumns[] = { COL_NAME, COL_SUBSCRIBE };					
	
	Vector imapFolders = null;
	
	Vector data = null;
	
	private char separator = '.';
	
	Account acnt = null;
	
	Hashtable storedFolders = null;
	
	public IMAPFoldersTableModel(Account acnt, Vector folders) throws Exception {
		this.acnt = acnt;		
		imapFolders = folders;
		
		if(imapFolders != null && imapFolders.size() > 0)
			imapFolders.remove(0);

		storedFolders = Context.storage.getAllImapFolders(acnt.id, false);		
		
		data = new Vector(imapFolders.size());		
		
		for(Enumeration en = imapFolders.elements(); en.hasMoreElements();) {
			Folder folder = (Folder)en.nextElement();
			separator = folder.getSeparator();
			IFolder iFolder = (IFolder)storedFolders.get(folder.getFullName());
			if(iFolder == null) {
				iFolder = IFolder.create(folder);				
			} else {
				iFolder.setImapTypeTemp(folder.getType());				
				iFolder.subscrSrv = folder.isSubscribed()?IFolder.SUBSCR_ON:IFolder.SUBSCR_OFF;
				iFolder.setSubsrTemp(iFolder.subscrSrv);				
				storedFolders.remove(folder.getFullName());
			}
			data.add(iFolder);			
		}
		
		for(Enumeration en = storedFolders.elements(); en.hasMoreElements();) {
			IFolder iFolder = (IFolder)en.nextElement();
			iFolder.setSubsrTemp(IFolder.SUBSCR_OFF);
			data.add(iFolder);
		}
	}
	
	public IMAPFoldersTableModel(Account acnt) throws Exception {
		this.acnt = acnt;
		Hashtable htFlds = Context.storage.getAllImapFolders(acnt.id, false);
		data = new Vector(htFlds.size() + 5);
		for (Enumeration en = htFlds.elements(); en.hasMoreElements();) {
			data.add(en.nextElement());
		}
	}
	
	public int getColumnCount() {
		return shownColumns.length;
	}

	public int getRowCount() {
		return data.size();
	}
	
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		IFolder iFolder = (IFolder)data.get(rowIndex);
		switch (shownColumns[columnIndex]) {
		case COL_SUBSCRIBE:
			if (iFolder.holdsMessages())
				iFolder
						.setSubsrTemp(((Boolean) aValue).booleanValue() ? IFolder.SUBSCR_ON
								: IFolder.SUBSCR_OFF);
		}
		fireTableRowsUpdated(rowIndex, rowIndex);		
	}
		
	public IFolder getFolderAt(int rowIndex) {
		return (IFolder)data.get(rowIndex);
	}	
	
	public Object getValueAt(int rowIndex, int columnIndex) {
		IFolder iFolder = (IFolder) data.get(rowIndex);
		switch (shownColumns[columnIndex]) {
		case COL_NAME:
			return iFolder.fullName;
		case COL_SUBSCRIBE:
			return new Boolean((iFolder.getSubscrTemp() == IFolder.SUBSCR_ON)?true:false);
		}
		return null;
	}

	public String getColumnName(int column) {
		switch(shownColumns[column]) {
		case COL_NAME:
			return Strings.getString("IMAPFoldersFrame.nameCol");
		case COL_SUBSCRIBE:
			return Strings.getString("IMAPFoldersFrame.subscrCol");	
		}
		return null;
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		if(shownColumns[columnIndex] == COL_SUBSCRIBE)
			return true;
		return false;
	}

    public void sortAllRowsBy(int colIndex, boolean ascending) {
        Collections.sort(data, new IFColumnSorter(colIndex, ascending));
        fireTableStructureChanged();
    }	
	
    public void addRow(IFolder iFolder) {
    	insertRow(getRowCount(), iFolder);
    }      
    
    public void insertRow(int row, IFolder iFolder) {
    	data.insertElementAt(iFolder, row);  
        fireTableRowsInserted(row, row);
    }
    
    public void removeRow(int row) {
        data.removeElementAt(row);
        fireTableRowsDeleted(row, row);
    }    

    public static void setColumnModel(JTable jTable1) {
		TableColumnModel tcm = jTable1.getColumnModel();
		for(int i = 0; i < tcm.getColumnCount(); i++) {
			setupColumn(tcm.getColumn(i), i);
		}
	}
    
    private static void setupColumn(TableColumn tableColumn, int column) {
		switch(shownColumns[column]) {
		case COL_NAME:
			tableColumn.setHeaderRenderer(
					new IFHeaderRenderer());
			break;
		case COL_SUBSCRIBE:
			tableColumn.setHeaderRenderer(
					new IFHeaderRenderer());
			tableColumn.setCellRenderer(
					new IFCheckBoxRenderer());
			tableColumn.setMaxWidth(50);
			break;
		}		
    }
    
    public Class getColumnClass(int columnIndex) {
		switch (shownColumns[columnIndex]) {
		case COL_NAME:
			return String.class;
		case COL_SUBSCRIBE:
			return Boolean.class;
		default:
			return String.class;
		}
	}    
    
    public void saveSubscription(boolean showUnsubscr) throws Exception {
    	//Refresh offline status
    	boolean offline = true;
    	acnt = Context.storage.getAccount(acnt.id);
    	if (!Context.workOffline && acnt.isOnline()) {
			try {
				IMAPContext.saveSubscription(acnt, data);
				offline = false;
			} catch (Exception exc) {
				if (!Context.gonnaWorkOffline(exc, acnt))
					throw exc;
			}
		}
    	
    	//Prepare storedFolders hashtable to be used to find a parent during the folder save process  
		storedFolders = Context.storage
				.getAllImapFolders(acnt.id, false);
		
		for (Enumeration en = data.elements(); en.hasMoreElements();) {
			IFolder iFolder = (IFolder) en.nextElement();
			if (showUnsubscr) {
				if (iFolder.getId() == null || iFolder.isSubscrChanged() || iFolder.isImapTypeChanged()) {
					if(offline)
						iFolder.dmStat = IFolder.DM_NOSYNC;
					saveFolder(iFolder);
				}
			} else {
				if (iFolder.isSubscrChanged() || iFolder.isImapTypeChanged()) {
					if(offline)
						iFolder.dmStat = IFolder.DM_NOSYNC;					
					saveFolder(iFolder);
				}
			}
		}
	}
    
    private IFolder saveFolder(IFolder iFolder) throws Exception {
    	if(iFolder.getId() == null) {
    		int idx = iFolder.fullName.lastIndexOf(separator);
    		if(idx == -1) {    			
    			IFolder rootFolder = Context.storage.getImapRootFolder(acnt.id); 
    			if(rootFolder == null) {
    				rootFolder = new IFolder();
    				rootFolder.setName("__ROOT");
    				rootFolder.fullName = "__ROOT";
    				rootFolder.acntId = acnt.id;
    				rootFolder = Context.storage.saveImapFolder(rootFolder);
    			}
    			iFolder.setFldId(rootFolder.id);
    		} else {
    			String parentFullName = iFolder.fullName.substring(0, idx);
    			IFolder parentFolder = (IFolder)storedFolders.get(parentFullName);
    			if(parentFolder == null) {
    				String parentName = null;
    				idx = parentFullName.indexOf(separator);
    				if(idx == -1) {
    					parentName = parentFullName;
    				} else {
    					parentName = parentFullName.substring(idx + 1);
    				}    				 
    				parentFolder = new IFolder();
    				parentFolder.setName(parentName);
    				parentFolder.fullName = parentFullName;
    				parentFolder.setSubsrTemp(IFolder.SUBSCR_OFF);
    				parentFolder = saveFolder(parentFolder);
    			}
    			iFolder.setFldId(parentFolder.getId());
    		}    		
			iFolder.acntId = acnt.id;    		
    	}
    	iFolder = Context.storage.saveImapFolder(iFolder);
    	storedFolders.put(iFolder.fullName, iFolder);
    	return iFolder;
    }
    
}

class IFHeaderRenderer extends JLabel implements TableCellRenderer {

	static final long serialVersionUID = 4451180638410833789L;

	public IFHeaderRenderer() {
		super();
	}

	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		switch (IMAPFoldersTableModel.shownColumns[column]) {
		case IMAPFoldersTableModel.COL_SUBSCRIBE:
			this.setHorizontalAlignment(SwingConstants.CENTER);
			this.setText(value.toString());
			break;
		default:
			this.setText(value.toString());
			break;
		}
		return this;
	}

}

class IFCheckBoxRenderer extends JCheckBox implements TableCellRenderer {

	static final long serialVersionUID = 62447933232L;

	public IFCheckBoxRenderer() {
		super();
	}

	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		switch (IMAPFoldersTableModel.shownColumns[column]) {
		case IMAPFoldersTableModel.COL_SUBSCRIBE:
			this.setSelected(((Boolean)value).booleanValue());
			this.setHorizontalAlignment(SwingConstants.CENTER);			
			break;			
		}
		return this;
	}
/*	
	public boolean isCellEditable(EventObject anEvent) {
		if(anEvent instanceof MouseEvent) { 
			if(((MouseEvent)anEvent).getClickCount() == 1)
		return true;
		}		
		return false;
	}	
*/	
}

// This comparator is used to sort vectors of data
class IFColumnSorter implements Comparator {
    int colIndex;
    boolean ascending;
    int iasc = 1;
    
    IFColumnSorter(int colIndex, boolean ascending) {
        this.colIndex = colIndex;
        this.ascending = ascending;
        if(!ascending)
        	iasc = -1;
    }
    
    public int compare(Object o1, Object o2) {

    	IFolder a = (IFolder)o1;
    	IFolder b = (IFolder)o2;
    	
        if (a == null && b == null) {
            return 0;
        } else if (a == null) {
            return 1;
        } else if (b == null) {
            return -1;
        } 
    	    	
		switch(colIndex) {
		case IMAPFoldersTableModel.COL_NAME:			
			return iasc *  a.fullName.compareTo(b.fullName);
		case IMAPFoldersTableModel.COL_SUBSCRIBE:
			return iasc * (a.getSubscrTemp() - b.getSubscrTemp());
		default:
			return 0;
		}        
        
    }
}