/*
 * @(#)InternatFrame.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.JPanel;
import javax.swing.JDialog;

import olivamail.gui.SettingsInternat;
import olivamail.gui.Strings;

import javax.swing.JButton;

public class InternatFrame extends JDialog {

	static final long serialVersionUID = -4169422052898769592L;
	private JPanel jContentPane = null;
	private SettingsInternat internatPanel = null;
	private JPanel jPanel1 = null;
	private JButton okButton = null;
	private JButton cancelButton = null;

	/**
	 * This is the default constructor
	 */
	public InternatFrame() {
		super();
		initialize();
		try {
			internatPanel.readSettings();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 179);
		this.setContentPane(getJContentPane());
		this.setTitle("Internationalization settings");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getInternatPanel(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes internatPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getInternatPanel() {
		if (internatPanel == null) {
			internatPanel = new SettingsInternat();
			internatPanel.setApplyAnyway();
		}
		return internatPanel;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.setPreferredSize(new java.awt.Dimension(10,35));
			jPanel1.add(getOkButton(), null);
			jPanel1.add(getCancelButton(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton")); //$NON-NLS-1$
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					okButton_actionPerformed(e);
				}
			});
		}
		return okButton;
	}

	void okButton_actionPerformed(java.awt.event.ActionEvent e) {
		try {
			internatPanel.applyChanges();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
			return;
		}
		dispose();
	}
	
	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			cancelButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					dispose();
				}
			});
		}
		return cancelButton;
	}

	public void center() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        Point center = ge.getCenterPoint();
        Rectangle bounds = ge.getMaximumWindowBounds();
//        this.setSize(400, 250);
        int w = Math.min(this.getWidth(), bounds.width);
        int h = Math.min(this.getHeight(), bounds.height);              
        int x = center.x - w/2;
        int y = center.y - h/2;
        this.setBounds(x, y, w, h);
        if(this.isVisible())
        	this.validate();
    }
		
}  //  @jve:decl-index=0:visual-constraint="15,16"
