/*
 * @(#)LuceneSearchEngine.java
 *
 * Copyright (C) 2007 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.mail.Message;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.RAMDirectory;

public class LuceneSearchEngine implements ContentSearchEngine {

	public boolean searchContent(Message message, String cp,
			String contentSearchText) throws Exception {
		olivamail.core.mail.Message msg = null;
		try {
			msg = Context.mailBean.getMessage(message, cp);
		} catch (Exception exc) {
			exc.printStackTrace();
			return false;
		}
		if (msg.messageParts != null) {
			for (java.util.Enumeration en = msg.messageParts.elements(); en
					.hasMoreElements();) {
				olivamail.core.mail.MessagePart textPart = (olivamail.core.mail.MessagePart) en
						.nextElement();
				if (textPart.fileName != null || textPart.cid != null
						|| textPart.alternative
						|| textPart.type == olivamail.core.mail.MessagePart.T_RFC822
						|| textPart.type == olivamail.core.mail.MessagePart.T_BINARY
						)
					//  || textPart.type == olivamail.core.mail.MessagePart.T_EMPTY // for message/partial 						
					continue;
				String textPartText = null;
				if (!textPart.isMimeType("message/partial")) {					
					textPartText = msg.getTextPart(textPart.partNo);
				}
				if (textPartText != null) {
					if(analyze(textPartText, contentSearchText))
						return true;
				}
			}
		}						  
		return false;
	}

	private boolean analyze(String sourceText, String searchText)
			throws Exception {
		boolean result = false;
		Analyzer analyzer = new StandardAnalyzer();
		//			Store the index in memory:
		Directory directory = new RAMDirectory();
		IndexWriter iwriter = new IndexWriter(directory, analyzer, true);
		iwriter.setMaxFieldLength(25000);
		Document doc = new Document();
		//String text = "This is the text to be indexed.";
		String text = sourceText;
		doc.add(new Field("fieldname", text, Field.Store.YES,
				Field.Index.TOKENIZED));
		iwriter.addDocument(doc);
		iwriter.optimize();
		iwriter.close();
		// Now search the index:
		IndexSearcher isearcher = new IndexSearcher(directory);
		// Parse a simple query that searches for "text":
		QueryParser parser = new QueryParser("fieldname", analyzer);
		//Query query = parser.parse("text");
		Query query = parser.parse(searchText);
		Hits hits = isearcher.search(query);
		if (hits.length() > 0)
			result = true;
		/*		    
		 assertEquals(1, hits.length());
		 // Iterate through the results:
		 for (int i = 0; i < hits.length(); i++) {
		 Document hitDoc = hits.doc(i);
		 assertEquals("This is the text to be indexed.", hitDoc.get("fieldname"));
		 }
		 */
		isearcher.close();
		directory.close();
		return result;
	}
		
}
