/*
 * @(#)MailContext.java
 *
 * Copyright (C) 2007 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.RandomAccessFile;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.activation.DataHandler;
import javax.mail.AuthenticationFailedException;
import javax.mail.FetchProfile;
import javax.mail.Flags;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.UIDFolder;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import olivamail.core.common.ComUtils;
import olivamail.core.mail.MailBean;
import olivamail.core.mail.MailUtils;
import olivamail.core.mail.MessageHeader;
import olivamail.core.mail.NewMessage;
import olivamail.core.mail.SimpleMessage;

public class MailContext {

	protected static MailBean getMailBeanCheckCredentials(Account acnt) {
		Context.restorePwdFromCache(acnt);
			
		if (ComUtils.nvl(acnt.inUser).equals("")) {
			acnt.inUser = acnt.outUser;
			if (ComUtils.nvl(acnt.outPass).equals("")) {
				if (AccountCredentialsDialog
						.showCredentialsDialog(
								acnt,
								AccountCredentialsDialog.TYPE_IN) == AccountCredentialsDialog.RESPONSE_CANCEL) {
					return null;
				}
				Context.storePwdInCache(acnt);
			} else
				acnt.inPass = acnt.outPass;
		} else {
			if (ComUtils.nvl(acnt.inPass).equals("")) {
				if (AccountCredentialsDialog
						.showCredentialsDialog(
								acnt,
								AccountCredentialsDialog.TYPE_IN) == AccountCredentialsDialog.RESPONSE_CANCEL) {
					return null;
				}
				Context.storePwdInCache(acnt);
			}
		}
		
		return Context.getMailBean(acnt);		
	}
	
	protected static MailBean getMailBean(Account acnt) throws Exception {
		MailBean mailBean = null;
		if((mailBean = getMailBeanCheckCredentials(acnt)) == null)
			return mailBean;
		
		//SSL
		Context.setSSL(acnt, true);					
/*
		pf = new ProgressFrame();
		pf.setVisible(true);
		pf.setAccount(acnt.name);
		pf.setOperationDescription(Strings.getString("ProgressFrame.descConnecting"));
*/	
		while(!logonSuccessfull(mailBean)) {
			if (AccountCredentialsDialog.showCredentialsDialog(
					acnt, AccountCredentialsDialog.TYPE_IN) == AccountCredentialsDialog.RESPONSE_CANCEL) {
				//pf.storeWindowPosition();
				//pf.dispose();
				return mailBean;
			}
			Context.storePwdInCache(acnt);
			mailBean = Context.getMailBean(acnt);
		}
		
		return mailBean;
	}
		
	private static ReceivingResult receiveForAccount(Account acnt, IFolder thisIFolder) {		
		ReceivingResult result = null;		
		MailBean mailBean = null;
		ProgressFrame pf = null;
		RandomAccessFile logFile = null;
		
		if(thisIFolder != null && !thisIFolder.holdsMessages())
			return result;
		
//		boolean ok = true;
		try {
			if(!Context.mainFrame.setReceiving()) {
				if (thisIFolder == null)
					AlertDialog.showWarningDialog(Strings
							.getString("MainFrame.simultReceiving"));
				return result;		
			}			
			
			if (acnt.isImap()) {
				if ((mailBean = IMAPContext.getMailBean(acnt, false)) == null)
					return result;
			} else {
				if ((mailBean = getMailBean(acnt)) == null)
					return result;
			}

			if (thisIFolder == null) {
				pf = new ProgressFrame();
				pf.setVisible(true);
				pf.setAccount(acnt.name);
				pf.setOperationDescription(Strings
						.getString("ProgressFrame.descConnecting"));
			}
			
			Context.mainFrame.resetCancelDelivery();
			Context.mainFrame.incReceiving(acnt.name);
						
			if (acnt.isImap()) {
				if (thisIFolder == null) {
					Enumeration eFolders = Context.storage.getAllImapFolders(
							acnt.id, true).elements();
					while (eFolders.hasMoreElements()) {
						IFolder iFolder = (IFolder) eFolders.nextElement();
						result = receiveForFolder(acnt, mailBean, pf, logFile,
								iFolder, result, false);
					}
				} else {
					result = receiveForFolder(acnt, mailBean, null, logFile,
							thisIFolder, result, true);
				}
			} else if (acnt.isPop3()) {
				result = receiveForFolder(acnt, mailBean, pf, logFile, null,
						null, false);
			}			
		} catch (Exception exc) {
//			ok = false;					
			exc.printStackTrace();
			try {
				if (mailBean != null) {
					mailBean.closeFolder(acnt.isPop3());
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
			if (Context.isConnectionProblem(exc)) {
				if (acnt.isImap())
					Context.gonnaWorkOffline(exc, acnt);
				else
					AlertDialog.showErrorDialog(Strings
							.getString("srvConnProblem"), exc);
			} else
				AlertDialog.showErrorDialog(exc);
		} finally {
			Context.mainFrame.setCancelDelivery();
/*			
			try {
				if(acnt.isImap() && mailBean != null) {
					mailBean.closeStore();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
*/						
			if(logFile != null) {
				try {
					logFile.close();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			if (pf != null) {
				try {
					pf.storeWindowPosition();
					pf.dispose();
				} catch (Exception e) {
					e.printStackTrace();
				}																	
			}
/*					
			if(showResults && ok) {
				try {
					selectInboxFolder();
					listMessages();
				} catch (Exception e) {
					e.printStackTrace();
				}											
			}	
*/
			if (result != null && result.getNew() > 0)
				Context.mainFrame.updateFldNewMsgInfo();
			
			Context.mainFrame.unsetReceiving();
			Context.mainFrame.decrReceiving();
		}
		return result;
	}
	
	private static MessageHeader getMsgHdr(MailBean mailBean,
			javax.mail.Message message, Vector saveInCpRules) throws Exception {
		MessageHeader msg = mailBean.getMessageHeader(message, null);
		String cpName = null;
		Rule rule = Rule.checkRules(saveInCpRules, msg);
		if (rule != null)
			cpName = rule.param;
		if (cpName != null) {
			msg = mailBean.getMessageHeader(message, cpName);
		}
		return msg;
	}
	
	private static ReceivingResult receiveForFolder(Account acnt,
			MailBean mailBean, ProgressFrame pf, RandomAccessFile logFile,
			IFolder iFolder, ReceivingResult result, boolean quick) throws Exception {
		if (pf != null)
			pf.setOperationDescription(Strings
					.getString("ProgressFrame.descReceiving"));
		
		javax.mail.Folder apiFolder = null;		
		long uidValidity = -1;
		
		if (iFolder != null) {
			try {
				apiFolder = mailBean.openFolder(iFolder.fullName, false);
			} catch (java.lang.IllegalStateException ise) {
				if (!ise.getMessage().equalsIgnoreCase("Not connected"))
					throw ise;
				ise.printStackTrace();
				mailBean.getStore().connect();
				apiFolder = mailBean.openFolder(iFolder.fullName, false);
			}
			uidValidity = ((com.sun.mail.imap.IMAPFolder)apiFolder).getUIDValidity();
			if(!ComUtils.nvl(iFolder.uidVal).equals(String.valueOf(uidValidity))) {
				iFolder.uidVal = String.valueOf(uidValidity);
				Context.storage.saveImapFolder(iFolder);
			}
		} else {
			apiFolder = mailBean.openFolder(false);
			uidValidity = ComUtils.nvl(acnt.incomingServer).toLowerCase().hashCode();
		}
		
		FetchProfile fp = new FetchProfile();
		fp.add(UIDFolder.FetchProfileItem.UID);
/*		
		if (acnt.inLeaveDays > 0)
			fp.add("Received");
*/			
		if (iFolder != null)
			fp.add(UIDFolder.FetchProfileItem.FLAGS);
		javax.mail.Message[] messages = apiFolder.getMessages();
		apiFolder.fetch(messages, fp);
		
		//int msgCount = mailBean.getMessageCount();
		int msgCount = messages.length;
		if (msgCount > 0) {
			Context.mainFrame.setProgressMax(msgCount);
			if (pf != null)
				pf.setMaximum(msgCount);
		}

		int skipped = 0;					
		boolean canceled = false;
		boolean exit = false;
//		boolean blindDelete = false;
		
		Vector deleteRules = null;
		Vector skipRules = null;
		Vector saveToFolderRules = null;
		Vector saveInCpRules = null;
		Vector fwdAttRules = null;
		Vector fwdCtntRules = null;
		Vector fwdReportRules = null;
		if (acnt.useRules.equals("1")) {
			deleteRules = Context.storage.getRulesList(acnt.id,
					Rule.ACTION_DELETE);
			skipRules = Context.storage.getRulesList(acnt.id,
					Rule.ACTION_SKIP);
			saveToFolderRules = Context.storage.getRulesList(acnt.id,
					Rule.ACTION_SAVE_TO_FOLDER);
			saveInCpRules = Context.storage.getRulesList(acnt.id,
					Rule.ACTION_SAVE_IN_CP);
			fwdAttRules = Context.storage.getRulesList(acnt.id,
					Rule.ACTION_FWD_ATT);
			fwdCtntRules = Context.storage.getRulesList(acnt.id,
					Rule.ACTION_FWD_CTNT);
			fwdReportRules = Context.storage.getRulesList(acnt.id,
					Rule.ACTION_FWD_REPORT);				
		}				

/*
        // HWM_UID STAFF		
		String lastHwmMsgUid = null;
		if(iFolder == null)
			lastHwmMsgUid = ComUtils.nvl(Context.storage.getAcntHwmUid(acnt.id));
		else
			lastHwmMsgUid = ComUtils.nvl(iFolder.hwmMsgUid);
*/	
		
		String hwmMsgUid = null;
		String lastHwmId = null;
		if(iFolder == null)
			lastHwmId = acnt.hwmMsgId;
			//lastHwmId = Context.storage.getAcntHwmId(acnt.id);
		String hwmId = null;
		int pos = -1;					
		
		Hashtable reportHeaders = new Hashtable(); 
		
		if (result == null)
			result = new ReceivingResult();
                    
		Hashtable htuids = null;
		String dummy = "";		
		if(iFolder != null)
			htuids = new Hashtable(msgCount);
		
		for (int i = msgCount; i >= 1; i--) {
			pos++;
			Context.mainFrame.setProgressVal(pos);
			if (pf != null) {
				pf.setValue(pos);

				pf.setRemainTime(-1);
				if (pf.isCanceled()) {
					canceled = true;
					break;
				}
			}
			if (Context.mainFrame.isDeliveryCanceled()) {
				canceled = true;
				break;
			}			
			
			javax.mail.Message curMessage = messages[i-1];
			
			MessageHeader fmsg = new MessageHeader(); 
			if (iFolder != null) {
				fmsg.UID = String
						.valueOf(((com.sun.mail.imap.IMAPFolder) apiFolder)
								.getUID(curMessage));
				htuids.put(fmsg.UID, dummy);				
			} else {
				fmsg.UID = ((com.sun.mail.pop3.POP3Folder) apiFolder)
						.getUID(curMessage);
			}
			
/*			
			if (acnt.inLeaveDays > 0)
				fmsg.receivedDate = mailBean.getReceivedDate(curMessage);
*/				
			
			MessageRow mr = Context.storage.getMessageByUid(fmsg.UID,
					uidValidity, lastHwmId, iFolder);						
									
/*			
            // HWM_UID STAFF
 
			if (blindDelete) {
				if (needDeleteByTime(fmsg, acnt)) {
					mailBean.deleteMessage(curMessage);
					result.incrementDeleted();
				} else {
					if (hwmMsgUid == null)
						hwmMsgUid = fmsg.UID;
					if (skipped == 1) {
						hwmMsgUid = fmsg.UID;
						skipped = 2;
					}
					// Assuming other messages are newer
					exit = true;
					break;
				}
				continue;
			}
											
			if (lastHwmMsgUid.length() > 0
					&& lastHwmMsgUid.equals(ComUtils.nvl(fmsg.UID))) {
				if (iFolder != null || acnt.inLeaveDays == -1) {
					if (hwmMsgUid == null)
						hwmMsgUid = fmsg.UID;
					if (skipped == 1) {
						hwmMsgUid = fmsg.UID;
						skipped = 2;
					}
					exit = true;
					break;
				} else {
					blindDelete = true;
					if (needDeleteByTime(fmsg, acnt)) {
						mailBean.deleteMessage(curMessage);
						result.incrementDeleted();
					} else {
						if (hwmMsgUid == null)
							hwmMsgUid = fmsg.UID;
						if (skipped == 1) {
							hwmMsgUid = fmsg.UID;
							skipped = 2;
						}
						// Assuming other messages are newer
						exit = true;
						break;
					}
					continue;
				}
			}			
									
			if (mr != null && !needDeleteByTime(fmsg, acnt)
					&& (mr.seen.equals(MessageRow.SEEN_UNREAD) || 
							mr.seen.equals(MessageRow.SEEN_READ) || 
							mr.seen.equals(MessageRow.SEEN_SPLITTED))) {
				if (hwmMsgUid == null)
					hwmMsgUid = fmsg.UID;
				if (skipped == 1) {
					hwmMsgUid = fmsg.UID;
					skipped = 2;
				}
				continue;
			}
*/			
			
			// DELETE PHASE
			if (iFolder == null) {
				if (mr != null) {
					// Time based delete
					if (needDeleteByTime(mr.getReceivedDate(), acnt) && !mr.isHeader()) {
						mailBean.deleteMessage(curMessage);
						result.incrementDeleted();
						continue;
					}
					// Delete because there is a message header in a trash
					// folder
					if (mr.isHeader()
							&& mr.folderId.equals(Context.storage
									.getTrashFolder())) {
						mailBean.deleteMessage(curMessage);
						result.incrementDeleted();
						Context.storage.deleteMessage(mr.id, false);
						// Remove the correspondent row from JTable
						Context.mainFrame.removeRowFromTable(mr);
						continue;
					}
				}

				// Rule based delete
				if (mr == null) {
					javax.mail.Message message = mailBean
							.getAPIMessage(i, null);
					MessageHeader msg = getMsgHdr(mailBean, message,
							saveInCpRules);
					
					if (Rule.checkRules(deleteRules, msg) != null) {
						mailBean.deleteMessage(message);
						result.incrementDeleted();
						StringBuffer sb = new StringBuffer();
						sb.append("From:       ").append(msg.From).append("\n")
								.append("To:         ").append(msg.To).append(
										"\n").append("Subject:    ").append(
										msg.Subject).append("\n").append(
										"SentDate:   ").append(
										msg.getSentDateToString()).append("\n")
								.append("Deleted At: ").append(
										(new Date()).toString()).append("\n")
								.append("\n\n");
						try {
							if (logFile == null) {
								logFile = new RandomAccessFile(Context.log_dir
										+ Context.log_filename, "rw");
								boolean reopen = false;
								if (logFile.length() > Context.log_file_limit) {
									logFile.close();
									reopen = true;
									File delFile = new File(Context.log_dir
											+ Context.log_filename);
									delFile.delete();
								}
								if (reopen)
									logFile = new RandomAccessFile(
											Context.log_dir
													+ Context.log_filename,
											"rw");
								logFile.skipBytes((int) logFile.length());
							}
							logFile.writeBytes(sb.toString());
						} catch (Exception e) {
							e.printStackTrace();
						}
						continue;
					}
				}
			}
			
			// SKIP PHASE

			Flags flags = null;
			if(iFolder != null)
				flags = curMessage.getFlags();
			
/*			
			String folderId = null;
			if(iFolder == null)
				folderId = Context.getInboxFolder();
			else
				folderId = iFolder.getId();
*/				
			
			//Skip if the message header is not forced for full receiving or receiving in quick mode
			if (mr != null
					&& (quick || !(mr != null && mr.hdr == MessageRow.HDR_FORCED))) {
				updateFlagsIfNeeded(iFolder, mr, flags);
				skipped = 1;
				continue;
			}
			
/*
 * javax.mail.Message message = mailBean.getAPIMessage(i, null); MessageHeader
 * msg = getMsgHdr(mailBean, message, saveInCpRules);
 */						
			MessageHeader msg = getMsgHdr(mailBean, curMessage, saveInCpRules);

			//block of getting folder by rule
			String folderId = null;
			Rule rule = null;			
			if (iFolder == null) {
				rule = Rule.checkRules(saveToFolderRules, msg);
				if (rule != null)
					folderId = rule.param;

				if (ComUtils.nvl(folderId).equals("")) {
					folderId = Context.storage.getInboxFolder();
				} else {
					Folder folder = Context.storage.getFolder(folderId);
					if (folder == null)
						folderId = Context.storage.getInboxFolder();
					else
						folderId = folder.getId();
				}
			} else {
				folderId = iFolder.getId();
			}			
			
			if (mr == null && quick) {
				Context.mainFrame.addRowToTableIfNeeded(Context.storage
						.insertMessageHeader(msg, folderId, uidValidity, flags),
						folderId);
				result.incrementSkipped();
				skipped = 1;
				continue;
			}						
			
			if (!(mr != null && mr.hdr == MessageRow.HDR_FORCED)) {				
				//Skip for IMAP sync mode SYNC_HDR_NEW
				if(iFolder != null && iFolder.syncMode == IFolder.SYNC_HDR_NEW) {
					if (mr == null) {
						Context.mainFrame.addRowToTableIfNeeded(Context.storage
								.insertMessageHeader(msg, folderId, uidValidity, flags),
								folderId);
						result.incrementSkipped();
					} else {
						updateFlagsIfNeeded(iFolder, mr, flags);						
					}
					skipped = 1;
					continue;					
				}
				//Skip for IMAP sync mode SYNC_MSG_NEW
				if(mr != null && iFolder != null && iFolder.syncMode == IFolder.SYNC_MSG_NEW) {
					updateFlagsIfNeeded(iFolder, mr, flags);					
					skipped = 1;
					continue;					
				}				
				//Skipping based on max size
				if (acnt.inMaxSize != -1 && msg.msgSize > acnt.inMaxSize) {
					if (mr == null) {
						Context.mainFrame.addRowToTableIfNeeded(Context.storage
								.insertMessageHeader(msg, folderId,
										uidValidity, flags), folderId);
						result.incrementSkipped();
					} else {
						updateFlagsIfNeeded(iFolder, mr, flags);						
					}
					skipped = 1;
					continue;
				}
				//Rule based skipping
				if (Rule.checkRules(skipRules, msg) != null) {
					if (mr == null) {
						Context.mainFrame.addRowToTableIfNeeded(Context.storage
								.insertMessageHeader(msg, folderId,
										uidValidity, flags), folderId);
						result.incrementSkipped();
					} else {
						updateFlagsIfNeeded(iFolder, mr, flags);						
					}
					skipped = 1;
					continue;
				}
			}

			// SAVE PHASE
						
			//old place for block of getting folder by rule
			
			if (Context.avgRxSpeed == -1) {
				Context.avgRxSpeed = Context.avgTxSpeed;
				if (pf != null)
					pf.setAvgRxSpeed(Context.avgRxSpeed);
			}						
			
			if (Context.avgRxSpeed > 0) {
				int seconds = (int) ((long) msg.msgSize / Context.avgRxSpeed);
				if (pf != null) {
					pf.setRemainTime(seconds);
					if (seconds > Context.timerLim)
						pf.startRemainTimer(seconds);
				}
			}
			
			// Delete a header because the message itself will be
			// saved
/*			
			if (mr != null) {
				Context.storage.deleteMessage(mr.id, false);
				//Remove the correspondent row from JTable
				Context.mainFrame.removeRowFromTable(mr);
			}
*/									
			
			Date d1 = new Date();
			try {
				if (mr == null)
					hwmId = Context.storage.saveMessage(msg, folderId,
							curMessage, uidValidity, flags, msg.cp, null);
				else {
					Context.storage.saveMessageSource(mr.id, curMessage);
					if (hwmId == null)
						hwmId = mr.id;
					else if (Integer.parseInt(mr.id) > Integer.parseInt(hwmId))
						hwmId = mr.id;
				}
			} catch (javax.mail.MessagingException me) {
				if(me.getMessage().equalsIgnoreCase("Unable to load BODYSTRUCTURE")) {
					me.printStackTrace();
					skipped = 1;
					continue;
				} else {
					throw me;
				}
			}			
			//hwmId = Context.storage.saveMessage(msg, folderId, message, acnt.incomingServer, cpName, null);
			Date d2 = new Date();
			result.incrementReceived();
			
			// Add row to JTable
			if(mr == null)
				Context.mainFrame.addRowToTableIfNeeded(hwmId, folderId);
			else {					
				mr = Context.storage.getMessageRow(mr.id);				
				Context.mainFrame.updateRowInTable(mr);
			}
			
			// if(acnt.inLeaveDays == 0)
			if (needDeleteByTime(msg.receivedDate, acnt)) {
				mailBean.deleteMessage(curMessage);
				// mailBean.deleteMessage(message);
				result.incrementDeleted();
			} else {
				if (hwmMsgUid == null)
					hwmMsgUid = msg.UID;
				if (skipped == 1) {
					hwmMsgUid = msg.UID;
					skipped = 2;
				}
			}
			
			if (pf != null)
				pf.stopRemainTimer();
			long delta = d2.getTime() - d1.getTime();
			if (delta > 0) {
				MessageRow mrJustSaved = Context.storage.getMessageRow(hwmId);
				Context.avgRxSpeed = (int) ((long) mrJustSaved.msgSize * 1000L / delta); // bytes per second
				if (pf != null)
					pf.setAvgRxSpeed(Context.avgRxSpeed);
			}
			
			int action = ComposeFrame.ACTION_NEW;
			if ((rule = Rule.checkRules(fwdAttRules, msg)) != null) {
				action = ComposeFrame.ACTION_FORWARD_AS_ATT;
			} 
			if ((rule = Rule.checkRules(fwdCtntRules, msg)) != null) {
				action = ComposeFrame.ACTION_FORWARD_PARTLY;
			} 

			if(rule != null) {
				NewMessage newMsg = new NewMessage();
				newMsg.To = rule.param;
				newMsg.Subject = MailUtils.getPrefixedSubject(msg.Subject, "Fwd");
				Account acntFrom = Context.storage.getAccount(rule.acntId);			
				
				try {
					MailContext.createNewMsgInOutbox(newMsg, acntFrom, null,
							null, null, action, curMessage, msg.cp, null);
/*
					MailContext.createNewMsgInOutbox(newMsg, acntFrom, null,
							null, null, action, message, cpName, null); 
 */					
				} catch (Exception exc) {
					exc.printStackTrace();
					AlertDialog.showErrorDialog(exc);
				}					
			}
							
			if ((rule = Rule.checkRules(fwdReportRules, msg)) != null) {
				Vector rulesReportHeaders = (Vector)reportHeaders.get(rule);
				if(rulesReportHeaders == null) {
					rulesReportHeaders = new Vector();
					reportHeaders.put(rule, rulesReportHeaders);
				}
				rulesReportHeaders.add(msg);
			}								
		}
		if (exit) {
			Context.mainFrame.setProgressVal(Context.mainFrame.getProgressMax());
			if (pf != null)
				pf.setValue(Context.mainFrame.getProgressMax());
		} else if(!canceled) {
			pos++;
			Context.mainFrame.setProgressVal(pos);
			if (pf != null)
				pf.setValue(pos);						
		}	
	
/*
        // HWM_UID STAFF
		if (!canceled) {
			if(skipped == 1 || hwmMsgUid == null) {
				if(iFolder == null) {
					Context.storage.setAcntHwmUid(acnt.id, "");
				} else {
					iFolder.hwmMsgUid = "";
					Context.storage.saveImapFolder(iFolder);
				}				
			} else {
				if(iFolder == null) {
					Context.storage.setAcntHwmUid(acnt.id, hwmMsgUid);
				} else {
					iFolder.hwmMsgUid = hwmMsgUid;
					Context.storage.saveImapFolder(iFolder);
				}
			}				
		}
*/		
				
		mailBean.closeFolder(iFolder == null);
		
		if (iFolder == null && acnt.inLeaveDays == 0 && hwmId != null
				&& !canceled && skipped == 0) {
			acnt.hwmMsgId = hwmId;
			Context.storage.updateAccountState(acnt);
			//Context.storage.setAcntHwmId(acnt.id, hwmId);
		}

		if(!exit && !canceled && iFolder != null) {
			Context.storage.deleteLostImapMessages(htuids, iFolder);
			Context.mainFrame.deleteLostImapMessages(htuids, iFolder);
		}

/*		
		Context.mainFrame.setStatusBarText(Strings.getString("statBarLastReceived") +
				" " + (new Date()).toString());
*/				
		
		//Create a report e-mail based on rule
		Vector rulesReportHeaders = null;
		for (Enumeration en = reportHeaders.keys(); en.hasMoreElements();) {
			Rule reportRule = (Rule)en.nextElement();
			rulesReportHeaders = (Vector)reportHeaders.get(reportRule);
			
			StringBuffer sb = new StringBuffer();
			MessageHeader rmh = null;
			sb.append("----- Report (").append(rulesReportHeaders.size()).append(") -----\n\n");
			
			for(Enumeration mhen = rulesReportHeaders.elements(); mhen.hasMoreElements();) {
				rmh = (MessageHeader)mhen.nextElement();
				sb.append("From: ")										
				.append(rmh.From).append("\nTo: ").append(rmh.To)
				.append("\nSent: ").append(rmh.getSentDateToString())
				.append("\nSubject: ").append(rmh.Subject).append("\n\n");					
			}
			
			NewMessage newMsg = new NewMessage();
			newMsg.To = reportRule.param;
			newMsg.Subject = "Report for received messages";
			//Account acntFrom = Context.storage.getAccount(reportRule.acntId);			
			
			try {
				MailContext.createNewMsgInOutbox(newMsg, acnt, sb.toString(),
						"text/plain", null, ComposeFrame.ACTION_NEW, null, null, null);
			} catch (Exception exc) {
				exc.printStackTrace();
				AlertDialog.showErrorDialog(exc);
			}									
		}
		
		return result;
	}
	
	protected static boolean logonSuccessfull(MailBean mailBean) throws Exception {
		try {
			mailBean.connectStore();
			return true;
		} catch	(javax.mail.AuthenticationFailedException afe) {
/*					
			if(afe.toString().toLowerCase().indexOf("login failed") != -1 ||
					afe.toString().toLowerCase().indexOf("invalid password") != -1)
*/						
/*					
			if(afe.toString().toLowerCase().indexOf("eof on socket") == -1)
				AlertDialog.showErrorDialog(Strings.getString("MainFrame.wrongUserPass"), afe);
			else throw afe;
*/
			if (afe.toString().toLowerCase().indexOf("eof on socket") != -1)
				throw afe;					
		}
		return false;
	}	
	
	public static void callReceive(final Account acnt, final IFolder thisIFolder, final boolean manual) {
		if(Context.workOffline) {
			AlertDialog.showWarningDialog(Strings.getString("MailContext.cannotReceiveOffline"));			
			return;
		}
		
		if (manual)
			Context.unsetTrayNewMsg();

		if (Context.mainFrame.isReceiving()) {
			if (thisIFolder == null)
				AlertDialog.showWarningDialog(Strings
						.getString("MainFrame.simultReceiving"));
			return;
		}
		
		final SwingWorker worker = new SwingWorker() {

			public Object construct() {
				ReceivingResult result = receiveForAccount(acnt, thisIFolder);
				if (!manual && result != null && result.getNew() > 0)
					Context.setTrayNewMsg(result);
				return null;
			}

		};
		worker.start();
	}	

	public static void callReceiveAll() {
		if(Context.workOffline) {
			AlertDialog.showWarningDialog(Strings.getString("MailContext.cannotReceiveOffline"));
			return;			
		}		
		
		Context.unsetTrayNewMsg();
		if (Context.mainFrame.isReceiving()) {
			AlertDialog.showWarningDialog(Strings
					.getString("MainFrame.simultReceiving"));
			return;
		}

		final SwingWorker worker = new SwingWorker() {

			public Object construct() {
				Vector acnts = null;
				try {
					acnts = Context.storage.getAccountList(null, null);
				} catch (Exception exc) {
					exc.printStackTrace();
					AlertDialog.showErrorDialog(exc);
					return null;
				}
				if (acnts != null) {
					ReceivingResult result = new ReceivingResult();
					for (Enumeration en = acnts.elements(); en
							.hasMoreElements();) {
						Vector row = (Vector) en.nextElement();
						String id = (String) row.elementAt(0);
						Account acnt = null;
						try {
							acnt = Context.storage.getAccount(id);
							if (acnt.status == Account.STAT_NO_RECEIVE)
								continue;
						} catch (Exception exc) {
							exc.printStackTrace();
							AlertDialog.showErrorDialog(exc);
							return null;
						}
						result.append(receiveForAccount(acnt, null));
					}
					if (result.getNew() > 0)
						Context.setTrayNewMsg(result);
				}
				return null;
			}

		};
		worker.start();
	}	
	
	public static void sendFromOutbox() {
		sendFromOutbox(false);
	}
	
	public static void sendFromOutbox(final boolean sendSilent) {
		Context.mainFrame.resetCancelDelivery();
		final ProgressFrame pf = new ProgressFrame();		
		pf.setVisible(true);
		final SwingWorker worker = new SwingWorker() {
			public Object construct() {
				try {
					Vector messages = Context.storage.getMessageList(
							Context.storage.getOutboxFolder(), null, null, -1);
					pf.setMaximum(messages.size());
					pf.setValue(0);
					int count = 1;
					
					AcntSelDialog.resetAcntPrefs();
					
					messageIteration: for (Enumeration en = messages.elements(); en
							.hasMoreElements(); count++) {
						pf.setValue(count - 1);
						
						if (pf.isCanceled() || Context.mainFrame.isDeliveryCanceled()) {
							break messageIteration;
						}
						MessageRow mr = (MessageRow) en.nextElement();

						// Properties props = new Properties();
						//olivamail.gui.Account acnt = Context.storage.getAccountForEmail(mr.addrFrom);
						olivamail.gui.Account acnt = null;
						if(!ComUtils.nvl(mr.acntId).equals(""))
							acnt = Context.storage.getAccount(mr.acntId);
						if(acnt != null) {							
							InternetAddress ia = new InternetAddress(mr.addrFrom);
							if(!ia.getAddress().equals(acnt.email)) {
								if(sendSilent) {
									//pf.setValue(count);
									continue messageIteration;
								}
								
								acnt = AcntSelDialog.getPrefAcnt(mr.addrFrom);
								
								if (acnt == null && AlertDialog.showWarningDialog(
										Strings.getString("MailContext.addrMismatchAcnt")
												.replaceFirst("\\$acntName", acnt.name)
												.replaceFirst("\\$addrTo", MailUtils.getHtmlAddressList(mr.addrTo, null))
												.replaceFirst("\\$subject", mr.subject)
												.replaceFirst("\\$addrFrom", MailUtils.getHtmlAddressList(mr.addrFrom, null)),
												AlertDialog.YES_NO_OPTION) == AlertDialog.NO_OPTION) {
									
									acnt = AcntSelDialog.showAccountDialog(mr.addrFrom);
									
									if (acnt == null || acnt == AcntSelDialog.DUMMY_ACCOUNT) {
										//pf.setValue(count);
										continue messageIteration;
									}
								}
							}							
						} else if (!sendSilent) {
							acnt = AcntSelDialog.getPrefAcnt(mr.addrFrom);
							
							if (acnt == null) {
								AlertDialog.showWarningDialog(Strings
										.getString("MailContext.needAcnt")
										.replaceFirst(
												"\\$addrTo",
												MailUtils.getHtmlAddressList(
														mr.addrTo, null))
										.replaceFirst("\\$subject", mr.subject)
										.replaceFirst(
												"\\$addrFrom",
												MailUtils.getHtmlAddressList(
														mr.addrFrom, null)));
								acnt = AcntSelDialog
										.showAccountDialog(mr.addrFrom);
							}
						}

						if (acnt == null || acnt == AcntSelDialog.DUMMY_ACCOUNT) {
							if(!sendSilent && acnt != AcntSelDialog.DUMMY_ACCOUNT)
								AlertDialog.showInfoDialog(Strings.getString("Context.noAccount"));
							//pf.setValue(count);
							continue messageIteration;
						}

						pf.setAccount(acnt.name);

						// SSL
						Properties props = Context.setSSL(acnt, false);

						props.put("mail.smtp.host", acnt.outgoingServer);
						if (acnt.smtpAuth.equals("1"))
							props.put("mail.smtp.auth", "true");
						else
							props.put("mail.smtp.auth", "false");						

						Context.restorePwdFromCache(acnt);

						if (acnt.smtpAuth.equals("1")) {
							if (ComUtils.nvl(acnt.outUser).equals("")) {
								acnt.outUser = acnt.inUser;
								if (ComUtils.nvl(acnt.inPass).equals("")) {
									if (AccountCredentialsDialog
											.showCredentialsDialog(
													acnt,
													AccountCredentialsDialog.TYPE_OUT) == AccountCredentialsDialog.RESPONSE_CANCEL) {
										//pf.setValue(count);
										continue messageIteration;
									}
									Context.storePwdInCache(acnt);
								} else
									acnt.outPass = acnt.inPass;
							} else {
								if (ComUtils.nvl(acnt.outPass).equals("")) {
									if (AccountCredentialsDialog
											.showCredentialsDialog(
													acnt,
													AccountCredentialsDialog.TYPE_OUT) == AccountCredentialsDialog.RESPONSE_CANCEL) {
										//pf.setValue(count);
										continue messageIteration;
									}
									Context.storePwdInCache(acnt);
								}
							}
						}

						Context.mainFrame.incSending(acnt.name);						
						boolean ok = false;
						while (!ok) {
							final String smtpUser = acnt.outUser;
							final String smtpPass = acnt.outPass;

							final Session smtpSession = Session.getInstance(
									props, new javax.mail.Authenticator() {
										protected javax.mail.PasswordAuthentication getPasswordAuthentication() {
											return new javax.mail.PasswordAuthentication(
													smtpUser, smtpPass);
										}
									});
							olivamail.core.mail.OlivaMimeMessage message = new olivamail.core.mail.OlivaMimeMessage(
									smtpSession, Context.storage
											.loadMessageInputStream(mr.id));
							// message.setHeader("X-Mailer", xMailer);
							try {
								if (Context.avgTxSpeed == -1) {
									Context.avgTxSpeed = Context.avgRxSpeed;
									pf.setAvgTxSpeed(Context.avgTxSpeed);
								}
								if (Context.avgTxSpeed > 0 && mr.msgSize > 0) {
									int seconds = (int) ((long) mr.msgSize / Context.avgTxSpeed);
									pf.setRemainTime(seconds);
									if (seconds > Context.timerLim)
										pf.startRemainTimer(seconds);
								} else
									pf.setRemainTime(-1);
								
								pf.setOperationDescription(Strings.getString("ProgressFrame.descSending"));
								
								Date d1 = new Date();
								MailBean.sendMessage(message);
								ok = true;
								Date d2 = new Date();
								Context.storage.moveMessage(mr.id,
										Context.storage.getSentFolder());
								pf.stopRemainTimer();
								
								//Remove the correspondent row from JTable
								Context.mainFrame.removeRowFromTable(mr);
								Context.mainFrame.addRowToTableIfNeeded(mr.id,
										Context.storage.getSentFolder());
														
								long delta = d2.getTime() - d1.getTime();
								if (delta > 0) {
									Context.avgTxSpeed = (int) ((long) mr.msgSize * 1000L / delta); // bytes
									// per
									// second
									pf.setAvgTxSpeed(Context.avgTxSpeed);
								}
							} catch (AuthenticationFailedException afe) {
								pf.stopRemainTimer();
								if (AccountCredentialsDialog
										.showCredentialsDialog(
												acnt,
												AccountCredentialsDialog.TYPE_OUT) == AccountCredentialsDialog.RESPONSE_CANCEL) {
									//pf.setValue(count);
									continue messageIteration;
								}
								Context.storePwdInCache(acnt);
							} catch (Exception exc) {
								ok = true;
								exc.printStackTrace();
								if(!sendSilent)								
									AlertDialog.showErrorDialog(exc);
								pf.stopRemainTimer();
							}
							//pf.setValue(count);
						} // Authentication retries
					} // Iteration through messages
				} catch (Exception exc) {
					exc.printStackTrace();
					if(!sendSilent)
						AlertDialog.showErrorDialog(exc);
				} finally {
					Context.mainFrame.decrSending();					
				}
				pf.storeWindowPosition();
				pf.dispose();
				if(Context.mainFrame != null) {
					if(!sendSilent)
						Context.mainFrame.updateFldNewMsgInfo();
					Context.mainFrame.setCancelDelivery();					
				}
				return null;
			}
		};
		worker.start();
	}
	
	private static boolean needDeleteByTime(Date receivedDate, Account acnt) {
		if(acnt.isImap())
			return false;
		if (acnt.inLeaveDays == -1)
			return false;
		if (acnt.inLeaveDays == 0)
			return true;
		if (receivedDate == null)
			return true;
		Date now = new Date();
		Calendar cal = GregorianCalendar.getInstance();
		cal.setTime(receivedDate);
		cal.add(Calendar.DATE, acnt.inLeaveDays);
		return now.after(cal.getTime());		
	}

/*
	private static boolean needDeleteByTime(MessageHeader msg, Account acnt) {
		if (acnt.inLeaveDays == -1)
			return false;
		if (acnt.inLeaveDays == 0)
			return true;
		Date receivedDate = msg.receivedDate;
		if (receivedDate == null)
			return true;
		Date now = new Date();
		Calendar cal = GregorianCalendar.getInstance();
		cal.setTime(receivedDate);
		cal.add(Calendar.DATE, acnt.inLeaveDays);
		return now.after(cal.getTime());		
	} 
 */	
	
	public static void createNewMsgInOutbox(NewMessage newMsg,
			Account acntFrom, String composeText, String contentType,
			Vector cacheFiles, int action, javax.mail.Message fwdMessage,
			String fwdMsgCp, SimpleMessage replyMessage) throws Exception {

		Vector vBodyParts = null;

		int attsSize = 0;
		if (cacheFiles != null)
			attsSize = cacheFiles.size();
		if (action == ComposeFrame.ACTION_FORWARD_PARTLY) {
			vBodyParts = new Vector(attsSize + 1);
			MimeBodyPart mbp = new MimeBodyPart();
			if (fwdMessage.isMimeType("multipart/*"))
				mbp.setContent((Multipart) fwdMessage.getContent());
			else
				mbp.setContent(Context.mailBean.getPartText(fwdMessage,
						fwdMsgCp), fwdMessage.getContentType());
			// mbp.setText(mailBean.getPartText(message, cp), sendCodePage);
			vBodyParts.add(mbp);
		}

		if (action == ComposeFrame.ACTION_FORWARD_AS_ATT) {
			vBodyParts = new Vector(attsSize + 1);
			MimeBodyPart mbp = new MimeBodyPart();
			mbp.setDataHandler(new DataHandler(
					new olivamail.core.mail.ByteArrayDataSource(MailUtils
							.getMessageInputStream(fwdMessage),
							"message/rfc822")));
			vBodyParts.add(mbp);
		}

		if (action == ComposeFrame.ACTION_FORWARD_INPLACE) {
			vBodyParts = new Vector(attsSize + 1);
			MimeBodyPart mbp = new MimeBodyPart();
			if (fwdMessage.isMimeType("multipart/*"))
				mbp.setContent((Multipart) fwdMessage.getContent());
			else
				mbp.setContent(Context.mailBean.getPartText(fwdMessage, fwdMsgCp),
						fwdMessage.getContentType());
			// mbp.setText(Context.mailBean.getPartText(fwdMessage, fwdMsgCp), Context.sendCodepage);
			vBodyParts.add(mbp);
		}		
		
		if (attsSize > 0) {
			if (vBodyParts == null)
				vBodyParts = new Vector(attsSize);			
			for (java.util.Enumeration en = cacheFiles.elements(); en
					.hasMoreElements();) {
				// String fname = fileField.getText();
				// File file = new File(fname);
				CacheFile cf = (CacheFile) en.nextElement();
				File file = cf.getFile();

				if (cf.getMimeType() != null
						&& cf.getMimeType().equalsIgnoreCase("message/rfc822")) {
					MimeBodyPart mbp = new MimeBodyPart();
					/*
					 * mbp .setDataHandler(new DataHandler( new
					 * olivamail.core.mail.ByteArrayDataSource( MailUtils
					 * .getMessageInputStream(fwdMessage), "message/rfc822")));
					 */
					FileInputStream fis = new FileInputStream(file);
					BufferedInputStream bis = new BufferedInputStream(fis,
							16 * 1024);
					mbp.setDataHandler(new DataHandler(
							new olivamail.core.mail.ByteArrayDataSource(bis,
									"message/rfc822")));
					vBodyParts.add(mbp);
				} else {
					String fname = cf.getName();
					FileInputStream fis = new FileInputStream(file);
					BufferedInputStream bis = new BufferedInputStream(fis,
							16 * 1024);
/*
					for (StringTokenizer fnst = new StringTokenizer(fname, "\\"); fnst
							.hasMoreTokens();) 
 */					
					for (StringTokenizer fnst = new StringTokenizer(fname, File.separator); fnst
							.hasMoreTokens();)
						fname = fnst.nextToken();
					String mimeType = cf.getMimeType();
					if (mimeType == null) {
						mimeType = "binary/octet-stream";
						if (fname.indexOf(".") != -1) {
							String ext = null;
							for (StringTokenizer fnst = new StringTokenizer(
									fname, "."); fnst.hasMoreTokens();)
								ext = fnst.nextToken();
							if (ext != null)
								mimeType = olivamail.core.mail.MailUtils
										.getMimeType(ext);
						}
					}
					MimeBodyPart mbp = new MimeBodyPart();					
					
					mbp.setDataHandler(new DataHandler(
							new olivamail.core.mail.ByteArrayDataSource(bis,
									mimeType)));
					byte fnBytes[] = fname.getBytes("us-ascii");
					String asciiFName = new String(fnBytes, "us-ascii");
					if (fname.compareTo(asciiFName) == 0)
						mbp.setFileName(fname);
					else {
						if (newMsg.asUnicode)
							mbp.setFileName(olivamail.core.mail.Coder
									.encodeBASE64(fname, "UTF-8"));
						else
							mbp.setFileName(olivamail.core.mail.Coder
									.encodeBASE64(fname, Context.sendCodepage));
					}
					vBodyParts.add(mbp);
				}
			}
		}
		/*
		 * Context.mailBean.sendMessage(toField.getText(), ccField.getText(),
		 * bccField.getText(), subjectField.getText(), jEditorPane1.getText(),
		 * vBodyParts, refId, null, null, cp, false, false, false);
		 */

		String msg = null;
		boolean hasAtt = false;

		if (vBodyParts != null)
			hasAtt = true;

		if (composeText != null) {
			if (contentType.equals("text/html")) {
				// Add message text in html
				if (vBodyParts == null)
					vBodyParts = new Vector(1);
				MimeMultipart mm = new MimeMultipart();
				mm.setSubType("alternative");
				MimeBodyPart mbp = new MimeBodyPart();
				mbp.setText(ComUtils.getTextFromHtml(composeText),
						Context.sendCodepage, "plain");
				mm.addBodyPart(mbp);
				mbp = new MimeBodyPart();
				mbp.setText(composeText, Context.sendCodepage, "html");
				mm.addBodyPart(mbp);
				mbp = new MimeBodyPart();
				mbp.setContent(mm);
				vBodyParts.add(mbp);
			} else if (contentType.equals("text/plain")) {
				msg = composeText;
			}
		}

		/*
		 * String fromAddr = (String) jcbAddrFrom.getSelectedItem(); Account
		 * acnt = Context.storage.getAccountForEmail(fromAddr);
		 */
		Account acnt = (Account) acntFrom;
		String fromAddr = acnt.email;
		if (!acnt.person.equals(""))
			fromAddr = acnt.person + " <" + acnt.email + ">";

		/*
		 * NewMessage newMsg = NewMessage.create(fromAddr, nm.To, nm.Cc, nm.Bcc,
		 * nm.Subject, msg, vBodyParts, null, nm.asUnicode, nm.requestNotify);
		 */

		newMsg.From = fromAddr;
		newMsg.text = msg;
		newMsg.bodyParts = vBodyParts;

		if (replyMessage != null) {
			newMsg.refId = replyMessage.MessageID;
			newMsg.References = replyMessage.References;
			newMsg.InReplyTo = replyMessage.InReplyTo;
		}
		if (!ComUtils.nvl(acnt.replyTo).equals(""))
			newMsg.ReplyTo = acnt.replyTo;
		if (!ComUtils.nvl(acnt.organization).equals(""))
			newMsg.addHeader("Organization", acnt.organization);
		// newMsg.importance = nm.importance;

		javax.mail.internet.MimeMessage mimeMsg = Context.mailBean
				.createMessage(newMsg);
		/*
		 * mimeMsg.setHeader("X-Mailer", Context.xMailer); if(requestNotify)
		 * mimeMsg.setHeader("Disposition-Notification-To", fromAddr);
		 */
		MessageHeader hdr = Context.mailBean.getMessageHeader(mimeMsg, null);
		hdr.UID = Context.mailBean.getOlivaMsgUID(mimeMsg);
		hdr.hasAtt = hasAtt;
		// hdr.msgSize = Context.getMessageSize(mimeMsg);
		Context.mainFrame.addRowToTableIfNeeded(Context.storage.saveMessage(
				hdr, Context.getOutboxFolder(), mimeMsg, "".hashCode(), null,
				null, acnt.id), Context.getOutboxFolder());
		// Context.mailBean.sendMessage(mimeMsg);
	}

	protected static void updateFlagsIfNeeded(IFolder iFolder, MessageRow mr,
			Flags flags) throws Exception {
		if (flags == null || iFolder == null || mr.flagsEqual(flags))
			return;
		mr.setFlags(flags);
		Context.storage.saveMessageState(mr);
		Context.mainFrame.updateMessageIfNeeded(mr);
	}
		
}
