/*
 * @(#)MessageContext.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.OutputStream;
import java.util.Vector;
import olivamail.core.common.IOUtils;
import olivamail.core.common.StreamHolder;
import olivamail.core.mail.Message;

public class MessageContext {

	public static ReceivingResult saveAllAtt(Message olivaMessage, Vector atts, File dir) throws Exception {
		ReceivingResult result = new ReceivingResult(); 
		for (int i = 0; i < atts.size(); i++) {
			olivamail.core.mail.MessagePart mp = (olivamail.core.mail.MessagePart) atts
					.get(i);
			File file = null;
	
			if (mp.type == olivamail.core.mail.MessagePart.T_BINARY) {
				MessagePartStreamHolder sh = new MessagePartStreamHolder(
						olivaMessage, mp.partNo);
				String filename = MessageContext.getAttFileName(dir.getPath(),
						mp.fileName, sh);
				if (filename == null)
					result.incrementSkipped();
				else {
					file = new File(filename);
					IOUtils.writeFile(sh.getInputStream(), file);
					result.incrementReceived();
				}
			}
			if (mp.type == olivamail.core.mail.MessagePart.T_RFC822
					|| mp.type == olivamail.core.mail.MessagePart.T_TEXT) {
				String filename = mp.fileName;					
				
				if (mp.type == olivamail.core.mail.MessagePart.T_RFC822)
					filename += ".eml";
				else
					filename = MessageContext.escapeBraces(filename);
	
				filename = MessageContext.getAttFileName(dir.getPath(), filename) ;
				if (filename == null)
					result.incrementSkipped();
				else {
					file = new File(filename);
					if (mp.type == olivamail.core.mail.MessagePart.T_RFC822) {
						OutputStream os = Context.getFileOutputStream(file);
						olivaMessage.getRfc822Part(mp.partNo).apiMessage
								.writeTo(os);
						Context.closeOutputStream(os);
					} else {
						ByteArrayInputStream bais = new ByteArrayInputStream(
								olivaMessage.getTextPart(mp.partNo).getBytes());
						IOUtils.writeFile(bais, file);
					}
					result.incrementReceived();
				}
			}
		}
		return result;
	}

	public static Vector extractAttachmets(olivamail.core.mail.Message msg) {
		Vector atts = new Vector(5); 
		
		olivamail.core.mail.MessagePart mp = null;
		if (msg.messageParts != null) {
			int attNo = 0;
			for (java.util.Enumeration en = msg.messageParts.elements(); en
					.hasMoreElements();) {
				olivamail.core.mail.MessagePart att = (olivamail.core.mail.MessagePart) en
						.nextElement();
				if (att.type == olivamail.core.mail.MessagePart.T_RFC822) {
					mp = new olivamail.core.mail.MessagePart(att.partNo,
							att.alternative, att.type, att.contentType,
							att.fileName);
					mp.fileName = Strings.getString("MessageFrame.rfc822message");
					atts.add(mp);
				} else if (att.fileName != null) {
					atts.add(att);
				} else if (att.cid != null || att.alternative
						|| att.type == olivamail.core.mail.MessagePart.T_BINARY) {
					if (att.type == olivamail.core.mail.MessagePart.T_TEXT) {
						String attFileName = "(ATT"
								+ String.valueOf(attNo++)
								+ "."
								+ olivamail.core.mail.MailUtils
										.getExtension(att.contentType)
								+ ")";
						mp = new olivamail.core.mail.MessagePart(att.partNo,
								att.alternative, att.type, att.contentType,
								att.fileName);
						mp.fileName = attFileName;
						atts.add(mp);
					} else if (att.type == olivamail.core.mail.MessagePart.T_BINARY) {
						String attFileName = "ATT"
								+ String.valueOf(attNo++)
								+ "."
								+ olivamail.core.mail.MailUtils
										.getExtension(att.contentType);
						mp = new olivamail.core.mail.MessagePart(att.partNo,
								att.alternative, att.type, att.contentType,
								att.fileName);
						mp.fileName = attFileName;
						atts.add(mp);
					}
				}
			}
			// String attList = sb.toString();
		}		
		return atts;
	}
	
	public static String getAttFileName(String path, String filename) throws Exception {
		return getAttFileName(path, filename, null);
	}	

	public static String getAttFileName(String path, String filename,
			StreamHolder sh) throws Exception {
		File file = new File(path + File.separator + filename);
		int count = 0;

		String ext = "";
		String fname = filename;
		int dotIdx = filename.lastIndexOf('.');
		if (dotIdx != -1) {
			ext = filename.substring(dotIdx);
			fname = filename.substring(0, dotIdx);
		}

		while (file.exists()) {
			if (sh != null && IOUtils.areStreamAndFileIdentical(sh, file)) {
				return null;
			}
			count++;
			if (count == 1000) {
				/*
				 * file = null; break;
				 */
				throw new Exception(Context.myName
						+ ": similar filename limit is reached");
			}
			String newName = fname + "-" + String.valueOf(count) + ext;
			file = new File(path + File.separator + newName);
		}
		return path + File.separator + file.getName();
	}
		
	public static String escapeBraces(String filename) {
		if(filename == null)
			filename = "";
		if(filename.charAt(0) == '(' && filename.charAt(filename.length() - 1) == ')')
			filename = filename.substring(1, filename.length() - 1);
		return filename;
	}

	public static MessageRow prepareMessageForOpen(MessageRow mr, RequestHandler rh) throws Exception {
		boolean imapMsgLoaded = false;
		IFolder iFolder = mr.getImapFolder();
		// Load IMAP Message if it is just a header
		Account acnt = null;
		if(iFolder != null)
			acnt = Context.storage.getAccount(iFolder.acntId);
		if (!Context.workOffline && acnt != null && acnt.isOnline() && mr.isHeader()) {
			javax.mail.Message[] messages = null;
			try {
				messages = IMAPContext.getMessages(iFolder, false,
						new String[] { mr.msgUid });
			} catch (Exception exc) {
				if (!Context.gonnaWorkOffline(exc, acnt))
					throw exc;
			}
			if (messages != null) {
				Context.storage.saveMessageSource(mr.id, messages[0]);
				mr = Context.storage.getMessageRow(mr.id);
				Context.mainFrame.updateMessageIfNeeded(mr);
				//mtm.setMessageRowAt(mr, row);
				imapMsgLoaded = true;
			}
		}

		if (imapMsgLoaded) {
			// NONE
		} else if (mr.folderId.equals(Context.getTrashFolder())
				&& mr.isHeader()) {
			AlertDialog
					.showInfoDialog(
							Strings
									.getString(
											"MessageFrame.justHeaderTrash").replaceAll("\\$1", "http://mainframe.gui.olivamail:1").replaceAll("\\$2", "http://mainframe.gui.olivamail:2").replaceAll("\\$3", "http://mainframe.gui.olivamail:3"), rh);
			return null;
		} else if (mr.hdr == MessageRow.HDR_HDR) {
			AlertDialog
					.showInfoDialog(
							Strings
									.getString("MessageFrame.justHeader").replaceAll("\\$1", "http://mainframe.gui.olivamail:1").replaceAll("\\$2", "http://mainframe.gui.olivamail:2").replaceAll("\\$3", "http://mainframe.gui.olivamail:3"), rh);
			return null;
		} else if (mr.hdr == MessageRow.HDR_FORCED) {
			AlertDialog
					.showInfoDialog(
							Strings
									.getString(
											"MessageFrame.justHeaderForced").replaceAll("\\$1", "http://mainframe.gui.olivamail:1").replaceAll("\\$2", "http://mainframe.gui.olivamail:2").replaceAll("\\$3", "http://mainframe.gui.olivamail:3"), rh);
			return null;
		}
		return mr;
	}
	
}
