/*
 * @(#)MessageRow.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.mail.Flags;
import javax.mail.Flags.Flag;

import olivamail.core.mail.MessageHeader;

public class MessageRow {

	String id;

	String addrFrom;

	String addrTo;

	String addrCc;

	String addrBcc;

	String subject;

	String sentDate;

	String receivedDate;

	int msgSize;

	String messageId;

	String msgUid;

	String hasAtt;

	String folderId;

	String codepage;

	String importance;

	String srcPlace = "0";

	String acntId;
	
	char hdr = HDR_COMPLETE;
	
	boolean sfAnsw = false;
	
	boolean sfSeen = false;
	
	boolean sfFlag = false;
	
	boolean sfDel = false;
	
	boolean sfDraft = false;
	
	char dmStat = DM_SYNC;
	
//	int flags = 0;
	
	long uidValidity = 0L;	

/*			
	public static final int FLAG_ANSWERED = 1;
	
	public static final int FLAG_DELETED = 2;
	
	public static final int FLAG_DRAFT = 4;
	
	public static final int FLAG_FLAGGED = 8;
	
	public static final int FLAG_RECENT = 16;
	
	public static final int FLAG_SEEN = 32;
	
	public static final int FLAG_USER = 64;
*/	
	public static final char HDR_HDR = 'H';
	
	public static final char HDR_FORCED = 'F';
	
	public static final char HDR_MULTI = 'M';
	
	public static final char HDR_COMPLETE = 'C';
	
	public static final char HDR_DELETED = 'D';
	
	public static final char DM_SYNC = 'S';
	
	public static final char DM_FLAG = 'F';
	
	public static final char DM_NEW = 'N';
/*	
	public static final String SEEN_UNREAD = "0";

	public static final String SEEN_READ = "1";

	public static final String SEEN_HEADER = "2";

	public static final String SEEN_HEADER_FORCED = "3";

	public static final String SEEN_SPLITTED = "4";
*/
	public static final String formatDMY = "dd-MM-yyyy HH:mm:ss";

	public static final String formatYMD = "yyyy-MM-dd HH:mm:ss";	
	
	public static final String formatHHmm = "HH:mm";
	
	public static final String formatDayMonth = "dd MMMMMM";
	
	public static final String formatDayMonthTime = "HH:mm dd MMMMMM";
	
	public static final String formatDMYnoSec = "dd-MM-yyyy HH:mm";
	
	public static final int MESSAGES_FLAGS_TOGGLE = 0;
	
	public static final int MESSAGES_FLAGS_SET = 1;	
	
	public static final int MESSAGES_FLAGS_UNSET = 2;
	
	public static final int MESSAGES_COPY = 3;
	
	public static final int MESSAGES_MOVE = 4;
	
	public static final int MESSAGES_APPEND = 5;
			
	public MessageRow() {
	}

	public Date getReceivedDate() {
		SimpleDateFormat sdf = new SimpleDateFormat(formatYMD);
		Date date = null;
		try {
			date = sdf.parse(receivedDate);
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return date;
	}

	public Date getSentDate() {
		SimpleDateFormat sdf = new SimpleDateFormat(formatYMD);
		Date date = null;
		try {
			date = sdf.parse(sentDate);
		} catch (Exception e) {
			e.printStackTrace();
		}		
		return date;
	}	
	
	public String getSentDateDMY() {
		return convertDMYtoYMD(sentDate, true);
	}

	public String getSentDateRelative() {
		return convertDMYtoRelative(sentDate);
	}
		
	public String getReceivedDateDMY() {
		return convertDMYtoYMD(receivedDate, true);
	}	

	public String getReceivedDateRelative() {
		return convertDMYtoRelative(receivedDate);
	}	
	
	public static String convertDMYtoRelative(String strDate) {
		SimpleDateFormat sdf = new SimpleDateFormat(formatYMD);
		Date date = null;
		try {
			date = sdf.parse(strDate);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return getRelativeDate(date);
	}

	public static String getRelativeDate(Date date) {
		StringBuffer sb = new StringBuffer();
		if (date != null) {
			String format = formatDMYnoSec;
			Date now = new Date();
			if (!now.before(date)) {
				long diffInHours = (now.getTime() - date.getTime()) / 3600000;
				long diffInDays = diffInHours / 24;
				long diffInYears = diffInDays / 365;
				if (diffInYears == 0) {
					if (diffInDays == 0) {
						if (diffInHours < 12)
							format = formatHHmm;
						else
							format = formatDayMonthTime;
					} else
						format = formatDayMonth;
				}
			}
			SimpleDateFormat sdf = new SimpleDateFormat(format);
			sdf.format(date, sb, new FieldPosition(0));
		}
		return sb.toString();
	}	
	
	public static String convertDMYtoYMD(String strDate, boolean reverse) {
		SimpleDateFormat sdf = new SimpleDateFormat(reverse?formatYMD:formatDMY);
		Date date = null;
		try {
			date = sdf.parse(strDate);
		} catch (Exception e) {
			e.printStackTrace();
		}
		StringBuffer sb = new StringBuffer();
		if (date != null) {
			sdf = new SimpleDateFormat(reverse?formatDMY:formatYMD);
			sdf.format(date, sb, new FieldPosition(0));
		}
		return sb.toString();
	}
	
/*	
	public static int compareSeens(String a, String b, boolean ascending) {
		return (ascending)?a.compareTo(b):b.compareTo(a);
	}
		
	public void setFlag(int flag, boolean flagOn) {
		int check = flags & flag;
		if((check == 0 && flagOn) || (check != 0 && !flagOn))
			flags = flags | flag;
	}
	
	public boolean isFlagSet(int flag) {
		return (flags & flag) != 0;
	}
	
	public boolean flagsEqual(Flags flags) {
		boolean result = false;
		if(isFlagSet(FLAG_ANSWERED) == flags.contains(Flag.ANSWERED))
			if(isFlagSet(FLAG_DELETED) == flags.contains(Flag.DELETED))
				if(isFlagSet(FLAG_DRAFT) == flags.contains(Flag.DRAFT))
					if(isFlagSet(FLAG_FLAGGED) == flags.contains(Flag.FLAGGED))
						if(isFlagSet(FLAG_SEEN) == flags.contains(Flag.SEEN))
							result = true;
		return result;
	}	

	public void setFlags(Flags flags) {
		setFlag(FLAG_ANSWERED, flags.contains(Flag.ANSWERED));
		setFlag(FLAG_DELETED, flags.contains(Flag.DELETED));
		setFlag(FLAG_DRAFT, flags.contains(Flag.DRAFT));
		setFlag(FLAG_FLAGGED, flags.contains(Flag.FLAGGED));
		setFlag(FLAG_SEEN, flags.contains(Flag.SEEN));
	}	
*/
	public boolean flagsEqual(Flags flags) {
		if(flags == null)
			return true;
		boolean result = false;
		if(sfAnsw == flags.contains(Flag.ANSWERED))
			if(sfDel == flags.contains(Flag.DELETED))
				if(sfDraft == flags.contains(Flag.DRAFT))
					if(sfFlag == flags.contains(Flag.FLAGGED))
						if(sfSeen == flags.contains(Flag.SEEN))
							result = true;
		return result;
	}	
	
	public void setFlags(Flags flags) {
		if(flags == null)
			return;
		sfAnsw = flags.contains(Flag.ANSWERED);
		sfDel = flags.contains(Flag.DELETED);
		sfDraft = flags.contains(Flag.DRAFT);
		sfFlag = flags.contains(Flag.FLAGGED);
		sfSeen = flags.contains(Flag.SEEN);
	}

	public void setFlagsToAPIMessage(javax.mail.Message message)
			throws Exception {
		message.setFlag(Flag.ANSWERED, sfAnsw);
		message.setFlag(Flag.DELETED, sfDel);
		message.setFlag(Flag.DRAFT, sfDraft);
		message.setFlag(Flag.FLAGGED, sfFlag);
		message.setFlag(Flag.SEEN, sfSeen);
	}	
	
	public static MessageRow createRow(MessageHeader hdr) {
		MessageRow mr = new MessageRow();
		mr.addrFrom = hdr.From;
		mr.addrTo = hdr.To;
		mr.addrCc = hdr.Cc;
		mr.addrBcc = hdr.Bcc;
		mr.subject = hdr.Subject;

		Date sentDate = hdr.sentDate;
		if (sentDate == null)
			sentDate = hdr.receivedDate;
		Date receivedDate = hdr.receivedDate;
		if (receivedDate == null)
			receivedDate = hdr.sentDate;

		mr.sentDate = olivamail.core.common.ComUtils.getDateString(sentDate,
				"en", MessageRow.formatYMD);

		mr.receivedDate = olivamail.core.common.ComUtils.getDateString(
				receivedDate, "en", MessageRow.formatYMD);

		mr.msgSize = hdr.msgSize;
		mr.messageId = hdr.MessageID;
		mr.msgUid = hdr.UID;
		if (hdr.hasAtt)
			mr.hasAtt = "1";
		else
			mr.hasAtt = "0";

		mr.importance = (hdr.isHighlyImportant()) ? "1" : "0";
		return mr;
	}
	
	public boolean isHeader() {
		return hdr == HDR_HDR || hdr == HDR_FORCED;  
	}

	public void changeMessageFlag(IFolder iFolder, boolean closeFolder,
			Flags.Flag flag, int operation) throws Exception {
		Account acnt = null;
		if(iFolder != null)
			acnt = Context.storage.getAccount(iFolder.acntId);
		boolean offline = true;
				
		if (!Context.workOffline && acnt != null && acnt.isOnline()) {
			dmStat = DM_SYNC;			
			try {
				IMAPContext.operateMessages(iFolder, closeFolder,
						new String[] { msgUid },
						operation, flag, null, null);
				offline = false;
			} catch (Exception exc) {
				if (!Context.gonnaWorkOffline(exc, acnt))
					throw exc;
			}
		}
		
		applyLocallyFlagChange(flag, operation, offline);
	}	
	
	public boolean willFlagBeSet(Flags.Flag flag, int operation) {
		boolean result = false;
		if (operation == MESSAGES_FLAGS_SET)
			result = true;
		else if (operation == MESSAGES_FLAGS_UNSET)
			result = false;
		else if (operation == MESSAGES_FLAGS_TOGGLE) {
			if (flag == Flags.Flag.ANSWERED)
				result = !sfAnsw;
			else if (flag == Flags.Flag.DELETED)
				result = !sfDel;
			else if (flag == Flags.Flag.DRAFT)
				result = !sfDraft;
			else if (flag == Flags.Flag.FLAGGED)
				result = !sfFlag;
			else if (flag == Flags.Flag.SEEN)
				result = !sfSeen;
		} else {
			if (flag == Flags.Flag.ANSWERED)
				result = sfAnsw;
			else if (flag == Flags.Flag.DELETED)
				result = sfDel;
			else if (flag == Flags.Flag.DRAFT)
				result = sfDraft;
			else if (flag == Flags.Flag.FLAGGED)
				result = sfFlag;
			else if (flag == Flags.Flag.SEEN)
				result = sfSeen;
		}
		return result;
	}
	
	public void applyLocallyFlagChange(Flags.Flag flag, int operation, boolean offline) {
		switch (operation) {
		case MESSAGES_FLAGS_TOGGLE:
			if (flag == Flags.Flag.ANSWERED)
				sfAnsw = !sfAnsw;
			else if (flag == Flags.Flag.DELETED)
				sfDel = !sfDel;
			else if (flag == Flags.Flag.DRAFT)
				sfDraft = !sfDraft;
			else if (flag == Flags.Flag.FLAGGED)
				sfFlag = !sfFlag;
			else if (flag == Flags.Flag.SEEN)
				sfSeen = !sfSeen;
			break;
		case MESSAGES_FLAGS_SET:
			if (flag == Flags.Flag.ANSWERED)
				sfAnsw = true;
			else if (flag == Flags.Flag.DELETED)
				sfDel = true;
			else if (flag == Flags.Flag.DRAFT)
				sfDraft = true;
			else if (flag == Flags.Flag.FLAGGED)
				sfFlag = true;
			else if (flag == Flags.Flag.SEEN)
				sfSeen = true;
			break;
		case MESSAGES_FLAGS_UNSET:
			if (flag == Flags.Flag.ANSWERED)
				sfAnsw = false;
			else if (flag == Flags.Flag.DELETED)
				sfDel = false;
			else if (flag == Flags.Flag.DRAFT)
				sfDraft = false;
			else if (flag == Flags.Flag.FLAGGED)
				sfFlag = false;
			else if (flag == Flags.Flag.SEEN)
				sfSeen = false;
			break;
		}
		
		if (offline && dmStat != MessageRow.DM_NEW)
			dmStat = DM_FLAG;		
	}
	
	public void changeMessageHdr(int operation) {
		switch (operation) {
		case MESSAGES_FLAGS_TOGGLE:
			if (hdr == HDR_HDR)
				hdr = HDR_FORCED;
			else if (hdr == HDR_FORCED)
				hdr = HDR_HDR;
			break;
		case MESSAGES_FLAGS_SET:
			if (isHeader())
				hdr = HDR_FORCED;
			break;
		case MESSAGES_FLAGS_UNSET:
			if (isHeader())
				hdr = HDR_HDR;
			break;
		}
	}
	
	public IFolder getImapFolder() throws Exception {
		if (folderId == null)
			return null;
		Folder folder = Context.storage.getFolder(folderId);
		if(!folder.isImap())
			return null;
		return Context.storage.getImapFolder(folderId);
	}

	public Folder getFolder() throws Exception {
		if (folderId == null)
			return null;
		return Context.storage.getFolder(folderId);
	}
	
}
