package olivamail.gui;

import java.awt.Component;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import olivamail.core.common.ComUtils;
import olivamail.core.mail.MailUtils;
import olivamail.core.mail.MessageHeader;

public class MessageTableModel extends AbstractTableModel {

	static final long serialVersionUID = 2211334L;
	
	public static MsgsHeaderRenderer mhr = null; 
	
	public static IconRenderer ir = null; 	

	public static final int COL_IMPORTANCE = 0;	
	
	public static final int COL_HDR = 1;	

	public static final int COL_HAS_ATT = 2;

	public static final int COL_ID = 3;

	public static final int COL_SENT_DATE = 4;

	public static final int COL_ADDR_FROM = 5;

	public static final int COL_ADDR_TO = 6;

	public static final int COL_SUBJECT = 7;
	
	public static final int COL_RECEIVED_DATE = 8;
	
	public static final int COL_MSG_SIZE = 9;
	
	public static final int COL_SF_SEEN = 10;
	
	public static final int COL_SF_FLAGGED = 11;
	
	public static final String columnCaps[] = { "IMP", "HDR", "ATT", "ID",
			"SENT", "FROM", "TO", "SUBJ", "RECD", "SIZE", "SEEN", "DEL" };

	public static final int colPos[] = new int[columnCaps.length];	
	
	public static final String columnNames[] = { "!",
			Strings.getString("MainFrame.seenField"),
			Strings.getString("MainFrame.attField"),
			Strings.getString("MainFrame.idField"),
			Strings.getString("MainFrame.sentDate"),
			Strings.getString("MainFrame.fromField"),
			Strings.getString("MainFrame.toField"),
			Strings.getString("MainFrame.subjectField"),
			Strings.getString("MainFrame.receivedDate"),
			Strings.getString("MainFrame.msgSize"),
			Strings.getString("MainFrame.sfSeen"),
			Strings.getString("MainFrame.sfFlagged") };

	public static final String sortv[][] = {
			{ "importance desc", "sf_seen desc, hdr desc", "has_att desc",
					"id desc", "sent_date desc", "addr_from desc",
					"addr_to desc", "subject desc", "received_date desc",
					"msg_size desc", "sf_seen desc", "sf_flag desc" },
			{ "importance asc", "sf_seen asc, hdr asc", "has_att asc",
					"id asc", "sent_date asc", "addr_from asc", "addr_to asc",
					"subject asc", "received_date asc", "msg_size asc",
					"sf_seen asc", "sf_flag asc" } };

	public static int shownColumns[] = { COL_IMPORTANCE, COL_HDR, COL_HAS_ATT,
			COL_RECEIVED_DATE, COL_ADDR_FROM, COL_ADDR_TO, COL_SUBJECT,
			COL_MSG_SIZE };	
				
	public static final Hashtable colCapHt = new Hashtable(columnCaps.length);	
		
	static boolean forceSentDate = false;
	
	static {
		for(int i = 0; i < columnCaps.length; i++) {
			colCapHt.put(columnCaps[i], new Integer(i));
		}
		int countDates = 0;
		for (int i = 0; i < colPos.length; i++)
			colPos[i] = -1;		
		for(int i = 0; i < shownColumns.length; i++) {
			colPos[shownColumns[i]] = i;
			if(shownColumns[i] == COL_SENT_DATE || shownColumns[i] == COL_RECEIVED_DATE)
				countDates++;
		}
		if(countDates >= 2)
			showBothDates = true;
	}

	public static void initRenderers() {
		Context.debugStr("mhr = new MsgsHeaderRenderer()");
		mhr = new MsgsHeaderRenderer();
		Context.debugStr("ir = new IconRenderer()");
		ir = new IconRenderer();
		Context.debugStr("After ir = new IconRenderer()");
	}
	
	public static void setShownColumns(String widths) {
		if (widths == null)
			return;

		Vector vShCol = new Vector(columnCaps.length);

		StringTokenizer st = new StringTokenizer(widths, ",");

		while (st.hasMoreTokens()) {
			String avp = st.nextToken();
			int idx = avp.indexOf("=");
			Object objCol = MessageTableModel.colCapHt.get(avp
					.substring(0, idx));
			vShCol.add(objCol);
		}

		if (vShCol.size() > 0) {
			int countDates = 0;			
			shownColumns = new int[vShCol.size()];
			for (int i = 0; i < colPos.length; i++)
				colPos[i] = -1;
			for (int i = 0; i < shownColumns.length; i++) {
				shownColumns[i] = ((Integer) vShCol.get(i)).intValue();
				colPos[shownColumns[i]] = i;
				if (shownColumns[i] == COL_SENT_DATE
						|| shownColumns[i] == COL_RECEIVED_DATE)
					countDates++;
			}
			if (countDates >= 2)
				showBothDates = true;
		}
	}
		
	public static boolean showBothDates = false;
	
	Vector data = null;
	
	MessageTableModel(Vector data, boolean forceSentDate) {
		this.data = data;	
		MessageTableModel.forceSentDate = forceSentDate; 
/*		
		if(!showBothDates && forceSentDate) {
			for(int i = 0; i < shownColumns.length; i++) {
				if(shownColumns[i] == COL_RECEIVED_DATE)
					shownColumns[i] = COL_SENT_DATE;
			}				
		} else {
			for(int i = 0; i < shownColumns.length; i++) {
				if(shownColumns[i] == COL_SENT_DATE)
					shownColumns[i] = COL_RECEIVED_DATE;
			}				
		}
*/				
	}
	
	public int getColumnCount() {
		return shownColumns.length;
	}

	public int getRowCount() {
		return data.size();
	}
	
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
/*		
		MessageRow mr = (MessageRow)data.get(rowIndex);
		switch(columnIndex) {
		case COL_IMPORTANCE:
			mr.importance = (String)aValue;
			break;
		case COL_HDR:
			mr.seen = (String)aValue;
			break;
		case COL_HAS_ATT:
			mr.hasAtt = (String)aValue;
			break;
		case COL_ID:
			mr.id = (String)aValue;
			break;
		case COL_SENT_DATE:
			mr.sentDate = (String)aValue;
			break;
		case COL_ADDR_FROM:
			mr.addrFrom = (String)aValue;
			break;
		case COL_ADDR_TO:
			mr.addrTo = (String)aValue;
			break;
		case COL_SUBJECT:
			mr.subject = (String)aValue;
			break;
		case COL_RECEIVED_DATE:
			mr.receivedDate = (String)aValue;
			break;
		}
		data.set(rowIndex, mr);
		fireTableRowsUpdated(rowIndex, rowIndex);
*/		
	}

	public void setMessageRowAt(MessageRow mr, int rowIndex) {
		data.set(rowIndex, mr);
		fireTableRowsUpdated(rowIndex, rowIndex);
	}
		
	public MessageRow getMessageRowAt(int rowIndex) {
		return (MessageRow)data.get(rowIndex);
	}	
	
    public Class getColumnClass(int columnIndex) {
		switch (shownColumns[columnIndex]) {
		case COL_SF_SEEN:
		case COL_SF_FLAGGED:
			return Boolean.class;
		default:
			return String.class;
		}
	}	

    public static int checkColumn(int column) {
    	if(forceSentDate && !showBothDates && column == COL_RECEIVED_DATE)
    		return COL_SENT_DATE;
    	return column;
    }
    
	public Object getValueAt(int rowIndex, int columnIndex) {
		MessageRow mr = (MessageRow)data.get(rowIndex);		
		int col = checkColumn(shownColumns[columnIndex]);		
		switch (col) {
		case COL_IMPORTANCE:
			return mr.importance;
		case COL_HDR:
			return String.valueOf(mr.hdr);
		case COL_HAS_ATT:
			return mr.hasAtt;
		case COL_ID:
			return mr.id;
		case COL_SENT_DATE:
			return mr.getSentDateRelative();
			//return mr.getSentDateDMY();
		case COL_ADDR_FROM:
			return mr.addrFrom;
		case COL_ADDR_TO:
			return mr.addrTo;
		case COL_SUBJECT:
			return mr.subject;
		case COL_RECEIVED_DATE:
			return mr.getReceivedDateRelative();
			//return mr.getReceivedDateDMY();
		case COL_MSG_SIZE:
			return ComUtils.getSizeString(mr.msgSize);
		case COL_SF_SEEN:
			return Boolean.valueOf(mr.sfSeen);
		case COL_SF_FLAGGED:
			return Boolean.valueOf(mr.sfFlag);			
		}
		return null;
		/*
		String date = null;
		if (isOutFolder())
			date = mr.getSentDateDMY();
		else
			date = mr.getReceivedDateDMY();
		*/						
	}

	public String getColumnName(int column) {
		int col = checkColumn(shownColumns[column]);		
		switch(col) {
		case COL_IMPORTANCE:
			return "!";
		case COL_HDR:
			return Strings.getString("MainFrame.seenField");	
		case COL_HAS_ATT:
			return Strings.getString("MainFrame.attField");
		case COL_ID:
			return Strings.getString("MainFrame.idField");
		case COL_SENT_DATE:
			return Strings.getString("MainFrame.sentDate");	
		case COL_ADDR_FROM:
			return Strings.getString("MainFrame.fromField");
		case COL_ADDR_TO:
			return Strings.getString("MainFrame.toField");
		case COL_SUBJECT:
			return Strings.getString("MainFrame.subjectField");
		case COL_RECEIVED_DATE:
			return Strings.getString("MainFrame.receivedDate");
		case COL_MSG_SIZE:
			return Strings.getString("MainFrame.msgSize");			
		}
		return null;
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}

/*
	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		super.setValueAt(value, rowIndex, columnIndex);
	}
*/		

    public void sortAllRowsBy(int colIndex, boolean ascending) {
        Collections.sort(data, new ColumnSorter(colIndex, ascending));
        fireTableStructureChanged();
    }	
	
    public void addRow(MessageRow mr) {
    	insertRow(getRowCount(), mr);
    }      

    public void addRowSorted(MessageRow mr, int colIndex, boolean ascending) {
		ColumnSorter cs = new ColumnSorter(colIndex, ascending);
		for (int i = 0; i < data.size(); i++) {
			int result = cs.compare(data.get(i), mr);
			if (result >= 0) {
				insertRow(i, mr);
				return;
			}
		}
		if (ascending)
			insertRow(0, mr);
		else
			insertRow(getRowCount(), mr);
	}          
    
    public void insertRow(int row, MessageRow mr) {
    	data.insertElementAt(mr, row);  
        fireTableRowsInserted(row, row);
    }
    
    public void removeRow(int row) {
        data.removeElementAt(row);
        fireTableRowsDeleted(row, row);
    }    

    public static void setColumnModel(JTable jTable1, MainFrame mf) {
		TableColumnModel tcm = jTable1.getColumnModel();
		for(int i = 0; i < tcm.getColumnCount(); i++) {
			Context.debugStr("Before getColumnCount( " + i);
			Context.debugStr("tcm.getColumn(i): " + tcm.getColumn(i).toString());
			setupColumn(tcm.getColumn(i), mf, i);
			Context.debugStr("After getColumnCount( " + i);
		}
	}
    
    private static void setupColumn(final TableColumn tableColumn, MainFrame mf, int column) {
    	Context.debugStr("MessageTableModel.setupColumn(column == " + column + ", name = " + columnCaps[shownColumns[column]]);
    	
		switch(shownColumns[column]) {
		case COL_IMPORTANCE:									
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setCellRenderer(ir);
			tableColumn.setMaxWidth(20);
			break;
		case COL_HDR:
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setCellRenderer(ir);
			tableColumn.setMaxWidth(20);    	
			break;
		case COL_HAS_ATT:			
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setCellRenderer(ir);
			tableColumn.setMaxWidth(20);
			break;
		case COL_ID:			
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setMaxWidth(40);
			break;
		case COL_SENT_DATE:			
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setMaxWidth(160);
			break;
		case COL_ADDR_FROM:
			tableColumn.setHeaderRenderer(mhr);
			break;
		case COL_ADDR_TO:
			tableColumn.setHeaderRenderer(mhr);
			break;
		case COL_SUBJECT:
			tableColumn.setHeaderRenderer(mhr);
			break;
		case COL_RECEIVED_DATE:			
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setMaxWidth(160);
			break;
		case COL_MSG_SIZE:			
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setMaxWidth(70);
			break;
		case COL_SF_SEEN:
		case COL_SF_FLAGGED:			
			tableColumn.setHeaderRenderer(mhr);
			tableColumn.setCellRenderer(ir);
			tableColumn.setMaxWidth(20);
			break;			
		}		
    }
    
}

class IconRenderer extends JLabel implements TableCellRenderer {

	static final long serialVersionUID = 6244793159432844502L;

	public IconRenderer() {
		//super();
	}

	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		try {
			String val = null;
			Boolean boolVal = null;
			if (value instanceof String)
				val = (String) value;
			else if (value instanceof Boolean)
				boolVal = (Boolean) value;
			switch (MessageTableModel.shownColumns[column]) {
			case MessageTableModel.COL_IMPORTANCE:
				if (val.equals("1"))
					this.setIcon(Context.getImageIcon("imptnt.gif"));
				else
					this.setIcon(null);
				break;
			case MessageTableModel.COL_HAS_ATT:
				if (val.equals("1"))
					this.setIcon(Context.getImageIcon("att.gif"));
				else
					this.setIcon(null);
				break;
			case MessageTableModel.COL_SF_SEEN:
				if (boolVal.booleanValue())
					this.setIcon(Context.getImageIcon("seen.gif"));
				else
					this.setIcon(null);
				break;
			case MessageTableModel.COL_SF_FLAGGED:
				if (boolVal.booleanValue())
					this.setIcon(Context.getImageIcon("flagged.gif"));
				else
					this.setIcon(null);
				break;				
			case MessageTableModel.COL_HDR:
				MessageRow mr = ((MessageTableModel) table.getModel())
						.getMessageRowAt(row);
				String iname = "m";
				if (mr.hdr == MessageRow.HDR_MULTI) {
					iname += "multi";
				} else {
					if (mr.hdr == MessageRow.HDR_COMPLETE && mr.sfSeen)
						iname += "seen";
					else if (mr.hdr == MessageRow.HDR_COMPLETE && !mr.sfSeen)
						iname += "uns";
					else if (mr.hdr == MessageRow.HDR_HDR) {
						iname += "hdrh";
						if (mr.sfSeen)
							iname += "s";
						else
							iname += "u";
					} else if (mr.hdr == MessageRow.HDR_FORCED) {
						iname += "hdrf";
						if (mr.sfSeen)
							iname += "s";
						else
							iname += "u";						
					}
					if(mr.sfAnsw)
						iname += "r";
				}
				if(mr.sfDel)
					iname += "d";									
				ImageIcon ic = null;
				if (!iname.equals("m") && !iname.equals("mr")
						&& !iname.equals("md"))
					ic = Context.getImageIcon(iname + ".gif");
				this.setIcon(ic);
				break;
/*
 * case MessageTableModel.COL_HDR: MessageRow mr = ((MessageTableModel)
 * table.getModel()) .getMessageRowAt(row); if (mr.hdr == MessageRow.HDR_HDR)
 * this.setIcon(new ImageIcon(olivamail.gui.MainFrame.class
 * .getResource("asc.gif"))); else if (mr.hdr == MessageRow.HDR_FORCED)
 * this.setIcon(new ImageIcon(olivamail.gui.MainFrame.class
 * .getResource("flag.gif"))); else if (mr.hdr == MessageRow.HDR_MULTI)
 * this.setIcon(new ImageIcon(olivamail.gui.MainFrame.class
 * .getResource("group.gif"))); else if (!mr.sfSeen) this.setIcon(new
 * ImageIcon(olivamail.gui.MainFrame.class .getResource("glasses.gif"))); else
 * this.setIcon(null); break;
 */				
			}
		} catch (Exception exc) {
			exc.printStackTrace();
		}
		return this;
	}
	
}

// This comparator is used to sort vectors of data
class ColumnSorter implements Comparator {
    int colIndex;
    boolean ascending;
    int iasc = 1;
    
    ColumnSorter(int colIndex, boolean ascending) {
        this.colIndex = colIndex;
        this.ascending = ascending;
        if(!ascending)
        	iasc = -1;
    }
    
    public int compare(Object o1, Object o2) {

    	MessageRow a = (MessageRow)o1;
    	MessageRow b = (MessageRow)o2;
    	
        if (a == null && b == null) {
            return 0;
        } else if (a == null) {
            return 1;
        } else if (b == null) {
            return -1;
        } 
    	    	
//        switch(MessageTableModel.shownColumns[colIndex]) {		
        switch(colIndex) {
		case MessageTableModel.COL_IMPORTANCE:			
			return MessageHeader.compareImportances(a.importance, b.importance, ascending);
		case MessageTableModel.COL_HDR:
			return iasc * (a.hdr - b.hdr);			
		case MessageTableModel.COL_HAS_ATT:
			return iasc * a.hasAtt.compareTo(b.hasAtt);
		case MessageTableModel.COL_ID:
			return iasc * a.id.compareTo(b.id);
		case MessageTableModel.COL_SENT_DATE:
			return iasc * MailUtils.nvlDate(a.getSentDate()).compareTo(MailUtils.nvlDate(b.getSentDate()));	
		case MessageTableModel.COL_ADDR_FROM:
			return iasc * a.addrFrom.toLowerCase().compareTo(b.addrFrom.toLowerCase());
		case MessageTableModel.COL_ADDR_TO:
			return iasc * a.addrTo.toLowerCase().compareTo(b.addrTo.toLowerCase());
		case MessageTableModel.COL_SUBJECT:
			return iasc * a.subject.toLowerCase().compareTo(b.subject.toLowerCase());
		case MessageTableModel.COL_RECEIVED_DATE:
			return iasc * MailUtils.nvlDate(a.getReceivedDate()).compareTo(MailUtils.nvlDate(b.getReceivedDate()));
		case MessageTableModel.COL_MSG_SIZE:
			return iasc * (a.msgSize - b.msgSize);
		case MessageTableModel.COL_SF_SEEN:
			return iasc * (Boolean.valueOf(a.sfSeen).compareTo(Boolean.valueOf(b.sfSeen)));
		case MessageTableModel.COL_SF_FLAGGED:
			return iasc * (Boolean.valueOf(a.sfFlag).compareTo(Boolean.valueOf(b.sfFlag)));			
		default:
			return 0;
		}        
        
    }
}