/*
 * @(#)MgrpFormFrame.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JFrame;
import java.awt.FlowLayout;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import java.awt.event.ActionEvent;

public class MgrpFormFrame extends JFrame {

	static final long serialVersionUID = 882184318907984058L;
	
	private final String WIN_ID = "mime_group";

	private JPanel jContentPane = null;

	private JLabel jLabel1 = null;

	private JLabel jLabel2 = null;

	private JLabel jLabel3 = null;

	private JTextField idField = null;

	private JTextField nameField = null;

	private JTextField statusField = null;

	private JButton okButton = null;

	private JButton cancelButton = null;

	private MimesFrame parent = null;

	/**
	 * This is the default constructor
	 */
	public MgrpFormFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(316, 143);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);				
			}
		});
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("MgrpFormFrame.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel3 = new JLabel();
			jLabel3.setText(Strings.getString("MgrpFormFrame.statusField")); //$NON-NLS-1$
			jLabel3.setPreferredSize(new java.awt.Dimension(70, 16));
			jLabel2 = new JLabel();
			jLabel2.setText(Strings.getString("MgrpFormFrame.nameField")); //$NON-NLS-1$
			jLabel2.setPreferredSize(new java.awt.Dimension(70, 16));
			jLabel1 = new JLabel();
			jLabel1.setText(Strings.getString("MgrpFormFrame.idField")); //$NON-NLS-1$
			jLabel1.setPreferredSize(new java.awt.Dimension(70, 16));
			jContentPane = new JPanel();
			jContentPane.setLayout(new FlowLayout());
			jContentPane.add(jLabel1, null);
			jContentPane.add(getIdField(), null);
			jContentPane.add(jLabel2, null);
			jContentPane.add(getNameField(), null);
			jContentPane.add(jLabel3, null);
			jContentPane.add(getStatusField(), null);
			jContentPane.add(getOkButton(), null);
			jContentPane.add(getCancelButton(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes idField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getIdField() {
		if (idField == null) {
			idField = new JTextField();
			idField.setPreferredSize(new java.awt.Dimension(220,20));
			idField.setEnabled(false);
			TextPopup.registerComponent(idField);
		}
		return idField;
	}

	/**
	 * This method initializes nameField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getNameField() {
		if (nameField == null) {
			nameField = new JTextField();
			nameField.setPreferredSize(new java.awt.Dimension(220,20));
			TextPopup.registerComponent(nameField);
		}
		return nameField;
	}

	/**
	 * This method initializes statusField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getStatusField() {
		if (statusField == null) {
			statusField = new JTextField();
			statusField.setPreferredSize(new java.awt.Dimension(220,20));
			statusField.setText("0");
			statusField.setEnabled(false);
			TextPopup.registerComponent(statusField);
		}
		return statusField;
	}

	/**
	 * This method initializes okButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton")); //$NON-NLS-1$
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					okButton_actionPerformed(e);
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return cancelButton;
	}

	public void editMimeGroup(String id) {
		try {
			olivamail.gui.MimeGroup mgrp = Context.storage.getMimeGroup(id);
			idField.setText(mgrp.id);
			nameField.setText(mgrp.name);
			statusField.setText(mgrp.status);
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

	public void setParent(MimesFrame parent) {
		this.parent = parent;
	}

	void okButton_actionPerformed(ActionEvent e) {
		olivamail.gui.MimeGroup mgrp = new olivamail.gui.MimeGroup();
		mgrp.id = idField.getText();
		mgrp.name = nameField.getText();
		mgrp.status = statusField.getText();
		try {
			Context.storage.saveMimeGroup(mgrp);
			parent.listGroups();
			this_windowClosing(null);
			dispose();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

} // @jve:decl-index=0:visual-constraint="10,10"
