/*
 * @(#)MimeSelectionForm.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import java.util.Vector;

public class MimeSelectionForm extends JFrame {

	static final long serialVersionUID = -1155460257117713217L;
	private final String WIN_ID = "mime_select";
	private JPanel jContentPane = null;
	private JScrollPane jScrollPane1 = null;
	private JTable jTable1 = null;
	private JPanel jPanel1 = null;
	private JButton jButtonSelect = null;
	private JButton jButtonCancel = null;
	private MtypeFormFrame parent = null;
	/**
	 * This is the default constructor
	 */
	public MimeSelectionForm() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	public void setParent(MtypeFormFrame parent) {
		this.parent = parent;
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("MimeSelectionForm.mimeTypes")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJScrollPane1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getJTable1());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jTable1	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	private JTable getJTable1() {
		if (jTable1 == null) {
			jTable1 = new JTable() {
				static final long serialVersionUID = 2L;
				public Component prepareRenderer(TableCellRenderer renderer,
						int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex,
							vColIndex);
					if (isCellSelected(rowIndex, vColIndex)) {// What else
						c.setBackground(Color.blue);
						c.setForeground(Color.white);
					} else {
						c.setBackground(Color.white);
						c.setForeground(Color.black);
					}
					return c;
				}
			};
			//jTable1.setFont(new java.awt.Font("Dialog", 0, 14)); //$NON-NLS-1$
		}
		return jTable1;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.add(getJButtonSelect(), null);
			jPanel1.add(getJButtonCancel(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jButtonSelect	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonSelect() {
		if (jButtonSelect == null) {
			jButtonSelect = new JButton();
			jButtonSelect.setText(Strings.getString("selectButton")); //$NON-NLS-1$
			jButtonSelect.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					if(parent != null) {						
						int row = jTable1.getSelectedRow();
						if (row != -1) {
							parent.setMimeTypeField((String)jTable1.getValueAt(row, 0));
						}
					}
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jButtonSelect;
	}

	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jButtonCancel;
	}

	public void listMimeTypes() {
		Vector headers = new Vector(1);
		headers.add("Mime Types"); //$NON-NLS-1$
		Vector rows = null;
		try {
			rows = new Vector(olivamail.core.mail.MailUtils.mimeTypes.length);
			for(int i = 0; i < olivamail.core.mail.MailUtils.mimeTypes.length; i++) {
				Vector row = new Vector(1);
				row.add(olivamail.core.mail.MailUtils.mimeTypes[i][0]); 
				rows.add(row);				
			}
			DefaultTableModel dtm = new DefaultTableModel();
			dtm.setDataVector(rows, headers);
			jTable1.setModel(dtm);
			/*
			TableColumnModel tcm = jTable1.getColumnModel();
			tcm.getColumn(0).setMaxWidth(50);
			tcm.getColumn(1).setMinWidth(150);
			*/
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

}
