/*
 * @(#)MimesFrame.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import java.util.Vector;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import java.awt.Component;
import java.awt.Font;
import java.awt.Color;
import java.awt.event.ActionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

public class MimesFrame extends JFrame {

	static final long serialVersionUID = -3243918227983534222L;
	
	private final String WIN_ID = "mimes";

	private JPanel jContentPane = null;

	private JPanel jPanel2 = null;

	private JPanel jPanel3 = null;

	private JPanel jPanel4 = null;

	private JSplitPane jSplitPane1 = null;

	private JScrollPane jScrollPane1 = null;

	private JScrollPane jScrollPane2 = null;

	private JTable jTable1 = null;

	private JTable jTable2 = null;

	private String mimeGroupId = null;
	
	private String copyMimeGroupId = null;

	private JMenuBar jJMenuBar1 = null;

	private JMenu jMenuFile = null;

	private JMenu jMenuMimeGroups = null;

	private JMenu jMenuMimeTypes = null;

	private JMenuItem jMenuItemExit = null;

	private JMenuItem jMenuItemMGCopy = null;

	private JMenuItem jMenuItemMGPaste = null;

	private JMenuItem jMenuItemMGCreate = null;

	private JMenuItem jMenuItemMGEdit = null;

	private JMenuItem jMenuItemMGDelete = null;

	private JMenuItem jMenuItemMGDefault = null;

	private JMenuItem jMenuItemMTCreate = null;

	private JMenuItem jMenuItemMTEdit = null;

	private JMenuItem jMenuItemMTDelete = null;

//	private int mimeGroupRow = -1;

	/**
	 * This is the default constructor
	 */
	public MimesFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));	
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(610, 313);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setJMenuBar(getJJMenuBar1());
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("MimesFrame.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel2(), java.awt.BorderLayout.WEST);
			jContentPane.add(getJPanel3(), java.awt.BorderLayout.EAST);
			jContentPane.add(getJPanel4(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getJSplitPane1(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel2
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();
		}
		return jPanel2;
	}

	/**
	 * This method initializes jPanel3
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new JPanel();
		}
		return jPanel3;
	}

	/**
	 * This method initializes jPanel4
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new JPanel();
		}
		return jPanel4;
	}

	public void listGroups() {
		Vector headers = new Vector(3);
		headers.add(Strings.getString("MimesFrame.idField")); //$NON-NLS-1$
		headers.add(Strings.getString("MimesFrame.nameField")); //$NON-NLS-1$
		headers.add(Strings.getString("MimesFrame.statusField")); //$NON-NLS-1$
		Vector rows = null;
		try {
			rows = Context.storage.getMimeGroupList(null, null, -1);
			DefaultTableModel dtm = new DefaultTableModel();
			dtm.setDataVector(rows, headers);
			jTable1.setModel(dtm);
			TableColumnModel tcm = jTable1.getColumnModel();
			tcm.getColumn(MgrpTableColumns.COL_ID).setMaxWidth(50);
			tcm.getColumn(MgrpTableColumns.COL_NAME).setMinWidth(150);
			tcm.getColumn(MgrpTableColumns.COL_STATUS).setMaxWidth(20);
			tcm.getColumn(MgrpTableColumns.COL_STATUS).setCellRenderer(
					new MgrpIconRenderer());						
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

	/**
	 * This method initializes jSplitPane
	 * 
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getJSplitPane1() {
		if (jSplitPane1 == null) {
			jSplitPane1 = new JSplitPane();
			jSplitPane1.setDividerLocation(80);
			jSplitPane1.setTopComponent(getJScrollPane1());
			jSplitPane1.setBottomComponent(getJScrollPane2());
			jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
		}
		return jSplitPane1;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getJTable1());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jScrollPane1
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane2() {
		if (jScrollPane2 == null) {
			jScrollPane2 = new JScrollPane();
			jScrollPane2.setViewportView(getJTable2());
		}
		return jScrollPane2;
	}

	/**
	 * This method initializes jTable
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getJTable1() {
		if (jTable1 == null) {
			jTable1 = new JTable() {
				static final long serialVersionUID = 4094890482761274506L;

				public boolean isCellEditable(int rowIndex, int vColIndex) {
					return false;
				}																
				
				public Component prepareRenderer(TableCellRenderer renderer,
						int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex,
							vColIndex);
					if (isCellSelected(rowIndex, vColIndex)) {// What else
						// should i put
						// in
						// here to check the value
						c.setBackground(Color.blue);
						c.setForeground(Color.white);
					} else {
						c.setBackground(Color.white);
						c.setForeground(Color.black);
					}
					if (((String) this.getValueAt(rowIndex, 2)).equals("D"))
						c.setFont(c.getFont().deriveFont(Font.BOLD));
					else
						c.setFont(c.getFont().deriveFont(Font.PLAIN));
					return c;
				}
			};
			//jTable1.setFont(new java.awt.Font("Dialog", 0, 14)); //$NON-NLS-1$
			SelectionListener listener = new SelectionListener(jTable1, this);
			jTable1.getSelectionModel().addListSelectionListener(listener);
			jTable1.addMouseListener(new java.awt.event.MouseAdapter() { 
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if (e.getClickCount() == 2) {
						editCurrentMgrpRow();
					}
				}
			});									
			/*
			 * jTable1.getColumnModel().getSelectionModel()
			 * .addListSelectionListener(listener);
			 */
			jTable1.setGridColor(jTable1.getBackground());
		}
		return jTable1;
	}

	/**
	 * This method initializes jTable1
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getJTable2() {
		if (jTable2 == null) {
			jTable2 = new JTable() {
				static final long serialVersionUID = -1789074023014241687L;

				public boolean isCellEditable(int rowIndex, int vColIndex) {
					return false;
				}																
				
				public Component prepareRenderer(TableCellRenderer renderer,
						int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex,
							vColIndex);
					if (isCellSelected(rowIndex, vColIndex)) {// What else should i put in
						// here to check the value
						c.setBackground(Color.blue);
						c.setForeground(Color.white);
					} else {
						c.setBackground(Color.white);
						c.setForeground(Color.black);
					}
					return c;
				}
			};
			//jTable2.setFont(new java.awt.Font("Dialog", 0, 14)); //$NON-NLS-1$
			jTable2.addMouseListener(new java.awt.event.MouseAdapter() { 
				public void mouseClicked(java.awt.event.MouseEvent e) {
					if (e.getClickCount() == 2) {
						editCurrentMtypeRow();
					}
				}
			});
			
			jTable2.setGridColor(jTable2.getBackground());
		}
		return jTable2;
	}

	public void listTypes() {
		if (mimeGroupId == null)
			return;
		Vector headers = new Vector(4);
		headers.add(Strings.getString("MimesFrame.idField")); //$NON-NLS-1$
		headers.add(Strings.getString("MimesFrame.mimeTypeField")); //$NON-NLS-1$
		headers.add(Strings.getString("MimesFrame.programField")); //$NON-NLS-1$
		headers.add(Strings.getString("MimesFrame.mgrpIdField")); //$NON-NLS-1$
		Vector rows = null;
		try {
			rows = Context.storage.getMimeTypeList(mimeGroupId, null, null, -1);
			DefaultTableModel dtm = new DefaultTableModel();
			dtm.setDataVector(rows, headers);
			jTable2.setModel(dtm);
			TableColumnModel tcm = jTable2.getColumnModel();
			tcm.getColumn(MtypeTableColumns.COL_ID).setMaxWidth(50);
			tcm.getColumn(MtypeTableColumns.COL_MIME_TYPE).setMinWidth(150);
			tcm.getColumn(MtypeTableColumns.COL_MIME_TYPE).setMaxWidth(250);
			tcm.getColumn(MtypeTableColumns.COL_MGRP_ID).setMaxWidth(50);
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

	void jMenuItemMGCreate_actionPerformed(java.awt.event.ActionEvent e) {
		MgrpFormFrame mff = new MgrpFormFrame();
		mff.setParent(this);
		mff.setVisible(true);
	}

	void jButton2_actionPerformed(ActionEvent e) {
	}

	void jButton3_actionPerformed(ActionEvent e) {
	}

	void jButton4_actionPerformed(ActionEvent e) {
	}

	void jButton5_actionPerformed(ActionEvent e) {
	}

	void jButton6_actionPerformed(ActionEvent e) {
	}

	public void refreshTypesList() {
		int row = jTable1.getSelectedRow();
		if (row == -1) {
			Vector headers = new Vector(4);
			headers.add(Strings.getString("MimesFrame.idField")); //$NON-NLS-1$
			headers.add(Strings.getString("MimesFrame.mimeTypeField")); //$NON-NLS-1$
			headers.add(Strings.getString("MimesFrame.programField")); //$NON-NLS-1$
			headers.add(Strings.getString("MimesFrame.mgrpIdField")); //$NON-NLS-1$
			Vector rows = null;
			DefaultTableModel dtm = new DefaultTableModel();
			dtm.setDataVector(rows, headers);
			jTable2.setModel(dtm);
			TableColumnModel tcm = jTable2.getColumnModel();
			tcm.getColumn(0).setMaxWidth(50);
			tcm.getColumn(1).setMinWidth(150);
			return;
		}
/*		
		if (row == mimeGroupRow)
			return;
*/			
		mimeGroupId = (String) jTable1.getValueAt(row, 0);
//		mimeGroupRow = row;
		// System.out.println(String.valueOf(row));
		listTypes();
	}

	void jButton7_actionPerformed(ActionEvent e) {
	}

	/**
	 * This method initializes jJMenuBar1	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */    
	private JMenuBar getJJMenuBar1() {
		if (jJMenuBar1 == null) {
			jJMenuBar1 = new JMenuBar();
			jJMenuBar1.add(getJMenuFile());
			jJMenuBar1.add(getJMenuMimeGroups());
			jJMenuBar1.add(getJMenuMimeTypes());
		}
		return jJMenuBar1;
	}

	/**
	 * This method initializes jMenuFile	
	 * 	
	 * @return javax.swing.JMenu	
	 */    
	private JMenu getJMenuFile() {
		if (jMenuFile == null) {
			jMenuFile = new JMenu();
			jMenuFile.setText(Strings.getString("menuFile")); //$NON-NLS-1$
			jMenuFile.add(getJMenuItemExit());
		}
		return jMenuFile;
	}

	/**
	 * This method initializes jMenuMimeGroups	
	 * 	
	 * @return javax.swing.JMenu	
	 */    
	private JMenu getJMenuMimeGroups() {
		if (jMenuMimeGroups == null) {
			jMenuMimeGroups = new JMenu();
			jMenuMimeGroups.setText(Strings.getString("MimesFrame.menuMimeGroups")); //$NON-NLS-1$
			jMenuMimeGroups.add(getJMenuItemMGCreate());
			jMenuMimeGroups.add(getJMenuItemMGEdit());
			jMenuMimeGroups.add(getJMenuItemMGDelete());
			jMenuMimeGroups.add(getJMenuItemMGDefault());
			jMenuMimeGroups.add(getJMenuItemMGCopy());
			jMenuMimeGroups.add(getJMenuItemMGPaste());
		}
		return jMenuMimeGroups;
	}

	/**
	 * This method initializes jMenuMimeTypes	
	 * 	
	 * @return javax.swing.JMenu	
	 */    
	private JMenu getJMenuMimeTypes() {
		if (jMenuMimeTypes == null) {
			jMenuMimeTypes = new JMenu();
			jMenuMimeTypes.setText(Strings.getString("MimesFrame.menuMimeTypes")); //$NON-NLS-1$
			jMenuMimeTypes.add(getJMenuItemMTCreate());
			jMenuMimeTypes.add(getJMenuItemMTEdit());
			jMenuMimeTypes.add(getJMenuItemMTDelete());
		}
		return jMenuMimeTypes;
	}

	/**
	 * This method initializes jMenuItemExit	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemExit() {
		if (jMenuItemExit == null) {
			jMenuItemExit = new JMenuItem();
			jMenuItemExit.setText(Strings.getString("menuClose")); //$NON-NLS-1$
			jMenuItemExit.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jMenuItemExit;
	}

	/**
	 * This method initializes jMenuItemMGCopy	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMGCopy() {
		if (jMenuItemMGCopy == null) {
			jMenuItemMGCopy = new JMenuItem();
			jMenuItemMGCopy.setText(Strings.getString("menuCopy")); //$NON-NLS-1$
			jMenuItemMGCopy.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMGCopy_actionPerformed(e);
				}
			});
		}
		return jMenuItemMGCopy;
	}

	void jMenuItemMGCopy_actionPerformed(java.awt.event.ActionEvent e) {
		int row = jTable1.getSelectedRow();
		if (row == -1) {
			AlertDialog.showInfoDialog(Strings.getString("MimesFrame.selectCopyFrom")); //$NON-NLS-1$
			return;		
		}
		copyMimeGroupId = (String) jTable1.getValueAt(row, 0);
		jMenuItemMGPaste.setEnabled(true);
	}
	
	/**
	 * This method initializes jMenuItemMGPaste	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMGPaste() {
		if (jMenuItemMGPaste == null) {
			jMenuItemMGPaste = new JMenuItem();
			jMenuItemMGPaste.setText(Strings.getString("menuPaste")); //$NON-NLS-1$
			jMenuItemMGPaste.setEnabled(false);
			jMenuItemMGPaste.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMGPaste_actionPerformed(e);
				}
			});
		}
		return jMenuItemMGPaste;
	}
	
	void jMenuItemMGPaste_actionPerformed(java.awt.event.ActionEvent e) {
		int row = jTable1.getSelectedRow();
		if (row == -1) {
			AlertDialog.showInfoDialog(Strings.getString("MimesFrame.selectCopyTo")); //$NON-NLS-1$
			return;		
		}
		String destinationMimeGroupId = (String) jTable1.getValueAt(row, 0);
		if (copyMimeGroupId.equals(destinationMimeGroupId)) {
			AlertDialog.showInfoDialog(Strings.getString("MimesFrame.groupsTheSame")); //$NON-NLS-1$
			return;		
		}		
		try {
			String count = String.valueOf(Context.storage.copyMimeTypes(copyMimeGroupId, destinationMimeGroupId));
			AlertDialog.showInfoDialog(count + Strings.getString("MimesFrame.typesCopied")); //$NON-NLS-1$
			refreshTypesList();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}		
		jMenuItemMGPaste.setEnabled(false);		
	}

	/**
	 * This method initializes jMenuItemMGCreate	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMGCreate() {
		if (jMenuItemMGCreate == null) {
			jMenuItemMGCreate = new JMenuItem();
			jMenuItemMGCreate.setText(Strings.getString("menuCreate")); //$NON-NLS-1$
			jMenuItemMGCreate.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMGCreate_actionPerformed(e);
				}
			});
		}
		return jMenuItemMGCreate;
	}

	/**
	 * This method initializes jMenuItemMGEdit	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMGEdit() {
		if (jMenuItemMGEdit == null) {
			jMenuItemMGEdit = new JMenuItem();
			jMenuItemMGEdit.setText(Strings.getString("menuEdit")); //$NON-NLS-1$
			jMenuItemMGEdit.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMGEdit_actionPerformed(e);
				}
			});
		}
		return jMenuItemMGEdit;
	}

	void editCurrentMgrpRow() {
		int row = jTable1.getSelectedRow();
		if (row == -1)
			return;
		MgrpFormFrame mff = new MgrpFormFrame();
		mff.setParent(this);
		mff.setVisible(true);
		mff.editMimeGroup((String) jTable1.getValueAt(row, 0));				
	}
	
	void jMenuItemMGEdit_actionPerformed(java.awt.event.ActionEvent e) {
		editCurrentMgrpRow();		
	}

	/**
	 * This method initializes jMenuItemMGDelete	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMGDelete() {
		if (jMenuItemMGDelete == null) {
			jMenuItemMGDelete = new JMenuItem();
			jMenuItemMGDelete.setText(Strings.getString("menuDelete")); //$NON-NLS-1$
			jMenuItemMGDelete.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMGDelete_actionPerformed(e);
				}
			});
		}
		return jMenuItemMGDelete;
	}

	void jMenuItemMGDelete_actionPerformed(java.awt.event.ActionEvent e) {
		int row = jTable1.getSelectedRow();
		if (row != -1) {
			if(AlertDialog.showConfirmDialog(Strings.getString("MimesFrame.mgrpDeleteConfirm"), AlertDialog.OK_CANCEL_OPTION) != AlertDialog.OK_OPTION) //$NON-NLS-1$
				return;					
			try {
				String mgrpId = (String) jTable1.getValueAt(
						row, 0);
				if(!Context.storage.isMimeGroupEmpty(mgrpId)) {
					if(AlertDialog.showConfirmDialog(Strings.getString("MimesFrame.groupNotEmpty"), AlertDialog.YES_NO_OPTION) != AlertDialog.YES_OPTION) //$NON-NLS-1$
						return;
					Context.storage.deleteMimeTypes(mgrpId);
				}				
				Context.storage.deleteMimeGroup(mgrpId);
				if(copyMimeGroupId != null) {
					if(mgrpId.equals(copyMimeGroupId)) {
						copyMimeGroupId = null;
						jMenuItemMGPaste.setEnabled(false);
					}
				}					
				listGroups();
				refreshTypesList();
			} catch (Exception exc) {
				exc.printStackTrace();
				AlertDialog.showErrorDialog(exc);
			}
		}		
	}

	/**
	 * This method initializes jMenuItemMGDefault	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMGDefault() {
		if (jMenuItemMGDefault == null) {
			jMenuItemMGDefault = new JMenuItem();
			jMenuItemMGDefault.setText(Strings.getString("menuDefault")); //$NON-NLS-1$
			jMenuItemMGDefault.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMGDefault_actionPerformed(e);
				}
			});
		}
		return jMenuItemMGDefault;
	}

	void jMenuItemMGDefault_actionPerformed(java.awt.event.ActionEvent e) {
		int row = jTable1.getSelectedRow();
		if (row != -1) {
			try {
				Context.storage.setDefaultMimeGroup((String) jTable1
						.getValueAt(row, 0));
				listGroups();
			} catch (Exception exc) {
				exc.printStackTrace();
				AlertDialog.showErrorDialog(exc);
			}
		}		
	}

	/**
	 * This method initializes jMenuItemMTCreate	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMTCreate() {
		if (jMenuItemMTCreate == null) {
			jMenuItemMTCreate = new JMenuItem();
			jMenuItemMTCreate.setText(Strings.getString("menuCreate")); //$NON-NLS-1$
			jMenuItemMTCreate.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMTCreate_actionPerformed(e);
				}
			});
		}
		return jMenuItemMTCreate;
	}

	void jMenuItemMTCreate_actionPerformed(java.awt.event.ActionEvent e) {
		if (mimeGroupId == null)
			return;
		MtypeFormFrame mff = new MtypeFormFrame();
		mff.setParent(this);
		mff.setVisible(true);
		mff.setMimeGroupId(mimeGroupId);		
	}
	
	/**
	 * This method initializes jMenuItemMTEdit	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMTEdit() {
		if (jMenuItemMTEdit == null) {
			jMenuItemMTEdit = new JMenuItem();
			jMenuItemMTEdit.setText(Strings.getString("menuEdit")); //$NON-NLS-1$
			jMenuItemMTEdit.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMTEdit_actionPerformed(e);
				}
			});
		}
		return jMenuItemMTEdit;
	}

	void editCurrentMtypeRow() {
		int row = jTable2.getSelectedRow();
		if (row == -1)
			return;
		MtypeFormFrame mff = new MtypeFormFrame();
		mff.setParent(this);
		mff.setVisible(true);
		mff.editMimeType((String) jTable2.getValueAt(row, 0));				
	}
	
	void jMenuItemMTEdit_actionPerformed(java.awt.event.ActionEvent e) {
		editCurrentMtypeRow();		
	}
	
	/**
	 * This method initializes jMenuItemMTDelete	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJMenuItemMTDelete() {
		if (jMenuItemMTDelete == null) {
			jMenuItemMTDelete = new JMenuItem();
			jMenuItemMTDelete.setText(Strings.getString("menuDelete")); //$NON-NLS-1$
			jMenuItemMTDelete.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jMenuItemMTDelete_actionPerformed(e);
				}
			});
		}
		return jMenuItemMTDelete;
	}
	
	void jMenuItemMTDelete_actionPerformed(java.awt.event.ActionEvent e) {
		int rows[] = jTable2.getSelectedRows();
		if (rows.length == 0) {
			AlertDialog.showInfoDialog(Strings.getString("MimesFrame.selectMtypeToDelete")); //$NON-NLS-1$
			return;
		}
		if (AlertDialog.showConfirmDialog(Strings.getString("MimesFrame.mtypeDeleteConfirm"), AlertDialog.OK_CANCEL_OPTION) != AlertDialog.OK_OPTION) //$NON-NLS-1$
			return;
		String ids[] = new String[rows.length];
		for (int i = 0; i < rows.length; i++) {
			ids[i] = (String) jTable2.getValueAt(rows[i], 0);
		}
		try {
			for (int i = 0; i < ids.length; i++)
				Context.storage.deleteMimeType(ids[i]);
			listTypes();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
	
} // @jve:decl-index=0:visual-constraint="10,10"

class SelectionListener implements ListSelectionListener {
	JTable table = null;

	MimesFrame mf = null;

	// It is necessary to keep the table since it is not possible
	// to determine the table from the event's source
	SelectionListener(JTable table, MimesFrame mf) {
		this.table = table;
		this.mf = mf;
	}

	public void valueChanged(ListSelectionEvent e) {
		// If cell selection is enabled, both row and column change events are
		// fired
		if (e.getSource() == table.getSelectionModel()
				&& table.getRowSelectionAllowed()) {
			// Column selection changed
			mf.refreshTypesList();
		} else if (e.getSource() == table.getColumnModel().getSelectionModel()
				&& table.getColumnSelectionAllowed()) {
			// Row selection changed
			mf.refreshTypesList();
		}

		if (e.getValueIsAdjusting()) {
			// The mouse button has not yet been released
		}
	}
}

class MgrpIconRenderer extends JLabel implements TableCellRenderer {

	static final long serialVersionUID = 6244793159432844223L;

	public MgrpIconRenderer() {
		super();
	}

	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		try {
			String val = (String) value;
			switch (column) {
			case MgrpTableColumns.COL_STATUS:
				if (val.equals("D")) //$NON-NLS-1$
					this.setIcon(new ImageIcon(olivamail.gui.MainFrame.class
							.getResource("check1.png"))); //$NON-NLS-1$
				else
					this.setIcon(null);
				break;
			}
		} catch (Exception exc) {
			exc.printStackTrace();
		}
		return this;
	}

}

class MgrpTableColumns {
	static final int COL_ID = 0;
	static final int COL_NAME = 1;	
	static final int COL_STATUS = 2;
}

class MtypeTableColumns {
	static final int COL_ID = 0;
	static final int COL_MIME_TYPE = 1;	
	static final int COL_PROGRAM = 2;
	static final int COL_MGRP_ID = 3;
}
