/*
 * @(#)MsgColLayoutFrame.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JButton;
import javax.swing.table.TableCellRenderer;
import javax.swing.JScrollPane;

public class MsgColLayoutFrame extends JFrame {

	static final long serialVersionUID = 1254804733565614902L;	
	
	private final String WIN_ID = "msglistcol";
	
	private JPanel jContentPane = null;

	private JPanel topPanel = null;

	private JPanel bottomPanel = null;

	private JSplitPane jSplitPane1 = null;

	private JButton jButtonLeft = null;

	private JButton jButtonRight = null;

	private JButton okButton = null;

	private JButton cancelButton = null;

	private JScrollPane jScrollPaneLeft = null;

	private JTable jTableAll = null;

	private JScrollPane jScrollPaneRight = null;

	private JTable jTableSel = null;

	private JButton jButtonUp = null;

	private JButton jButtonDown = null;

	/**
	 * This is the default constructor
	 */
	public MsgColLayoutFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(450, 239);
		this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("MsgColLayoutFrame.title"));
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getTopPanel(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getBottomPanel(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getJSplitPane1(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}

	/**
	 * This method initializes topPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getTopPanel() {
		if (topPanel == null) {
			topPanel = new JPanel();
		}
		return topPanel;
	}

	/**
	 * This method initializes bottomPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getBottomPanel() {
		if (bottomPanel == null) {
			bottomPanel = new JPanel();
			bottomPanel.setPreferredSize(new java.awt.Dimension(10,35));
			bottomPanel.add(getJButtonLeft(), null);
			bottomPanel.add(getJButtonRight(), null);
			bottomPanel.add(getJButtonDown(), null);
			bottomPanel.add(getJButtonUp(), null);
			bottomPanel.add(getOkButton(), null);
			bottomPanel.add(getCancelButton(), null);
		}
		return bottomPanel;
	}

	/**
	 * This method initializes jSplitPane1	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */    
	private JSplitPane getJSplitPane1() {
		if (jSplitPane1 == null) {
			jSplitPane1 = new JSplitPane();
			jSplitPane1.setDividerLocation(155);
			jSplitPane1.setRightComponent(getJScrollPaneRight());
			jSplitPane1.setLeftComponent(getJScrollPaneLeft());
		}
		return jSplitPane1;
	}

	/**
	 * This method initializes jButtonLeft	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonLeft() {
		if (jButtonLeft == null) {
			jButtonLeft = new JButton();
			jButtonLeft.setText("<--");
			jButtonLeft.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					int row = jTableSel.getSelectedRow();
					if (row == -1)
						return;
					MsgColLayoutTableModel mcltmSel = (MsgColLayoutTableModel) jTableSel
							.getModel();
					MsgCol mc = mcltmSel.getMsgColAt(row);
					MsgColLayoutTableModel mcltmAll = (MsgColLayoutTableModel) jTableAll
							.getModel();
					mcltmAll.addRow(mc);
					mcltmSel.removeRow(row);
					row = jTableAll.getRowCount() - 1; 
					jTableAll.setRowSelectionInterval(row, row);					
				}
			});
		}
		return jButtonLeft;
	}

	/**
	 * This method initializes jButtonRight	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonRight() {
		if (jButtonRight == null) {
			jButtonRight = new JButton();
			jButtonRight.setText("-->");
			jButtonRight.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					int row = jTableAll.getSelectedRow();
					if (row == -1)
						return;
					MsgColLayoutTableModel mcltmAll = (MsgColLayoutTableModel) jTableAll
							.getModel();
					MsgCol mc = mcltmAll.getMsgColAt(row);
					MsgColLayoutTableModel mcltmSel = (MsgColLayoutTableModel) jTableSel
							.getModel();
					mcltmSel.addRow(mc);
					mcltmAll.removeRow(row);					
					row = jTableSel.getRowCount() - 1; 
					jTableSel.setRowSelectionInterval(row, row);
				}
			});
		}
		return jButtonRight;
	}

	/**
	 * This method initializes okButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton"));
			okButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					try {				
						if(jTableSel.getRowCount() == 0)
							return;						
						StringBuffer sb = new StringBuffer(); 
						for(int i = 0; i < jTableSel.getRowCount(); i++) {
							if(i != 0)
								sb.append(",");
							MsgCol mc = ((MsgColLayoutTableModel)jTableSel.getModel()).getMsgColAt(i);
							sb.append(MessageTableModel.columnCaps[mc.column]).append("=").append("20");
						}
						Context.mainFrame.setShownColumns(sb.toString());
						this_windowClosing(null);
						dispose();
					} catch (Exception exc) {
						exc.printStackTrace();
						AlertDialog.showErrorDialog(exc);
					}
				}
			});
		}
		return okButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton"));
			cancelButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return cancelButton;
	}
		
	public void listColumnsAll() {
		try {
			MsgColLayoutTableModel mcltm = new MsgColLayoutTableModel(false);
			jTableAll.setModel(mcltm);;
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}		

	public void listColumnsSel() {
		try {
			MsgColLayoutTableModel mcltm = new MsgColLayoutTableModel(true);
			jTableSel.setModel(mcltm);;
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}

	/**
	 * This method initializes jScrollPaneLeft	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPaneLeft() {
		if (jScrollPaneLeft == null) {
			jScrollPaneLeft = new JScrollPane();
			jScrollPaneLeft.setViewportView(getJTableAll());
		}
		return jScrollPaneLeft;
	}

	/**
	 * This method initializes jTableAll	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	private JTable getJTableAll() {
		if (jTableAll == null) {
			jTableAll = new JTable() {
				static final long serialVersionUID = 310240634738289292L;

				public boolean isCellEditable(int rowIndex, int vColIndex) {
					return false;
				}

				public Component prepareRenderer(TableCellRenderer renderer,
						int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex,
							vColIndex);
					if (isCellSelected(rowIndex, vColIndex)) {// What else
						c.setBackground(Color.blue);
						c.setForeground(Color.white);
					} else {
						c.setBackground(Color.white);
						c.setForeground(Color.black);
					}
					int style = Font.PLAIN;
					c.setFont(c.getFont().deriveFont(style));
					return c;
				}
			};
			jTableAll.setGridColor(jTableAll.getBackground());
		}
		return jTableAll;
	}

	/**
	 * This method initializes jScrollPaneRight
	 * 
	 * @return javax.swing.JScrollPane
	 */    
	private JScrollPane getJScrollPaneRight() {
		if (jScrollPaneRight == null) {
			jScrollPaneRight = new JScrollPane();
			jScrollPaneRight.setViewportView(getJTableSel());
		}
		return jScrollPaneRight;
	}

	/**
	 * This method initializes jTableSel	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	private JTable getJTableSel() {
		if (jTableSel == null) {
			jTableSel = new JTable() {
				static final long serialVersionUID = 310211234735489292L;

				public boolean isCellEditable(int rowIndex, int vColIndex) {
					return false;
				}

				public Component prepareRenderer(TableCellRenderer renderer,
						int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex,
							vColIndex);
					if (isCellSelected(rowIndex, vColIndex)) {// What else
						c.setBackground(Color.blue);
						c.setForeground(Color.white);
					} else {
						c.setBackground(Color.white);
						c.setForeground(Color.black);
					}
					int style = Font.PLAIN;
					c.setFont(c.getFont().deriveFont(style));
					return c;
				}
			};
			jTableSel.setGridColor(jTableSel.getBackground());
		}
		return jTableSel;
	}

	/**
	 * This method initializes jButtonUp
	 * 
	 * @return javax.swing.JButton
	 */    
	private JButton getJButtonUp() {
		if (jButtonUp == null) {
			jButtonUp = new JButton();
			jButtonUp.setText("/\\");
			jButtonUp.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					int row = jTableSel.getSelectedRow();
					if(row < 1)
						return;
					MsgColLayoutTableModel mcltm = (MsgColLayoutTableModel)jTableSel.getModel();
					mcltm.moveRow(row, row, --row);
					jTableSel.setRowSelectionInterval(row, row);
				}
			});
		}
		return jButtonUp;
	}

	/**
	 * This method initializes jButtonDown	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonDown() {
		if (jButtonDown == null) {
			jButtonDown = new JButton();
			jButtonDown.setText("\\/");
			jButtonDown.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					int row = jTableSel.getSelectedRow();
					if(row == -1 || row >= jTableSel.getRowCount() - 1)
						return;
					MsgColLayoutTableModel mcltm = (MsgColLayoutTableModel)jTableSel.getModel();
					mcltm.moveRow(row, row, ++row);
					jTableSel.setRowSelectionInterval(row, row);
				}
			});
		}
		return jButtonDown;
	}		
		
}  //  @jve:decl-index=0:visual-constraint="10,10"
