/*
 * @(#)MsgColLayoutTableModel.java
 *
 * Copyright (C) 2008 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.util.Vector;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

public class MsgColLayoutTableModel extends AbstractTableModel {	
	static final long serialVersionUID = 2211334328942112L;
	
	public static final int COL_NAME = 0;				
		
	public static final int shownColumns[] = { COL_NAME };						
	
	Vector data = null;
		
	public MsgColLayoutTableModel(boolean shown) {
		data = new Vector(MessageTableModel.columnCaps.length);

		if (!shown) {
			for (int i = 0; i < MessageTableModel.columnCaps.length; i++) {
				boolean skip = false;
				for (int j = 0; j < MessageTableModel.shownColumns.length; j++) {
					if (i == MessageTableModel.shownColumns[j]) {
						skip = true;
					}
				}
				if (skip)
					continue;
				MsgCol mc = new MsgCol(i);
				data.add(mc);
			}
		} else {
			for (int j = 0; j < MessageTableModel.shownColumns.length; j++) {
				MsgCol mc = new MsgCol(MessageTableModel.shownColumns[j]);
				data.add(mc);
			}
			return;

		}
	}
		
	public int getColumnCount() {
		return shownColumns.length;
	}

	public int getRowCount() {
		return data.size();
	}
	
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		fireTableRowsUpdated(rowIndex, rowIndex);		
	}
		
	public MsgCol getMsgColAt(int rowIndex) {
		return (MsgCol)data.get(rowIndex);
	}	
	
	public Object getValueAt(int rowIndex, int columnIndex) {
		MsgCol mc = (MsgCol) data.get(rowIndex);
		switch (shownColumns[columnIndex]) {
		case COL_NAME:
			return mc.name;
		}
		return null;
	}

	public String getColumnName(int column) {
		switch(shownColumns[column]) {
		case COL_NAME:
			return Strings.getString("IMAPFoldersFrame.nameCol");
		}
		return null;
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}
	
    public void addRow(MsgCol mc) {
    	insertRow(getRowCount(), mc);
    }      
    
    public void insertRow(int row, MsgCol mc) {
    	data.insertElementAt(mc, row);  
        fireTableRowsInserted(row, row);
    }
    
    public void removeRow(int row) {
        data.removeElementAt(row);
        fireTableRowsDeleted(row, row);
    }    

    public static void setColumnModel(JTable jTable1) {
		TableColumnModel tcm = jTable1.getColumnModel();
		for(int i = 0; i < tcm.getColumnCount(); i++) {
			setupColumn(tcm.getColumn(i), i);
		}
	}
    
    private static void setupColumn(TableColumn tableColumn, int column) {
		switch(shownColumns[column]) {
		case COL_NAME:
			tableColumn.setHeaderRenderer(
					new IFHeaderRenderer());
			break;
		}
    }
    
    public Class getColumnClass(int columnIndex) {
		switch (shownColumns[columnIndex]) {
		case COL_NAME:
			return String.class;
		default:
			return String.class;
		}
	}               
    
    public void moveRow(int start, int end, int to) {
		int shift = to - start;
		int first, last;
		if (shift < 0) {
			first = to;
			last = end;
		} else {
			first = start;
			last = to + end - start;
		}
		rotate(data, first, last + 1, shift);

		fireTableRowsUpdated(first, last);
	}    
    
    private static void rotate(Vector v, int a, int b, int shift) {
		int size = b - a;
		int r = size - shift;
		int g = gcd(size, r);
		for (int i = 0; i < g; i++) {
			int to = i;
			Object tmp = v.elementAt(a + to);
			for (int from = (to + r) % size; from != i; from = (to + r) % size) {
				v.setElementAt(v.elementAt(a + from), a + to);
				to = from;
			}
			v.setElementAt(tmp, a + to);
		}
	}
    
    private static int gcd(int i, int j) {
		return (j == 0) ? i : gcd(j, i % j);
	}
        
}

class MsgCol {
	public int column = -1;

	public String name = null;

	public MsgCol(int column) {
		this.column = column;
		this.name = MessageTableModel.columnNames[column];
	}
}
