/*
 * @(#)MsgSearchFrame.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JComboBox;

import olivamail.core.common.ComUtils;

import javax.swing.JProgressBar;

public class MsgSearchFrame extends JFrame implements AddressReceiver {

	static final long serialVersionUID = 9181389998470815541L;
	private final String WIN_ID = "message_search";
	private JPanel jContentPane = null;
	private JPanel jPanel1 = null;
	private JPanel jPanel2 = null;
	private JTextField jtfFrom = null;
	private JTextField jtfTo = null;
	private JLabel jlSubject = null;
	private JTextField jtfSubject = null;
	private JButton jbFrom = null;
	private JButton jbTo = null;
	private JButton jbSearch = null;
	private JButton jbClose = null;
	private MainFrame parent = null;
	private JLabel jlSizeMore = null;
	private JComboBox jcbSizeMore = null;
	private JLabel jlSizeLess = null;
	private JComboBox jcbSizeLess = null;
	private static final String[] sizeArray = {"", "50K", "100K", "200K", "500K", "1M", "2M", "5M", "10M", "20M", "50M"};
	private JLabel jlSize = null;
	private JLabel jlSearchText = null;
	private JTextField jtfSearchText = null;
	private JLabel jlSpacer1 = null;
	private JProgressBar jpbProgress = null;
	private JButton jbHelp = null;

	/**
	 * This is the default constructor
	 */
	public MsgSearchFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(495, 223);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setResizable(false);
		this.setTitle(Strings.getString("MsgSearchFrame.title")); //$NON-NLS-1$
		this.setContentPane(getJContentPane());
	}
	
	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
	
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel2(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jlSpacer1 = new JLabel();
			jlSpacer1.setText("");
			jlSpacer1.setPreferredSize(new java.awt.Dimension(160,16));
			jlSearchText = new JLabel();
			jlSearchText.setText(Strings.getString("MsgSearchFrame.searchInContent"));
			jlSearchText.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlSize = new JLabel();
			jlSize.setText(Strings.getString("MsgSearchFrame.size"));
			jlSize.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlSize.setPreferredSize(new java.awt.Dimension(60,16));
			jlSizeLess = new JLabel();
			jlSizeLess.setText("<");
			jlSizeMore = new JLabel();
			jlSizeMore.setText(">");
			jlSubject = new JLabel();
			jlSubject.setText(Strings.getString("ComposeFrame.subjectFiled")); //$NON-NLS-1$
			jlSubject.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlSubject.setPreferredSize(new java.awt.Dimension(80,16));
			jPanel1 = new JPanel();
			jPanel1.add(getJbFrom(), null);
			jPanel1.add(getJtfFrom(), null);
			jPanel1.add(getJbTo(), null);
			jPanel1.add(getJtfTo(), null);
			jPanel1.add(jlSubject, null);
			jPanel1.add(getJtfSubject(), null);
			jPanel1.add(jlSize, null);
			jPanel1.add(jlSizeMore, null);
			jPanel1.add(getJcbSizeMore(), null);
			jPanel1.add(jlSizeLess, null);
			jPanel1.add(getJcbSizeLess(), null);
			jPanel1.add(jlSpacer1, null);
			jPanel1.add(jlSearchText, null);
			jPanel1.add(getJtfSearchText(), null);
			jPanel1.add(getJpbProgress(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();
			jPanel2.setPreferredSize(new java.awt.Dimension(10,40));
			jPanel2.add(getJbHelp(), null);
			jPanel2.add(getJbSearch(), null);
			jPanel2.add(getJbClose(), null);
		}
		return jPanel2;
	}

	/**
	 * This method initializes jtfFrom	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfFrom() {
		if (jtfFrom == null) {
			jtfFrom = new JTextField();
			jtfFrom.setPreferredSize(new java.awt.Dimension(390,20));
			TextPopup.registerComponent(jtfFrom);
		}
		return jtfFrom;
	}

	/**
	 * This method initializes jtfTo	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfTo() {
		if (jtfTo == null) {
			jtfTo = new JTextField();
			jtfTo.setPreferredSize(new java.awt.Dimension(390,20));
			TextPopup.registerComponent(jtfTo);
		}
		return jtfTo;
	}

	/**
	 * This method initializes jtfSubject	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfSubject() {
		if (jtfSubject == null) {
			jtfSubject = new JTextField();
			jtfSubject.setPreferredSize(new java.awt.Dimension(390,20));
			TextPopup.registerComponent(jtfSubject);
		}
		return jtfSubject;
	}

	/**
	 * This method initializes jbFrom	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbFrom() {
		if (jbFrom == null) {
			jbFrom = new JButton();
			jbFrom.setText(Strings.getString("ComposeFrame.fromFiled")); //$NON-NLS-1$
			jbFrom.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jbFrom_actionPerformed(e);
				}
			});
			jbFrom.setPreferredSize(new java.awt.Dimension(80,20));
		}
		return jbFrom;
	}

	void jbFrom_actionPerformed(java.awt.event.ActionEvent e) {
		AddressesFrame af = new AddressesFrame();
		af.setParent(this, Address.ADDR_FROM);
		af.setVisible(true);
		af.listAddresses();						
	}
	
	/**
	 * This method initializes jbTo	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbTo() {
		if (jbTo == null) {
			jbTo = new JButton();
			jbTo.setText(Strings.getString("ComposeFrame.toFiled")); //$NON-NLS-1$
			jbTo.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jbTo_actionPerformed(e);
				}
			});
			jbTo.setPreferredSize(new java.awt.Dimension(80,20));
		}
		return jbTo;
	}

	void jbTo_actionPerformed(java.awt.event.ActionEvent e) {
		AddressesFrame af = new AddressesFrame();
		af.setParent(this, Address.ADDR_TO);
		af.setVisible(true);
		af.listAddresses();								
	}
	
	/**
	 * This method initializes jbSearch	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbSearch() {
		if (jbSearch == null) {
			jbSearch = new JButton();
			jbSearch.setText(Strings.getString("searchButton")); //$NON-NLS-1$
			jbSearch.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jbSearch_actionPerformed(e);
				}
			});
		}
		return jbSearch;
	}

	void jbSearch_actionPerformed(java.awt.event.ActionEvent e) {
		if(parent == null)
			return;
		StringBuffer sb = new StringBuffer(); 
		String str = jtfFrom.getText().trim().toUpperCase();
		if(!str.equals(""))
			sb.append("UPPER(ADDR_FROM) like '%").append(str).append("%'");
		str = jtfTo.getText().trim().toUpperCase();
		if(!str.equals("")) {
			if(sb.length() > 0)
				sb.append(" or ");
			sb.append("UPPER(ADDR_TO) like '%").append(str).append("%'");
			sb.append(" or ");
			sb.append("UPPER(ADDR_CC) like '%").append(str).append("%'");
		}
		str = jtfSubject.getText().trim().toUpperCase();
		if(!str.equals("")) {
			if(sb.length() > 0)
				sb.append(" or ");
			sb.append("UPPER(SUBJECT) like '%").append(str).append("%'");
		}
		
		str = (String)jcbSizeMore.getSelectedItem();
		int size1 = parseSize(str);
		str = (String)jcbSizeLess.getSelectedItem();
		int size2 = parseSize(str);		
		
		if(size1 != -1 && size2 != -1) {
			if(sb.length() > 0)
				sb.append(" or ");
			sb.append("(MSG_SIZE > ").append(String.valueOf(size1)).append(" and MSG_SIZE < ").append(String.valueOf(size2)).append(")");
		} else if(size1 != -1 || size2 != -1) {
			if(sb.length() > 0)
				sb.append(" or ");
			sb.append("MSG_SIZE ").append((size1 != -1)?">":"<").append(" ").append(String.valueOf((size1 != -1)?size1:size2));
		} 	
		
		if(sb.length() == 0) {
			AlertDialog.showWarningDialog(Strings.getString("MsgSearchFrame.enterSearchCond")); //$NON-NLS-1$			
			return;
		}
		String searchText = jtfSearchText.getText().trim();
		parent.searchMessages(sb.toString(), searchText,
				jpbProgress);
		if(searchText.equals("")) {
			this_windowClosing(null);			
			dispose();
		}
	}
	
	void setParent(MainFrame parent) {
		this.parent = parent;		
	}
	
	/**
	 * This method initializes jbClose	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbClose() {
		if (jbClose == null) {
			jbClose = new JButton();
			jbClose.setText(Strings.getString("closeButton"));
			jbClose.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jbClose;
	}

	public void addAddress(String addr, int addrType) {
		String str = null;
		switch (addrType) {
		case Address.ADDR_FROM:
			str = jtfFrom.getText();
			break;
		case Address.ADDR_TO:
			str = jtfTo.getText();
			break;
/*			
		case Address.ADDR_CC:
			str = jtfCc.getText();
			break;
*/			
		default:
			return;
		}
		
		if(str != null) {
			str = str.trim();
			int len = str.length();			
			if(len > 0) {
				char last = str.charAt(len - 1);
				if(last == ',' || last == ';')
					str += " " + addr;
				else
					str += ", " + addr;
			} else
				str = addr;
		} else
			str = addr;
		
		switch (addrType) {
		case Address.ADDR_FROM:
			jtfFrom.setText(str);
			break;
		case Address.ADDR_TO:
			jtfTo.setText(str);
			break;
/*			
		case Address.ADDR_CC:
			jtfCc.setText(str);
			break;
*/			
		}				
	}

	/**
	 * This method initializes jcbSizeMore	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbSizeMore() {
		if (jcbSizeMore == null) {
			jcbSizeMore = new JComboBox(sizeArray);
			jcbSizeMore.setPreferredSize(new java.awt.Dimension(100,20));
			jcbSizeMore.setEditable(true);
		}
		return jcbSizeMore;
	}

	private int parseSize(String str) {
		str = ComUtils.nvl(str).trim().toLowerCase();
		if(!str.matches("\\d+\\s*[km]?"))
				return -1;
		char last = str.charAt(str.length() - 1);
		int m = 1;
		if(last == 'k') {
			m = 1024;
			str = str.substring(0, str.length() - 1);
		}
		if(last == 'm') {
			m = 1048576;
			str = str.substring(0, str.length() - 1);
		}
		return m * Integer.parseInt(str.trim());
	}
	
	/**
	 * This method initializes jcbLess	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbSizeLess() {
		if (jcbSizeLess == null) {
			jcbSizeLess = new JComboBox(sizeArray);
			jcbSizeLess.setPreferredSize(new java.awt.Dimension(100,20));
			jcbSizeLess.setEditable(true);
		}
		return jcbSizeLess;
	}

	/**
	 * This method initializes jtfSearchText	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfSearchText() {
		if (jtfSearchText == null) {
			jtfSearchText = new JTextField();
			jtfSearchText.setPreferredSize(new java.awt.Dimension(322,20));
			TextPopup.registerComponent(jtfSearchText);
		}
		return jtfSearchText;
	}

	/**
	 * This method initializes jpbProgress	
	 * 	
	 * @return javax.swing.JProgressBar	
	 */    
	private JProgressBar getJpbProgress() {
		if (jpbProgress == null) {
			jpbProgress = new JProgressBar();
			jpbProgress.setPreferredSize(new java.awt.Dimension(476,20));
		}
		return jpbProgress;
	}

	/**
	 * This method initializes jbHelp	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbHelp() {
		if (jbHelp == null) {
			jbHelp = new JButton();
			jbHelp.setText(Strings.getString("helpButton"));
			jbHelp.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					AlertDialog.showInfoDialog(HelpStrings.getString("contentSearch"));
				}
			});
		}
		return jbHelp;
	}
		
}  //  @jve:decl-index=0:visual-constraint="10,10"
