/*
 * @(#)MySQLStorage.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.io.*;
import java.sql.*;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;

import com.mysql.jdbc.Driver;

public class MySQLStorage extends olivamail.gui.Storage {
		
	public MySQLStorage() {
		mConn = null;
		mConnString = "jdbc:mysql://localhost/mail";
		mSchema = "mail.";
	}

	public synchronized int logon(String user, String pass) throws Exception {
		if (true)
			throw new Exception(Strings.getString("classDevOff").replaceFirst(
					"\\$class", this.getClass().getName()));
		mUser = user;
		mPass = pass;
		DriverManager.registerDriver(new Driver());
		mConn = DriverManager.getConnection(mConnString, mUser, mPass);
		mConn.setAutoCommit(false);
		return 0;
	}

	public synchronized void close() throws Exception {
		if (mConn == null)
			return;
		if (!mConn.isClosed()) {
			mConn.rollback();
			mConn.close();
		}
		mConn = null;
	}

	public synchronized javax.mail.internet.MimeMessage loadMessage(String id)
			throws Exception {
		MimeMessage message = null;		
		
		MessageRow mr = getMessageRow(id);
		if(mr.srcPlace.equals("1")) {
			File file = new File(msgsDir + File.separator + id + ".eml");
			FileInputStream inputstream = new FileInputStream(file);
			Session session = Session
					.getDefaultInstance(System.getProperties());
			message = new MimeMessage(session, inputstream);
			inputstream.close();
			return message;			
		}
				
		String sql = "select SOURCE from " + mSchema + "sources where ID = "
				+ id;
		PreparedStatement ps = mConn.prepareStatement(sql);
		ResultSet resultset = ps.executeQuery();
		if (resultset.next()) {
			InputStream inputstream = resultset.getBlob(1).getBinaryStream();
			Session session = Session
					.getDefaultInstance(System.getProperties());
			message = new MimeMessage(session, inputstream);
		}
		resultset.close();
		ps.close();
		return message;
	}

	public synchronized java.io.InputStream loadMessageInputStream(String id)
			throws Exception {
		MessageRow mr = getMessageRow(id);
		if(mr.srcPlace.equals("1")) {
			File file = new File(msgsDir + File.separator + id + ".eml");
			FileInputStream inputstream = new FileInputStream(file);
			return inputstream;			
		}
						
		String sql = "select SOURCE from " + mSchema + "sources where ID = "
				+ id;
		PreparedStatement ps = mConn.prepareStatement(sql);
		ResultSet resultset = ps.executeQuery();
		InputStream inputstream = null;
		if (resultset.next()) {
			inputstream = resultset.getBlob(1).getBinaryStream();
		}
		resultset.close();
		ps.close();
		return inputstream;
	}

	public synchronized void deleteMessage(String id, boolean checkOnlySrc) throws Exception {
		try {
			String sql = null;
			PreparedStatement ps = null;
			MessageRow mr = getMessageRow(id);
			if (mr.srcPlace.equals("0")) {
				sql = "delete from " + mSchema + "messages where ID = ?";
				ps = mConn.prepareStatement(sql);
				ps.setString(1, id);
				ps.execute();
				ps.close();
			}
			sql = "delete from " + mSchema + "sources where ID = ?";
			ps = mConn.prepareStatement(sql);
			ps.setString(1, id);
			ps.execute();
			ps.close();
			if(mr.srcPlace.equals("1")) {
				File file = new File(msgsDir + File.separator + id + ".eml");
				if(file.exists())
					file.delete();
			}						
			mConn.commit();
		} catch (Exception exc) {
			try {
				mConn.rollback();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}		 		
	}
			
	protected String getLastInsertedId() throws Exception {
		String id = "";
		PreparedStatement ps = mConn
				.prepareStatement("SELECT LAST_INSERT_ID()");
		ResultSet resultset = ps.executeQuery();
		if (resultset.next()) {
			id = resultset.getString(1);
		}
		ps.close();
		resultset.close();
		return id;
	}

	public synchronized int copyMimeTypes(String fromMimeGroupId, String toMimeGroupId) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select MIME_TYPE, PROGRAM from " + mSchema
						+ "mime_types where MGRP_ID = ?");
		ps.setString(1, fromMimeGroupId);
		ResultSet resultset = ps.executeQuery();		
		while (resultset.next()) {
			PreparedStatement ps1 = mConn
					.prepareStatement("insert into "
							+ mSchema
							+ "mime_types (MIME_TYPE, PROGRAM, MGRP_ID) values (?, ?, ?)");
			ps1.setString(1, resultset.getString(1));
			ps1.setString(2, resultset.getString(2));
			ps1.setString(3, toMimeGroupId);
			ps1.execute();
			ps1.close();
		}
		resultset.close();
		ps.close();
		ps = mConn.prepareStatement("select count(1) from "
				+ mSchema + "mime_types where MGRP_ID = ?");
		ps.setString(1, fromMimeGroupId);
		resultset = ps.executeQuery();
		int count = 0;
		if (resultset.next()) {
			count = resultset.getInt(1);
		}
		resultset.close();
		ps.close();

		mConn.commit();

		return count;
	}

	public synchronized void populateSchema(int schemaVer) throws Exception {
		String sqls[] = new String[] {
				"CREATE TABLE accounts (ID BIGINT(10) NOT NULL AUTO_INCREMENT, EMAIL VARCHAR(60) NOT NULL DEFAULT '', SERVER_TYPE VARCHAR(10) NOT NULL DEFAULT '', INCOMING_SERVER VARCHAR(60) NOT NULL DEFAULT '', OUTGOING_SERVER VARCHAR(60) NOT NULL DEFAULT '', SMTP_AUTH CHAR(1) NOT NULL DEFAULT '0', IN_USER VARCHAR(30), IN_PASS VARCHAR(30), OUT_USER VARCHAR(30), OUT_PASS VARCHAR(30), STATUS CHAR(1) NOT NULL DEFAULT '0', NAME VARCHAR(60) NOT NULL DEFAULT '', IN_PORT INT(5) NOT NULL DEFAULT '-1', OUT_PORT INT(5) NOT NULL DEFAULT '-1', IN_MAX_SIZE INT(10) NOT NULL DEFAULT '-1', IN_LEAVE_DAYS INT(3) NOT NULL DEFAULT '0', PRIMARY KEY (ID))",
				"CREATE TABLE addresses (ID BIGINT(10) NOT NULL AUTO_INCREMENT, FNAME VARCHAR(60), LNAME VARCHAR(60), MNAME VARCHAR(60), EMAIL VARCHAR(60), HOME_COUNTRY VARCHAR(60), HOME_REGION VARCHAR(60), HOME_CITY VARCHAR(60), HOME_INDEX VARCHAR(10), HOME_STREET VARCHAR(60), HOME_PHONE VARCHAR(15), MOBILE_PHONE VARCHAR(15), PRIMARY KEY (ID))",
				"CREATE TABLE folders (ID BIGINT(10) NOT NULL AUTO_INCREMENT, NAME VARCHAR(30) NOT NULL DEFAULT '', PRIMARY KEY (ID))",
				"INSERT INTO folders (ID, NAME) VALUES (1,'__INBOX')",
				"INSERT INTO folders (ID, NAME) VALUES (2,'__SENT')",
				"INSERT INTO folders (ID, NAME) VALUES (3,'__DRAFT')",
				"INSERT INTO folders (ID, NAME) VALUES (4,'__TRASH')",
				"INSERT INTO folders (ID, NAME) VALUES (5,'__OUTBOX')",
				"CREATE TABLE messages (ID BIGINT(10) NOT NULL AUTO_INCREMENT, ADDR_FROM VARCHAR(255), ADDR_TO VARCHAR(255), ADDR_CC VARCHAR(255), ADDR_BCC VARCHAR(255), SUBJECT VARCHAR(255), SENT_DATE VARCHAR(19), MSG_SIZE BIGINT(10), MESSAGE_ID VARCHAR(255), MSG_UID VARCHAR(255) NOT NULL DEFAULT '', HAS_ATT CHAR(1) NOT NULL DEFAULT '0', FOLDER_ID BIGINT(10) NOT NULL DEFAULT '0', SEEN CHAR(1) NOT NULL DEFAULT '0', CODEPAGE VARCHAR(30) NOT NULL DEFAULT '', PRIMARY KEY (ID))",
				"ALTER TABLE messages ADD CONSTRAINT msg_fld_fk FOREIGN KEY (FOLDER_ID) REFERENCES folders (ID)",
				"CREATE INDEX msg_fld_fk_i on messages (FOLDER_ID)",
				"CREATE TABLE mime_groups (ID BIGINT(10) NOT NULL AUTO_INCREMENT, NAME VARCHAR(60) NOT NULL DEFAULT '', STATUS CHAR(1) NOT NULL DEFAULT '0', PRIMARY KEY (ID))",
				"INSERT INTO mime_groups VALUES (1,'Windows XP (DEFAULT)','D')",
				"INSERT INTO mime_groups VALUES (2,'Linux','0')",
				"CREATE TABLE mime_types (ID BIGINT(10) NOT NULL AUTO_INCREMENT, MIME_TYPE VARCHAR(40), PROGRAM VARCHAR(255), MGRP_ID BIGINT(10) NOT NULL DEFAULT '0', PRIMARY KEY (ID))",
				"ALTER TABLE mime_types ADD CONSTRAINT mtypes_mgrp_fk FOREIGN KEY (MGRP_ID) REFERENCES mime_groups (ID)",
				"CREATE INDEX mtypes_mgrp_fk_i ON mime_types (MGRP_ID)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (1,'image/','C:\\\\WINDOWS\\\\system32\\\\rundll32.exe C:\\\\WINDOWS\\\\system32\\\\shimgvw.dll,ImageView_Fullscreen %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (2,'application/msword','C:\\\\Program Files\\\\Microsoft Office\\\\Office10\\\\WINWORD.EXE %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (3,'application/vnd.ms-excel','C:\\\\Program Files\\\\Microsoft Office\\\\Office10\\\\EXCEL.EXE %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (4,'','C:\\\\Program Files\\\\Internet Explorer\\\\iexplore.exe %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (5,'image/','/usr/bin/mozilla %1',2)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (6,'','/usr/bin/mozilla %1',2)",				
				"CREATE TABLE settings (ID BIGINT(10) NOT NULL AUTO_INCREMENT, NAME VARCHAR(40) NOT NULL DEFAULT '', VALUE VARCHAR(40), PRIMARY KEY (ID))",
				"CREATE TABLE sources (ID BIGINT(10) NOT NULL DEFAULT '0', SOURCE longblob, PRIMARY KEY (ID))"
		};
		String sqls104[] = new String[] {
				"CREATE TABLE groups (ID BIGINT(10) NOT NULL AUTO_INCREMENT, NAME VARCHAR(30) NOT NULL, COMMENT VARCHAR(255), PRIMARY KEY (ID))",
				"CREATE TABLE grp_addr (GRP_ID BIGINT(10) NOT NULL, ADDR_ID BIGINT(10) NOT NULL)",
				"ALTER TABLE grp_addr ADD CONSTRAINT grp_addr_grp_fk FOREIGN KEY (GRP_ID) REFERENCES groups (ID)",
				"ALTER TABLE grp_addr ADD CONSTRAINT grp_addr_addr_fk FOREIGN KEY (ADDR_ID) REFERENCES addresses (ID)",
				"ALTER TABLE folders ADD FLD_ID BIGINT(10)",
				"ALTER TABLE folders ADD CONSTRAINT fld_fld_fk FOREIGN KEY (FLD_ID) REFERENCES folders (ID)",
				"ALTER TABLE accounts ADD HWM_MSG_UID VARCHAR(255)",
				"CREATE TABLE rules (ID BIGINT(10) NOT NULL AUTO_INCREMENT, HEADER VARCHAR(255) NOT NULL, OPERAND TINYINT(2) NOT NULL, VALUE VARCHAR(255) NOT NULL, ACTION TINYINT(2) NOT NULL, PARAM VARCHAR(255), ACNT_ID BIGINT(10) NOT NULL, PRIMARY KEY (ID))",
				"ALTER TABLE rules ADD CONSTRAINT rule_acnt_fk FOREIGN KEY (ACNT_ID) REFERENCES accounts (ID)",
				"ALTER TABLE accounts ADD USE_RULES CHAR(1) NOT NULL DEFAULT '1'"				
		};
		String sqls110[] = new String[] {				
				"ALTER TABLE messages MODIFY CODEPAGE VARCHAR(30) DEFAULT NULL NULL",
				"UPDATE messages SET CODEPAGE = NULL where CODEPAGE = 'koi8-r'",
				"ALTER TABLE messages ADD SRV_HASH BIGINT(10) NOT NULL DEFAULT 0",
				"ALTER TABLE accounts ADD HWM_MSG_ID BIGINT(10) NULL",				
				"ALTER TABLE groups CHANGE COMMENT COMMENTS VARCHAR(255)"				
		};
		String sqls111[] = new String[] {				
				"ALTER TABLE accounts ADD USE_SIG CHAR(1) NOT NULL DEFAULT '0'",
				"CREATE TABLE signatures (ACNT_ID BIGINT(10) NOT NULL, SIG_TYPE CHAR(1) NOT NULL, SIG_LOCAL VARCHAR(1024), SIG_INT VARCHAR(1024), STATUS CHAR(1) NOT NULL DEFAULT '0', PRIMARY KEY (ACNT_ID, SIG_TYPE))",
				"ALTER TABLE signatures ADD CONSTRAINT sig_acnt_fk FOREIGN KEY (ACNT_ID) REFERENCES accounts (ID)"
		};
		String sqls112[] = new String[] {				
				"ALTER TABLE accounts ADD IN_SSL CHAR(1) NOT NULL DEFAULT '0'",
				"ALTER TABLE accounts ADD OUT_SSL CHAR(1) NOT NULL DEFAULT '0'",
				"ALTER TABLE messages ADD IMPORTANCE CHAR(1) NOT NULL DEFAULT '0'"
		};		
		String sqls113_1[] = new String[] {				
				"ALTER TABLE messages ADD SRC_PLACE CHAR(1) NOT NULL DEFAULT '0'"
		};
		String sqls113_2[] = new String[] {				
				"ALTER TABLE sources ADD CONSTRAINT src_msg_fk FOREIGN KEY (ID) REFERENCES messages (ID)"
		};
		String sqls120[] = new String[] {				
				"ALTER TABLE accounts ADD PERSON VARCHAR(72) DEFAULT ''",
				"ALTER TABLE accounts ADD REPLY_TO VARCHAR(60) DEFAULT ''",
				"ALTER TABLE accounts ADD ORGANIZATION VARCHAR(128) DEFAULT ''",
				"CREATE INDEX msg_fld_seen_i ON messages (FOLDER_ID, SEEN)",
				"ALTER TABLE messages ADD RECEIVED_DATE VARCHAR(19)"
		};				
		String sqls121[] = new String[] {						
				"ALTER TABLE messages ADD ACNT_ID BIGINT(10)"
		};		
		if(schemaVer == getReleaseVersion())
			return;						
		if (schemaVer == SCHEMA_EMPTY) {
			try {
				execSqlArray(sqls);
				mConn.commit();
				schemaVer = SCHEMA_1_0_3;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_0_3) {
			try {
				execSqlArray(sqls104);
				System.out.println("Creating local folder, if needed...");
				String localFldId = getLocalFolder();
				System.out.println("OK!");
				System.out.println("Updating other folders, if needed...");
				PreparedStatement ps = mConn
						.prepareStatement("UPDATE folders SET FLD_ID = ? WHERE ID <> ? AND FLD_ID IS NULL");
				ps.setString(1, localFldId);
				ps.setString(2, localFldId);
				ps.execute();
				ps.close();
				mConn.commit();
				System.out.println("OK!");
				System.out.println("Setting schema version to 1.0.4...");
				setSettingsValue("schema_version", "1.0.4");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_0_4;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_0_4) {
			try {
				execSqlArray(sqls110);
				mConn.commit();
				System.out.println("Setting schema version to 1.1.0 ...");
				setSettingsValue("schema_version", "1.1.0");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_0;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_1_0) {
			try {
				execSqlArray(sqls111);
				mConn.commit();
				System.out.println("Setting schema version to 1.1.1 ...");
				setSettingsValue("schema_version", "1.1.1");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_1;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_1_1) {
			try {
				execSqlArray(sqls112);
				mConn.commit();
				System.out.println("Setting schema version to 1.1.2 ...");
				setSettingsValue("schema_version", "1.1.2");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_2;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}	
		if (schemaVer == SCHEMA_1_1_2) {
			try {
				execSqlArray(sqls113_1);
				mConn.commit();
				System.out.println("Deleting missing message sources ...");
				this.deleteMissingSources();
				System.out.println("OK!");
				execSqlArray(sqls113_2);
				mConn.commit();				
				System.out.println("Setting schema version to 1.1.3 ...");
				setSettingsValue("schema_version", "1.1.3");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_3;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}				
		if (schemaVer == SCHEMA_1_1_3) {
			try {
				execSqlArray(sqls120);
				mConn.commit();
				convertSentDate();
				System.out.println("Setting schema version to 1.2.0 ...");
				setSettingsValue("schema_version", "1.2.0");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_2_0;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_2_0) {
			try {
				execSqlArray(sqls121);
				mConn.commit();
				convertSentDate();
				System.out.println("Setting schema version to 1.2.1 ...");
				setSettingsValue("schema_version", "1.2.1");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_2_1;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}		
	}

	protected boolean isIdAutoincremented() {
		return true;
	}

	protected String getId() throws Exception {
		return null;
	}
				
}