/*
 * @(#)OracleStorage.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.io.*;
import java.sql.*;

import javax.mail.Session;
import javax.mail.internet.MimeMessage;

import olivamail.core.common.ComUtils;
import olivamail.core.mail.MailUtils;
import oracle.jdbc.driver.OracleDriver;
import oracle.sql.BLOB;

public class OracleStorage extends olivamail.gui.Storage {
		
	public OracleStorage() {
		mConn = null;
		mConnString = "jdbc:oracle:thin:@localhost:1521:SID";
		mSchema = "mail.";
	}

	public synchronized int logon(String user, String pass) throws Exception {
		if (true)
			throw new Exception(Strings.getString("classDevOff").replaceFirst(
					"\\$class", this.getClass().getName()));
		mUser = user;
		mPass = pass;
		DriverManager.registerDriver(new OracleDriver());
		mConn = DriverManager.getConnection(mConnString, mUser, mPass);
		mConn.setAutoCommit(false);
		return 0;
	}

	public synchronized void close() throws Exception {
		if (mConn == null)
			return;
		if (!mConn.isClosed()) {
			mConn.rollback();
			mConn.close();
		}
		mConn = null;
	}

	public synchronized javax.mail.internet.MimeMessage loadMessage(String id)
			throws Exception {
		MimeMessage message = null;		
		
		MessageRow mr = getMessageRow(id);
		if(mr.srcPlace.equals("1")) {
			File file = new File(msgsDir + File.separator + id + ".eml");
			FileInputStream inputstream = new FileInputStream(file);
			Session session = Session
					.getDefaultInstance(System.getProperties());
			message = new MimeMessage(session, inputstream);
			inputstream.close();
			return message;			
		}
						
		String sql = "select source from " + mSchema + "messages where id = "
				+ id;
		PreparedStatement ps = mConn.prepareStatement(sql);
		ResultSet resultset = ps.executeQuery();
		if (resultset.next()) {
			InputStream inputstream = resultset.getBlob(1).getBinaryStream();
			Session session = Session
					.getDefaultInstance(System.getProperties());
			message = new MimeMessage(session, inputstream);
		}
		resultset.close();
		ps.close();
		return message;
	}

	public synchronized java.io.InputStream loadMessageInputStream(String id)
			throws Exception {
		MessageRow mr = getMessageRow(id);
		if(mr.srcPlace.equals("1")) {
			File file = new File(msgsDir + File.separator + id + ".eml");
			FileInputStream inputstream = new FileInputStream(file);
			return inputstream;			
		}
						
		String sql = "select source from " + mSchema + "messages where id = "
				+ id;
		PreparedStatement ps = mConn.prepareStatement(sql);
		ResultSet resultset = ps.executeQuery();
		InputStream inputstream = null;
		if (resultset.next()) {
			inputstream = resultset.getBlob(1).getBinaryStream();
		}
		resultset.close();
		ps.close();
		return inputstream;
	}

	public synchronized void deleteMessage(String id, boolean checkOnlySrc) throws Exception {
		MessageRow mr = getMessageRow(id);
		String sql = "delete from " + mSchema + "messages where id = ?";
		PreparedStatement ps = mConn.prepareStatement(sql);
		ps.setString(1, id);
		ps.execute();
		ps.close();
		if(mr.srcPlace.equals("1")) {
			File file = new File(msgsDir + File.separator + id + ".eml");
			if(file.exists())
				file.delete();
		}								
		mConn.commit();
	}

	protected String getId() throws Exception {
		PreparedStatement ps = mConn.prepareStatement("select "
				+ mSchema + "mail_seq.nextval from dual");
		ResultSet resultset = ps.executeQuery();
		String id = null;
		if (resultset.next())
			id = resultset.getString(1);
		ps.close();
		resultset.close();
		return id;
	}
		
	public synchronized String saveMessage(olivamail.core.mail.MessageHeader hdr,
			String folderId, javax.mail.Message message, String serverName, String codepage, String acntId)
			throws Exception {
		File file = null;
		//hdr = substrMsgHeader(hdr);
		try {
			String id = getId();
			
			String srcPlace = "0";
			if(dbMessageMaxSize != -1 && hdr.msgSize > dbMessageMaxSize)
				srcPlace = "1";

			PreparedStatement ps = mConn
					.prepareStatement("insert into "
							+ mSchema
							+ "messages (ID, ADDR_FROM, ADDR_TO, ADDR_CC, ADDR_BCC, SUBJECT, SENT_DATE, RECEIVED_DATE, MSG_SIZE, MESSAGE_ID, MSG_UID, HAS_ATT, SOURCE, FOLDER_ID, SEEN, CODEPAGE, SRV_HASH, IMPORTANCE, SRC_PLACE, ACNT_ID) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, EMPTY_BLOB(), ?, ?, ?, ?, ?, ?, ?)");
			ps.setString(1, id);
			ps.setString(2, hdr.From);
			ps.setString(3, hdr.To);
			ps.setString(4, hdr.Cc);
			ps.setString(5, hdr.Bcc);
			ps.setString(6, hdr.Subject);
			ps.setString(7, olivamail.core.common.ComUtils.getDateString(
					hdr.sentDate, "en", MessageRow.formatYMD));
			ps.setString(8, olivamail.core.common.ComUtils.getDateString(
					hdr.receivedDate, "en", MessageRow.formatYMD));			
			ps.setInt(9, hdr.msgSize);
			ps.setString(10, hdr.MessageID);
			ps.setString(11, hdr.UID);
			if (hdr.hasAtt)
				ps.setString(12, "1");
			else
				ps.setString(12, "0");
			ps.setString(13, folderId);
			if(MailUtils.getContentType(message).equals("message/partial"))
				ps.setString(14, "4");
			else
				ps.setString(14, "0");
			ps.setString(15, codepage);
			ps.setInt(16, ComUtils.nvl(serverName).toLowerCase().hashCode());
			ps.setString(17, (hdr.isHighlyImportant())?"1":"0");
			ps.setString(18, srcPlace);
			ps.setString(19, acntId);
			ps.execute();
			ps.close();

			if(srcPlace.equals("1")) {
				file = new File(msgsDir + File.separator + id + ".eml");
				OutputStream os = Context.getFileOutputStream(file);
				message.writeTo(os);
				Context.closeOutputStream(os);
			} else {
				String sql = "select source from " + mSchema
						+ "messages where id = " + id + " for update";
				Statement statement = mConn.createStatement();
				ResultSet resultset = statement.executeQuery(sql);
				if (resultset.next()) {
					OutputStream outputstream = ((BLOB) resultset.getBlob(1))
							.getBinaryOutputStream();
					// mailBean.getMessageSource(msgNo, msgUID, outputstream);
					message.writeTo(outputstream);
					outputstream.flush();
					outputstream.close();
				}
				statement.close();
				resultset.close();
			}
			
			mConn.commit();
			return id;
		} catch (Exception exc) {
			try {
				mConn.rollback();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}				
	}

	public synchronized String saveMessageHeader(olivamail.core.mail.MessageHeader hdr,
			String folderId, String serverName) throws Exception {
		//hdr = substrMsgHeader(hdr);
		String id = getId();

		PreparedStatement ps = mConn
				.prepareStatement("insert into "
						+ mSchema
						+ "messages (ID, ADDR_FROM, ADDR_TO, ADDR_CC, ADDR_BCC, SUBJECT, SENT_DATE, RECEIVED_DATE, MSG_SIZE, MESSAGE_ID, MSG_UID, HAS_ATT, SOURCE, FOLDER_ID, SEEN, CODEPAGE, SRV_HASH, IMPORTANCE) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, EMPTY_BLOB(), ?, ?, ?, ?, ?)");
		ps.setString(1, id);
		ps.setString(2, hdr.From);
		ps.setString(3, hdr.To);
		ps.setString(4, hdr.Cc);
		ps.setString(5, hdr.Bcc);
		ps.setString(6, hdr.Subject);
		//      ps.setDate(7, new java.sql.Date(sentDate.getTime()));
		ps.setString(7, olivamail.core.common.ComUtils.getDateString(
				hdr.sentDate, "en", MessageRow.formatYMD));
		ps.setString(8, olivamail.core.common.ComUtils.getDateString(
				hdr.receivedDate, "en", MessageRow.formatYMD));		
		ps.setInt(9, hdr.msgSize);
		ps.setString(10, hdr.MessageID);
		ps.setString(11, hdr.UID);
		if (hdr.hasAtt)
			ps.setString(12, "1");
		else
			ps.setString(12, "0");
		ps.setString(13, folderId);
		ps.setString(14, "2");
		ps.setString(15, null);
		ps.setInt(16, ComUtils.nvl(serverName).toLowerCase().hashCode());
		ps.setString(17, (hdr.isHighlyImportant())?"1":"0");
		ps.execute();
		ps.close();
		mConn.commit();
		return id;
	}
					
	public synchronized int copyMimeTypes(String fromMimeGroupId, String toMimeGroupId) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("insert into " + mSchema + "mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) select " + mSchema + "mail_seq.nextval, MIME_TYPE, PROGRAM, ? from " + mSchema
						+ "mime_types where MGRP_ID = ?");
		ps.setString(1, toMimeGroupId);
		ps.setString(2, fromMimeGroupId);
		ps.execute();
		ps = mConn
		.prepareStatement("select count(1) from " + mSchema
				+ "mime_types where MGRP_ID = ?");
		ps.setString(1, fromMimeGroupId);		
		ResultSet resultset = ps.executeQuery();
		int count = 0;
		if (resultset.next()) {
			count = resultset.getInt(1);
		}
		resultset.close();
		ps.close();
		
		mConn.commit();

		return count;
	}	

	public synchronized void populateSchema(int schemaVer) throws Exception {
		String sqls[] = new String[] {
				"CREATE TABLE accounts (ID NUMBER(10) NOT NULL, EMAIL VARCHAR2(60) NOT NULL, SERVER_TYPE VARCHAR2(10) NOT NULL, INCOMING_SERVER VARCHAR2(60) NOT NULL, OUTGOING_SERVER VARCHAR2(60) NOT NULL, SMTP_AUTH CHAR(1) DEFAULT '0' NOT NULL, IN_USER VARCHAR2(30), IN_PASS VARCHAR2(30), OUT_USER VARCHAR2(30), OUT_PASS VARCHAR2(30), STATUS CHAR(1) DEFAULT '0' NOT NULL, NAME VARCHAR2(60) NOT NULL, IN_PORT NUMBER(5) DEFAULT -1 NOT NULL, OUT_PORT NUMBER(5) DEFAULT -1 NOT NULL, IN_MAX_SIZE NUMBER(10) DEFAULT -1 NOT NULL, IN_LEAVE_DAYS NUMBER(3) DEFAULT '0' NOT NULL, PRIMARY KEY (ID))",
				"CREATE TABLE addresses (ID NUMBER(10) NOT NULL, FNAME VARCHAR2(60), LNAME VARCHAR2(60), MNAME VARCHAR2(60), EMAIL VARCHAR2(60), HOME_COUNTRY VARCHAR2(60), HOME_REGION VARCHAR2(60), HOME_CITY VARCHAR2(60), HOME_INDEX VARCHAR2(10), HOME_STREET VARCHAR2(60), HOME_PHONE VARCHAR2(15), MOBILE_PHONE VARCHAR2(15), PRIMARY KEY (ID))",
				"CREATE TABLE folders (ID NUMBER(10) NOT NULL, NAME VARCHAR2(30) NOT NULL, PRIMARY KEY (ID))",
				"INSERT INTO folders (ID, NAME) VALUES (1,'__INBOX')",
				"INSERT INTO folders (ID, NAME) VALUES (2,'__SENT')",
				"INSERT INTO folders (ID, NAME) VALUES (3,'__DRAFT')",
				"INSERT INTO folders (ID, NAME) VALUES (4,'__TRASH')",
				"INSERT INTO folders (ID, NAME) VALUES (5,'__OUTBOX')",
				"CREATE TABLE messages (ID NUMBER(10) NOT NULL, ADDR_FROM VARCHAR2(255), ADDR_TO VARCHAR2(255), ADDR_CC VARCHAR2(255), ADDR_BCC VARCHAR2(255), SUBJECT VARCHAR2(255), SENT_DATE VARCHAR2(19), MSG_SIZE NUMBER(10), MESSAGE_ID VARCHAR2(255), MSG_UID VARCHAR2(255) NOT NULL, HAS_ATT CHAR(1) DEFAULT '0' NOT NULL, FOLDER_ID NUMBER(10) NOT NULL, SEEN CHAR(1) DEFAULT '0' NOT NULL, CODEPAGE VARCHAR2(30) NOT NULL, SOURCE BLOB, PRIMARY KEY (ID))",
				"ALTER TABLE messages ADD CONSTRAINT msg_fld_fk FOREIGN KEY (FOLDER_ID) REFERENCES folders (ID)",
				"CREATE INDEX msg_fld_fk_i on messages (FOLDER_ID)",
				"CREATE TABLE mime_groups (ID NUMBER(10) NOT NULL, NAME VARCHAR2(60) NOT NULL, STATUS CHAR(1) DEFAULT '0' NOT NULL, PRIMARY KEY (ID))",
				"INSERT INTO mime_groups VALUES (1,'Windows XP (DEFAULT)','D')",
				"INSERT INTO mime_groups VALUES (2,'Linux','0')",				
				"CREATE TABLE mime_types (ID NUMBER(10) NOT NULL, MIME_TYPE VARCHAR2(40), PROGRAM VARCHAR2(255), MGRP_ID NUMBER(10) NOT NULL, PRIMARY KEY (ID))",
				"ALTER TABLE mime_types ADD CONSTRAINT mtypes_mgrp_fk FOREIGN KEY (MGRP_ID) REFERENCES mime_groups (ID)",
				"CREATE INDEX mtypes_mgrp_fk_i ON mime_types (MGRP_ID)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (1,'image/','C:\\\\WINDOWS\\\\system32\\\\rundll32.exe C:\\\\WINDOWS\\\\system32\\\\shimgvw.dll,ImageView_Fullscreen %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (2,'application/msword','C:\\\\Program Files\\\\Microsoft Office\\\\Office10\\\\WINWORD.EXE %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (3,'application/vnd.ms-excel','C:\\\\Program Files\\\\Microsoft Office\\\\Office10\\\\EXCEL.EXE %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (4,'','C:\\\\Program Files\\\\Internet Explorer\\\\iexplore.exe %1',1)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (5,'image/','/usr/bin/mozilla %1',2)",
				"INSERT INTO mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) VALUES (6,'','/usr/bin/mozilla %1',2)",				
				"CREATE TABLE settings (ID NUMBER(10) NOT NULL, NAME VARCHAR2(40) NOT NULL, VALUE VARCHAR2(40), PRIMARY KEY (ID))",
				"create sequence mail_seq nocache nominvalue nomaxvalue nocycle START WITH 100"
		};
		String sqls104[] = new String[] {
				"CREATE TABLE groups (ID NUMBER(10) NOT NULL, NAME VARCHAR2(30) NOT NULL, COMMENTS VARCHAR2(255), PRIMARY KEY (ID))",
				"CREATE TABLE grp_addr (GRP_ID NUMBER(10) NOT NULL, ADDR_ID NUMBER(10) NOT NULL)",
				"ALTER TABLE grp_addr ADD CONSTRAINT grp_addr_grp_fk FOREIGN KEY (GRP_ID) REFERENCES groups (ID)",
				"ALTER TABLE grp_addr ADD CONSTRAINT grp_addr_addr_fk FOREIGN KEY (ADDR_ID) REFERENCES addresses (ID)",
				"ALTER TABLE folders ADD FLD_ID NUMBER(10)",
				"ALTER TABLE folders ADD CONSTRAINT fld_fld_fk FOREIGN KEY (FLD_ID) REFERENCES folders (ID)",
				"ALTER TABLE accounts ADD HWM_MSG_UID VARCHAR2(255)",
				"CREATE TABLE rules (ID NUMBER(10) NOT NULL, HEADER VARCHAR2(255) NOT NULL, OPERAND NUMBER(2) NOT NULL, VALUE VARCHAR2(255) NOT NULL, ACTION NUMBER(2) NOT NULL, PARAM VARCHAR2(255), ACNT_ID NUMBER(10) NOT NULL, PRIMARY KEY (ID))",
				"ALTER TABLE rules ADD CONSTRAINT rule_acnt_fk FOREIGN KEY (ACNT_ID) REFERENCES accounts (ID)",
				"ALTER TABLE accounts ADD USE_RULES CHAR(1) DEFAULT '1' NOT NULL"				
		};
		String sqls110[] = new String[] {
				"ALTER TABLE messages MODIFY CODEPAGE VARCHAR2(30) DEFAULT NULL NULL",
				"UPDATE messages SET CODEPAGE = NULL where CODEPAGE = 'koi8-r'",
				"ALTER TABLE messages ADD SRV_HASH NUMBER(10) DEFAULT 0 NOT NULL",
				"ALTER TABLE accounts ADD HWM_MSG_ID NUMBER(10) NULL"								
		};		
		String sqls111[] = new String[] {
				"ALTER TABLE accounts ADD USE_SIG CHAR(1) DEFAULT '0' NOT NULL",
				"CREATE TABLE signatures (ACNT_ID NUMBER(10) NOT NULL, SIG_TYPE CHAR(1) NOT NULL, SIG_LOCAL VARCHAR2(1024), SIG_INT VARCHAR2(1024), STATUS CHAR(1) DEFAULT '0' NOT NULL, PRIMARY KEY (ACNT_ID, SIG_TYPE))",
				"ALTER TABLE signatures ADD CONSTRAINT sig_acnt_fk FOREIGN KEY (ACNT_ID) REFERENCES accounts (ID)"
		};
		String sqls112[] = new String[] {
				"ALTER TABLE accounts ADD IN_SSL CHAR(1) DEFAULT '0' NOT NULL",
				"ALTER TABLE accounts ADD OUT_SSL CHAR(1) DEFAULT '0' NOT NULL",
				"ALTER TABLE messages ADD IMPORTANCE CHAR(1) DEFAULT '0' NOT NULL"
		};		
		String sqls113[] = new String[] {				
				"ALTER TABLE messages ADD SRC_PLACE CHAR(1) DEFAULT '0' NOT NULL",
				"ALTER TABLE sources ADD CONSTRAINT src_msg_fk FOREIGN KEY (ID) REFERENCES messages (ID)"
		};
		String sqls120[] = new String[] {				
				"ALTER TABLE accounts ADD PERSON VARCHAR2(72) DEFAULT ''",
				"ALTER TABLE accounts ADD REPLY_TO VARCHAR2(60) DEFAULT ''",
				"ALTER TABLE accounts ADD ORGANIZATION VARCHAR2(128) DEFAULT ''",
				"CREATE INDEX msg_fld_seen_i ON messages (FOLDER_ID, SEEN)",
				"ALTER TABLE messages ADD RECEIVED_DATE VARCHAR2(19)"
		};
		String sqls121[] = new String[] {						
				"ALTER TABLE messages ADD ACNT_ID NUMBER(10)"
		};		
		if(schemaVer == getReleaseVersion())
			return;						
		if (schemaVer == SCHEMA_EMPTY) {
			try {
				execSqlArray(sqls);
				mConn.commit();
				schemaVer = SCHEMA_1_0_3;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_0_3) {
			try {
				execSqlArray(sqls104);
				System.out.println("Creating local folder, if needed...");
				String localFldId = getLocalFolder();
				System.out.println("OK!");
				System.out.println("Updating other folders, if needed...");
				PreparedStatement ps = mConn
						.prepareStatement("UPDATE folders SET FLD_ID = ? WHERE ID <> ? AND FLD_ID IS NULL");
				ps.setString(1, localFldId);
				ps.setString(2, localFldId);
				ps.execute();
				ps.close();
				mConn.commit();
				System.out.println("OK!");
				System.out.println("Setting schema version to 1.0.4...");
				setSettingsValue("schema_version", "1.0.4");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_0_4;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_0_4) {
			try {
				execSqlArray(sqls110);
				mConn.commit();
				System.out.println("Setting schema version to 1.1.0 ...");
				setSettingsValue("schema_version", "1.1.0");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_0;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_1_0) {
			try {
				execSqlArray(sqls111);
				mConn.commit();
				System.out.println("Setting schema version to 1.1.1 ...");
				setSettingsValue("schema_version", "1.1.1");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_1;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}	
		if (schemaVer == SCHEMA_1_1_1) {
			try {
				execSqlArray(sqls112);
				mConn.commit();
				System.out.println("Setting schema version to 1.1.2 ...");
				setSettingsValue("schema_version", "1.1.2");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_2;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}	
		if (schemaVer == SCHEMA_1_1_2) {
			try {
				execSqlArray(sqls113);
				mConn.commit();
				System.out.println("Setting schema version to 1.1.3 ...");
				setSettingsValue("schema_version", "1.1.3");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_1_3;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}	
		if (schemaVer == SCHEMA_1_1_3) {
			try {
				execSqlArray(sqls120);
				mConn.commit();
				convertSentDate();
				System.out.println("Setting schema version to 1.2.0 ...");
				setSettingsValue("schema_version", "1.2.0");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_2_0;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		if (schemaVer == SCHEMA_1_2_0) {
			try {
				execSqlArray(sqls121);
				mConn.commit();
				convertSentDate();
				simplifySignatures();
				System.out.println("Setting schema version to 1.2.1 ...");
				setSettingsValue("schema_version", "1.2.1");
				System.out.println("OK!");
				schemaVer = SCHEMA_1_2_1;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}				
	}

	public synchronized String getLastInsertedId() throws Exception {
		String id = "";
		PreparedStatement ps = mConn
				.prepareStatement("select " + mSchema + "mail_seq.currval from dual");				
		ResultSet resultset = ps.executeQuery();
		if (resultset.next()) {
			id = resultset.getString(1);
		}
		ps.close();
		resultset.close();
		return id;
	}

	protected boolean isIdAutoincremented() {
		return false;
	}
		
}