/*
 * @(#)ProgressFrame.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JProgressBar;
import javax.swing.JButton;
import javax.swing.Timer;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import javax.swing.JLabel;

public class ProgressFrame extends JFrame {

	static final long serialVersionUID = 985089246530720170L;
	private final String WIN_ID = "progress";
	private JPanel jContentPane = null;
	private JProgressBar jProgressBar1 = null;
	private JButton jButtonCancel = null;
	private boolean isCanceled = false;
	private Timer timer = null;	
	private int timerSeconds = 0;
	private JLabel jlMsgRemainTime = null;
	private JLabel jlMsgRemainTimeVal = null;
	private JLabel jlAvgRxSpeed = null;
	private JLabel jlAvgRxSpeedVal = null;
	private JLabel jlAvgTxSpeed = null;
	private JLabel jlAvgTxSpeedVal = null;
	private JLabel jlAccount = null;
	private JLabel jlAccountVal = null;
	private JLabel jlOperDesc = null;	

	/**
	 * This is the default constructor
	 */
	public ProgressFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
		setAvgRxSpeed(Context.avgRxSpeed);
		setAvgTxSpeed(Context.avgTxSpeed);
	}

	public void setOperationDescription(String desc) {
		jlOperDesc.setText(desc);
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(370, 215);
		this.setResizable(false);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				storeWindowPosition();
			}
		});
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("ProgressFrame.title")); //$NON-NLS-1$
	}

	public void storeWindowPosition() {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jlOperDesc = new JLabel();
			jlOperDesc.setText("");
			jlOperDesc.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
			jlOperDesc.setPreferredSize(new java.awt.Dimension(340,16));
			jlAccountVal = new JLabel();
			jlAccountVal.setText(""); //$NON-NLS-1$
			jlAccountVal.setPreferredSize(new java.awt.Dimension(200,16));
			jlAccount = new JLabel();
			jlAccount.setText(Strings.getString("ProgressFrame.Account")); //$NON-NLS-1$
			jlAccount.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlAccount.setPreferredSize(new java.awt.Dimension(120,16));
			jlAvgTxSpeedVal = new JLabel();
			jlAvgTxSpeedVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
			jlAvgTxSpeedVal.setPreferredSize(new java.awt.Dimension(120,16));
			jlAvgTxSpeed = new JLabel();
			jlAvgTxSpeed.setText(Strings.getString("ProgressFrame.AvgTxSpeed")); //$NON-NLS-1$
			jlAvgTxSpeed.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlAvgTxSpeed.setPreferredSize(new java.awt.Dimension(200,16));
			jlAvgRxSpeedVal = new JLabel();
			jlAvgRxSpeedVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
			jlAvgRxSpeedVal.setPreferredSize(new java.awt.Dimension(120,16));
			jlAvgRxSpeed = new JLabel();
			jlAvgRxSpeed.setText(Strings.getString("ProgressFrame.AvgRxSpeed")); //$NON-NLS-1$
			jlAvgRxSpeed.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlAvgRxSpeed.setPreferredSize(new java.awt.Dimension(200,16));
			jlMsgRemainTimeVal = new JLabel();
			jlMsgRemainTimeVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
			jlMsgRemainTimeVal.setPreferredSize(new java.awt.Dimension(120,16));
			jlMsgRemainTime = new JLabel();
			jlMsgRemainTime.setText(Strings.getString("ProgressFrame.estMsgTime")); //$NON-NLS-1$
			jlMsgRemainTime.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlMsgRemainTime.setPreferredSize(new java.awt.Dimension(200,16));
			jContentPane = new JPanel();
			jContentPane
					.setLayout(new FlowLayout());
			jContentPane.add(jlAccount, null);
			jContentPane.add(jlAccountVal, null);
			jContentPane.add(jlOperDesc, null);
			jContentPane.add(getJProgressBar1(), null);
			jContentPane.add(jlMsgRemainTime, null);
			jContentPane.add(jlMsgRemainTimeVal, null);
			jContentPane.add(jlAvgRxSpeed, null);
			jContentPane.add(jlAvgRxSpeedVal, null);
			jContentPane.add(jlAvgTxSpeed, null);
			jContentPane.add(jlAvgTxSpeedVal, null);
			jContentPane.add(getJButtonCancel(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jProgressBar1
	 * 
	 * @return javax.swing.JProgressBar
	 */
	private JProgressBar getJProgressBar1() {
		if (jProgressBar1 == null) {
			jProgressBar1 = new JProgressBar();
			jProgressBar1.setStringPainted(true);
			jProgressBar1.setPreferredSize(new java.awt.Dimension(340,21));
			jProgressBar1.setName("jProgressBar1"); //$NON-NLS-1$
		}
		return jProgressBar1;
	}

	public void setMaximum(int maximum) {
		jProgressBar1.setMaximum(maximum);
	}

	public void setValue(int pos) {
		jProgressBar1.setValue(pos);
	}

	public void setAccount(String acntName) {
		jlAccountVal.setText(acntName);
	}	
	
	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jButtonCancel_actionPerformed(e);
				}
			});
		}
		return jButtonCancel;
	}

	public void setRemainTime(int seconds) {
		if (seconds == -1)
			jlMsgRemainTimeVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
		else
			jlMsgRemainTimeVal.setText(seconds + " sec");		
	}
	
	public void setAvgRxSpeed(int avgRxSpeed) {
		if(avgRxSpeed == -1)
			jlAvgRxSpeedVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
		else
			jlAvgRxSpeedVal.setText((avgRxSpeed / 1024) + " Kb/sec");
	}

	public void setAvgTxSpeed(int avgTxSpeed) {
		if(avgTxSpeed == -1)
			jlAvgTxSpeedVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
		else		
			jlAvgTxSpeedVal.setText((avgTxSpeed / 1024) + " Kb/sec");
	}	
	
	public void startRemainTimer(int seconds) {
		timerSeconds = seconds;
		if (timer != null)
			if (timer.isRunning())
				timer.stop();
		timer = new Timer(1000, new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				timer_actionPerformed(e);
			}
		});
		timer.start();
	}
	
	void timer_actionPerformed(ActionEvent e) {
		if(timerSeconds == 0) {
			timer.stop();
			return;
		}
		timerSeconds--;
		jlMsgRemainTimeVal.setText(timerSeconds + " sec");
	}	
	
	public void stopRemainTimer() {
		if (timer != null)
			if (timer.isRunning())
				timer.stop();		
	}
	
	synchronized void jButtonCancel_actionPerformed(java.awt.event.ActionEvent e) {
		isCanceled = true;
		jButtonCancel.setEnabled(false);
	}
	
	synchronized boolean isCanceled() {
		return isCanceled;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
