/*
 * @(#)Rule.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.util.Enumeration;
import java.util.Vector;
import java.util.regex.Pattern;

import olivamail.core.common.ComUtils;
import olivamail.core.mail.MessageHeader;

public class Rule {

	public Rule() {
	}

	String id;
	String header;
	int operand = -1;	
	String value;
	int action = -1;
	String param;
	String acntId;
	
	public static final String[] headers = {"From", "To", "Cc", "Subject", "MessageID"};

	public static int ACTION_DELETE = 0;
	public static int ACTION_SKIP = 1;
	public static int ACTION_SAVE_TO_FOLDER = 2;
	public static int ACTION_SAVE_IN_CP = 3;
	public static int ACTION_FWD_ATT = 4;
	public static int ACTION_FWD_CTNT = 5;
	public static int ACTION_FWD_REPORT = 6;
	
	public static final String[] operands = {
		Strings.getString("ruleOperContains"),
		Strings.getString("ruleOperNotContains"),
		Strings.getString("ruleOperStartsWith"),
		Strings.getString("ruleOperNotStartsWith"),
		Strings.getString("ruleOperRegexp")
		};
	public static final String[] actions = {
		Strings.getString("ruleActionDelete"),
		Strings.getString("ruleActionSkip"),
		Strings.getString("ruleActionSaveToFolder"),
		Strings.getString("ruleActionSaveInCp"),
		Strings.getString("ruleActionFwdAtt"),
		Strings.getString("ruleActionFwdCtnt"),
		Strings.getString("ruleActionFwdReport")
		};
	
	public static String getRuleText(Rule rule) {
		if(rule == null)
			return null;
		String param = ComUtils.nvl(rule.param);
		if(param.length() > 0)
			param = " " + param;
		StringBuffer sb = new StringBuffer(Strings.getString("ruleTextIf"));
		sb.append(" (").append(rule.header).append(") ").append(operands[rule.operand])
		.append(" (").append(rule.value).append("), ").append(Strings.getString("ruleTextThen"))
		.append(" ").append(actions[rule.action]).append(param);
		return sb.toString();
	}
	
	public String getRuleText() {
		return getRuleText(this);
	}

/*	
	public static boolean checkRules(Vector rules, MessageHeader mh) {
		if(rules == null || mh == null)
			return false;
		boolean result = false;
		for(Enumeration<Rule> en = rules.elements(); en.hasMoreElements();) {
			Rule rule = (Rule)en.nextElement();
			result = checkRule(rule, mh);
			if(result)
				break;
		}
		return result;
	}
	
	public static boolean checkRule(Rule rule, MessageHeader mh) {
		String header = getHeader(rule.header, mh);
		if(header == null)
			return false;
		switch(rule.operand) {
		//Contains
		case 0:
			return (header.indexOf(rule.value) != -1);
			//NotContains
		case 1:
			return (header.indexOf(rule.value) == -1);
			//StartsWith		
		case 2:
			return header.startsWith(rule.value);
			//NotStartsWith		
		case 3:
			return !header.startsWith(rule.value);
			//Regexp		
		case 4:
			return (Pattern.matches(rule.value, header));			
		default:
			return false;
		}
	}
*/	
	
	public static String getHeader(String header, MessageHeader mh) {
		if(header.equalsIgnoreCase("From"))
			return mh.From;
		else if(header.equalsIgnoreCase("To"))
			return mh.To;
		else if(header.equalsIgnoreCase("Cc"))
			return mh.Cc;
		else if(header.equalsIgnoreCase("Subject"))
			return mh.Subject;
		else if(header.equalsIgnoreCase("MessageID"))
			return mh.MessageID;
		else
			return null;
	}

	public static Rule checkRules(Vector rules, MessageHeader mh) {
		if(rules == null || mh == null)
			return null;
		Rule result = null;
		for(Enumeration en = rules.elements(); en.hasMoreElements();) {
			Rule rule = (Rule)en.nextElement();
			result = checkRule(rule, mh);
			if(result != null)
				break;
		}
		return result;
	}
		
	public static Rule checkRule(Rule rule, MessageHeader mh) {
		String header = getHeader(rule.header, mh);
		if (header == null)
			return null;
		switch (rule.operand) {
		// Contains
		case 0:
			if (header.indexOf(rule.value) != -1)
				return rule;
			break;
		// NotContains
		case 1:
			if (header.indexOf(rule.value) == -1)
				return rule;
			break;
		// StartsWith
		case 2:
			if (header.startsWith(rule.value))
				return rule;
			break;
		// NotStartsWith
		case 3:
			if (!header.startsWith(rule.value))
				return rule;
			break;
		// Regexp
		case 4:
			if (Pattern.matches(rule.value, header))
				return rule;
			break;
		default:
			return null;
		}
		return null;
	}
		
}
