/*
 * @(#)RuleFormFrame.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JButton;

public class RuleFormFrame extends JFrame {

	static final long serialVersionUID = 3L;	
	private final String WIN_ID = "rule_form";
	private JPanel jContentPane = null;
	private JPanel jPanelCenter = null;
	private JPanel jPanelSouth = null;
	private JLabel jLabelHeader = null;
	private JComboBox jcbHeader = null;
	private JLabel jLabelOperand = null;
	private JComboBox jcbOperand = null;
	private JLabel jLabelValue = null;
	private JTextField jtfValue = null;
	private JLabel jLabelAction = null;
	private JComboBox jcbAction = null;
	private JLabel jLabelParam = null;
	private JButton jButtonOk = null;
	private JButton jButtonCancel = null;
	private ParentList parent = null;
	private JTable parentTable = null;
	private int parentRow = -1;
	private JLabel jLabelId = null;
	private JTextField jtfId = null;
	private String acntId = null;
	private JTextField jtfParam = null;
	
	/**
	 * This is the default constructor
	 */
	public RuleFormFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(293, 232);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("RuleFormFrame.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanelCenter(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanelSouth(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	void setParent(ParentList parent) {
		this.parent = parent;
	}
	
	/**
	 * This method initializes jPanelCenter	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelCenter() {
		if (jPanelCenter == null) {
			jLabelId = new JLabel();
			jLabelId.setText(Strings.getString("RuleFormFrame.idField")); //$NON-NLS-1$
			jLabelId.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelId.setPreferredSize(new java.awt.Dimension(70,16));
			jLabelParam = new JLabel();
			jLabelParam.setText(Strings.getString("RuleFormFrame.paramField")); //$NON-NLS-1$
			jLabelParam.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelParam.setPreferredSize(new java.awt.Dimension(70,16));
			jLabelAction = new JLabel();
			jLabelAction.setText(Strings.getString("RuleFormFrame.actionField")); //$NON-NLS-1$
			jLabelAction.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelAction.setPreferredSize(new java.awt.Dimension(70,16));
			jLabelValue = new JLabel();
			jLabelValue.setText(Strings.getString("RuleFormFrame.valueField")); //$NON-NLS-1$
			jLabelValue.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelValue.setPreferredSize(new java.awt.Dimension(70,16));
			jLabelOperand = new JLabel();
			jLabelOperand.setText(Strings.getString("RuleFormFrame.operandField")); //$NON-NLS-1$
			jLabelOperand.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelOperand.setPreferredSize(new java.awt.Dimension(70,16));
			jLabelHeader = new JLabel();
			jLabelHeader.setText(Strings.getString("RuleFormFrame.headerField")); //$NON-NLS-1$
			jLabelHeader.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jLabelHeader.setPreferredSize(new java.awt.Dimension(70,16));
			jPanelCenter = new JPanel();
			jPanelCenter.add(jLabelId, null);
			jPanelCenter.add(getJtfId(), null);
			jPanelCenter.add(jLabelHeader, null);
			jPanelCenter.add(getJComboBoxHeader(), null);
			jPanelCenter.add(jLabelOperand, null);
			jPanelCenter.add(getJcbOperand(), null);			
			jPanelCenter.add(jLabelValue, null);
			jPanelCenter.add(getJtfValue(), null);
			jPanelCenter.add(jLabelAction, null);
			jPanelCenter.add(getJcbAction(), null);
			jPanelCenter.add(jLabelParam, null);
			jPanelCenter.add(getJtfParam(), null);
		}
		return jPanelCenter;
	}

	/**
	 * This method initializes jPanelSouth	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelSouth() {
		if (jPanelSouth == null) {
			jPanelSouth = new JPanel();
			jPanelSouth.setPreferredSize(new java.awt.Dimension(10,40));
			jPanelSouth.add(getJButtonOk(), null);
			jPanelSouth.add(getJButtonCancel(), null);
		}
		return jPanelSouth;
	}

	/**
	 * This method initializes jComboBoxHeader	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJComboBoxHeader() {
		if (jcbHeader == null) {
			jcbHeader = new JComboBox(Rule.headers);
			jcbHeader.setEditable(true);
			jcbHeader.setPreferredSize(new java.awt.Dimension(200,20));
		}
		return jcbHeader;
	}

	/**
	 * This method initializes jcbOperand	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbOperand() {
		if (jcbOperand == null) {
			jcbOperand = new JComboBox(Rule.operands);
			jcbOperand.setPreferredSize(new java.awt.Dimension(200,20));
		}
		return jcbOperand;
	}

	/**
	 * This method initializes jtfValue	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfValue() {
		if (jtfValue == null) {
			jtfValue = new JTextField();
			jtfValue.setPreferredSize(new java.awt.Dimension(200,20));
			TextPopup.registerComponent(jtfValue);
		}
		return jtfValue;
	}

	/**
	 * This method initializes jcbAction	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbAction() {
		if (jcbAction == null) {
			jcbAction = new JComboBox(Rule.actions);
			jcbAction.setPreferredSize(new java.awt.Dimension(200,20));
		}
		return jcbAction;
	}

	/**
	 * This method initializes jButtonOk	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonOk() {
		if (jButtonOk == null) {
			jButtonOk = new JButton();
			jButtonOk.setText(Strings.getString("okButton")); //$NON-NLS-1$
			jButtonOk.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jButtonOk_actionPerformed(e);
				}
			});
		}
		return jButtonOk;
	}

	void jButtonOk_actionPerformed(java.awt.event.ActionEvent e) {
		Rule rule = new Rule();
		rule.id = jtfId.getText();
		rule.header = ((String)jcbHeader.getSelectedItem()).trim();
		rule.operand = jcbOperand.getSelectedIndex();
		rule.value = Context.checkField(jtfValue);
		if (rule.value == null) {
			return;
		}
		rule.action = jcbAction.getSelectedIndex();
		rule.param = jtfParam.getText().trim();
		if (acntId != null)
			rule.acntId = acntId; 
		try {
			rule = Context.storage.saveRule(rule);
			parent.updateRow(parentTable, parentRow, rule);
			this_windowClosing(null);
			dispose();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}		
	}
	
	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jButtonCancel;
	}

	public void editRule(JTable table, int row, String id) {
		this.parentTable = table;
		this.parentRow = row;
		try {
			Rule rule = Context.storage.getRule(id);
			jtfId.setText(rule.id);
			jcbHeader.setSelectedItem(rule.header);
			jcbOperand.setSelectedIndex(rule.operand);
			jtfValue.setText(rule.value);
			jcbAction.setSelectedIndex(rule.action);
			jtfParam.setText(rule.param);
			acntId = rule.acntId;
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}

	}
	
	/**
	 * This method initializes jtfId	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfId() {
		if (jtfId == null) {
			jtfId = new JTextField();
			jtfId.setPreferredSize(new java.awt.Dimension(200,20));
			jtfId.setEnabled(false);
		}
		return jtfId;
	}
	
	public void setAcntId(String acntId) {
		this.acntId = acntId;
	}

	/**
	 * This method initializes jtfParam	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfParam() {
		if (jtfParam == null) {
			jtfParam = new JTextField();
			jtfParam.setPreferredSize(new java.awt.Dimension(200,20));
			TextPopup.registerComponent(jtfParam);
		}
		return jtfParam;
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
