/*
 * @(#)SettingsFrame.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeCellRenderer;

public class SettingsFrame extends JFrame {

	static final long serialVersionUID = 5270346247832127699L;

	private final String WIN_ID = "settings";
	
	private JPanel jContentPane = null;

	private JPanel jPanel2 = null;

	private JButton okButton = null;

	private JButton cancelButton = null;

	private JButton applyButton = null;

	private boolean stateChanged = false;
	
//	private boolean autoStateChanged = false;
	
	private MainFrame parent = null;

	private JSplitPane jSplitPane1 = null;

	private JTree jTree1 = null;

	private JPanel jPanel1 = null;

	private JPanel jPanel3 = null;

	private JPanel jPanel4 = null;

	private SettingsPanel[] panels = new SettingsPanel[Settings.PANELS_COUNT];

	private JPanel jPanelRight = null;

	/**
	 * This is the default constructor
	 */
	public SettingsFrame(MainFrame parent) {
		super();
		this.parent = parent;
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));		
		selectFirstNode();
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(483, 351);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setResizable(false);
		this
				.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("SettingsFrame.title")); //$NON-NLS-1$
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
	
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel2(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getJSplitPane1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.WEST);
			jContentPane.add(getJPanel3(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getJPanel4(), java.awt.BorderLayout.EAST);
		}
		return jContentPane;
	}

	public void getSettings() {
/* NEU		
		try {
			if (jTabbedPane1.getSelectedComponent() == jPanel1) {
				boolean rp = olivamail.core.mail.MailUtils.parseBoolean(Context
						.getSettingsValue("receive_periodically"), false); //$NON-NLS-1$
//				autoStateChanged = true;
				periodicallyBox.setSelected(rp);
//				autoStateChanged = false;
				periodField.setText(Context.getSettingsValue("receive_period")); //$NON-NLS-1$
				periodField.setEnabled(rp);
			}
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
*/				
	}
	
	void jTabbedPane1_stateChanged(javax.swing.event.ChangeEvent e) {
		if(stateChanged) {			
			AlertDialog.showWarningDialog(Strings.getString("SettingsFrame.applyFirst")); //$NON-NLS-1$
			return;
		}
		getSettings();	
	}

	void periodicallyBox_actionPerformed(java.awt.event.ActionEvent e) {
/*
		if(autoStateChanged) {
			autoStateChanged = false;
			return;
		}
*/		
		stateChanged = true;
		applyButton.setEnabled(true);		
// NEU		periodField.setEnabled(periodicallyBox.isSelected());
	}

	void periodField_keyTyped(java.awt.event.KeyEvent e) {
		stateChanged = true;
		applyButton.setEnabled(true);
	}

	/**
	 * This method initializes jPanel2
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();
			jPanel2.setPreferredSize(new java.awt.Dimension(245,40));
			jPanel2.add(getOkButton(), null);
			jPanel2.add(getApplyButton(), null);
			jPanel2.add(getCancelButton(), null);
		}
		return jPanel2;
	}

	/**
	 * This method initializes jButtonOk
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getOkButton() {
		if (okButton == null) {
			okButton = new JButton();
			okButton.setText(Strings.getString("okButton")); //$NON-NLS-1$
			okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (applyChanges()) {
						this_windowClosing(null);
						dispose();
					}
				}
			});
		}
		return okButton;
	}

	private boolean applyChanges() {
		for (int i = 0; i < panels.length; i++) {
			if (panels[i] != null) {
				try {
					panels[i].applyChanges();
				} catch (Exception exc) {
					exc.printStackTrace();
					AlertDialog.showErrorDialog(exc);
					showPanel(i);
					return false;
				}
			}
		}
		applyButton.setEnabled(false);		
		return true;
	}
	
	public void setStateChanged() {
		applyButton.setEnabled(true);		
	}

	void applyButton_actionPerformed(java.awt.event.ActionEvent e) {
		applyChanges();
	}	
	
	/**
	 * This method initializes cancelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes applyButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getApplyButton() {
		if (applyButton == null) {
			applyButton = new JButton();
			applyButton.setText(Strings.getString("applyButton")); //$NON-NLS-1$
			applyButton.setEnabled(false);
			applyButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					applyButton_actionPerformed(e);
				}
			});
		}
		return applyButton;
	}

	void jButtonOptimize_actionPerformed(java.awt.event.ActionEvent e) {
		if (!Context.storage.isOptimizable()) {
			AlertDialog.showInfoDialog(Strings
					.getString("SettingsFrame.noOptimization")); //$NON-NLS-1$					
			return;
		}							
		if(AlertDialog.showConfirmDialog(Strings.getString("SettingsFrame.optimizeConfirm"), AlertDialog.OK_CANCEL_OPTION) != AlertDialog.OK_OPTION) //$NON-NLS-1$
			return;
		try {
			Context.storage.optimize();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}		
	}

	/**
	 * This method initializes jSplitPane1	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */    
	private JSplitPane getJSplitPane1() {
		if (jSplitPane1 == null) {
			jSplitPane1 = new JSplitPane();
			jSplitPane1.setDividerSize(0);
			jSplitPane1.setLeftComponent(getJTree1());
			jSplitPane1.setRightComponent(getJPanelRight());
			jSplitPane1.setDividerLocation(120);
		}
		return jSplitPane1;
	}

	/**
	 * This method initializes jTree1	
	 * 	
	 * @return javax.swing.JTree	
	 */    
	private JTree getJTree1() {
		if (jTree1 == null) {
			Settings settings = new Settings("Settings", Settings.S_ROOT); 
			DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(settings);			
			settings = new Settings(Strings.getString("SettingsFrame.pageIncoming"), Settings.S_INCOMING); 
			DefaultMutableTreeNode dmtn = new DefaultMutableTreeNode(settings);			
			rootNode.add(dmtn);
			settings = new Settings(Strings.getString("SettingsFrame.pageRead"), Settings.S_READ); 
			dmtn = new DefaultMutableTreeNode(settings);			
			rootNode.add(dmtn);			
			settings = new Settings(Strings.getString("SettingsFrame.pageDatabase"), Settings.S_STORAGE); 
			dmtn = new DefaultMutableTreeNode(settings);
			rootNode.add(dmtn);
			settings = new Settings(Strings.getString("SettingsFrame.pageInternat"), Settings.S_INTERNAT); 
			dmtn = new DefaultMutableTreeNode(settings);
			rootNode.add(dmtn);			
			jTree1 = new JTree(rootNode);
			jTree1.setPreferredSize(new java.awt.Dimension(119,10));
			jTree1.setRootVisible(false);
			jTree1.setCellRenderer(new SettingsTreeCellRenderer());
			jTree1.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() { 
				public void valueChanged(javax.swing.event.TreeSelectionEvent e) {    
					jTree1_valueChanged(e);
				}
			});
		}
		return jTree1;
	}

		
	
	void jTree1_valueChanged(TreeSelectionEvent e) {
		if (jTree1.getSelectionPath() == null)
			return;
		DefaultMutableTreeNode dutn = (DefaultMutableTreeNode) jTree1
				.getSelectionPath().getLastPathComponent();
		Settings settings = (Settings) dutn.getUserObject();
		showPanel(settings.getType());		
	}

	private void showPanel(int type) {
		SettingsPanel sp = panels[type];
		int div = jSplitPane1.getDividerLocation();		
		if (sp != null) { 
			jSplitPane1.setRightComponent((JPanel) sp);
			jSplitPane1.setDividerLocation(div);
			return;
		}

		switch (type) {
		case Settings.S_INCOMING:
			SettingsIncoming si = new SettingsIncoming();
			si.setMainFrame(parent);
			sp = si;			
			break;
		case Settings.S_READ:
			sp = new SettingsRead();
			break;			
		case Settings.S_STORAGE:
			sp = new SettingsStorage();
			break;
		case Settings.S_INTERNAT:
			sp = new SettingsInternat();
			break;			
		}				
		
		if (sp == null) {
			jSplitPane1.setRightComponent(getJPanelRight());			
			return;
		}
		
		sp.setParent(this);
		panels[type] = sp;
		jSplitPane1.setRightComponent((JPanel) sp);
		jSplitPane1.setDividerLocation(div);
		try {
			sp.readSettings();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
	}
	
	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
		}
		return jPanel1;
	}

	/**
	 * This method initializes jPanel3	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jPanel3 = new JPanel();
		}
		return jPanel3;
	}

	/**
	 * This method initializes jPanel4	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel4() {
		if (jPanel4 == null) {
			jPanel4 = new JPanel();
		}
		return jPanel4;
	}

	/**
	 * This method initializes jPanelRight	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanelRight() {
		if (jPanelRight == null) {
			jPanelRight = new JPanel();
		}
		return jPanelRight;
	}	
	
	private void selectFirstNode() {
		jTree1.setSelectionRow(0);
		jTree1.scrollRowToVisible(0);
	}
		
} // @jve:decl-index=0:visual-constraint="10,10"

class SettingsTreeCellRenderer extends JLabel implements TreeCellRenderer {
	static final long serialVersionUID = 3458391262076892631L;

	SettingsTreeCellRenderer() {
		setOpaque(true);
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean selected, boolean expanded, boolean leaf, int row,
			boolean hasFocus) {
		if (selected)
			setIcon(Context.getImageIcon("arrow_141.gif"));
		else
			setIcon(Context.getImageIcon("arrow_141.gif"));
		setText(value.toString());
		setBackground(selected ? Color.LIGHT_GRAY : Color.WHITE);
		return this;
	}
		
}
