/*
 * @(#)SettingsIncoming.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JLabel;

public class SettingsIncoming extends JPanel implements SettingsPanel {

	static final long serialVersionUID = 5641570478220254063L;
	private JCheckBox periodicallyBox = null;
	private JTextField periodField = null;
	private JLabel jLabel1 = null;
	private boolean stateChanged = false;
	private MainFrame mainFrame = null;
	private SettingsFrame parent = null;
	/**
	 * This is the default constructor
	 */
	public SettingsIncoming() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		jLabel1 = new JLabel();
		jLabel1.setText(Strings.getString("SettingsFrame.min")); //$NON-NLS-1$
		this.setSize(300, 200);
		this.setPreferredSize(new java.awt.Dimension(300,200));
		this.add(getPeriodicallyBox(), null);
		this.add(getPeriodField(), null);
		this.add(jLabel1, null);
	}

	/**
	 * This method initializes periodicallyBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getPeriodicallyBox() {
		if (periodicallyBox == null) {
			periodicallyBox = new JCheckBox();
			periodicallyBox.setText(Strings.getString("SettingsFrame.receivePeriodically")); //$NON-NLS-1$
			periodicallyBox.setPreferredSize(new java.awt.Dimension(168,16));
			periodicallyBox.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
			periodicallyBox.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					periodicallyBox_actionPerformed(e);
				}
			});
		}
		return periodicallyBox;
	}

	void periodicallyBox_actionPerformed(java.awt.event.ActionEvent e) {
		setStateChanged();
		periodField.setEnabled(periodicallyBox.isSelected());
	}
		
	/**
	 * This method initializes periodField
	 * 
	 * @return javax.swing.JTextField
	 */    
	private JTextField getPeriodField() {
		if (periodField == null) {
			periodField = new JTextField();
			periodField.setPreferredSize(new java.awt.Dimension(30,20));
			periodField.addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyTyped(java.awt.event.KeyEvent e) {    
					setStateChanged();
				}
			});
			TextPopup.registerComponent(periodField);
		}
		return periodField;
	}

	void setStateChanged() {
		stateChanged = true;
		if(parent != null)
			parent.setStateChanged();
	}
		
	public void applyChanges() throws Exception {
		if (!stateChanged)
			return;
		Context.storage.setSettingsValue("receive_periodically", String //$NON-NLS-1$
				.valueOf(periodicallyBox.isSelected()));
		Context.storage.setSettingsValue("receive_period", String //$NON-NLS-1$
				.valueOf(periodField.getText()));
		if (mainFrame != null)
			mainFrame.updateTimer();
		stateChanged = false;
	}
	
	public void setMainFrame(MainFrame mainFrame) {
		this.mainFrame = mainFrame;
	}

	public void readSettings() throws Exception {
		boolean rp = olivamail.core.common.ComUtils.parseBoolean(Context
				.getSettingsValue("receive_periodically"), false); //$NON-NLS-1$
		periodicallyBox.setSelected(rp);
		periodField.setText(Context.getSettingsValue("receive_period")); //$NON-NLS-1$
		periodField.setEnabled(rp);
	}

	public void setParent(SettingsFrame parent) {
		this.parent = parent;	
	}	
		
}
