/*
 * @(#)SettingsInternat.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.Arrays;
import java.util.StringTokenizer;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComboBox;

import olivamail.core.common.IOUtils;
import olivamail.core.common.ComUtils;

public class SettingsInternat extends JPanel implements SettingsPanel {

	static final long serialVersionUID = 5641570478220154063L;
	private boolean stateChanged = false;
	private boolean applyAnyway = false;	
	private SettingsFrame parent = null;
	private JLabel jlLanguage = null;
	private JComboBox jcbLang = null;
	private JLabel jlCpDefault = null;
	private JComboBox jcbCpDefault = null;
	private JLabel jlCpSend = null;
	private JComboBox jcbCpSend = null;
	private JLabel jlCountry = null;
	private JComboBox jcbCountry = null;
	
	/**
	 * This is the default constructor
	 */
	public SettingsInternat() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		jlCountry = new JLabel();
		jlCountry.setText("Country:");
		jlCountry.setPreferredSize(new java.awt.Dimension(110,16));
		jlCpSend = new JLabel();
		jlCpSend.setText("Send codepage:");
		jlCpSend.setPreferredSize(new java.awt.Dimension(110,16));
		jlCpDefault = new JLabel();
		jlCpDefault.setText("Default codepage:");
		jlCpDefault.setPreferredSize(new java.awt.Dimension(110,16));
		jlLanguage = new JLabel();
		jlLanguage.setText("Language:");
		jlLanguage.setPreferredSize(new java.awt.Dimension(110,16));
		this.setSize(300, 200);
		this.setPreferredSize(new java.awt.Dimension(300,200));
		this.add(jlCountry, null);
		this.add(getJcbCountry(), null);
		this.add(jlLanguage, null);
		this.add(getJcbLang(), null);
		this.add(jlCpDefault, null);
		this.add(getJcbCpDefault(), null);
		this.add(jlCpSend, null);
		this.add(getJcbCpSend(), null);
	}

	public void setApplyAnyway() {
		applyAnyway = true;
	}
	
	public void applyChanges() throws Exception {
		if (!stateChanged && !applyAnyway)
			return;
		File sf = Context.getSettingsFile();
		String settings = IOUtils.readFile(sf, "UTF-8");
		StringTokenizer st = new StringTokenizer(settings, "\r\n");
		StringBuffer sb = new StringBuffer();
		String country = ComUtils.nvl((String)jcbCountry.getSelectedItem()).toUpperCase();
		String lang = ComUtils.nvl((String)jcbLang.getSelectedItem()).toLowerCase();
		String defCp = ComUtils.nvl((String)jcbCpDefault.getSelectedItem());
		String sendCp = ComUtils.nvl((String)jcbCpSend.getSelectedItem());
		boolean countryFound = false;
		boolean langFound = false;
		boolean defCpFound = false;
		boolean sendCpFound = false;
		while (st.hasMoreTokens()) {
			String line = st.nextToken();
			if (!country.equals("")) {
				if (line.matches("^olivamail.gui.country=.*$")) {
					countryFound = true;
					line = line.replaceFirst("^olivamail.gui.country=.*$",
							"olivamail.gui.country=" + country);
				}
			}			
			if (!lang.equals("")) {
				if (line.matches("^olivamail.gui.language=.*$")) {
					langFound = true;
					line = line.replaceFirst("^olivamail.gui.language=.*$",
							"olivamail.gui.language=" + lang);
				}
			}
			if (!defCp.equals("")) {
				if (line.matches("^olivamail.gui.defaultCodepage=.*$")) {
					defCpFound = true;
					line = line.replaceFirst(
							"^olivamail.gui.defaultCodepage=.*$",
							"olivamail.gui.defaultCodepage=" + defCp);
				}
			}
			if (!sendCp.equals("")) {
				if (line.matches("^olivamail.gui.sendCodepage=.*$")) {
					sendCpFound = true;
					line = line.replaceFirst("^olivamail.gui.sendCodepage=.*$",
							"olivamail.gui.sendCodepage=" + sendCp);
				}
			}
			sb.append(line + "\n");
		}
		if(!countryFound && !country.equals("")) {
			sb.append("olivamail.gui.country=" + country + "\n");
		}		
		if(!langFound && !lang.equals("")) {
			sb.append("olivamail.gui.language=" + lang + "\n");
		}
		if(!defCpFound && !defCp.equals("")) {
			sb.append("olivamail.gui.defaultCodepage=" + defCp + "\n");
		}
		if(!sendCpFound && !sendCp.equals("")) {
			sb.append("olivamail.gui.sendCodepage=" + sendCp + "\n");
		}				
		ByteArrayInputStream bais = new ByteArrayInputStream(sb.toString()
				.getBytes("UTF-8")); 
		IOUtils.writeFile(bais, sf);
		if (!country.equals("")) {
			Context.country = country;
		}		
		if (!lang.equals("")) {
			Strings.setLanguage(lang);
			HelpStrings.setLanguage(lang);
			Context.language = lang;
		}
		if (!defCp.equals("")) {
			Context.defaultCodepage = defCp;
			if (Context.mailBean != null)
				Context.mailBean.setDefaultCodePage(defCp);			
		}
		if (!sendCp.equals("")) {
			Context.sendCodepage = sendCp;
			if (Context.mailBean != null)
				Context.mailBean.setSendCodePage(sendCp);			
		}
		stateChanged = false;
	}
	
	public void readSettings() throws Exception {
	}

	public void setParent(SettingsFrame parent) {
		this.parent = parent;	
	}

	/**
	 * This method initializes jcbLang	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbLang() {
		if (jcbLang == null) {
			String files[] = Context.getLanguageFiles();
			String langs[] = null;
			if(files != null) {
				langs = new String[files.length];
				for(int i = 0; i < files.length; i++) {
					int sIdx = files[i].indexOf('_');
					int eIdx = files[i].indexOf('.');
					if(sIdx != -1 && eIdx != -1 && eIdx - sIdx == 3)
						langs[i] = files[i].substring(sIdx + 1, eIdx);
					else
						langs[i] = files[i];
				}
				Arrays.sort(langs);
			}
			if(langs == null)
				jcbLang = new JComboBox();
			else
				jcbLang = new JComboBox(langs);
			if (Context.language != null)
				jcbLang.setSelectedItem(Context.language);			
			jcbLang.setPreferredSize(new java.awt.Dimension(150,20));
			jcbLang.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					setStateChanged();
				}
			});
			jcbLang.setEditable(true);
		}
		return jcbLang;
	}

	void setStateChanged() {
		stateChanged = true;
		if(parent != null)
			parent.setStateChanged();
	}
	
	/**
	 * This method initializes jcbCpDefault	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbCpDefault() {
		if (jcbCpDefault == null) {
			jcbCpDefault = new JComboBox(Charsets.getCpArraySorted());
			jcbCpDefault.setSelectedItem(Context.defaultCodepage);
			jcbCpDefault.setPreferredSize(new java.awt.Dimension(150,20));
			jcbCpDefault.setEditable(true);
			jcbCpDefault.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jcbCpDefault_actionPerformed(e);
				}
			});
		}
		return jcbCpDefault;
	}

	void jcbCpDefault_actionPerformed(java.awt.event.ActionEvent e) {
		stateChanged = true;
		if (parent != null)
			parent.setStateChanged();		
	}
		
	/**
	 * This method initializes jcbCpSend	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbCpSend() {
		if (jcbCpSend == null) {
			jcbCpSend = new JComboBox(Charsets.getCpArraySorted());
			jcbCpSend.setSelectedItem(Context.sendCodepage);
			jcbCpSend.setPreferredSize(new java.awt.Dimension(150,20));
			jcbCpSend.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jcbCpSend_actionPerformed(e);
				}
			});
			jcbCpSend.setEditable(true);
		}
		return jcbCpSend;
	}	

	void jcbCpSend_actionPerformed(java.awt.event.ActionEvent e) {
		stateChanged = true;
		if (parent != null)
			parent.setStateChanged();				
	}

	/**
	 * This method initializes jcbCountry	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbCountry() {
		if (jcbCountry == null) {
			jcbCountry = new JComboBox(Context.countries);
			if (Context.country != null)
				jcbCountry.setSelectedItem(Context.country);						
			jcbCountry.setPreferredSize(new java.awt.Dimension(150,20));
			jcbCountry.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					setStateChanged();
				}
			});
			jcbCountry.setEditable(true);
		}
		return jcbCountry;
	}
		
}
