/*
 * @(#)SettingsIncoming.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;

import olivamail.core.common.ComUtils;

public class SettingsRead extends JPanel implements SettingsPanel {

	static final long serialVersionUID = 5641570478220254063L;
	private boolean stateChanged = false;
	private SettingsFrame parent = null;
	private JLabel jlCtntPane = null;
	private JComboBox jcbCtntPane = null;
	private JCheckBox jcbHidePictures = null;
	private JCheckBox jcbShowMessageInline = null;
	
	/**
	 * This is the default constructor
	 */
	public SettingsRead() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		jlCtntPane = new JLabel();
		jlCtntPane.setText(Strings.getString("SettingsFrame.messageViewer")); //$NON-NLS-1$
		jlCtntPane.setPreferredSize(new java.awt.Dimension(280,16));
		this.setSize(300, 200);
		this.setPreferredSize(new java.awt.Dimension(300,200));
		this.add(jlCtntPane, null);
		this.add(getJcbCtntPane(), null);
		this.add(getJcbHidePictures(), null);
		this.add(getJcbShowMessageInline(), null);
	}

	void setStateChanged() {
		stateChanged = true;
		if(parent != null)
			parent.setStateChanged();
	}
		
	public void applyChanges() throws Exception {
		if (!stateChanged)
			return;
		String ctntPane = ComUtils.nvl((String)jcbCtntPane.getSelectedItem()).trim();
		if (!ctntPane.equals("")) {
			Context.storage.setSettingsValue("content_pane", ctntPane); //$NON-NLS-1$
			Context.contentPane = ctntPane; 			
		}
		Context.storage.setSettingsValue("hide_pictures", String //$NON-NLS-1$
				.valueOf(jcbHidePictures.isSelected()));
		Context.hidePictures = jcbHidePictures.isSelected();
		Context.storage.setSettingsValue("show_message_inline", String //$NON-NLS-1$
				.valueOf(jcbShowMessageInline.isSelected()));
		Context.setShowMessageInline(jcbShowMessageInline.isSelected());		
		stateChanged = false;
	}
	
	public void readSettings() throws Exception {		
	}

	public void setParent(SettingsFrame parent) {
		this.parent = parent;	
	}

	/**
	 * This method initializes jcbCtntPane	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbCtntPane() {
		if (jcbCtntPane == null) {
			jcbCtntPane = new JComboBox(Context.ctntPanes);			
			jcbCtntPane.setPreferredSize(new java.awt.Dimension(280,20));
			jcbCtntPane.setEditable(true);
			jcbCtntPane.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					setStateChanged();
				}
			});
			jcbCtntPane.setSelectedItem(Context.contentPane);
		}
		return jcbCtntPane;
	}

	/**
	 * This method initializes jcbHidePictures	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbHidePictures() {
		if (jcbHidePictures == null) {
			jcbHidePictures = new JCheckBox();
			jcbHidePictures.setText(Strings.getString("SettingsFrame.hidePictures"));
			jcbHidePictures.setPreferredSize(new java.awt.Dimension(280,16));
			jcbHidePictures.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					setStateChanged();
				}
			});
			jcbHidePictures.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
			jcbHidePictures.setSelected(Context.hidePictures);					
		}
		return jcbHidePictures;
	}

	/**
	 * This method initializes jcbShowMessageInline	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbShowMessageInline() {
		if (jcbShowMessageInline == null) {
			jcbShowMessageInline = new JCheckBox();
			jcbShowMessageInline.setPreferredSize(new java.awt.Dimension(280,16));
			jcbShowMessageInline.setText(Strings.getString("SettingsFrame.showMessageInline"));
			jcbShowMessageInline.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					setStateChanged();
				}
			});
			jcbShowMessageInline.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
			jcbShowMessageInline.setSelected(Context.isShowMessageInline());
			jcbShowMessageInline.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if(jcbShowMessageInline.isSelected())
						jcbCtntPane.setSelectedItem(Context.ctntPanes[0]);
				}
			});
		}
		return jcbShowMessageInline;
	}	
		
}
