/*
 * @(#)SettingsStorage.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JComboBox;

public class SettingsStorage extends JPanel implements SettingsPanel {

	static final long serialVersionUID = 4765712854418232360L;
	private JButton jButtonOptimize = null;
	private SettingsFrame parent = null;
	private JLabel jlDbMsgMaxSize = null;
	private JTextField jtfDbMsgMaxSize = null;
	private JComboBox jcbSizeUnits = null;
	private JLabel jlSpace1 = null;
	private boolean stateChanged = false;	
	
	/**
	 * This is the default constructor
	 */
	public SettingsStorage() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		jlSpace1 = new JLabel();
		jlSpace1.setText("----------------------------------------------------------------");
		jlSpace1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
		jlSpace1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
		jlSpace1.setPreferredSize(new java.awt.Dimension(270,16));
		jlDbMsgMaxSize = new JLabel();
		jlDbMsgMaxSize.setText(Strings.getString("SettingsFrame.dbMsgMaxSize"));
		jlDbMsgMaxSize.setPreferredSize(new java.awt.Dimension(270,16));
		this.setSize(300, 200);
		this.setPreferredSize(new java.awt.Dimension(300,200));
		this.add(jlDbMsgMaxSize, null);
		this.add(getJtfDbMsgMaxSize(), null);
		this.add(getJcbSizeUnits(), null);
		this.add(jlSpace1, null);
		this.add(getJButtonOptimize(), null);
	}

	/**
	 * This method initializes jButtonOptimize	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButtonOptimize() {
		if (jButtonOptimize == null) {
			jButtonOptimize = new JButton();
			jButtonOptimize.setText(Strings.getString("SettingsFrame.optimizeButton")); //$NON-NLS-1$
			jButtonOptimize.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jButtonOptimize_actionPerformed(e);
				}
			});
		}
		return jButtonOptimize;
	}

	void jButtonOptimize_actionPerformed(java.awt.event.ActionEvent e) {
		//Prevent warnings in IDE
		parent.isVisible();
		if (!Context.storage.isOptimizable()) {
			AlertDialog.showInfoDialog(Strings.getString("SettingsFrame.noOptimization")); //$NON-NLS-1$					
			return;
		}							
		if(AlertDialog.showConfirmDialog(Strings.getString("SettingsFrame.optimizeConfirm"), AlertDialog.OK_CANCEL_OPTION) != AlertDialog.OK_OPTION) //$NON-NLS-1$
			return;
		try {
			Context.storage.optimize();
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}		
	}

	public void applyChanges() throws Exception {
		if (!stateChanged)
			return;		
		String str = jtfDbMsgMaxSize.getText().trim();
		int size = -1;
		if(!str.equals("")) {
			size = Integer.parseInt(str);
		}
		if(size != -1) {
			size *= 1024 ^ jcbSizeUnits.getSelectedIndex();
		}
		Context.storage.setSettingsValue("db_message_max_size", String.valueOf(size));
		Context._db_message_max_size = size;		
		Context.storage.setDbMessageMaxSize(Context._db_message_max_size);
		stateChanged = false;		
	}

	public void readSettings() throws Exception {
		int size = Context._db_message_max_size; 
		if (size > 1048576) {
			size /= 1048576;
		} else if (size > 1024) {
			size /= 1024;
		}		
		jtfDbMsgMaxSize.setText(String.valueOf(size));
	}	
	
	void setStateChanged() {
		stateChanged = true;
		if(parent != null)
			parent.setStateChanged();
	}
		
	public void setParent(SettingsFrame parent) {
		this.parent = parent;	
	}

	/**
	 * This method initializes jtfDbMsgMaxSize	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfDbMsgMaxSize() {
		if (jtfDbMsgMaxSize == null) {
			jtfDbMsgMaxSize = new JTextField();
			jtfDbMsgMaxSize.addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyTyped(java.awt.event.KeyEvent e) {    
					setStateChanged();
				}
			});			
			jtfDbMsgMaxSize.setPreferredSize(new java.awt.Dimension(80,20));
			jtfDbMsgMaxSize.addFocusListener(new java.awt.event.FocusAdapter() { 
				public void focusLost(java.awt.event.FocusEvent e) {    
					String str = jtfDbMsgMaxSize.getText().trim();
					if(!str.matches("^-?\\d+$")) {
						AlertDialog.showWarningDialog(Strings.getString("SettingsFrame.warnSize"));			
						jtfDbMsgMaxSize.requestFocus();
						return;
					}		
				}
			});
			TextPopup.registerComponent(jtfDbMsgMaxSize);
		}
		return jtfDbMsgMaxSize;
	}

	/**
	 * This method initializes jcbSizeUnits	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbSizeUnits() {
		if (jcbSizeUnits == null) {
			jcbSizeUnits = new JComboBox(Context.sizeUnits);
			jcbSizeUnits.setPreferredSize(new java.awt.Dimension(60,20));
			
			int size = Context._db_message_max_size; 
			int idx = 0;
			if (size > 1048576) {
				idx = 2;
			} else if (size > 1024) {
				idx = 1;
			}		
			if(size == -1)
				jcbSizeUnits.setSelectedIndex(2);
			else
				jcbSizeUnits.setSelectedIndex(idx);			
			
			jcbSizeUnits.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					setStateChanged();
				}
			});
		}
		return jcbSizeUnits;
	}	
		
}  //  @jve:decl-index=0:visual-constraint="10,10"
