/*
 * @(#)SignaturesFrame.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;

public class SignaturesFrame extends JFrame {

	static final long serialVersionUID = 6767896132312341640L;	
	private final String WIN_ID = "signatures";
	private JPanel jContentPane = null;
	private JPanel jPanel1 = null;
	private JLabel jlLocalText = null;
	private JScrollPane jScrollPane1 = null;
	private JEditorPane jepLocalText = null;
	private JPanel jPanel2 = null;
	private JButton jbOk = null;
	private JButton jbCancel = null;
	private JButton jbApply = null;
	private JPanel jPanel3 = null;
	private JLabel jLabelAccount = null;
	private JComboBox jComboBoxAcnt = null;
	private JCheckBox jcbDefault = null;
	private Signature signature = null;
	private boolean stateChanged = false;

	private JLabel jlSigType = null;

	private JComboBox jcbSigType = null;

	private JLabel jlHdrHint = null;	
	
	/**
	 * This is the default constructor
	 */
	public SignaturesFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(431, 354);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("Signatures.title")); //$NON-NLS-1$
	}
	
	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
	
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel1(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel2(), java.awt.BorderLayout.SOUTH);
			jContentPane.add(getJPanel3(), java.awt.BorderLayout.NORTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jlLocalText = new JLabel();
			jlLocalText.setText(Strings.getString("Signatures.sigLocal")); //$NON-NLS-1$
			jPanel1 = new JPanel();
			jPanel1.add(jlLocalText, null);
			jPanel1.add(getJScrollPane1(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setPreferredSize(new java.awt.Dimension(400,170));
			jScrollPane1.setViewportView(getJepLocalText());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jEditorPane1	
	 * 	
	 * @return javax.swing.JEditorPane	
	 */    
	private JEditorPane getJepLocalText() {
		if (jepLocalText == null) {
			jepLocalText = new JEditorPane();
			jepLocalText.addKeyListener(new java.awt.event.KeyAdapter() { 
				public void keyTyped(java.awt.event.KeyEvent e) {    
					setStateChanged();
				}
			});
			TextPopup.registerComponent(jepLocalText);
		}
		return jepLocalText;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			jPanel2 = new JPanel();
			jPanel2.setPreferredSize(new java.awt.Dimension(10,40));
			jPanel2.add(getJbOk(), null);
			jPanel2.add(getJbApply(), null);
			jPanel2.add(getJbCancel(), null);
		}
		return jPanel2;
	}

	/**
	 * This method initializes jbOk	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbOk() {
		if (jbOk == null) {
			jbOk = new JButton();
			jbOk.setText(Strings.getString("okButton")); //$NON-NLS-1$
			jbOk.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					jbOk_actionPerformed(e);
				}
			});
		}
		return jbOk;
	}

	void jbOk_actionPerformed(java.awt.event.ActionEvent e) {
		if(applyChanges()) {
			this_windowClosing(null);			
			dispose();
		}
	}
	
	/**
	 * This method initializes jbCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbCancel() {
		if (jbCancel == null) {
			jbCancel = new JButton();
			jbCancel.setText(Strings.getString("cancelButton")); //$NON-NLS-1$
			jbCancel.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jbCancel;
	}

	/**
	 * This method initializes jbApply	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbApply() {
		if (jbApply == null) {
			jbApply = new JButton();
			jbApply.setText(Strings.getString("applyButton")); //$NON-NLS-1$
			jbApply.setEnabled(false);
			jbApply.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					applyChanges();
				}
			});
		}
		return jbApply;
	}
	
	boolean applyChanges() {
		if (!stateChanged)
			return true;
		boolean result = false;
		// try {
		if (signature == null) {
			// String acntEmail = (String) jComboBoxAcnt.getSelectedItem();
			Account acnt = (Account) jComboBoxAcnt.getSelectedItem();
			String sigType = String.valueOf(jcbSigType.getSelectedIndex());
			if (acnt == null || sigType == null) {
				// Show warning that acnt or sigType are not selected.
				return result;
			}
			// String acntId = Context.storage.getAccountForEmail(acntEmail).id;
			signature = new Signature();
			signature.acntId = acnt.id;
			signature.sigType = sigType;
		}
		if (jcbDefault.isSelected())
			signature.status = "D";
		else
			signature.status = "0";
		String sigText = jepLocalText.getText().trim();
		if (sigText.equals(""))
			sigText = null;
		signature.sigLocal = sigText;
		try {
			Context.storage.saveSignature(signature);
			result = true;
			stateChanged = false;
			jbApply.setEnabled(false);
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}
		return result;
	}		

	/**
	 * This method initializes jPanel3
	 * 
	 * @return javax.swing.JPanel
	 */    
	private JPanel getJPanel3() {
		if (jPanel3 == null) {
			jlHdrHint = new JLabel();
			jlHdrHint.setText(Strings.getString("Signatures.sigHdrHint")); //$NON-NLS-1$
			jlHdrHint.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
			jlHdrHint.setForeground(java.awt.Color.blue);
			jlHdrHint.setPreferredSize(new java.awt.Dimension(360,16));
			jlSigType = new JLabel();
			jlSigType.setText(Strings.getString("Signatures.sigType")); //$NON-NLS-1$
			jlSigType.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlSigType.setPreferredSize(new java.awt.Dimension(95,16));
			jLabelAccount = new JLabel();
			jLabelAccount.setText(Strings.getString("accountComboBox"));
			jLabelAccount.setPreferredSize(new java.awt.Dimension(105,16));
			jPanel3 = new JPanel();
			jPanel3.setPreferredSize(new java.awt.Dimension(10,80));
			jPanel3.add(jLabelAccount, null);
			jPanel3.add(getJComboBoxAcnt(), null);
			jPanel3.add(jlSigType, null);
			jPanel3.add(getJcbSigType(), null);
			jPanel3.add(getJcbDefault(), null);
			jPanel3.add(jlHdrHint, null);
		}
		return jPanel3;
	}

	/**
	 * This method initializes jComboBoxAcnt	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJComboBoxAcnt() {
		if (jComboBoxAcnt == null) {
			Object senderAddresses[] = null;
			try {
				senderAddresses = Context.storage.getAccounts(null).toArray();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			Account defAcnt = null;
			try {
				defAcnt = Context.storage.getDefaultAccount();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			if (senderAddresses == null)
				jComboBoxAcnt = new JComboBox();
			else
				jComboBoxAcnt = new JComboBox(senderAddresses);
			if (defAcnt != null)
				jComboBoxAcnt.setSelectedItem(defAcnt);
			jComboBoxAcnt.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(java.awt.event.ItemEvent e) {
					jComboBoxAcnt_itemStateChanged(e);
				}
			});
			jComboBoxAcnt.setPreferredSize(new java.awt.Dimension(240,20));
		}
		return jComboBoxAcnt;
	}
	
	void jComboBoxAcnt_itemStateChanged(java.awt.event.ItemEvent e) {
		listSignatures();
	}

	void initState() {
		signature = null;
		jcbDefault.setSelected(false);
		jepLocalText.setText("");
		stateChanged = false;
		jbApply.setEnabled(false);
	}
	
	public void listSignatures() {
		//String acntEmail = (String) jComboBoxAcnt.getSelectedItem();
		Account acnt = (Account) jComboBoxAcnt.getSelectedItem();
		String sigType = String.valueOf(jcbSigType.getSelectedIndex());
		if(acnt == null || sigType == null)
			return;
		try {			
			initState();			
			//String acntId = Context.storage.getAccountForEmail(acntEmail).id;
			Signature sig = Context.storage.getSignature(acnt.id, sigType);
			if (sig == null) {
				if (sigType.equals("0"))
					sigType = "1";
				else if (sigType.equals("1"))
					sigType = "0";
				sig = Context.storage.getSignature(acnt.id, sigType);
				if(sig != null) {
					if(sig.status.equalsIgnoreCase("D"))
						jcbDefault.setSelected(true);
				}
				return;
			}
			signature = sig;
			if(sig.status.equalsIgnoreCase("D"))
				jcbDefault.setSelected(true);
			if(sig.sigLocal != null)
				jepLocalText.setText(sig.sigLocal);
		} catch (Exception exc) {
			exc.printStackTrace();
			AlertDialog.showErrorDialog(exc);
		}					
	}
	
	/**
	 * This method initializes jcbDefault	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	private JCheckBox getJcbDefault() {
		if (jcbDefault == null) {
			jcbDefault = new JCheckBox();
			jcbDefault.setText(Strings.getString("fieldDefault")); //$NON-NLS-1$
			jcbDefault.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					setStateChanged();
				}
			});
			jcbDefault.setHorizontalTextPosition(javax.swing.SwingConstants.LEADING);
		}
		return jcbDefault;
	}	
	
	public void setStateChanged() {
		if (stateChanged)
			return;
		stateChanged = true;
		jbApply.setEnabled(true);
	}

	/**
	 * This method initializes jcbSigType	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJcbSigType() {
		if (jcbSigType == null) {
			String[] sigTypes = { Strings.getString("Signatures.sigTypeHdr"), //$NON-NLS-1$
					Strings.getString("Signatures.sigTypeFooter"), //$NON-NLS-1$
			};
			jcbSigType = new JComboBox(sigTypes);
			jcbSigType.setPreferredSize(new java.awt.Dimension(180,20));
			jcbSigType.addItemListener(new java.awt.event.ItemListener() { 
				public void itemStateChanged(java.awt.event.ItemEvent e) {    
					listSignatures();
				}
			});
		}
		return jcbSigType;
	}
		
}  //  @jve:decl-index=0:visual-constraint="11,12"
