/*
 * @(#)SqlFrame.java
 *
 * Copyright (C) 2007 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

public class SqlFrame extends JFrame {

	static final long serialVersionUID = 4854804113131435902L;
	
	private final String WIN_ID = "sql_query";	
	
	private JPanel jContentPane = null;

	private JPanel topPanel = null;

	private JScrollPane jScrollPane1 = null;

	private JTable jTable1 = null;

	private JLabel jlSql = null;

	private JTextField jtfSql = null;

	private JButton jbRun = null;

	/**
	 * This is the default constructor
	 */
	public SqlFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setSize(626, 400);
		this.setContentPane(getJContentPane());
		this.setTitle(Strings.getString("SqlFrame.title"));
	}
	
	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}	

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getTopPanel(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getJScrollPane1(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes topPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getTopPanel() {
		if (topPanel == null) {
			jlSql = new JLabel();
			jlSql.setText(Strings.getString("SqlFrame.SQL"));
			topPanel = new JPanel();
			topPanel.setPreferredSize(new java.awt.Dimension(10,35));
			topPanel.add(jlSql, null);
			topPanel.add(getJtfSql(), null);
			topPanel.add(getJbRun(), null);
		}
		return topPanel;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */    
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setViewportView(getJTable1());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes jTable1	
	 * 	
	 * @return javax.swing.JTable	
	 */    
	private JTable getJTable1() {
		if (jTable1 == null) {
			jTable1 = new JTable() {
				static final long serialVersionUID = -7755773025991125503L;
				
				public Component prepareRenderer(TableCellRenderer renderer,
						int rowIndex, int vColIndex) {
					Component c = super.prepareRenderer(renderer, rowIndex,
							vColIndex);
					if (isCellSelected(rowIndex, vColIndex)) {// What else
																// should i put
																// in here to
																// check the
																// value
						c.setBackground(Color.blue);
						c.setForeground(Color.white);
					} else {
						c.setBackground(Color.white);
						c.setForeground(Color.black);
					}
					return c;
				}
				
			};
		}
		return jTable1;
	}

	/**
	 * This method initializes jtfSql	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	private JTextField getJtfSql() {
		if (jtfSql == null) {
			jtfSql = new JTextField();
			jtfSql.setPreferredSize(new java.awt.Dimension(400,20));
		}
		return jtfSql;
	}

	/**
	 * This method initializes jbRun	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJbRun() {
		if (jbRun == null) {
			jbRun = new JButton();
			jbRun.setText(Strings.getString("SqlFrame.runButton"));
			jbRun.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					executeQuery();
				}
			});
		}
		return jbRun;
	}

	private void executeQuery() {
		String sql = Context.trimField(jtfSql);
		if (sql.equals(""))
			return;
		try {
			DefaultTableModel dtm = new DefaultTableModel();
			TableData td = Context.storage.executeQuery(sql);
			dtm.setDataVector(td.data, td.headers);
			jTable1.setModel(dtm);
		} catch (Exception exc) {
			AlertDialog.showErrorDialog(exc);
		}
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
