/*
 * @(#)Storage.java
 *
 * Copyright (C) 2005 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.OutputStream;
import java.sql.DatabaseMetaData;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.mail.Flags;
import javax.mail.Message;
import javax.mail.internet.InternetAddress;
import javax.swing.JProgressBar;
import olivamail.core.common.ComUtils;
import olivamail.core.common.IOUtils;
import olivamail.core.mail.MailUtils;

public abstract class Storage {
	
	public static final int SCHEMA_EMPTY = 0;
	
	public static final int SCHEMA_1_0_3 = 1;
	
	public static final int SCHEMA_1_0_4 = 2;
	
	public static final int SCHEMA_1_1_0 = 3;
	
	public static final int SCHEMA_1_1_1 = 4;
	
	public static final int SCHEMA_1_1_2 = 5;
	
	public static final int SCHEMA_1_1_3 = 6;
	
	public static final int SCHEMA_1_2_0 = 7;
	
	public static final int SCHEMA_1_2_1 = 8;
	
	public static final int SCHEMA_2_0_0 = 9;
	
	public static final int SCHEMA_2_1_0 = 10;	
	
	public static final int SCHEMA_RELEASE = 10;

	protected static final String INS_MESSAGE = "messages (ADDR_FROM, ADDR_TO, ADDR_CC, ADDR_BCC, SUBJECT, SENT_DATE, RECEIVED_DATE, MSG_SIZE, MESSAGE_ID, MSG_UID, HAS_ATT, FOLDER_ID, HDR, CODEPAGE, SRV_HASH, IMPORTANCE, SRC_PLACE, ACNT_ID, SF_ANSW, SF_SEEN, SF_FLAG, SF_DEL, SF_DRAFT, DM_STAT) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";	
	
	protected static final String UPD_MESSAGE = "UPDATE messages SET ADDR_FROM = ?, ADDR_TO = ?, ADDR_CC = ?, ADDR_BCC = ?, SUBJECT = ?, SENT_DATE = ?, RECEIVED_DATE = ?, MSG_SIZE = ?, MESSAGE_ID = ?, MSG_UID = ?, HAS_ATT = ?, FOLDER_ID = ?, HDR = ?, CODEPAGE = ?, SRV_HASH = ?, IMPORTANCE = ?, SRC_PLACE = ?, ACNT_ID = ?, SF_ANSW = ?, SF_SEEN = ?, SF_FLAG = ?, SF_DEL = ?, SF_DRAFT = ?, DM_STAT = ? WHERE ID = ?";
	
	protected static final String SEL_MESSAGE = "select ID, ADDR_FROM, ADDR_TO, ADDR_CC, ADDR_BCC, SUBJECT, SENT_DATE, RECEIVED_DATE, MSG_SIZE, MESSAGE_ID, MSG_UID, HAS_ATT, FOLDER_ID, HDR, CODEPAGE, SRV_HASH, IMPORTANCE, SRC_PLACE, ACNT_ID, SF_ANSW, SF_SEEN, SF_FLAG, SF_DEL, SF_DRAFT, DM_STAT from ";

	protected static final String SEL_FOLDER = "select ID, NAME, FLD_ID, FTYPE from ";	
	
	protected static final String SEL_IFOLDER = "SELECT ID, NAME, FLD_ID, FULL_NAME, SUBSCR, SYNC_MODE, IMAP_TYPE, UIDVAL, ACNT_ID, DM_STAT FROM ";
	
	protected static final String SEL_ACCOUNT = "select ID, NAME, PERSON, EMAIL, REPLY_TO, SERVER_TYPE, INCOMING_SERVER, OUTGOING_SERVER, SMTP_AUTH, IN_USER, IN_PASS, OUT_USER, OUT_PASS, STATUS, IN_PORT, OUT_PORT, IN_MAX_SIZE, IN_LEAVE_DAYS, HWM_MSG_UID, USE_RULES, USE_SIG, IN_SSL, OUT_SSL, ORGANIZATION, I_SH_UNS, HWM_MSG_ID, DM_STAT from ";		
	
	public synchronized static int getReleaseVersion() {
		return SCHEMA_RELEASE;
	}
	
	protected int dbMessageMaxSize = -1;
	
	protected String msgsDir = null;
	
	protected Connection mConn = null;

	protected String mUser = null;

	protected String mPass = null;

	protected String mConnString = null;

	protected String mSchema = null;

	public abstract int logon(String user, String pass) throws Exception;	
	
	public abstract void populateSchema(int schemaVer) throws Exception;

	public abstract int copyMimeTypes(String fromMimeGroupId, String toMimeGroupId) throws Exception;			
	
	protected abstract String getLastInsertedId() throws Exception;			

	protected abstract boolean isIdAutoincremented();		
	
	protected abstract String getId() throws Exception;						

	public abstract void close() throws Exception;

	public abstract javax.mail.internet.MimeMessage loadMessage(String id)
			throws Exception;

	public abstract java.io.InputStream loadMessageInputStream(String id)
			throws Exception;

	public abstract void deleteMessage(String id, boolean checkOnlySrc) throws Exception;		
					
	public synchronized String getDatabaseProductName() throws Exception {
		DatabaseMetaData dbmd = mConn.getMetaData();		 		
		return dbmd.getDatabaseProductName();
	}

	public synchronized String getDatabaseProductVersion() throws Exception {
		DatabaseMetaData dbmd = mConn.getMetaData();		 		
		return dbmd.getDatabaseProductVersion();
	}
	
	public synchronized String getDriverName() throws Exception {
		DatabaseMetaData dbmd = mConn.getMetaData();		 		
		return dbmd.getDriverName();
	}
	
	public synchronized String getDriverVersion() throws Exception {
		DatabaseMetaData dbmd = mConn.getMetaData();		 		
		return dbmd.getDriverVersion();
	}	
	
	public synchronized void deleteGroup(String id) throws Exception {
		try {
			PreparedStatement ps = mConn
					.prepareStatement("delete from " + mSchema
							+ "grp_addr where GRP_ID = ?");
			ps.setString(1, id);
			ps.execute();
			ps.close();
			ps = mConn.prepareStatement("delete from " + mSchema
					+ "groups where ID = ?");
			ps.setString(1, id);
			ps.execute();
			ps.close();

			mConn.commit();
		} catch (Exception e) {
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			throw e;
		}
	}

	public synchronized void deleteLostImapMessages(Hashtable htuids, IFolder iFolder) throws Exception {				
		PreparedStatement ps = mConn.prepareStatement("select ID, MSG_UID, SRV_HASH from " + mSchema + "messages where FOLDER_ID = ?");
		ps.setString(1, iFolder.getId());
		ResultSet rs = ps.executeQuery();
		while(rs.next()) {
			String id = rs.getString(1);
			String msgUid = rs.getString(2);
			String srvHash = rs.getString(3);			
			if(!ComUtils.nvl(iFolder.uidVal).equals(srvHash) || !htuids.containsKey(msgUid)) {
				deleteMessage(id, false);
			}
		}
	}	
	
	public synchronized Folder saveFolder(
			Folder folder) throws Exception {
		saveFolderNC(folder);
		mConn.commit();
		return folder;
	}	
	
	protected Folder saveFolderNC(
			Folder folder) throws Exception {		
		PreparedStatement ps = null;
		if (folder.getId() == null || folder.getId().equals("")) {
			String sql = null;
			int id1 = 0;
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"folders (NAME, FLD_ID, FTYPE) values (?, ?, ?)")
						.toString();
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"folders (ID, NAME, FLD_ID, FTYPE) values (?, ?, ?, ?)")
						.toString();
			}			
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				folder.setId(getId());
				ps.setString(1, folder.getId());
			}
			ps.setString(1 + id1, folder.getName());
			ps.setString(2 + id1, folder.getFldId());
			ps.setString(3 + id1, String.valueOf(folder.getFType()));			
		} else {
			ps = mConn.prepareStatement("update " + mSchema
					+ "folders set NAME = ?, FLD_ID = ?, FTYPE = ? where ID = ?");
			ps.setString(1, folder.getName());
			ps.setString(2, folder.getFldId());
			ps.setString(3, String.valueOf(folder.getFType()));
			ps.setString(4, folder.getId());
		}
		ps.execute();
		ps.close();
		if (folder.getId() == null || folder.getId().equals("")) {
			folder.setId(getLastInsertedId());
		}
//		mConn.commit();
		return folder;
	}
	
	public synchronized Folder getFolder(String id) throws Exception {
		Folder folder = null;
		PreparedStatement ps = mConn.prepareStatement(SEL_FOLDER + mSchema
				+ "folders where ID = ?");
		ps.setString(1, id);
		ResultSet resultset = ps.executeQuery();
		if (resultset.next()) {
			folder = selectFolder(resultset);
		}
		resultset.close();
		ps.close();

		return folder;
	}
	
	public synchronized Vector getAddressList1(String searchStr, String orderBy, int limit,
			String grpId, int variant) throws Exception {
		String sqlQuery = null;
		if (variant == 0) {
			if (grpId != null) {
				sqlQuery = ((new StringBuffer(
						"select ID, FNAME, LNAME, EMAIL from "))
						.append(mSchema).append(
								"addresses a where not exists (select 1 from ")
						.append(mSchema)
						.append("grp_addr where GRP_ID = ? and ADDR_ID = a.ID)"))
						.toString();
			} else {
				sqlQuery = ((new StringBuffer(
						"select ID, FNAME, LNAME, EMAIL from "))
						.append(mSchema).append("addresses")).toString();
			}
		} else if (variant == 1) {
			if (grpId == null)
				return null;
			sqlQuery = (new StringBuffer(
					"select a.ID, a.FNAME, a.LNAME, a.EMAIL, 0 from "))
					.append(mSchema).append("addresses a, ").append(mSchema)
					.append("grp_addr g where GRP_ID = ? and g.ADDR_ID = a.ID")
					.toString();
		}
		String whereClause = "";
		if (searchStr != null) {
			searchStr = searchStr.toUpperCase();
			whereClause = String.valueOf(String.valueOf(new StringBuffer(
					" where ").append(searchStr)));
		}
		if (orderBy == null)
			orderBy = "";
		if (orderBy.length() == 0)
			orderBy = "id desc";
		String orderByClause = " order by " + orderBy;
		sqlQuery += whereClause + orderByClause;
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		if (grpId != null)
			ps.setString(1, grpId);
		ResultSet rs = ps.executeQuery();
		Vector data = new Vector(100);
		int count = 0;
		while (rs.next()) {
			count++;
			if (limit != -1 && count > limit)
				break;
			Vector row = new Vector(3);
			row.add(rs.getString(1));
			String fname = ComUtils.nvl(rs.getString(2));
			String lname = ComUtils.nvl(rs.getString(3));
			if(fname.length() == 0 || lname.length() == 0)
				row.add(fname + lname);
			else
				row.add(fname + " " + lname);
			row.add(rs.getString(4));
			data.add(row);
		}

		rs.close();
		return data;
	}	
		
	public synchronized Vector getContactsList(String searchStr,
			String orderBy, int limit) throws Exception {
		StringBuffer sb = new StringBuffer(
				"select ID, FNAME, LNAME, EMAIL, 0 from ");
		sb.append(mSchema).append("addresses");
		if (searchStr != null && !searchStr.equals("")) {
			sb.append(" where UPPER(FNAME || ' ' || LNAME) like '%").append(
					searchStr.toUpperCase()).append("%'").append(
					" or UPPER(EMAIL) like '%").append(searchStr.toUpperCase())
					.append("%'");
		}
		sb.append(" union all select ID, NAME, '', '', 1 from ")
				.append(mSchema).append("groups");
		if (searchStr != null && !searchStr.equals("")) {
			sb.append(" where UPPER(NAME) like '%").append(
					searchStr.toUpperCase()).append("%'");
		}
		PreparedStatement ps = mConn.prepareStatement(sb.toString());
		ResultSet rs = ps.executeQuery();
		Vector data = new Vector(100);
		int count = 0;
		while (rs.next()) {
			count++;
			if (limit != -1 && count > limit)
				break;
			Vector row = new Vector(4);
			row.add(rs.getString(1));
			String fname = ComUtils.nvl(rs.getString(2));
			String lname = ComUtils.nvl(rs.getString(3));
			if(fname.length() == 0 || lname.length() == 0)
				row.add(fname + lname);
			else
				row.add(fname + " " + lname);
			row.add(rs.getString(4));
			row.add(rs.getString(5));
			data.add(row);
		}

		rs.close();
		return data;
	}
		
	public synchronized Folder[] getFoldersChildren(String id) throws Exception {		
		Vector result = new Vector(); 
		PreparedStatement ps = mConn
				.prepareStatement(SEL_FOLDER + mSchema
						+ "folders where FLD_ID = ?");
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		while (rs.next()) {			
			result.add(selectFolder(rs));
		}
		rs.close();
		ps.close();

		Folder[] folders = new Folder[result.size()];
		Enumeration en = result.elements();
		int i = 0;
		while (en.hasMoreElements()) {
			folders[i] = (Folder) en.nextElement();
			i++;
		}		
		return folders;
	}
	
	public synchronized IFolder[] getImapFoldersChildren(String id)
			throws Exception {
		Vector result = new Vector(); 
		PreparedStatement ps = mConn.prepareStatement(SEL_IFOLDER + mSchema
				+ "imap_flds where FLD_ID = ?");
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		while (rs.next()) {
			result.add(selectImapFolder(rs));
		}
		rs.close();
		ps.close();

		IFolder[] iFolders = new IFolder[result.size()];
		Enumeration en = result.elements();
		int i = 0;
		while (en.hasMoreElements()) {
			iFolders[i] = (IFolder) en.nextElement();
			i++;
		}				
		
		return iFolders;
	}
		
	public synchronized olivamail.gui.Group getGroup(String id) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID, NAME, COMMENTS from "
						+ mSchema + "groups where id = ?");
		ps.setString(1, id);
		ResultSet resultset = ps.executeQuery();
		olivamail.gui.Group grp = null;
		if (resultset.next()) {
			grp = new olivamail.gui.Group();
			grp.id = resultset.getString(1);
			grp.name = resultset.getString(2);
			grp.comments = resultset.getString(3);
		}
		ps.close();
		resultset.close();
		return grp;
	}
		
	public synchronized String getGroupAddresses(String id) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("select a.EMAIL from "
				+ mSchema + "addresses a, " + mSchema + "grp_addr ga where ga.GRP_ID = ? and ga.ADDR_ID = a.ID");
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		StringBuffer sb = new StringBuffer();
		while (rs.next()) {
			sb.append(rs.getString(1)).append(", ");
		}
		ps.close();
		rs.close();
		return sb.toString();
	}
		
	public synchronized String getLocalFolder() throws Exception {
		return getOrCreateFolderId("__LOCAL");
	}	
	
	public synchronized String getInboxFolder() throws Exception {
		return getOrCreateFolderId("__INBOX");
	}

	public synchronized String getOutboxFolder() throws Exception {
		return getOrCreateFolderId("__OUTBOX");
	}

	public synchronized String getDraftFolder() throws Exception {
		return getOrCreateFolderId("__DRAFT");
	}

	public synchronized String getSentFolder() throws Exception {
		return getOrCreateFolderId("__SENT");
	}

	public synchronized String getTrashFolder() throws Exception {
		return getOrCreateFolderId("__TRASH");
	}
		
	public synchronized olivamail.gui.Group saveGroup(Group grp, String[] addrs) throws Exception {
		try {
			PreparedStatement ps = null;
			if (grp.id == null || grp.id.equals("")) {
				String sql = null;
				int id1 = 0;
				if (isIdAutoincremented()) {
					sql = (new StringBuffer("insert into "))
							.append(mSchema)
							.append(
									"groups (NAME, COMMENTS) values (?, ?)")
							.toString();
				} else {
					id1 = 1;
					sql = (new StringBuffer("insert into "))
							.append(mSchema)
							.append(
									"groups (ID, NAME, COMMENTS) values (?, ?, ?)")
							.toString();
				}			
				ps = mConn.prepareStatement(sql);
				if (!isIdAutoincremented()) {
					grp.id = getId();
					ps.setString(1, grp.id);
				}
				ps.setString(1 + id1, grp.name);
				ps.setString(2 + id1, grp.comments);
			} else {
				ps = mConn.prepareStatement("update " + mSchema
						+ "groups set NAME = ?, COMMENTS = ? where ID = ?");
				ps.setString(1, grp.name);
				ps.setString(2, grp.comments);
				ps.setString(3, grp.id);
			}
			ps.execute();
			ps.close();
			
			if (grp.id == null || grp.id.length() == 0) {
				grp.id = getLastInsertedId();
			}

			ps = mConn.prepareStatement("delete from " + mSchema
					+ "grp_addr where GRP_ID = ?");
			ps.setString(1, grp.id);
			ps.execute();
			ps.close();

			if (addrs != null) {
				ps = mConn.prepareStatement("insert into "
						+ mSchema + "grp_addr (GRP_ID, ADDR_ID) values (?, ?)");
				for (int i = 0; i < addrs.length; i++) {
					ps.setString(1, grp.id);
					ps.setString(2, addrs[i]);
					ps.execute();
				}
				ps.close();
			}
			
			mConn.commit();
		} catch (Exception e) {
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			throw e;
		}
		return grp;
	}
		
	/*
	 * (non-Javadoc)
	 * 
	 * @see olivamail.gui.Storage#saveAddress(olivamail.gui.Address)
	 */
	public synchronized olivamail.gui.Address saveAddress(Address addr) throws Exception {
		PreparedStatement ps = null;
		if(addr.id == null || addr.id.equals("")) {
			String sql = null;
			int id1 = 0;
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"addresses (FNAME, LNAME, MNAME, EMAIL, HOME_COUNTRY, HOME_REGION, HOME_CITY, HOME_INDEX, HOME_STREET, HOME_PHONE, MOBILE_PHONE) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
						.toString();
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"addresses (ID, FNAME, LNAME, MNAME, EMAIL, HOME_COUNTRY, HOME_REGION, HOME_CITY, HOME_INDEX, HOME_STREET, HOME_PHONE, MOBILE_PHONE) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
						.toString();
			}			
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				addr.id = getId();
				ps.setString(1, addr.id);
			}
			ps.setString(1 + id1, addr.fname);
			ps.setString(2 + id1, addr.lname);
			ps.setString(3 + id1, addr.mname);
			ps.setString(4 + id1, addr.email);
			ps.setString(5 + id1, addr.homeCountry);
			ps.setString(6 + id1, addr.homeRegion);
			ps.setString(7 + id1, addr.homeCity);
			ps.setString(8 + id1, addr.homeIndex);
			ps.setString(9 + id1, addr.homeStreet);
			ps.setString(10 + id1, addr.homePhone);
			ps.setString(11 + id1, addr.mobilePhone);
		} else {
			ps = mConn
					.prepareStatement("update "
							+ mSchema
							+ "addresses set FNAME = ?, LNAME = ?, MNAME = ?, EMAIL = ?, HOME_COUNTRY = ?, HOME_REGION = ?, HOME_CITY = ?, HOME_INDEX = ?, HOME_STREET = ?, HOME_PHONE = ?, MOBILE_PHONE = ? where ID = ?");
			ps.setString(1, addr.fname);
			ps.setString(2, addr.lname);
			ps.setString(3, addr.mname);
			ps.setString(4, addr.email);
			ps.setString(5, addr.homeCountry);
			ps.setString(6, addr.homeRegion);
			ps.setString(7, addr.homeCity);
			ps.setString(8, addr.homeIndex);
			ps.setString(9, addr.homeStreet);
			ps.setString(10, addr.homePhone);
			ps.setString(11, addr.mobilePhone);
			ps.setString(12, addr.id);
		}
		ps.execute();
		ps.close();
		if(addr.id == null || addr.id.equals(""))
			addr.id = getLastInsertedId();
		mConn.commit();
		return addr; 
	}
		
	public synchronized Vector getFolderList() throws Exception {
		String localFldId = getLocalFolder();
		PreparedStatement ps = mConn
				.prepareStatement(SEL_FOLDER + mSchema + "folders WHERE ID <> ? AND FTYPE = '" + Folder.FTYPE_LOCAL + "'");
		ps.setString(1, localFldId);
		ResultSet resultset = ps.executeQuery();
		Vector rows = new Vector();
		while (resultset.next()) {
			rows.add(selectFolder(resultset));
		}
		resultset.close();
		ps.close();
		return rows;
	}

	public synchronized Vector getAllFolderList() throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement(SEL_FOLDER + mSchema + "folders");
		ResultSet resultset = ps.executeQuery();
		Vector rows = new Vector(10);
		while (resultset.next()) {
			rows.add(selectFolder(resultset));
		}
		resultset.close();
		ps.close();
		return rows;
	}
		
	public synchronized int getSchemaVersion() throws Exception {
		int result = Storage.SCHEMA_EMPTY;
				
		DatabaseMetaData dbmd = mConn.getMetaData();		 
		ResultSet resultset = dbmd.getTables(null, null, "settings", new String[]{"TABLE"});
		if (resultset.next()) {
			result = Storage.SCHEMA_1_0_3;
		}
		resultset.close();
		resultset = dbmd.getTables(null, null, "SETTINGS", new String[]{"TABLE"});
		if (resultset.next()) {
			result = Storage.SCHEMA_1_0_3;
		}
		resultset.close();
		
		if(result == Storage.SCHEMA_EMPTY)
			return result;
		
		String schemaVer = getSettingsValue("schema_version");
		if(schemaVer == null)
			return result;

		if (schemaVer.equals("2.1.0"))
			result = Storage.SCHEMA_2_1_0;		
		if (schemaVer.equals("2.0.0"))
			result = Storage.SCHEMA_2_0_0;
		else if (schemaVer.equals("1.2.1"))
			result = Storage.SCHEMA_1_2_1;
		else if (schemaVer.equals("1.2.0"))
			result = Storage.SCHEMA_1_2_0;
		else if (schemaVer.equals("1.1.3"))
			result = Storage.SCHEMA_1_1_3;
		else if (schemaVer.equals("1.1.2"))
			result = Storage.SCHEMA_1_1_2;
		else if (schemaVer.equals("1.1.1"))
			result = Storage.SCHEMA_1_1_1;
		else if (schemaVer.equals("1.1.0"))
			result = Storage.SCHEMA_1_1_0;
		else if (schemaVer.equals("1.0.4"))
			result = Storage.SCHEMA_1_0_4;
		
		return result;		
		
/*		
		resultset = dbmd.getTables(null, null, "grp_addr", new String[]{"TABLE"});
		if (resultset.next()) {
			schemaVer = getSettingsValue("schema_version");			
			if(schemaVer != null && schemaVer.equals("1.0.4"))
				result = Storage.SCHEMA_1_0_4;
		}
		resultset.close();		
		resultset = dbmd.getTables(null, null, "GRP_ADDR", new String[]{"TABLE"});
		if (resultset.next()) {
			schemaVer = getSettingsValue("schema_version");			
			if(schemaVer != null && schemaVer.equals("1.0.4"))
				result = Storage.SCHEMA_1_0_4;
		}
		resultset.close();
*/		
	}
	
	public synchronized void execSqlArray(String[] sqls) throws Exception {
		for (int i = 0; i < sqls.length; i++) {
			java.sql.Statement stmt = mConn.createStatement();
			System.out.println("Executing statement:");
			System.out.println("\t" + sqls[i]);
			System.out.flush();
			stmt.execute(sqls[i]);
			System.out.println("OK!");
			System.out.flush();
			stmt.close();
		}
	}

	public synchronized Vector getMessageList(String folderId,
			String searchStr, String orderBy, int limit) throws Exception {
		return getMessageList(folderId, searchStr, orderBy, limit, null, null, null);
	}
	
	public synchronized Vector getMessageList(String folderId,
			String searchStr, String orderBy, int limit,
			ContentSearchEngine cse, String contentSearchText,
			JProgressBar progressBar) throws Exception {
		StringBuffer sb = new StringBuffer(SEL_MESSAGE);						
		sb.append(mSchema).append("messages");
		StringBuffer whereClause = new StringBuffer(" where HDR != '"
				+ MessageRow.HDR_DELETED + "'");
		if (folderId != null)
			whereClause.append(" and FOLDER_ID = ?");
		if (searchStr != null)
			whereClause.append(" and (").append(searchStr).append(")");
		sb.append(whereClause);		
/*		
 		StringBuffer whereClause = new StringBuffer(); 
		if (folderId != null)
			whereClause.append(" where FOLDER_ID = ?");
		if (searchStr != null) {
			if (whereClause.length() == 0)
				whereClause.append(" where ");
			else
				whereClause.append(" and ");
			whereClause.append("(").append(searchStr).append(")");
		}
		if(whereClause.length() > 0)
			sb.append(whereClause);
*/			
		if (orderBy == null || orderBy.equals(""))
			orderBy = "id desc";	
		String orderByClause = " order by " + orderBy;
		sb.append(orderByClause);
		
		PreparedStatement ps = mConn.prepareStatement(sb.toString());		
		if(folderId != null)
			ps.setString(1, folderId);
		
		ResultSet rs = ps.executeQuery();
		Vector data = new Vector(50);
		if(cse != null && contentSearchText != null)
			limit = -1;
		
		int progressMax = 30;
		if(progressBar != null)
			progressBar.setMaximum(progressMax);
		
		int count = 0;
		while (rs.next()) {
			count++;
			if (limit != -1 && count > limit)
				break;			
			
			MessageRow mr = selectMessage(rs);
			
			if(cse != null && contentSearchText != null) {
				if(mr.hdr == MessageRow.HDR_COMPLETE) {
					if(cse.searchContent(this.loadMessage(mr.id), mr.codepage, contentSearchText))
						data.add(mr);
				}
			} else 			
				data.add(mr);
		
			if (progressBar != null) {
				if (count > progressMax) {
					progressMax *= 1.5;
					progressBar.setMaximum(progressMax);
				}
				progressBar.setValue(count);
			}
		}
		if (progressBar != null)
			progressBar.setValue(progressMax);

		rs.close();
		ps.close();
		return data;
	}	
	
	public synchronized MessageRow getMessageRow(String id) throws Exception {
		String sqlQuery = (new StringBuffer(SEL_MESSAGE)).append(mSchema).append("messages where ID = ?").toString();
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, id);
		return selectMessage(ps);
	}	
	
	public synchronized MessageRow getMessageByUid(String uid, 
			long uidValidity, String hwmId, Folder folder) throws Exception {
		String hwmStr = "";
		if (hwmId != null)
			hwmStr = " and ID > " + String.valueOf(hwmId);
		StringBuffer sb = new StringBuffer(SEL_MESSAGE);
		sb.append(mSchema).append("messages where MSG_UID = ? and SRV_HASH = ?").append(hwmStr);
		if(folder != null)
			sb.append(" and folder_id = ?");
		String sqlQuery = sb.toString();
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, uid);
		ps.setLong(2, uidValidity);
		if(folder != null)
			ps.setString(3, folder.getId());
		return selectMessage(ps);
	}	
		
	protected MessageRow selectMessage(PreparedStatement ps) throws Exception {
		ResultSet rs = ps.executeQuery();
		MessageRow mr = null;
		if (rs.next()) {
			mr = selectMessage(rs);
		}
		rs.close();
		ps.close();
		return mr;
	}	

	protected MessageRow selectMessage(ResultSet rs) throws Exception {
		MessageRow mr = new MessageRow();
		mr.id = rs.getString(1);
		mr.addrFrom = rs.getString(2);
		mr.addrTo = rs.getString(3);
		mr.addrCc = rs.getString(4);
		mr.addrBcc = rs.getString(5);
		mr.subject = rs.getString(6);
		mr.sentDate = rs.getString(7);
		mr.receivedDate = rs.getString(8);
		mr.msgSize = rs.getInt(9);
		mr.messageId = rs.getString(10);
		mr.msgUid = rs.getString(11);
		mr.hasAtt = rs.getString(12);
		mr.folderId = rs.getString(13);
		mr.hdr = rs.getString(14).charAt(0);
		mr.codepage = rs.getString(15);
		mr.uidValidity = rs.getLong(16);		
		mr.importance = rs.getString(17);
		mr.srcPlace = rs.getString(18);
		mr.acntId = rs.getString(19);
		mr.sfAnsw = rs.getBoolean(20);
		mr.sfSeen = rs.getBoolean(21);
		mr.sfFlag = rs.getBoolean(22);
		mr.sfDel = rs.getBoolean(23);
		mr.sfDraft = rs.getBoolean(24);
		mr.dmStat = rs.getString(25).charAt(0);
		return mr;		
	}		
	
	public synchronized String getMessageCodepage(String id) throws Exception {
		String sqlQuery = String.valueOf(String.valueOf((new StringBuffer(
				"select CODEPAGE from ")).append(mSchema).append(
				"messages where ID = ?")));
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		String codepage = "koi8-r";
		if (rs.next()) {
			codepage = rs.getString(1);
		}
		ps.close();
		rs.close();
		return codepage;
	}

	public synchronized String getSettingsValue(String name) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("Select ID, VALUE from " + mSchema + "settings where NAME = ?");
		ps.setString(1, name);
		ResultSet rs = ps.executeQuery();
		String value = null;
		if (rs.next()) {
			value = rs.getString(2);
		}
		ps.close();
		rs.close();
		return value;
	}

	public synchronized void setDatabaseUrl(String URL) {
		mConnString = URL;
	}

	public synchronized void setSchema(String schema) {
		mSchema = schema + ".";
	}

	public synchronized boolean isConnected() throws Exception {
		if(mConn == null)
			return false;
		if(mConn.isClosed())
			return false;
		return true;
	}	

	public synchronized Vector getAddressList(String searchStr, String orderBy, int limit)
			throws Exception {
		String sqlQuery = String
				.valueOf(String
						.valueOf((new StringBuffer(
								"select ID, FNAME, LNAME, MNAME, EMAIL, HOME_COUNTRY, HOME_REGION, HOME_CITY, HOME_INDEX, HOME_STREET, HOME_PHONE, MOBILE_PHONE from "))
								.append(mSchema).append("addresses")));
		String whereClause = "";
		if (searchStr != null) {
			searchStr = searchStr.toUpperCase();
			whereClause = String.valueOf(String.valueOf(new StringBuffer(
					" where ").append(searchStr)));
		}
		if (orderBy == null)
			orderBy = "";
		if (orderBy.length() == 0)
			orderBy = "id desc";
		String orderByClause = " order by " + orderBy;
		sqlQuery += whereClause + orderByClause;
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ResultSet rs = ps.executeQuery();
		Vector data = new Vector(100);
		int count = 0;
		while (rs.next()) {
			count++;
			if (limit != -1 && count > limit)
				break;
			Vector row = new Vector(7);
			for (int i = 1; i <= 12; i++) {
				row.add(rs.getString(i));
			}
			data.add(row);
		}

		rs.close();
		return data;
	}

/*	
	public synchronized void setAcntHwmUid(String id, String hwmMsgUid) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("update " + mSchema
				+ "accounts set HWM_MSG_UID = ? where ID = ?");
		ps.setString(1, hwmMsgUid);
		ps.setString(2, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}

	public synchronized void setAcntHwmId(String id, String hwmMsgId) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("update " + mSchema
				+ "accounts set HWM_MSG_ID = ? where ID = ?");
		ps.setString(1, hwmMsgId);
		ps.setString(2, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}

	public synchronized void setAcntShowUnsubscr(String id, String showUnsubscr) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("update " + mSchema
				+ "accounts set I_SH_UNS = ? where ID = ?");
		ps.setString(1, showUnsubscr);
		ps.setString(2, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}	
		
	public synchronized String getAcntHwmUid(String id) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("select HWM_MSG_UID from " + mSchema
				+ "accounts where ID = ?");
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		String result = null;
		if(rs.next())
			result = rs.getString(1);
		rs.close();
		ps.close();
		return result;
	}	

	public synchronized String getAcntHwmId(String id) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("select HWM_MSG_ID from " + mSchema
				+ "accounts where ID = ?");
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		String result = null;
		if(rs.next())
			result = rs.getString(1);
		rs.close();
		ps.close();
		return result;
	}
*/

	public synchronized Vector getImapAccountList() throws Exception {
		Vector result = new Vector();
		PreparedStatement ps = mConn
				.prepareStatement(SEL_ACCOUNT
						+ mSchema + "accounts where SERVER_TYPE = 'imap'");
		ResultSet rs = ps.executeQuery();
		while (rs.next())
			result.add(selectAccount(rs));
		rs.close();
		ps.close();
		return result;
	}
				
	public synchronized Vector getAccountList(String searchStr, String orderBy)
			throws Exception {
		String sqlQuery = (new StringBuffer("select ID, NAME, STATUS from "))
				.append(mSchema).append("accounts").toString();
		if (searchStr != null)
			sqlQuery += " where " + searchStr;
		if (orderBy == null)
			orderBy = "id";
		sqlQuery += " order by " + orderBy;
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ResultSet rs = ps.executeQuery();
		Vector data = new Vector(100);
		while (rs.next()) {
			Vector row = new Vector(3);
			for (int i = 1; i <= 3; i++)
				row.add(rs.getString(i));
			data.add(row);
		}
		rs.close();
		ps.close();
		return data;
	}

/*	
	public synchronized Vector getSenderAddresses() throws Exception {
		String sql = "select ID, NAME, PERSON, EMAIL, REPLY_TO, SERVER_TYPE, INCOMING_SERVER, OUTGOING_SERVER, SMTP_AUTH, IN_USER, IN_PASS, OUT_USER, OUT_PASS, STATUS, IN_PORT, OUT_PORT, IN_MAX_SIZE, IN_LEAVE_DAYS, HWM_MSG_UID, USE_RULES, USE_SIG, IN_SSL, OUT_SSL, ORGANIZATION from "
				+ mSchema + "accounts";
		PreparedStatement ps = mConn.prepareStatement(sql);
		ResultSet resultset = ps.executeQuery();
		Vector addresses = new Vector(10);
		while (resultset.next()) {
			addresses.add(selectAccount(resultset));
		}
		resultset.close();
		ps.close();
		return addresses;
	}
*/	
		
	public synchronized boolean hasMessage(String msgUID) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select 1 from " + mSchema
						+ "messages where msg_uid = ?");
		ps.setString(1, msgUID);
		ResultSet resultset = ps.executeQuery();
		String res = "0";
		if (resultset.next()) {
			res = resultset.getString(1);
		}
		resultset.close();
		ps.close();
		if (res.compareTo("1") == 0)
			return true;
		return false;
	}

	protected olivamail.gui.Account selectAccount(ResultSet resultset)
			throws Exception {
		olivamail.gui.Account acnt = null;
		acnt = new olivamail.gui.Account();
		acnt.id = resultset.getString(1);
		acnt.name = resultset.getString(2);
		acnt.person = resultset.getString(3);
		acnt.email = resultset.getString(4);
		acnt.replyTo = resultset.getString(5);
		acnt.serverType = resultset.getString(6);
		acnt.incomingServer = resultset.getString(7);
		acnt.outgoingServer = resultset.getString(8);
		acnt.smtpAuth = resultset.getString(9);
		acnt.inUser = resultset.getString(10);
		acnt.inPass = resultset.getString(11);
		acnt.outUser = resultset.getString(12);
		acnt.outPass = resultset.getString(13);
		acnt.status = resultset.getString(14).charAt(0);
		acnt.inPort = resultset.getInt(15);
		acnt.outPort = resultset.getInt(16);
		acnt.inMaxSize = resultset.getInt(17);
		acnt.inLeaveDays = resultset.getInt(18);
		acnt.hwmMsgUid = resultset.getString(19);
		acnt.useRules = resultset.getString(20);
		acnt.useSig = resultset.getString(21);
		acnt.inSSL = resultset.getString(22);
		acnt.outSSL = resultset.getString(23);
		acnt.organization = resultset.getString(24);
		acnt.iShowUnsubscr = resultset.getString(25);
		acnt.hwmMsgId = resultset.getString(26);
		acnt.dmStat = resultset.getString(27).charAt(0);
		return acnt;
	}

	protected olivamail.gui.Account selectAccount(PreparedStatement ps)
			throws Exception {
		ResultSet resultset = ps.executeQuery();
		olivamail.gui.Account acnt = null;
		if (resultset.next())
			acnt = selectAccount(resultset);
		resultset.close();
		ps.close();
		return acnt;
	}	
	
	public synchronized olivamail.gui.Account getDefaultAccount()
			throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement(SEL_ACCOUNT
						+ mSchema + "accounts where STATUS = 'D'");
		
		return selectAccount(ps);
	}

	public synchronized olivamail.gui.Account getAccount(String id)
			throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement(SEL_ACCOUNT
						+ mSchema + "accounts where id = ?");
		ps.setString(1, id);
		return selectAccount(ps);
	}	

	public synchronized olivamail.gui.Rule getRule(String id) throws Exception {
		String sqlQuery = String
				.valueOf((new StringBuffer(
						"select ID, HEADER, OPERAND, VALUE, ACTION, PARAM, ACNT_ID from "))
						.append(mSchema).append("rules where ID = ?")
						.toString());
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		Rule rule = null;
		if (rs.next()) {
			rule = new Rule();
			rule.id = rs.getString(1);
			rule.header = rs.getString(2);
			rule.operand = rs.getInt(3);
			rule.value = rs.getString(4);
			rule.action = rs.getInt(5);
			rule.param = rs.getString(6);
			rule.acntId = rs.getString(7);
		}
		rs.close();
		ps.close();
		return rule;
	}

	public synchronized Signature getSignature(String acntId, String sigType) throws Exception {
		String sqlQuery = String.valueOf((new StringBuffer(
				"select ACNT_ID, SIG_TYPE, SIG_LOCAL, STATUS from "))
				.append(mSchema).append("signatures where ACNT_ID = ? and SIG_TYPE = ?")
				.toString());
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, acntId);
		ps.setString(2, sigType);
		ResultSet rs = ps.executeQuery();
		Signature sig = null;
		if (rs.next()) {
			sig = new Signature();
			sig.acntId = rs.getString(1);
			sig.sigType = rs.getString(2);
			sig.sigLocal = rs.getString(3);
			sig.status = rs.getString(4);
		}
		rs.close();
		ps.close();
		return sig;
	}	
	
	public synchronized Signature getSignatureToUse(String acntId, String sigType) throws Exception {
		String sqlQuery = String
		.valueOf((new StringBuffer(
				"select USE_SIG from "))
				.append(mSchema).append("accounts where ID = ?")
				.toString());
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, acntId);
		ResultSet rs = ps.executeQuery();
		String useSig = null;
		if (rs.next()) {
			useSig = rs.getString(1);
		}
		rs.close();
		ps.close();		
		if(useSig == null || useSig.equals("0"))
			return null;
		if(useSig.equals("1")) {
			sqlQuery = String.valueOf((new StringBuffer(
					"select ACNT_ID, SIG_TYPE, SIG_LOCAL, STATUS from "))
					.append(mSchema).append("signatures where ACNT_ID = ? and SIG_TYPE = ?")
					.toString());
			ps = mConn.prepareStatement(sqlQuery);
			ps.setString(1, acntId);
			ps.setString(2, sigType);
		} else if (useSig.equals("2")) {
			sqlQuery = String.valueOf((new StringBuffer(
					"select ACNT_ID, SIG_TYPE, SIG_LOCAL, STATUS from "))
					.append(mSchema).append("signatures where STATUS = 'D' and SIG_TYPE = ?")
					.toString());
			ps = mConn.prepareStatement(sqlQuery);
			ps.setString(1, sigType);
		} else {
			return null;
		}
		rs = ps.executeQuery();
		Signature sig = null;
		if (rs.next()) {
			sig = new Signature();
			sig.acntId = rs.getString(1);
			sig.sigType = rs.getString(2);
			sig.sigLocal = rs.getString(3);
			sig.status = rs.getString(4);
		}
		rs.close();
		ps.close();
		return sig;
	}
	
	public synchronized Signature saveSignature(Signature sig) throws Exception {
		try {
			String sql = (new StringBuffer("select 1 from ")).append(mSchema)
					.append("signatures where ACNT_ID = ? and SIG_TYPE = ?")
					.toString();
			PreparedStatement ps = mConn.prepareStatement(sql);
			ps.setString(1, sig.acntId);
			ps.setString(2, sig.sigType);
			ResultSet rs = ps.executeQuery();
			boolean insert = true;
			if (rs.next())
				insert = false;
			rs.close();
			ps.close();			
			if (insert) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"signatures (ACNT_ID, SIG_TYPE, SIG_LOCAL, STATUS) values (?, ?, ?, ?)")
						.toString();
				ps = mConn.prepareStatement(sql);
				ps.setString(1, sig.acntId);
				ps.setString(2, sig.sigType);
				ps.setString(3, sig.sigLocal);
				ps.setString(4, sig.status);
			} else {
				ps = mConn
						.prepareStatement("update "
								+ mSchema
								+ "signatures set SIG_LOCAL = ?, STATUS = ? where ACNT_ID = ? and SIG_TYPE = ?");
				ps.setString(1, sig.sigLocal);
				ps.setString(2, sig.status);
				ps.setString(3, sig.acntId);
				ps.setString(4, sig.sigType);
			}
			ps.execute();
			ps.close();
			if (sig.status.equalsIgnoreCase("D")) {
				ps = mConn.prepareStatement("update " + mSchema
						+ "signatures set STATUS = '0' where ACNT_ID <> ?");
				ps.setString(1, sig.acntId);
				ps.execute();
				ps.close();
				ps = mConn.prepareStatement("update " + mSchema
						+ "signatures set STATUS = 'D' where ACNT_ID = ?");
				ps.setString(1, sig.acntId);
				ps.execute();
				ps.close();				
			} else {
				ps = mConn.prepareStatement("update " + mSchema
						+ "signatures set STATUS = '0' where ACNT_ID = ?");
				ps.setString(1, sig.acntId);
				ps.execute();
				ps.close();				
			}
			mConn.commit();
			return sig;
		} catch (Exception e) {
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			throw e;
		}		
	}	
	
	public synchronized Vector getAccounts(String email)
			throws Exception {
		if(email != null) {
			InternetAddress ia = new InternetAddress(email);
			email = ia.getAddress();
		}
		String sql = SEL_ACCOUNT
			+ mSchema + "accounts";
		if(email != null)
			sql += " where email = ?";
		PreparedStatement ps = mConn.prepareStatement(sql);
		if(email != null)
			ps.setString(1, email);
		Vector result = new Vector((email == null)?10:3);
		ResultSet rs = ps.executeQuery();
		while(rs.next())
			result.add(selectAccount(rs));
		rs.close();
		ps.close();	
		return result;
	}
		
	public synchronized void deleteFolder(String id) throws Exception {
		if(!isFolderEmpty(id))
			throw new ChildRecordExist();
		Folder folder = getFolder(id);
		try {
			PreparedStatement ps = null;
			if (folder.getFType() == Folder.FTYPE_IMAP) {
				ps = mConn.prepareStatement("delete from " + mSchema
						+ "imap_flds where ID = ?");
				ps.setString(1, id);
				ps.execute();
				ps.close();
			}
			ps = mConn.prepareStatement("delete from " + mSchema
					+ "folders where ID = ?");
			ps.setString(1, id);
			ps.execute();
			ps.close();
			mConn.commit();
		} catch (Exception e) {
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			throw e;
		} 
	}
		
	public synchronized void deleteAccount(String id) throws Exception {
		try {
			IFolder imapRoot = getImapRootFolder(id);
			if(imapRoot != null)
				deleteFolder(imapRoot.id);
			
			PreparedStatement ps = mConn
					.prepareStatement("delete from " + mSchema
							+ "rules where ACNT_ID = ?");
			ps.setString(1, id);
			ps.execute();
			ps.close();
			ps = mConn.prepareStatement("delete from " + mSchema
					+ "accounts where ID = ?");
			ps.setString(1, id);
			ps.execute();
			ps.close();
			mConn.commit();
		} catch (Exception e) {
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			throw e;
		} 
	}

	public synchronized void deleteRule(String id) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("delete from " + mSchema
				+ "rules where ID = ?");
		ps.setString(1, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}
		
	public synchronized void setDefaultAccount(String id) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("update "
				+ mSchema + "accounts set STATUS = '0' where STATUS = 'D'");
		ps.execute();
		ps = mConn.prepareStatement("update " + mSchema
				+ "accounts set STATUS = 'D' where ID = ?");
		ps.setString(1, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}
	
	protected String getFolderId(String folder) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID from " + mSchema
						+ "folders where NAME = ?");
		ps.setString(1, folder);
		ResultSet resultset = ps.executeQuery();
		String id = null;
		if (resultset.next()) {
			id = resultset.getString(1);
		}
		resultset.close();
		ps.close();
		return id;
	}	

	protected String substrString(String hdr, int len) {
		if(hdr != null && hdr.length() > len)
			hdr = hdr.substring(0, len);
		return hdr;
	}
	
/*	
	protected MessageHeader substrMsgHeader(MessageHeader hdr) {
		hdr.From = substrString(hdr.From, 255);
		hdr.To = substrString(hdr.To, 255);
		hdr.Cc = substrString(hdr.Cc, 255);
		hdr.Subject = substrString(hdr.Subject, 255);
		return hdr;
	}
*/	
	
	protected String getFolderId(String parentFolderId, String folderName) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID from " + mSchema
						+ "folders where FLD_ID = ? and NAME = ?");
		ps.setString(1, parentFolderId);
		ps.setString(2, folderName);
		ResultSet resultset = ps.executeQuery();
		String id = null;
		if (resultset.next()) {
			id = resultset.getString(1);
		}
		resultset.close();
		ps.close();
		return id;
	}
		
	public synchronized Address getAddress(String id) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID, FNAME, LNAME, MNAME, EMAIL, HOME_COUNTRY, HOME_REGION, HOME_CITY, HOME_INDEX, HOME_STREET, HOME_PHONE, MOBILE_PHONE from "
						+ mSchema + "addresses where id = ?");
		ps.setString(1, id);
		ResultSet resultset = ps.executeQuery();
		olivamail.gui.Address addr = null;
		if (resultset.next()) {
			addr = new olivamail.gui.Address();
			addr.id = resultset.getString(1);
			addr.fname = resultset.getString(2);
			addr.lname = resultset.getString(3);
			addr.mname = resultset.getString(4);
			addr.email = resultset.getString(5);
			addr.homeCountry = resultset.getString(6);
			addr.homeRegion = resultset.getString(7);
			addr.homeCity = resultset.getString(8);
			addr.homeIndex = resultset.getString(9);
			addr.homeStreet = resultset.getString(10);
			addr.homePhone = resultset.getString(11);
			addr.mobilePhone = resultset.getString(12);
		}
		ps.close();
		resultset.close();
		return addr;
	}
		
	public synchronized void deleteAddress(String id) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("delete from " + mSchema
						+ "addresses where ID = ?");
		ps.setString(1, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}
	
	public synchronized void deleteMimeGroup(String id) throws Exception {
		if(!isMimeGroupEmpty(id))
			throw new ChildRecordExist();
				
		PreparedStatement ps = mConn
				.prepareStatement("delete from " + mSchema
						+ "mime_groups where ID = ?");
		ps.setString(1, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}

	public synchronized void deleteMimeType(String id) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("delete from " + mSchema
						+ "mime_types where ID = ?");
		ps.setString(1, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}

	public synchronized MimeGroup getMimeGroup(String id) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID, NAME, STATUS from " + mSchema
						+ "mime_groups where id = ?");
		ps.setString(1, id);
		ResultSet resultset = ps.executeQuery();
		olivamail.gui.MimeGroup mgrp = null;
		if (resultset.next()) {
			mgrp = new olivamail.gui.MimeGroup();
			mgrp.id = resultset.getString(1);
			mgrp.name = resultset.getString(2);
			mgrp.status = resultset.getString(3);
		}
		ps.close();
		resultset.close();
		return mgrp;
	}

	public synchronized Vector getMimeGroupList(String searchStr, String orderBy, int limit)
			throws Exception {
		String sqlQuery = String.valueOf(String.valueOf((new StringBuffer(
				"select ID, NAME, STATUS from ")).append(mSchema).append(
				"mime_groups")));
		String whereClause = "";
		if (searchStr != null) {
			searchStr = searchStr.toUpperCase();
			whereClause = String.valueOf(String.valueOf(new StringBuffer(
					" where ").append(searchStr)));
		}
		if (orderBy == null)
			orderBy = "";
		if (orderBy.length() == 0)
			orderBy = "ID desc";
		String orderByClause = " order by " + orderBy;
		sqlQuery += whereClause + orderByClause;
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ResultSet rs = ps.executeQuery();
		Vector data = new Vector(100);
		int count = 0;
		while (rs.next()) {
			count++;
			if (limit != -1 && count > limit)
				break;
			Vector row = new Vector(3);
			for (int i = 1; i <= 3; i++) {
				row.add(rs.getString(i));
			}
			data.add(row);
		}

		rs.close();
		return data;
	}

	public synchronized MimeType getMimeType(String id) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID, MIME_TYPE, PROGRAM, MGRP_ID from "
						+ mSchema + "mime_types where id = ?");
		ps.setString(1, id);
		ResultSet resultset = ps.executeQuery();
		olivamail.gui.MimeType mtype = null;
		if (resultset.next()) {
			mtype = new olivamail.gui.MimeType();
			mtype.id = resultset.getString(1);
			mtype.mimeType = resultset.getString(2);
			mtype.program = resultset.getString(3);
			mtype.mgrpId = resultset.getString(4);
		}
		ps.close();
		resultset.close();
		return mtype;
	}

	public synchronized Vector getMimeTypeList(String mGrpId, String searchStr,
			String orderBy, int limit) throws Exception {
		String sqlQuery = String.valueOf(String.valueOf((new StringBuffer(
				"select ID, MIME_TYPE, PROGRAM, MGRP_ID from "))
				.append(mSchema).append("mime_types where MGRP_ID = ?")));
		String whereClause = "";
		if (searchStr != null) {
			searchStr = searchStr.toUpperCase();
			whereClause = String.valueOf(String.valueOf(new StringBuffer(
					" and ").append(searchStr)));
		}
		if (orderBy == null)
			orderBy = "";
		if (orderBy.length() == 0)
			orderBy = "ID desc";
		String orderByClause = " order by " + orderBy;
		sqlQuery += whereClause + orderByClause;
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, mGrpId);
		ResultSet rs = ps.executeQuery();
		Vector data = new Vector(100);
		int count = 0;
		while (rs.next()) {
			count++;
			if (limit != -1 && count > limit)
				break;
			Vector row = new Vector(4);
			for (int i = 1; i <= 4; i++) {
				row.add(rs.getString(i));
			}
			data.add(row);
		}

		rs.close();
		return data;
	}

	public synchronized void setDefaultMimeGroup(String id) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("update "
				+ mSchema + "mime_groups set STATUS = '0' where STATUS = 'D'");
		ps.execute();
		ps = mConn.prepareStatement("update " + mSchema
				+ "mime_groups set STATUS = 'D' where ID = ?");
		ps.setString(1, id);
		ps.execute();
		ps.close();
		mConn.commit();
	}

	public synchronized olivamail.gui.MimeGroup getDefaultMimeGroup()
			throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID, NAME, STATUS from " + mSchema
						+ "mime_groups where STATUS = 'D'");
		ResultSet resultset = ps.executeQuery();
		olivamail.gui.MimeGroup mgrp = null;
		if (resultset.next()) {
			mgrp = new olivamail.gui.MimeGroup();
			mgrp.id = resultset.getString(1);
			mgrp.name = resultset.getString(2);
			mgrp.status = resultset.getString(3);
		}
		ps.close();
		resultset.close();
		return mgrp;
	}
	
	public synchronized void deleteFolderContent(String folderId) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select ID from " + mSchema
						+ "messages where FOLDER_ID = ?");
		ps.setString(1, folderId);
		ResultSet rs = ps.executeQuery();
		while(rs.next()) {
			deleteMessage(rs.getString(1), true);
		}
		rs.close();
		ps.close();
		
		ps = mConn
				.prepareStatement("select ID from " + mSchema
						+ "folders where FLD_ID = ?");				
		ps.setString(1, folderId);				
		rs = ps.executeQuery();
		while(rs.next()) {
			deleteFolderContent(rs.getString(1));
			deleteFolder(rs.getString(1));
		}
		rs.close();
		ps.close();
		
		mConn.commit();
	}

	public synchronized void expungeImapFolder(String folderId)
			throws Exception {
		PreparedStatement ps = mConn.prepareStatement("select ID from "
				+ mSchema + "messages where FOLDER_ID = ? and SF_DEL = TRUE");
		ps.setString(1, folderId);
		ResultSet rs = ps.executeQuery();
		while (rs.next()) {
			deleteMessage(rs.getString(1), false);
		}
		rs.close();
		ps.close();
		mConn.commit();
	}
		
	public synchronized void deleteMimeTypes(String mgrpId) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("delete from " + mSchema
						+ "mime_types where MGRP_ID = ?");
		ps.setString(1, mgrpId);
		ps.execute();
		ps.close();

		mConn.commit();
	}	

/*	
	public synchronized void setMessageSeen(String id, boolean Seen)
			throws Exception {
		PreparedStatement ps = mConn.prepareStatement("update " + mSchema
				+ "messages set SEEN = ? where ID = ? and SEEN = ?");
		if (Seen)
			ps.setString(1, "1");
		else
			ps.setString(1, "0");
		ps.setString(2, id);
		if (Seen)
			ps.setString(3, "0");
		else
			ps.setString(3, "1");
		ps.execute();
		mConn.commit();
		ps.close();
	}
*/

	public synchronized void saveMessageState(MessageRow mr) throws Exception {
		try {
			if (mr.dmStat != MessageRow.DM_SYNC) {
				IFolder iFolder = getImapFolder(mr.folderId);
				if (iFolder == null)
					mr.dmStat = MessageRow.DM_SYNC;
				else {
					PreparedStatement ps = mConn.prepareStatement("update "
							+ mSchema + "imap_flds set DM_STAT = '"
							+ IFolder.DM_NOSYNC + "' where ID = ?");
					ps.setString(1, mr.folderId);
					ps.execute();
					ps.close();

					ps = mConn.prepareStatement("update " + mSchema
							+ "accounts set DM_STAT = '" + Account.DM_NOSYNC
							+ "' where ID = ?");
					ps.setString(1, iFolder.acntId);
					ps.execute();
					ps.close();
				}
			}

			PreparedStatement ps = mConn
					.prepareStatement("update "
							+ mSchema
							+ "messages set HDR = ?, SF_ANSW = ?, SF_SEEN = ?, SF_FLAG = ?, SF_DEL = ?, SF_DRAFT = ?, DM_STAT = ? WHERE ID = ?");
			ps.setString(1, String.valueOf(mr.hdr));
			ps.setBoolean(2, mr.sfAnsw);
			ps.setBoolean(3, mr.sfSeen);
			ps.setBoolean(4, mr.sfFlag);
			ps.setBoolean(5, mr.sfDel);
			ps.setBoolean(6, mr.sfDraft);
			ps.setString(7, String.valueOf(mr.dmStat));
			ps.setString(8, mr.id);
			ps.execute();
			ps.close();
			mConn.commit();
		} catch (Exception e) {
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			throw e;
		}
	}
		
	public synchronized void moveMessage(String id, String folderId)
			throws Exception {
		String sql = "update " + mSchema
				+ "messages set FOLDER_ID = ? where id = ?";
		PreparedStatement ps = mConn.prepareStatement(sql);
		ps.setString(1, folderId);
		ps.setString(2, id);
		ps.execute();
		mConn.commit();
		ps.close();
	}	

	public synchronized void setMessageCodepage(String id, String cp,
			String from, String to, String subject) throws Exception {
		String sqlQuery = String.valueOf(String.valueOf((new StringBuffer(
				"select CODEPAGE from ")).append(mSchema).append(
				"messages where ID = ?")));
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, id);
		ResultSet rs = ps.executeQuery();
		if (rs.next()) {
			String codepage = rs.getString(1);
			if(!ComUtils.nvl(codepage).equalsIgnoreCase(ComUtils.nvl(cp))) {
				ps = mConn
						.prepareStatement("update messages set ADDR_FROM = ?, ADDR_TO = ?, SUBJECT = ?, CODEPAGE = ? where ID = ?");
				ps.setString(1, from);
				ps.setString(2, to);
				ps.setString(3, subject);
				ps.setString(4, cp);
				ps.setString(5, id);
				ps.execute();
				mConn.commit();
			}
		}
		ps.close();
		rs.close();
	}

	public synchronized void updateMessageRow(MessageRow mr) throws Exception {
		PreparedStatement ps = ps = mConn.prepareStatement(UPD_MESSAGE);
		setMessageValues(ps, mr, 0);
		ps.setString(25, mr.id);
		ps.execute();
		mConn.commit();
	}
			
	public synchronized void optimize() throws Exception {
	}

	public synchronized boolean isOptimizable() {
		return false;
	}
	
	public synchronized Vector getRulesList(String acntId) throws Exception {
		String sqlQuery = String
				.valueOf((new StringBuffer(
						"select ID, HEADER, OPERAND, VALUE, ACTION, PARAM, ACNT_ID from "))
						.append(mSchema).append("rules where ACNT_ID = ?")
						.toString());
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, acntId);
		ResultSet rs = ps.executeQuery();
		Vector data = null;
		while (rs.next()) {
			if(data == null)
				data = new Vector(20);
			Rule rule = new Rule();
			rule.id = rs.getString(1);
			rule.header = rs.getString(2);
			rule.operand = rs.getInt(3);
			rule.value = rs.getString(4);
			rule.action = rs.getInt(5);
			rule.param = rs.getString(6);
			rule.acntId = rs.getString(7);
			data.add(rule);
		}
		rs.close();
		ps.close();
		return data;
	}	

	public synchronized Vector getRulesList(String acntId, int action) throws Exception {
		String sqlQuery = String
				.valueOf((new StringBuffer(
						"select ID, HEADER, OPERAND, VALUE, ACTION, PARAM, ACNT_ID from "))
						.append(mSchema).append("rules where ACNT_ID = ? and ACTION = ?")
						.toString());
		PreparedStatement ps = mConn.prepareStatement(sqlQuery);
		ps.setString(1, acntId);
		ps.setInt(2, action);
		ResultSet rs = ps.executeQuery();
		Vector data = null;
		while (rs.next()) {
			if(data == null)
				data = new Vector(20);
			Rule rule = new Rule();
			rule.id = rs.getString(1);
			rule.header = rs.getString(2);
			rule.operand = rs.getInt(3);
			rule.value = rs.getString(4);
			rule.action = rs.getInt(5);
			rule.param = rs.getString(6);
			rule.acntId = rs.getString(7);
			data.add(rule);
		}
		rs.close();
		ps.close();
		return data;
	}	
		
	protected boolean defAcntExists() throws Exception {
		String sql = (new StringBuffer("select 1 from ")).append(mSchema)
				.append("accounts where STATUS = 'D'").toString();
		PreparedStatement ps = mConn.prepareStatement(sql);
		ResultSet rs = ps.executeQuery();
		boolean result = false;
		if (rs.next()) {
			result = true;
		}
		rs.close();
		ps.close();
		return result;
	}
	
	public synchronized Account saveAccount(olivamail.gui.Account acnt, boolean updateCred) throws Exception {
		PreparedStatement ps = null;
		if (acnt.id == null || acnt.id.equals("")) {
			String sql = null;
			int id1 = 0;
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"accounts (NAME, PERSON, EMAIL, REPLY_TO, SERVER_TYPE, INCOMING_SERVER, OUTGOING_SERVER, SMTP_AUTH, IN_USER, IN_PASS, OUT_USER, OUT_PASS, STATUS, IN_PORT, OUT_PORT, IN_MAX_SIZE, IN_LEAVE_DAYS, USE_RULES, USE_SIG, IN_SSL, OUT_SSL, ORGANIZATION, I_SH_UNS, DM_STAT) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
						.toString();
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"accounts (ID, NAME, PERSON, EMAIL, REPLY_TO, SERVER_TYPE, INCOMING_SERVER, OUTGOING_SERVER, SMTP_AUTH, IN_USER, IN_PASS, OUT_USER, OUT_PASS, STATUS, IN_PORT, OUT_PORT, IN_MAX_SIZE, IN_LEAVE_DAYS, USE_RULES, USE_SIG, IN_SSL, OUT_SSL, ORGANIZATION, I_SH_UNS, DM_STAT) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
						.toString();
			}			
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				acnt.id = getId();
				ps.setString(1, acnt.id);
			}
			
			if(!defAcntExists() && acnt.status != Account.STAT_NO_RECEIVE) {
				acnt.status = Account.STAT_DEFAULT_ACNT;
			}
			
			ps.setString(1 + id1, acnt.name);
			ps.setString(2 + id1, acnt.person);
			ps.setString(3 + id1, acnt.email);
			ps.setString(4 + id1, acnt.replyTo);
			ps.setString(5 + id1, acnt.serverType);
			ps.setString(6 + id1, acnt.incomingServer);
			ps.setString(7 + id1, acnt.outgoingServer);
			ps.setString(8 + id1, acnt.smtpAuth);
			ps.setString(9 + id1, acnt.inUser);
			ps.setString(10 + id1, acnt.inPass);
			ps.setString(11 + id1, acnt.outUser);
			ps.setString(12 + id1, acnt.outPass);
			ps.setString(13 + id1, String.valueOf(acnt.status));
			ps.setInt(14 + id1, acnt.inPort);
			ps.setInt(15 + id1, acnt.outPort);
			ps.setInt(16 + id1, acnt.inMaxSize);
			ps.setInt(17 + id1, acnt.inLeaveDays);
			ps.setString(18 + id1, acnt.useRules);
			ps.setString(19 + id1, acnt.useSig);
			ps.setString(20 + id1, acnt.inSSL);
			ps.setString(21 + id1, acnt.outSSL);
			ps.setString(22 + id1, acnt.organization);
			ps.setString(23 + id1, acnt.iShowUnsubscr);
			ps.setString(24 + id1, String.valueOf(acnt.dmStat));
		} else {
			String hwmStr = "";			
			ps = mConn
				.prepareStatement("select INCOMING_SERVER from " + mSchema + "accounts where ID = ?");
			ps.setString(1, acnt.id);
			ResultSet rs = ps.executeQuery();
			if(rs.next()) {
				String incomingServer = rs.getString(1);
				if(!ComUtils.nvl(incomingServer).equals(ComUtils.nvl(acnt.incomingServer))) {
					hwmStr = ", HWM_MSG_UID = null, HWM_MSG_ID = null";
				}				
			}
			rs.close();
			ps.close();
			
			ps = mConn
					.prepareStatement("update " + mSchema + 
							"accounts set NAME = ?, PERSON = ?, EMAIL = ?, REPLY_TO = ?, SERVER_TYPE = ?, INCOMING_SERVER = ?, OUTGOING_SERVER = ?, " + 
							(updateCred?"SMTP_AUTH = ?, IN_USER = ?, IN_PASS = ?, OUT_USER = ?, OUT_PASS = ?, ":"") +
							"STATUS = ?, IN_PORT = ?, OUT_PORT = ?, IN_MAX_SIZE = ?, IN_LEAVE_DAYS = ?, USE_RULES = ?" + 
							hwmStr + ", USE_SIG = ?, IN_SSL = ?, OUT_SSL = ?, ORGANIZATION = ?, HWM_MSG_ID = ?, HWM_MSG_UID = ?, I_SH_UNS = ?, DM_STAT = ? where ID = ?");
			ps.setString(1, acnt.name);
			ps.setString(2, acnt.person);
			ps.setString(3, acnt.email);
			ps.setString(4, acnt.replyTo);
			ps.setString(5, acnt.serverType);
			ps.setString(6, acnt.incomingServer);
			ps.setString(7, acnt.outgoingServer);
			int offset = 0;
			if (updateCred) {
				ps.setString(8, acnt.smtpAuth);
				ps.setString(9, acnt.inUser);
				ps.setString(10, acnt.inPass);
				ps.setString(11, acnt.outUser);
				ps.setString(12, acnt.outPass);
				offset = 5;
			}
			ps.setString(8 + offset, String.valueOf(acnt.status));
			ps.setInt(9 + offset, acnt.inPort);
			ps.setInt(10 + offset, acnt.outPort);
			ps.setInt(11 + offset, acnt.inMaxSize);
			ps.setInt(12 + offset, acnt.inLeaveDays);
			ps.setString(13 + offset, acnt.useRules);
			ps.setString(14 + offset, acnt.useSig);
			ps.setString(15 + offset, acnt.inSSL);
			ps.setString(16 + offset, acnt.outSSL);
			ps.setString(17 + offset, acnt.organization);
			ps.setString(18 + offset, acnt.hwmMsgId);
			ps.setString(19 + offset, acnt.hwmMsgUid);			
			ps.setString(20 + offset, acnt.iShowUnsubscr);
			ps.setString(21 + offset, String.valueOf(acnt.dmStat));
			ps.setString(22 + offset, acnt.id);
		}
		ps.execute();
		ps.close();
		if (acnt.id == null || acnt.id.equals("")) {
			acnt.id = getLastInsertedId();
		}
		mConn.commit();
		return acnt;
	}
	
	public synchronized void updateAccountState(Account acnt) throws Exception {
		PreparedStatement ps = mConn
		.prepareStatement("update " + mSchema + 
				"accounts set STATUS = ?, USE_RULES = ?, USE_SIG = ?, HWM_MSG_ID = ?, HWM_MSG_UID = ?, I_SH_UNS = ?, DM_STAT = ? where ID = ?");
		ps.setString(1, String.valueOf(acnt.status));	
		ps.setString(2, acnt.useRules);
		ps.setString(3, acnt.useSig);
		ps.setString(4, acnt.hwmMsgId);
		ps.setString(5, acnt.hwmMsgUid);
		ps.setString(6, acnt.iShowUnsubscr);
		ps.setString(7, String.valueOf(acnt.dmStat));
		ps.setString(8, acnt.id);
		ps.execute();
		ps.close();

		mConn.commit();
	}
	
	public synchronized Rule saveRule(Rule rule) throws Exception {
		PreparedStatement ps = null;
		if (rule.id == null || rule.id.equals("")) {
			String sql = null;
			int id1 = 0;
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"rules (HEADER, OPERAND, VALUE, ACTION, PARAM, ACNT_ID) values (?, ?, ?, ?, ?, ?)")
						.toString();
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"rules (ID, HEADER, OPERAND, VALUE, ACTION, PARAM, ACNT_ID) values (?, ?, ?, ?, ?, ?, ?)")
						.toString();
			}			
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				rule.id = getId();
				ps.setString(1, rule.id);
			}
			ps.setString(1 + id1, rule.header);
			ps.setInt(2 + id1, rule.operand);
			ps.setString(3 + id1, rule.value);
			ps.setInt(4 + id1, rule.action);
			ps.setString(5 + id1, rule.param);
			ps.setString(6 + id1, rule.acntId);
		} else {
			ps = mConn
					.prepareStatement("update "
							+ mSchema
							+ "rules set HEADER = ?, OPERAND = ?, VALUE = ?, ACTION = ?, PARAM = ?, ACNT_ID = ? where ID = ?");
			ps.setString(1, rule.header);
			ps.setInt(2, rule.operand);
			ps.setString(3, rule.value);
			ps.setInt(4, rule.action);
			ps.setString(5, rule.param);
			ps.setString(6, rule.acntId);
			ps.setString(7, rule.id);
		}
		ps.execute();
		ps.close();
		if (rule.id == null || rule.id.equals("")) {
			rule.id = getLastInsertedId();
		}		
		mConn.commit();
		return rule;
	}	
	
	public synchronized void setSettingsValue(String name, String value) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("Select ID, VALUE from " + mSchema + "settings where NAME = ?");
		ps.setString(1, name);
		ResultSet rs = ps.executeQuery();
		String id = null;
		if (rs.next()) {
			id = rs.getString(1);
		}
		ps.close();
		rs.close();
		String sql = null;
		if(id == null) {
			int id1 = 0;
			if(isIdAutoincremented())
				sql = "INSERT INTO " + mSchema + "settings (NAME, VALUE) values (?, ?)";
			else {
				sql = "INSERT INTO " + mSchema + "settings (ID, NAME, VALUE) values (?, ?, ?)";
				id1 = 1;
			}
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented())
				ps.setString(1, getId());
			ps.setString(1 + id1, name);
			ps.setString(2 + id1, value);
		}
		else {
			sql = "UPDATE " + mSchema + "settings set NAME = ?, VALUE = ? where id = ?";
			ps = mConn.prepareStatement(sql);			
			ps.setString(1, name);
			ps.setString(2, value);
			ps.setString(3, id);
		}
		ps.execute();
		ps.close();
		mConn.commit();
	}
		
	public synchronized MimeGroup saveMimeGroup(MimeGroup mgrp) throws Exception {
		PreparedStatement ps = null;
		if (mgrp.id == null || mgrp.id.equals("")) {
			String sql = null;
			int id1 = 0;
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"mime_groups (NAME, STATUS) values (?, ?)")
						.toString();
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"mime_groups (ID, NAME, STATUS) values (?, ?, ?)")
						.toString();
			}			
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				mgrp.id = getId();
				ps.setString(1, mgrp.id);
			}
			ps.setString(1 + id1, mgrp.name);
			ps.setString(2 + id1, mgrp.status);
		} else {
			ps = mConn.prepareStatement("update " + mSchema
					+ "mime_groups set NAME = ?, STATUS = ? where ID = ?");
			ps.setString(1, mgrp.name);
			ps.setString(2, mgrp.status);
			ps.setString(3, mgrp.id);
		}
		ps.execute();
		ps.close();
		if (mgrp.id == null || mgrp.id.equals("")) {
			mgrp.id = getLastInsertedId();			
		}
		mConn.commit();
		return mgrp;
	}	

	public synchronized MimeType saveMimeType(MimeType mtype) throws Exception {
		PreparedStatement ps = null;
		if (mtype.id == null || mtype.id.equals("")) {
			String sql = null;
			int id1 = 0;
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"mime_types (MIME_TYPE, PROGRAM, MGRP_ID) values (?, ?, ?)")
						.toString();
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"mime_types (ID, MIME_TYPE, PROGRAM, MGRP_ID) values (?, ?, ?, ?)")
						.toString();
			}			
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				mtype.id = getId();
				ps.setString(1, mtype.id);
			}
			ps.setString(1 + id1, mtype.mimeType);
			ps.setString(2 + id1, mtype.program);
			ps.setString(3 + id1, mtype.mgrpId);
		} else {
			ps = mConn
					.prepareStatement("update "
							+ mSchema
							+ "mime_types set MIME_TYPE = ?, PROGRAM = ?, MGRP_ID = ? where ID = ?");
			ps.setString(1, mtype.mimeType);
			ps.setString(2, mtype.program);
			ps.setString(3, mtype.mgrpId);
			ps.setString(4, mtype.id);
		}
		ps.execute();
		ps.close();
		if (mtype.id == null || mtype.id.equals("")) {
			mtype.id = getLastInsertedId();			
		}		
		mConn.commit();
		return mtype;
	}
	
	public synchronized boolean isMimeGroupEmpty(String mgrpId) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select 1 from " + mSchema
						+ "mime_types where MGRP_ID = ?");
		ps.setString(1, mgrpId);
		ResultSet resultset = ps.executeQuery();
		boolean result = true;
		if (resultset.next()) {
			result = false;
		}
		resultset.close();
		ps.close();

		return result;
	}	
	
	protected String getOrCreateFolderId(String folder) throws Exception {
		String id = getFolderId(folder);
		if (id == null) {
			String sql = null;
			int id1 = 0;
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"folders (NAME) values (?)")
						.toString();
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"folders (ID,NAME) values (?, ?)")
						.toString();
			}
			PreparedStatement ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				ps.setString(1, getId());
			}			
			ps.setString(1 + id1, folder);
			ps.execute();
			ps.close();
			id = getFolderId(folder);
			mConn.commit();
		}
		return id;
	}

	public synchronized boolean isFolderEmpty(String folderId) throws Exception {
		PreparedStatement ps = mConn
				.prepareStatement("select 1 from " + mSchema
						+ "messages where FOLDER_ID = ?");
		ps.setString(1, folderId);
		ResultSet rs = ps.executeQuery();
		boolean empty = true;
		if (rs.next()) {
			empty = false;
		}
		rs.close();
		ps.close();
		
		if(!empty)
			return empty;
		
		ps = mConn.prepareStatement("select 1 from " + mSchema
				+ "folders where FLD_ID = ?");
		ps.setString(1, folderId);
		rs = ps.executeQuery();
		if (rs.next()) {
			empty = false;
		}
		rs.close();
		ps.close();		

		return empty;
	}

	public void setDbMessageMaxSize(int dbMessageMaxSize) {
		this.dbMessageMaxSize = dbMessageMaxSize;
	}

	public int getDbMessageMaxSize() {
		return dbMessageMaxSize;
	}

	public String getMsgsDir() {
		return msgsDir;
	}

	public void setMsgsDir(String msgsDir) {
		this.msgsDir = msgsDir;
	}
	
	protected synchronized void deleteMissingSources() throws Exception {
		PreparedStatement ps = mConn.prepareStatement("select ID from " + mSchema
				+ "sources");
		PreparedStatement ps1 = mConn.prepareStatement("select 1 from " + mSchema
				+ "messages where ID = ?");
		PreparedStatement ps2 = mConn.prepareStatement("delete from " + mSchema
				+ "sources where ID = ?");
		ResultSet rs = ps.executeQuery();
		while(rs.next()) {
			String id = rs.getString(1);
			ps1.setString(1, id);
			ResultSet rs1 = ps1.executeQuery();
			if(!rs1.next()) {
				ps2.setString(1, id);
				ps2.execute();
			}
		}
		ps2.close();
		ps1.close();
		rs.close();
		ps.close();
		mConn.commit();
	}	

	public synchronized int getNewMsgCount(String folderId) throws Exception {
		PreparedStatement ps = mConn.prepareStatement("select count(1) from "
				+ mSchema + "messages where FOLDER_ID = ? and HDR in ('"
				+ MessageRow.HDR_HDR + "', '" + MessageRow.HDR_MULTI + "', '"
				+ MessageRow.HDR_COMPLETE + "') and SF_SEEN = FALSE");
		ps.setString(1, folderId);
		ResultSet resultset = ps.executeQuery();
		int count = 0;
		if (resultset.next()) {
			count = resultset.getInt(1);
		}
		ps.close();
		resultset.close();
		return count;
	}
	
	protected void setMessageValues(PreparedStatement ps, MessageRow mr, int startPos) throws Exception {
		ps.setString(startPos + 1, substrString(mr.addrFrom, 255));
		ps.setString(startPos + 2, substrString(mr.addrTo, 255));
		ps.setString(startPos + 3, substrString(mr.addrCc, 255));
		ps.setString(startPos + 4, mr.addrBcc);
		ps.setString(startPos + 5, substrString(mr.subject, 255));		
		ps.setString(startPos + 6, mr.sentDate);
		ps.setString(startPos + 7, mr.receivedDate);		
		ps.setInt(startPos + 8, mr.msgSize);
		ps.setString(startPos + 9, mr.messageId);				
		ps.setString(startPos + 10, mr.msgUid);
		ps.setString(startPos + 11, mr.hasAtt);
		ps.setString(startPos + 12, mr.folderId);
		ps.setString(startPos + 13, String.valueOf(mr.hdr));
		ps.setString(startPos + 14, mr.codepage);
		ps.setLong(startPos + 15, mr.uidValidity);
		ps.setString(startPos + 16, mr.importance);
		ps.setString(startPos + 17, mr.srcPlace);
		ps.setString(startPos + 18, mr.acntId);		
		ps.setBoolean(startPos + 19, mr.sfAnsw);
		ps.setBoolean(startPos + 20, mr.sfSeen);
		ps.setBoolean(startPos + 21, mr.sfFlag);
		ps.setBoolean(startPos + 22, mr.sfDel);
		ps.setBoolean(startPos + 23, mr.sfDraft);
		ps.setString(startPos + 24, String.valueOf(mr.dmStat));
	}
		
	public synchronized String saveMessageSource(String id,
			javax.mail.Message message) throws Exception {
		File file = null;
		try {
			char hdr = MessageRow.HDR_COMPLETE;
			if (MailUtils.getContentType(message).equals("message/partial"))
				hdr = MessageRow.HDR_MULTI;

			ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
			message.writeTo(baos);
			int msgSize = baos.size();

			String srcPlace = "0";
			if (dbMessageMaxSize != -1 && msgSize > dbMessageMaxSize)
				srcPlace = "1";

			PreparedStatement ps = mConn.prepareStatement("update " + mSchema
					+ "messages set HDR = '" + hdr
					+ "', MSG_SIZE = ?, SRC_PLACE = ? where ID = ?");
			ps.setInt(1, msgSize);
			ps.setString(2, srcPlace);
			ps.setString(3, id);
			ps.execute();

			if (srcPlace.equals("1")) {
				file = new File(msgsDir + File.separator + id + ".eml");
				OutputStream os = Context.getFileOutputStream(file);
				message.writeTo(os);
				Context.closeOutputStream(os);
			} else {
				ps = mConn.prepareStatement("insert into " + mSchema
						+ "sources (ID, SOURCE) values (?, ?)");
				ps.setString(1, id);
				ps.setBytes(2, baos.toByteArray());
				ps.execute();
				ps.close();
			}

			mConn.commit();
			return id;
		} catch (Exception exc) {
			try {
				mConn.rollback();
			} catch (Exception e) {
				e.printStackTrace();
			}
			try {
				if (file != null)
					file.delete();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
	}
		
	public synchronized String saveMessage(olivamail.core.mail.MessageHeader hdr,
			String folderId, javax.mail.Message message, long uidValidity, Flags flags, String codepage, String acntId)
			throws Exception {				
		File file = null;
//		hdr = substrMsgHeader(hdr);
		try {					
			ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
						
			message.writeTo(baos);
			hdr.msgSize = baos.size();
			//For composed messages MessageID is empty before message.writeTo(baos);
			if (hdr.MessageID.equals(""))
				hdr.MessageID = Context.mailBean.getCoder().getPartHeader(
						message, "Message-ID", hdr.cp);						
			
			String srcPlace = "0";
			if(dbMessageMaxSize != -1 && hdr.msgSize > dbMessageMaxSize)
				srcPlace = "1";
			
			PreparedStatement ps = null;
			ps = mConn.prepareStatement("insert into " + mSchema + INS_MESSAGE);

			MessageRow mr = MessageRow.createRow(hdr);
			mr.folderId = folderId;			
			if(MailUtils.getContentType(message).equals("message/partial"))
				mr.hdr = MessageRow.HDR_MULTI;
			else
				mr.hdr = MessageRow.HDR_COMPLETE;				
			mr.codepage = codepage;
			mr.uidValidity = uidValidity;
			mr.srcPlace = srcPlace;
			mr.acntId = acntId;
			mr.setFlags(flags);
			
			setMessageValues(ps, mr, 0);		
			
			ps.execute();
			ps.close();

			String id = getLastInsertedId();
						
			if(srcPlace.equals("1")) {
				file = new File(msgsDir + File.separator + id + ".eml");
				OutputStream os = Context.getFileOutputStream(file);
				message.writeTo(os);
				Context.closeOutputStream(os);
			} else {
				ps = mConn.prepareStatement("insert into " + mSchema
						+ "sources (ID, SOURCE) values (?, ?)");
				ps.setString(1, id);
				ps.setBytes(2, baos.toByteArray());
				ps.execute();
				ps.close();
			} 
			
			mConn.commit();
			return id;
		} catch (Exception exc) {
			try {
				mConn.rollback();
			} catch (Exception e) {
				e.printStackTrace();
			}
			try {
				if(file != null)
					file.delete();
			} catch (Exception e) {
				e.printStackTrace();
			}						
			throw exc;
		}
	}
	
	public synchronized TableData executeQuery(String sql) throws Exception {
		TableData result = new TableData(); 
		PreparedStatement ps = mConn.prepareStatement(sql);
		ResultSet rs = ps.executeQuery();
		int colCount = rs.getMetaData().getColumnCount();
		result.headers = new Vector(colCount);
		for(int i = 1; i <= colCount; i++) {
			result.headers.add(rs.getMetaData().getColumnName(i));
		}
		result.data = new Vector(50);		
		while (rs.next()) {
			Vector row = new Vector(colCount);			
			for (int i = 1; i <= colCount; i++)
				row.add(rs.getString(i));
			result.data.add(row);
		}
		rs.close();
		ps.close();
		return result;
	}
		
	public synchronized String insertMessageHeader(
			olivamail.core.mail.MessageHeader hdr, String folderId,
			long uidvalidity, Flags flags) throws Exception {
		MessageRow mr = MessageRow.createRow(hdr);
		mr.folderId = folderId;
		mr.hdr = MessageRow.HDR_HDR;
		mr.codepage = hdr.cp;
		mr.uidValidity = uidvalidity;
		mr.setFlags(flags);
		return insertMessageRow(mr);
	}

	private synchronized String insertMessageRow(MessageRow mr) throws Exception {
		PreparedStatement ps = null;
		ps = mConn.prepareStatement("insert into " + mSchema + INS_MESSAGE);
		
		setMessageValues(ps, mr, 0);		
		
		ps.execute();
		ps.close();
		mConn.commit();
		return getLastInsertedId();
	}
		
	protected boolean shouldDelOnlyMscSrc(MessageRow mr) throws Exception {
		boolean result = false;
		if (mr.uidValidity != 0 && !mr.msgUid.equals("")) {
			Folder folder = getFolder(mr.folderId);
			if (folder.getFType() != Folder.FTYPE_IMAP)
				result = true;
		}
		return result;
	}
	
	// for version upgrade
	protected void convertSentDate() {
		System.out.println("Converting SENT_DATE field of the messages table ...");
		System.out.flush();
		PreparedStatement ps = null;
		try {			
			ps = mConn.prepareStatement("select SENT_DATE, ID from " + mSchema
					+ "messages where SENT_DATE IS NOT NULL and SENT_DATE <> ''");
			ResultSet rs = ps.executeQuery();
			PreparedStatement ps1 = mConn.prepareStatement("update " + mSchema
					+ "messages set SENT_DATE = ? where ID = ?");
			while (rs.next()) {
				ps1.setString(1, MessageRow.convertDMYtoYMD(rs.getString(1), false));
				ps1.setString(2, rs.getString(2));
				ps1.execute();
			}
			ps1.close();
			rs.close();
			ps.close();
			mConn.commit();
			System.out.println("OK!");
			System.out.flush();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
		}
	}	
		
	// for version upgrade 	
	protected void simplifySignatures() throws Exception {
		try {
			System.out.println("Simplifying signatures mechanism ...");
			System.out.flush();
			
			// Delete bad types if any
			String sql = "delete from " + mSchema
					+ "signatures where SIG_TYPE <> '0' and SIG_TYPE <> '1'";
			PreparedStatement ps = mConn.prepareStatement(sql);
			ps.execute();
			mConn.commit();

			// Duplicate accounts with international signatures
			sql = "select distinct ACNT_ID from " + mSchema
					+ "signatures where SIG_INT is not null and SIG_INT <> ''";
			ps = mConn.prepareStatement(sql);
			ResultSet rs = ps.executeQuery();
			PreparedStatement ps1 = null;
			while (rs.next()) {
				String oldAcntId = rs.getString(1);
				Account acnt = getAccount(oldAcntId);
				acnt.id = null;
				if (acnt.name != null) {
					if (acnt.name.length() <= 51)
						acnt.name += " (Inter.)";
					else if (acnt.name.length() <= 59)
						acnt.name += "1";
				} else {
					acnt.name = "<noname> (Inter.)";
				}
				acnt.status = Account.STAT_NO_RECEIVE;
				acnt.useRules = "0";
				acnt = saveAccount(acnt, false);
				ps1 = mConn
						.prepareStatement("select SIG_TYPE, SIG_INT, STATUS from "
								+ mSchema
								+ "signatures where SIG_INT is not null and SIG_INT <> '' and ACNT_ID = ?");
				ps1.setString(1, oldAcntId);
				ResultSet rs1 = ps1.executeQuery();
				while (rs1.next()) {
					Signature sig = new Signature();
					sig.acntId = acnt.id;
					sig.sigType = rs1.getString(1);
					sig.sigLocal = rs1.getString(2);
					sig.status = rs1.getString(3);
					saveSignature(sig);
				}
				rs1.close();
				ps1.close();
			}
			rs.close();
			ps.close();
			ps = mConn.prepareStatement("delete from " + mSchema
					+ "signatures where SIG_LOCAL is null or SIG_LOCAL = ''");
			ps.execute();			
			ps.close();
			mConn.commit();
			
			// Drop column SIG_INT of signatures table
			ps = mConn.prepareStatement("alter table " + mSchema
					+ "signatures drop column SIG_INT");
			ps.execute();
			ps.close();
			mConn.commit();
			System.out.println("OK!");
			System.out.flush();			
		} catch (Exception e) {
			e.printStackTrace();
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
		}
		
	}	

	public synchronized IFolder getImapRootFolder(String acntId) throws Exception {
		PreparedStatement ps = mConn.prepareStatement(SEL_IFOLDER + mSchema
				+ "imap_flds WHERE ACNT_ID = ? AND FLD_ID IS NULL");
		ps.setString(1, acntId);
		return selectImapFolder(ps);
	}
	
	public synchronized Hashtable getAllImapFolders(String acntId, boolean onlySubscr)
			throws Exception {
		Hashtable folders = new Hashtable();
		IFolder root = getImapRootFolder(acntId);
		if(root == null)
			return folders;
		return getImapFolders(folders, root.id, onlySubscr);
	}
	
	public synchronized IFolder getImapFolder(String id)
			throws Exception {
		PreparedStatement ps = mConn.prepareStatement(SEL_IFOLDER + mSchema
				+ "imap_flds WHERE ID = ?");
		ps.setString(1, id);
		return selectImapFolder(ps);
	}

	protected Folder selectFolder(ResultSet resultset) throws Exception {
		Folder folder = new Folder(resultset.getString(1), resultset
				.getString(2), resultset.getString(3));
		folder.setFType(resultset.getString(4).charAt(0));
		return folder;
	}	
	
	protected IFolder selectImapFolder(ResultSet resultset) throws Exception {
		IFolder iFolder = null;
		iFolder = new IFolder(resultset.getString(1), resultset.getString(2),
				resultset.getString(3));
		iFolder.fullName = resultset.getString(4);
		iFolder.subscr = resultset.getString(5).charAt(0);
		iFolder.syncMode = resultset.getString(6).charAt(0);
		iFolder.imapType = resultset.getInt(7);
		iFolder.uidVal = resultset.getString(8);
		iFolder.acntId = resultset.getString(9);		
		iFolder.dmStat = resultset.getString(10).charAt(0);		
		return iFolder;
	}	
	
	protected IFolder selectImapFolder(PreparedStatement ps) throws Exception {
		ResultSet resultset = ps.executeQuery();
		IFolder iFolder = null;
		if (resultset.next())
			iFolder = selectImapFolder(resultset);
		resultset.close();
		ps.close();
		return iFolder;
	}		
	
	protected Hashtable getImapFolders(Hashtable folders, String parentFldId,
			boolean onlySubscr) throws Exception {
		if (parentFldId == null)
			return folders;
		PreparedStatement ps = mConn.prepareStatement(SEL_IFOLDER + mSchema
				+ "imap_flds WHERE FLD_ID = ?");
		ps.setString(1, parentFldId);
		ResultSet resultset = ps.executeQuery();
		IFolder iFolder = null;
		while (resultset.next()) {
			iFolder = selectImapFolder(resultset);
			if (onlySubscr) {
				if (iFolder.subscr == IFolder.SUBSCR_ON)
					folders.put(iFolder.fullName, iFolder);
			} else
				folders.put(iFolder.fullName, iFolder);
			folders = getImapFolders(folders, iFolder.getId(), onlySubscr);
		}
		resultset.close();
		ps.close();
		return folders;
	}
	
	public synchronized IFolder saveImapFolder(IFolder iFolder)
			throws Exception {
		try {
			iFolder = saveImapFolderNC(iFolder);
			mConn.commit();
		} catch (Exception e) {
			try {
				mConn.rollback();
			} catch (Exception exc) {
				exc.printStackTrace();
			}
			throw e;
		}
		iFolder.saveSubscrTemp();
		iFolder.saveImapTypeTemp();
		return iFolder;
	}
	
	protected IFolder saveImapFolderNC(IFolder iFolder) throws Exception {
		PreparedStatement ps = null;
		if (iFolder.getId() == null || iFolder.getId().equals("")) {
			saveFolderNC(iFolder);
			
			String sql = null;
			int id1 = 0;
/*			
			if (isIdAutoincremented()) {
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"imap_flds (NAME, FLD_ID, FULL_NAME, SUBSCR, SYNC_MODE, IMAP_TYPE, UIDVAL, ACNT_ID) values (?, ?, ?, ?, ?, ?, ?, ?)")
						.toString();								
			} else {
				id1 = 1;
				sql = (new StringBuffer("insert into "))
						.append(mSchema)
						.append(
								"imap_flds (ID, NAME, FLD_ID, FULL_NAME, SUBSCR, SYNC_MODE, IMAP_TYPE, UIDVAL, ACNT_ID) values (?, ?, ?, ?, ?, ?, ?, ?, ?)")
						.toString();
			}			
			ps = mConn.prepareStatement(sql);
			if (!isIdAutoincremented()) {
				iFolder.setId(getId());
				ps.setString(1, iFolder.getId());
			}
*/
			id1 = 1;
			sql = (new StringBuffer("insert into "))
					.append(mSchema)
					.append(
							"imap_flds (ID, NAME, FLD_ID, FULL_NAME, SUBSCR, SYNC_MODE, IMAP_TYPE, UIDVAL, ACNT_ID, DM_STAT) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
					.toString();
			ps = mConn.prepareStatement(sql);			
			ps.setString(1, iFolder.getId());			
			
			ps.setString(1 + id1, iFolder.getName());
			ps.setString(2 + id1, iFolder.getFldId());
			ps.setString(3 + id1, iFolder.fullName);
			ps.setString(4 + id1, String.valueOf(iFolder.getSubscrTemp()));
			ps.setString(5 + id1, String.valueOf(iFolder.syncMode));
			ps.setInt(6 + id1, iFolder.getImapTypeTemp());
			ps.setString(7 + id1, iFolder.uidVal);
			ps.setString(8 + id1, iFolder.acntId);
			ps.setString(9 + id1, String.valueOf(iFolder.dmStat));
		} else {
			saveFolderNC(iFolder);			
			ps = mConn
					.prepareStatement("update "
							+ mSchema
							+ "imap_flds set NAME = ?, FLD_ID = ?, FULL_NAME = ?, SUBSCR = ?, SYNC_MODE = ?, IMAP_TYPE = ?, UIDVAL = ?, ACNT_ID = ?, DM_STAT = ? where ID = ?");
			ps.setString(1, iFolder.getName());
			ps.setString(2, iFolder.getFldId());
			ps.setString(3, iFolder.fullName);
			ps.setString(4, String.valueOf(iFolder.getSubscrTemp()));
			ps.setString(5, String.valueOf(iFolder.syncMode));
			ps.setInt(6, iFolder.getImapTypeTemp());
			ps.setString(7, iFolder.uidVal);
			ps.setString(8, iFolder.acntId);
			ps.setString(9, String.valueOf(iFolder.dmStat));
			ps.setString(10, iFolder.getId());
		}
		ps.execute();
		ps.close();
		if (iFolder.getId() == null || iFolder.getId().equals("")) {
			iFolder.setId(getLastInsertedId());
		}
		return iFolder;
	}
		
	public String copyMessage(String msgId, String toFolderId, boolean dmNew) throws Exception {
		File toFile = null;
		try {
			MessageRow mr = getMessageRow(msgId);
			if(dmNew) {
				IFolder iFolder = getImapFolder(toFolderId);
				if (iFolder == null) {
					mr.dmStat = MessageRow.DM_SYNC;
					mr.sfDel = false;
				} else {
					PreparedStatement ps = mConn.prepareStatement("update "
							+ mSchema + "imap_flds set DM_STAT = '"
							+ IFolder.DM_NOSYNC + "' where ID = ?");
					ps.setString(1, toFolderId);
					ps.execute();
					ps.close();

					ps = mConn.prepareStatement("update " + mSchema
							+ "accounts set DM_STAT = '" + Account.DM_NOSYNC
							+ "' where ID = ?");
					ps.setString(1, iFolder.acntId);
					ps.execute();
					ps.close();

					mr.dmStat = MessageRow.DM_NEW;
				}
			} else {
				mr.dmStat = MessageRow.DM_SYNC;
				mr.sfDel = false;
			}
			
			mr.folderId = toFolderId;
			
			PreparedStatement ps = mConn.prepareStatement("insert into "
					+ mSchema + INS_MESSAGE);
			setMessageValues(ps, mr, 0);
			ps.execute();
			ps.close();
			String id = getLastInsertedId();
			String srcPlace = "0";
			if (dbMessageMaxSize != -1 && mr.msgSize > dbMessageMaxSize)
				srcPlace = "1";

			if (srcPlace.equals("1")) {
				File fromFile = new File(msgsDir + File.separator + msgId
						+ ".eml");
				toFile = new File(msgsDir + File.separator + id + ".eml");
				IOUtils.copyFile(fromFile, toFile);
			} else {
				ps = mConn
						.prepareStatement("insert into "
								+ mSchema
								+ "sources (id, source) select ?, source from sources where id = ?");
				ps.setString(1, id);
				ps.setString(2, msgId);
				ps.execute();
				ps.close();
			}

			mConn.commit();
			return id;
		} catch (Exception exc) {
			try {
				mConn.rollback();
			} catch (Exception e) {
				e.printStackTrace();
			}
			try {
				if (toFile != null)
					toFile.delete();
			} catch (Exception e) {
				e.printStackTrace();
			}
			throw exc;
		}
	}

	// for version upgrade
	protected void updateMessageID() throws Exception {
		Vector list = getMessageList(null,
				"MESSAGE_ID = ''", null, -1);
		for (Enumeration en = list.elements(); en.hasMoreElements();) {
			MessageRow mr = (MessageRow) en.nextElement();
			if(mr.isHeader())
				continue;
			try {
				Message message = loadMessage(mr.id);
				if(message == null) {
					Context.debugStr("message == null, mr.id = " + mr.id);
					continue;
				}
				mr.messageId = Context.mailBean.getCoder().getPartHeader(message,
						"Message-ID", mr.codepage);
				Context.debugStr(mr.messageId + ", mr.id = " + mr.id);
				updateMessageRow(mr);				
			} catch (java.io.FileNotFoundException ioe) {
				ioe.printStackTrace();
			}
		}
		
	}
	
}