/*
 * @(#)TechInfoFrame.java
 *
 * Copyright (C) 2006 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.JLabel;
import java.awt.FlowLayout;
import javax.swing.JButton;
import java.awt.Dimension;
import javax.swing.SwingConstants;

public class TechInfoFrame extends JDialog {

	static final long serialVersionUID = 7286908051106969089L;
	private final String WIN_ID = "tech_info";
	private JPanel jContentPane = null;
	private JLabel jlAvgTxSpeed = null;
	private JLabel jlAvgTxSpeedVal = null;
	private JButton jButton = null;
	private JLabel jlAvgRxSpeedVal = null;
	private JLabel jlAvgRxSpeed = null;

	/**
	 * This is the default constructor
	 */
	public TechInfoFrame() {
		super();
		initialize();
		setLocation(Context.getWindowPosition(WIN_ID));
		setAvgRxSpeed(Context.avgRxSpeed);
		setAvgTxSpeed(Context.avgTxSpeed);		
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(275, 112);
		this.addWindowListener(new java.awt.event.WindowAdapter() { 
			public void windowClosing(java.awt.event.WindowEvent e) {    
				this_windowClosing(e);
			}
		});
		this.setTitle(Strings.getString("TechInfoFrame.title")); //$NON-NLS-1$
		this.setResizable(false);
		this.setContentPane(getJContentPane());
	}

	void this_windowClosing(java.awt.event.WindowEvent e) {
		Context.setWindowPosition(WIN_ID, getLocation());
	}
		
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jlAvgRxSpeed = new JLabel();
			jlAvgRxSpeed.setPreferredSize(new Dimension(150, 16));
			jlAvgRxSpeed.setText(Strings.getString("ProgressFrame.AvgRxSpeed"));
			jlAvgRxSpeed.setHorizontalAlignment(SwingConstants.TRAILING);
			jlAvgRxSpeedVal = new JLabel();
			jlAvgRxSpeedVal.setPreferredSize(new Dimension(90, 16));
			jlAvgRxSpeedVal.setText(Strings.getString("n_a"));
			jlAvgTxSpeed = new JLabel();
			jlAvgTxSpeed.setPreferredSize(new java.awt.Dimension(150,16));
			jlAvgTxSpeed.setText(Strings.getString("ProgressFrame.AvgTxSpeed")); //$NON-NLS-1$
			jlAvgTxSpeed.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
			jlAvgTxSpeedVal = new JLabel();
			jlAvgTxSpeedVal.setPreferredSize(new java.awt.Dimension(90,16));
			jlAvgTxSpeedVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
			jContentPane = new JPanel();			
			jContentPane.setLayout(new FlowLayout());			
			jContentPane.add(jlAvgRxSpeed, null);
			jContentPane.add(jlAvgRxSpeedVal, null);			
			jContentPane.add(jlAvgTxSpeed, null);
			jContentPane.add(jlAvgTxSpeedVal, null);
			jContentPane.add(getJButton(), null);			
		}
		return jContentPane;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setText(Strings.getString("closeButton")); //$NON-NLS-1$
			jButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {
					this_windowClosing(null);
					dispose();
				}
			});
		}
		return jButton;
	}

	public void setAvgRxSpeed(int avgRxSpeed) {
		if(avgRxSpeed == -1)
			jlAvgRxSpeedVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
		else
			jlAvgRxSpeedVal.setText((avgRxSpeed / 1024) + " Kb/sec");
	}

	public void setAvgTxSpeed(int avgTxSpeed) {
		if(avgTxSpeed == -1)
			jlAvgTxSpeedVal.setText(Strings.getString("n_a")); //$NON-NLS-1$
		else		
			jlAvgTxSpeedVal.setText((avgTxSpeed / 1024) + " Kb/sec");
	}		
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
