/*
 * @(#)TextPopup.java
 *
 * Copyright (C) 2007 Sergey Bredikhin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 */

package olivamail.gui;

import java.awt.Component;
import java.awt.event.MouseEvent;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.text.JTextComponent;

public class TextPopup extends JPopupMenu {
	static final long serialVersionUID = 23332234242423L;
	private JMenuItem jmiCopy = null;
	private JMenuItem jmiPaste = null;
	private JMenuItem jmiSelAll = null;
	private JMenuItem jmiCut = null;
	private JMenuItem jmiDelete = null;
	private JTextComponent tc = null;
	
	static {
		setDefaultLightWeightPopupEnabled(false);		
	}
		
	/**
	 * This method initializes 
	 * 
	 */
	public TextPopup() {		
		super();		
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.add(getJmiCut());
        this.add(getJmiCopy());
        this.add(getJmiPaste());
        this.add(getJmiDelete());
        this.addSeparator();
        this.add(getJmiSelAll());		
	}

	/**
	 * This method initializes jmiCopy	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiCopy() {
		if (jmiCopy == null) {
			jmiCopy = new JMenuItem();
			jmiCopy.setText(Strings.getString("TextPopup.copy"));
			jmiCopy.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					tc.copy();
				}
			});
		}
		return jmiCopy;
	}

	/**
	 * This method initializes jmiPaste	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiPaste() {
		if (jmiPaste == null) {
			jmiPaste = new JMenuItem();
			jmiPaste.setText(Strings.getString("TextPopup.paste"));
			jmiPaste.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					tc.paste();
				}
			});
		}
		return jmiPaste;
	}

	/**
	 * This method initializes jmiSelAll	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiSelAll() {
		if (jmiSelAll == null) {
			jmiSelAll = new JMenuItem();
			jmiSelAll.setText(Strings.getString("TextPopup.selectAll"));
			jmiSelAll.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					tc.selectAll();
				}
			});
		}
		return jmiSelAll;
	}
	
	public static void registerComponent(final Component comp) {
		comp.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent e) {
				if (e.getButton() != MouseEvent.BUTTON3)
					return;
				TextPopup textPopup = new TextPopup();
				textPopup.tc = (JTextComponent)comp;
				boolean editable = textPopup.tc.isEditable(); 
				textPopup.jmiCut.setEnabled(editable);
				textPopup.jmiPaste.setEnabled(editable);
				textPopup.jmiDelete.setEnabled(editable);
				textPopup.show(comp, e.getX(), e.getY());
			}
		});					
	}
	
	/**
	 * This method initializes jmiCut	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiCut() {
		if (jmiCut == null) {
			jmiCut = new JMenuItem();
			jmiCut.setText(Strings.getString("TextPopup.cut"));
			jmiCut.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					tc.cut();
				}
			});
		}
		return jmiCut;
	}

	/**
	 * This method initializes jmiDelete	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */    
	private JMenuItem getJmiDelete() {
		if (jmiDelete == null) {
			jmiDelete = new JMenuItem();
			jmiDelete.setText(Strings.getString("TextPopup.delete"));
			jmiDelete.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					tc.cut();
				}
			});
		}
		return jmiDelete;
	}	
		
}
